import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.Vector;
import java.io.*;
import itcr.mathexplorer.constantes.*;
import itcr.mathexplorer.exceptions.*;
import itcr.mathexplorer.funciones.*;
import itcr.mathexplorer.operadores.*;
import itcr.mathexplorer.tablas.*;
import itcr.mathexplorer.herramientas.Opciones;
import itcr.mathexplorer.paneles.*;
import itcr.mathexplorer.expresion.*;

/**
 *@(#)MathExplorer.java
 *
 *Programa principal de MathExplorer
 *
 *Este programa es un explorador matemático donde el usuario
 *puede digitar cualquier expresión matemática y realizar operaciones
 *y cálculos con ella.
 *
 *Autores:
 *
 *M.Sc. Alexander Borbón Alpízar.
 *M.Sc. Cristhian Páez Páez.
 *M.Sc. Walter Mora Flores
 *Instituto Tecnológico de Costa Rica.
 *Escuela de Matemática.
 *Cartago, Costa Rica.
 *Tel: (506)2550-2225
 *Fax: (506)2550-2493
 *Correos electrónicos:
 *aborbon@itcr.ac.cr
 *cpaez@itcr.ac.cr
 *wmora2@yahoo.com
 *
 *@author Alexander Borbón Alpízar
 *@author Cristhian Páez Páez
 *@author Walter Mora Flores
 *
 *@version %I%, %G%
 */
public class MathExplorer extends JFrame implements ActionListener, InternalFrameListener{
	static final long serialVersionUID=10000;
	
	//Vector que maneja la dirección de todas las ventanas
	Vector<VentanaInternaExpresiones> ventanas;

	//Esta es la tabla que contiene a todas las funciones permitidas
	TablaFunciones funciones;
	
	//Esta es la tabla que contiene a todas las constantes permitidas
	TablaConstantes constantes;
	
	//Esta es la tabla que contiene a todos los operadores permitidos
	TablaOperadores operadores;
	
	//Esta es la tabla que contiene a todas las funciones definidas por el usuario
	TablaFuncionesUsuario funcionesUsuario;
	
	//Elementos del menú
	JMenuBar elMenu;
	JMenu menuArchivo, menuEdicion, menuFunciones, menuGraficar, menuGraficar3D, menuOpciones, menuVentanas, menuAyuda;
	JMenuItem menuNuevo, menuAbrir, menuGuardar, menuGuardarComo, menuSalir;
	JMenuItem menuCortar, menuCopiar, menuPegar, menuSeleccionarTodo;
	JMenu menuConstantes;
	JMenuItem menuPi, menuE;
	JMenu menuOperadores;
	JMenuItem menuSuma, menuResta, menuMultiplicacion, menuDivision, menuPotencia,
		menuPorcentaje, menuFactorial;
	JMenu menuOperadoresLogicos;
	JMenuItem menuIgual, menuDistinto, menuMenor, menuMenorIgual, menuMayor, menuMayorIgual,
		menuY, menuO, menuXO, menuNegacion;
	JMenu menuFuncionesAlgebraicas;
	JMenuItem menuValorAbsoluto, menuLogaritmo, menuModulo, menuRandom, menuRaiz, menuSigno;
	JMenu menuFuncionesTrigonometricas;
	JMenuItem menuSen, menuCos, menuTan, menuSec, menuCsc,
		menuCot, menuASen, menuACos, menuATan, menuASec, menuACsc,
		menuACot, menuSenH, menuCosH, menuTanH, menuSecH, menuCscH,
		menuCotH;
	JMenu menuTeoriaNumeros;
	JMenuItem menuEsPrimo, menuPrimo, menuPrimos, menuFactorNumero, menuFuncionPi, 
		menuRestoChino, menuCoeficientes;
	JMenu menuMetodosNumericos;
	JMenuItem menuParteEntera, menuRedondear, menuTruncar;
	JMenu menuMatricesConjuntos;
	JMenuItem menuDeterminante, menuInversa, menuReduccionFilas, menuTranspuesta, menuUnion,
		menuInterseccion, menufij, menufikfj, menukfi, menuNuevaMatriz;
	JMenu menuMatematica;
	JMenuItem menuEvaluar, menuDerivar;
	JMenu menuFuncionesLogicas;
	JMenuItem menuEsEntero, menuEsNumero, menuEsMonomio, menuEsPolinomio, menuSi;
	JMenuItem menuDefinirConstante, menuDefinirFuncion;
	JMenuItem menuGraficarActual, menuBorraActual, menuBorraTodas, menuZoomIn, 
		menuZoomInV, menuZoomInH, menuZoomOut, menuZoomOutV, menuZoomOutH;
	JMenu menuGraficar3DZoomInPrincipal, menuGraficar3DZoomOutPrincipal;
	JMenuItem menuGraficar3DActual, menuGraficar3DBorraActual, menuGraficar3DBorraTodas, 
		menuGraficar3DZoomIn, menuGraficar3DZoomInX, menuGraficar3DZoomInY,
		menuGraficar3DZoomInZ, menuGraficar3DZoomOut, menuGraficar3DZoomOutX,
		menuGraficar3DZoomOutY, menuGraficar3DZoomOutZ;
	JMenuItem menuOpcionesGenerales;
	JMenuItem menuCascada, menuMosaicoHorizontal, menuMosaicoVertical, menuIconos, 
		      menuSiguiente, menuAnterior;
	JMenuItem menuIndice, menuContenido, menuAcercaDe;
	
	//Marcos principales del programa
	JPanel marcoPrincipal1, marcoPrincipal2, marcoPrincipal3;
	
	//Ventana Interna de Expresiones que está actualmente seleccionada
	VentanaInternaExpresiones ventanaActual;
	VentanaGraficos2D Graficos2D; //Ventana para mostrar los gráficos en 2D
	VentanaGraficos3D Graficos3D; //Ventana para mostrar los gráficos en 3D
	
	
	//Barras de herramientas
	JToolBar barraDeHerramientas, barraDeUtilidades, barraDeEstado, barraDeGraficos, barraDeGraficos3D;
	
	//Elementos de la barra de herramientas y de las barras de gráficos
	JButton barraDeHerramientasNuevo, barraDeHerramientasAbrir, barraDeHerramientasGuardar, 
			barraDeHerramientasCortar, barraDeHerramientasCopiar, barraDeHerramientasPegar, 
			barraDeHerramientasBorrar, barraDeHerramientasGraficar, barraDeHerramientasGraficar3D;
	JButton barraDeGraficosGraficar, barraDeGraficosBorraGrafica, barraDeGraficosBorraTodas, 
		barraDeGraficosZoomIn, barraDeGraficosZoomInV, barraDeGraficosZoomInH, 
		barraDeGraficosZoomOut, barraDeGraficosZoomOutV, barraDeGraficosZoomOutH;
	JButton barraDeGraficosGraficar3D, barraDeGraficosBorraGrafica3D, barraDeGraficosBorraTodas3D, 
		barraDeGraficosZoomIn3D, barraDeGraficosZoomOut3D;
	
	//Elementos de la barra de utilidades
	JTextField barraDeUtilidadesExpresion;
	JButton barraDeUtilidadesAceptar, barraDeUtilidadesCancelar;
	
	//Elementos de la barra de estado
	JLabel barraDeEstadoMensajes;
	
	//Dirección donde se guarda el directorio de trabajo del programa.
	String directorioPrograma;
	
	//Iconos
	ImageIcon icono, iconoNuevo, iconoAbrir, iconoGuardar, iconoCortar, iconoCopiar, 
		iconoPegar, iconoBorraExpresion, iconoGraficar, iconoBorraGrafico, iconoBorraTodas, 
		iconoZoomIn, iconoZoomInV, iconoZoomInH, iconoZoomOut, iconoZoomOutV, iconoZoomOutH,
		iconoAceptar, iconoCancelar, iconoGraficar3D, iconoBorraGrafico3D, iconoBorraTodas3D, 
		iconoZoomIn3D, iconoZoomOut3D;
	
	//El panel que maneja los JDesktopPane con las expresiones
	JDesktopPane panelPrincipal;
	
	//Este panel
	
	JFrame esteJFrame;
	
	/**
	 * Constructor vacío que genera una nueva
	 * ventana del programa MathExplorer
	 */
	public MathExplorer(){
		try{
			this.setTitle("Math Explorer");
			this.pack();
			this.setSize(640, 480);
			this.setLocationRelativeTo(null);
			this.setLayout(new BorderLayout());
			//this.setBackground(fondo);
			
			directorioPrograma=System.getProperty("user.dir");
			
			ventanas=new Vector<VentanaInternaExpresiones>();
			
			//Llena las tablas con las funciones, los operadores y las constantes
			llenarTablas();
			
			//Carga de íconos
			icono=new ImageIcon(directorioPrograma+"/images/Icono.gif");
			iconoNuevo=new ImageIcon(directorioPrograma+"/images/Nuevo.gif");
			iconoAbrir=new ImageIcon(directorioPrograma+"/images/Abrir.gif");
			iconoGuardar=new ImageIcon(directorioPrograma+"/images/Guardar.gif");
			iconoCortar=new ImageIcon(directorioPrograma+"/images/Cortar.gif");
			iconoCopiar=new ImageIcon(directorioPrograma+"/images/Copiar.gif");
			iconoPegar=new ImageIcon(directorioPrograma+"/images/Pegar.gif");
			iconoBorraExpresion=new ImageIcon(directorioPrograma+"/images/BorraExp.gif");
			iconoGraficar=new ImageIcon(directorioPrograma+"/images/Graficar.gif");
			iconoBorraGrafico=new ImageIcon(directorioPrograma+"/images/BorraGrafico.gif");
			iconoBorraTodas=new ImageIcon(directorioPrograma+"/images/BorraTodas.gif");
			iconoZoomIn=new ImageIcon(directorioPrograma+"/images/ZoomIn.gif");
			iconoZoomInV=new ImageIcon(directorioPrograma+"/images/ZoomInV.gif");
			iconoZoomInH=new ImageIcon(directorioPrograma+"/images/ZoomInH.gif");
			iconoZoomOut=new ImageIcon(directorioPrograma+"/images/ZoomOut.gif");
			iconoZoomOutV=new ImageIcon(directorioPrograma+"/images/ZoomOutV.gif");
			iconoZoomOutH=new ImageIcon(directorioPrograma+"/images/ZoomOutH.gif");
			iconoAceptar=new ImageIcon(directorioPrograma+"/images/Aceptar.gif");
			iconoCancelar=new ImageIcon(directorioPrograma+"/images/Cancelar.gif");
			iconoGraficar3D=new ImageIcon(directorioPrograma+"/images/Graficar3D.gif");
			iconoBorraGrafico3D=new ImageIcon(directorioPrograma+"/images/BorraGrafico3D.gif");
			iconoBorraTodas3D=new ImageIcon(directorioPrograma+"/images/BorraTodas3D.gif");
			iconoZoomIn3D=new ImageIcon(directorioPrograma+"/images/ZoomIn3D.gif");
			iconoZoomOut3D=new ImageIcon(directorioPrograma+"/images/ZoomOut3D.gif");
			
			this.setIconImage(icono.getImage());
			
			//CREACION DEL MENU
			
			crearMenu();
			
			//CREACION DE LAS BARRAS DE HERRAMIENTAS
			
			crearBarraDeHerramientas();
			crearBarraDeGraficos();
			crearBarraDeGraficos3D();
			crearBarraDeUtilidades();
			crearBarraDeEstado();
						
			panelPrincipal = new JDesktopPane();
			
			//Esta es la primera ventana interna que se crea
			ventanaNueva();
			
			marcoPrincipal1=new JPanel();
			marcoPrincipal1.setLayout(new BorderLayout());
			
			marcoPrincipal2=new JPanel();
			marcoPrincipal2.setLayout(new BorderLayout());
			
			marcoPrincipal3=new JPanel();
			marcoPrincipal3.setLayout(new BorderLayout());
			
			marcoPrincipal3.add(panelPrincipal, BorderLayout.CENTER);
			marcoPrincipal3.add(barraDeGraficos3D, BorderLayout.EAST);
			
			marcoPrincipal2.add(barraDeUtilidades, BorderLayout.NORTH);
			marcoPrincipal2.add(barraDeGraficos, BorderLayout.EAST);
			marcoPrincipal2.add(marcoPrincipal3, BorderLayout.CENTER);
			
			marcoPrincipal1.add(barraDeHerramientas, BorderLayout.NORTH);
			marcoPrincipal1.add(marcoPrincipal2,BorderLayout.CENTER);
			
			this.add(elMenu, BorderLayout.NORTH);
			this.add(barraDeEstado, BorderLayout.SOUTH);
			this.add(marcoPrincipal1,BorderLayout.CENTER);
			
			this.setVisible(true);
			
			barraDeUtilidadesExpresion.requestFocus();
			
			this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
			
			this.addWindowListener( new WindowAdapter() {
	      		public void windowClosing( WindowEvent evt ){
					salirPrograma();
	      		}
	    	} );
	    	
	    	esteJFrame=this;
		}catch(Exception e){
			e.printStackTrace();
			JOptionPane.showMessageDialog(this,"Se produjo un error al inicializar el programa","Error al iniciar",JOptionPane.ERROR_MESSAGE);
			System.exit(0);
		}
	}
	
	/**
	 * Función principal de la clase MathExplorer
	 * @param argv
	 */
	public static void main( String argv[] ) {
		new MathExplorer();
	}
	
	
	//FUNCION PARA LA CREACION DE LAS BARRAS DE HERRAMIENTAS

	private void crearBarraDeHerramientas() {
		barraDeHerramientas=new JToolBar();
		barraDeHerramientas.setRollover(true);
		
		barraDeHerramientasNuevo=new JButton(iconoNuevo);
		barraDeHerramientasNuevo.setToolTipText("Crear un documento nuevo");
		barraDeHerramientasNuevo.addActionListener(this);
		
		barraDeHerramientasAbrir=new JButton(iconoAbrir);
		barraDeHerramientasAbrir.setToolTipText("Abrir un documento guardado");
		barraDeHerramientasAbrir.addActionListener(this);
		
		barraDeHerramientasGuardar=new JButton(iconoGuardar);
		barraDeHerramientasGuardar.setToolTipText("Guardar el documento activo");
		barraDeHerramientasGuardar.addActionListener(this);

		barraDeHerramientasCortar=new JButton(iconoCortar);
		barraDeHerramientasCortar.setToolTipText("Cortar las expresiones seleccionadas");
		barraDeHerramientasCortar.addActionListener(this);
		barraDeHerramientasCortar.setEnabled(false);

		barraDeHerramientasCopiar=new JButton(iconoCopiar);
		barraDeHerramientasCopiar.setToolTipText("Copiar las expresiones seleccionadas");
		barraDeHerramientasCopiar.addActionListener(this);
		barraDeHerramientasCopiar.setEnabled(false);
		
		barraDeHerramientasPegar=new JButton(iconoPegar);
		barraDeHerramientasPegar.setToolTipText("Pegar las expresiones copiadas");
		barraDeHerramientasPegar.addActionListener(this);
		barraDeHerramientasPegar.setEnabled(false);
		
		barraDeHerramientasBorrar=new JButton(iconoBorraExpresion);
		barraDeHerramientasBorrar.setToolTipText("Borra las expresiones seleccionadas");
		barraDeHerramientasBorrar.addActionListener(this);
		barraDeHerramientasBorrar.setEnabled(false);
		
		barraDeHerramientasGraficar=new JButton(iconoGraficar);
		barraDeHerramientasGraficar.setToolTipText("Grafica la expresión actual");
		barraDeHerramientasGraficar.addActionListener(this);
		
		barraDeHerramientasGraficar3D=new JButton(iconoGraficar3D);
		barraDeHerramientasGraficar3D.setToolTipText("Grafica la expresión actual en 3D");
		barraDeHerramientasGraficar3D.addActionListener(this);
		
		barraDeHerramientas.add(barraDeHerramientasNuevo);
		barraDeHerramientas.addSeparator();
		barraDeHerramientas.add(barraDeHerramientasAbrir);
		barraDeHerramientas.add(barraDeHerramientasGuardar);
		barraDeHerramientas.addSeparator();
		barraDeHerramientas.add(barraDeHerramientasCortar);
		barraDeHerramientas.add(barraDeHerramientasCopiar);
		barraDeHerramientas.add(barraDeHerramientasPegar);
		barraDeHerramientas.add(barraDeHerramientasBorrar);
		barraDeHerramientas.addSeparator();
		barraDeHerramientas.add(barraDeHerramientasGraficar);
		barraDeHerramientas.add(barraDeHerramientasGraficar3D);
		//barraDeHerramientas.add(barraDeHerramientasBorrarGrafico);
		
	}
	
	private void crearBarraDeGraficos(){
		barraDeGraficos=new JToolBar();
		barraDeGraficos.setRollover(true);
		
		barraDeGraficosGraficar=new JButton(iconoGraficar);
		barraDeGraficosGraficar.setToolTipText("Grafica la expresión actual");
		barraDeGraficosGraficar.addActionListener(this);
		
		barraDeGraficosBorraGrafica=new JButton(iconoBorraGrafico);
		barraDeGraficosBorraGrafica.setToolTipText("Borra la última gráfica realizada.");
		barraDeGraficosBorraGrafica.addActionListener(this);
		
		barraDeGraficosBorraTodas=new JButton(iconoBorraTodas);
		barraDeGraficosBorraTodas.setToolTipText("Borra todas las gráficas realizadas.");
		barraDeGraficosBorraTodas.addActionListener(this);
		
		barraDeGraficosZoomIn=new JButton(iconoZoomIn);
		barraDeGraficosZoomIn.setToolTipText("Hace zoom hacia adentro.");
		barraDeGraficosZoomIn.addActionListener(this);
		
		barraDeGraficosZoomInV=new JButton(iconoZoomInV);
		barraDeGraficosZoomInV.setToolTipText("Hace zoom vertical hacia adentro.");
		barraDeGraficosZoomInV.addActionListener(this);
		
		barraDeGraficosZoomInH=new JButton(iconoZoomInH);
		barraDeGraficosZoomInH.setToolTipText("Hace zoom horizontal hacia adentro.");
		barraDeGraficosZoomInH.addActionListener(this);
		
		barraDeGraficosZoomOut=new JButton(iconoZoomOut);
		barraDeGraficosZoomOut.setToolTipText("Hace zoom hacia afuera.");
		barraDeGraficosZoomOut.addActionListener(this);
		
		barraDeGraficosZoomOutV=new JButton(iconoZoomOutV);
		barraDeGraficosZoomOutV.setToolTipText("Hace zoom vertical hacia afuera.");
		barraDeGraficosZoomOutV.addActionListener(this);
		
		barraDeGraficosZoomOutH=new JButton(iconoZoomOutH);
		barraDeGraficosZoomOutH.setToolTipText("Hace zoom horizontal hacia afuera.");
		barraDeGraficosZoomOutH.addActionListener(this);
		
		barraDeGraficos.add(barraDeGraficosGraficar);
		barraDeGraficos.addSeparator();
		barraDeGraficos.add(barraDeGraficosBorraGrafica);
		barraDeGraficos.add(barraDeGraficosBorraTodas);
		barraDeGraficos.addSeparator();
		barraDeGraficos.add(barraDeGraficosZoomIn);
		barraDeGraficos.add(barraDeGraficosZoomInV);
		barraDeGraficos.add(barraDeGraficosZoomInH);
		barraDeGraficos.addSeparator();
		barraDeGraficos.add(barraDeGraficosZoomOut);
		barraDeGraficos.add(barraDeGraficosZoomOutV);
		barraDeGraficos.add(barraDeGraficosZoomOutH);
		
		
		barraDeGraficos.setOrientation(JToolBar.VERTICAL);
		barraDeGraficos.setVisible(false);
	}
	
	private void crearBarraDeGraficos3D(){
		barraDeGraficos3D=new JToolBar();
		barraDeGraficos3D.setRollover(true);
		
		barraDeGraficosGraficar3D=new JButton(iconoGraficar3D);
		barraDeGraficosGraficar3D.setToolTipText("Grafica la expresión actual");
		barraDeGraficosGraficar3D.addActionListener(this);
		
		barraDeGraficosBorraGrafica3D=new JButton(iconoBorraGrafico3D);
		barraDeGraficosBorraGrafica3D.setToolTipText("Borra la última gráfica realizada.");
		barraDeGraficosBorraGrafica3D.addActionListener(this);
		
		barraDeGraficosBorraTodas3D=new JButton(iconoBorraTodas3D); //=new JButton(iconoBorraTodas);
		barraDeGraficosBorraTodas3D.setToolTipText("Borra todas las gráficas realizadas.");
		barraDeGraficosBorraTodas3D.addActionListener(this);
		
		barraDeGraficosZoomIn3D=new JButton(iconoZoomIn3D);
		barraDeGraficosZoomIn3D.setToolTipText("Hace zoom hacia adentro.");
		barraDeGraficosZoomIn3D.addActionListener(this);
		
		barraDeGraficosZoomOut3D=new JButton(iconoZoomOut3D);
		barraDeGraficosZoomOut3D.setToolTipText("Hace zoom hacia afuera.");
		barraDeGraficosZoomOut3D.addActionListener(this);
		
		barraDeGraficos3D.add(barraDeGraficosGraficar3D);
		barraDeGraficos3D.addSeparator();
		barraDeGraficos3D.add(barraDeGraficosBorraGrafica3D);
		barraDeGraficos3D.add(barraDeGraficosBorraTodas3D);
		barraDeGraficos3D.addSeparator();
		barraDeGraficos3D.add(barraDeGraficosZoomIn3D);
		barraDeGraficos3D.addSeparator();
		barraDeGraficos3D.add(barraDeGraficosZoomOut3D);
		
		
		barraDeGraficos3D.setOrientation(JToolBar.VERTICAL);
		barraDeGraficos3D.setVisible(false);
	}
	
	private void crearBarraDeUtilidades() {
		JLabel etiqueta=new JLabel(" Expresión:  ");
		
		barraDeUtilidades=new JToolBar();
		barraDeUtilidades.setRollover(true);
		
		barraDeUtilidadesExpresion=new JTextField();
		barraDeUtilidadesExpresion.setFont(new Font("Arial",Font.BOLD,14));
		barraDeUtilidadesExpresion.setToolTipText("Aquí se digitan las expresiones nuevas");
		barraDeUtilidadesExpresion.setMinimumSize(new Dimension(150,30));
		barraDeUtilidadesExpresion.addActionListener(this);
		
		barraDeUtilidadesAceptar=new JButton(iconoAceptar);
		barraDeUtilidadesAceptar.setToolTipText("Aceptar la expresión digitada");
		barraDeUtilidadesAceptar.addActionListener(this);
		
		barraDeUtilidadesCancelar=new JButton(iconoCancelar);
		barraDeUtilidadesCancelar.setToolTipText("Cancelar la expresión digitada");
		barraDeUtilidadesCancelar.addActionListener(this);
		
		barraDeUtilidades.add(etiqueta);
		barraDeUtilidades.add(barraDeUtilidadesExpresion);
		barraDeUtilidades.add(barraDeUtilidadesAceptar);
		barraDeUtilidades.add(barraDeUtilidadesCancelar);
	}
	
	private void crearBarraDeEstado() {
		barraDeEstado=new JToolBar();
		barraDeEstado.setFloatable(false);
		
		barraDeEstadoMensajes=new JLabel("Bienvenido a Math Explorer");
		
		barraDeEstado.add(barraDeEstadoMensajes);
	}

	
	//FUNCION PARA LA CREACION DEL MENU

	private void crearMenu() {
		
		String espacio=new String("        ");

		menuArchivo=new JMenu("Archivo");
		menuArchivo.setMnemonic(KeyEvent.VK_A);
		
		menuNuevo=new JMenuItem("Nuevo", iconoNuevo);
		menuNuevo.setMnemonic('N');
		menuNuevo.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, ActionEvent.CTRL_MASK));
		menuNuevo.addActionListener(this);
	
		menuAbrir=new JMenuItem("Abrir", iconoAbrir);
		menuAbrir.setMnemonic('b');
		menuAbrir.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A, ActionEvent.CTRL_MASK));
		menuAbrir.addActionListener(this);
		
		menuGuardar=new JMenuItem("Guardar", iconoGuardar);
		menuGuardar.setMnemonic('G');
		menuGuardar.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_G, ActionEvent.CTRL_MASK));
		menuGuardar.addActionListener(this);
		
		menuGuardarComo=new JMenuItem(espacio + "Guardar como...",'c');
		menuGuardarComo.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_G, ActionEvent.CTRL_MASK+ActionEvent.ALT_MASK));
		menuGuardarComo.addActionListener(this);
		
		menuSalir=new JMenuItem(espacio + "Salir",'S');
		menuSalir.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
		menuSalir.addActionListener(this);
		
		menuArchivo.add(menuNuevo);
		menuArchivo.add(menuAbrir);
		menuArchivo.add(menuGuardar);
		menuArchivo.add(menuGuardarComo);
		menuArchivo.add(new JSeparator());
		menuArchivo.add(menuSalir);

		
		menuEdicion=new JMenu("Editar");
		menuEdicion.setMnemonic(new Character('E'));
		
		menuCortar=new JMenuItem("Cortar", iconoCortar);
		menuCortar.setMnemonic('t');
		menuCortar.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, ActionEvent.CTRL_MASK));
		menuCortar.addActionListener(this);
		menuCortar.setEnabled(false);
		
		menuCopiar=new JMenuItem("Copiar", iconoCopiar);
		menuCopiar.setMnemonic('C');
		menuCopiar.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, ActionEvent.CTRL_MASK));
		menuCopiar.addActionListener(this);
		menuCopiar.setEnabled(false);
		
		menuPegar=new JMenuItem("Pegar", iconoPegar);
		menuPegar.setMnemonic('P');
		menuPegar.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V, ActionEvent.CTRL_MASK));
		menuPegar.addActionListener(this);
		menuPegar.setEnabled(false);
		
		menuSeleccionarTodo=new JMenuItem(espacio + "Seleccionar todo",'S');
		menuSeleccionarTodo.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E, ActionEvent.CTRL_MASK));
		menuSeleccionarTodo.addActionListener(this);
		menuSeleccionarTodo.setEnabled(false);
		
		menuEdicion.add(menuCortar);
		menuEdicion.add(menuCopiar);
		menuEdicion.add(menuPegar);
		menuEdicion.add(new JSeparator());
		menuEdicion.add(menuSeleccionarTodo);
		
		//Menu de Funciones
		
		menuFunciones=new JMenu("Funciones");
		menuFunciones.setMnemonic(new Character('F'));
		
		menuConstantes=new JMenu("Constantes");
		menuConstantes.setMnemonic(new Character('C'));
		
		menuPi=new JMenuItem("Pi");
		menuPi.addActionListener(this);
		
		menuE=new JMenuItem("E");
		menuE.addActionListener(this);
		
		menuConstantes.add(menuPi);
		menuConstantes.add(menuE);
		
		menuOperadores=new JMenu("Operadores");
		menuOperadores.setMnemonic(new Character('O'));
		
		menuSuma=new JMenuItem("Suma (+)...");
		menuSuma.addActionListener(this);
		
		menuResta=new JMenuItem("Resta (-)...");
		menuResta.addActionListener(this);
		
		menuMultiplicacion=new JMenuItem("Multiplicación (·)...");
		menuMultiplicacion.addActionListener(this);
		
		menuDivision=new JMenuItem("División (/)...");
		menuDivision.addActionListener(this);
		
		menuPotencia=new JMenuItem("Potencia (^)...");
		menuPotencia.addActionListener(this);
		
		menuPorcentaje=new JMenuItem("Porcentaje (%)...");
		menuPorcentaje.addActionListener(this);
		
		menuFactorial=new JMenuItem("Factorial (!)...");
		menuFactorial.addActionListener(this);
		
		menuOperadores.add(menuSuma);
		menuOperadores.add(menuResta);
		menuOperadores.add(menuMultiplicacion);
		menuOperadores.add(menuDivision);
		menuOperadores.add(menuPotencia);
		menuOperadores.add(menuPotencia);
		menuOperadores.add(menuFactorial);
		
		menuOperadoresLogicos=new JMenu("Operadores Lógicos");
		menuOperadores.setMnemonic(new Character('L'));
		
		menuIgual=new JMenuItem("Igual (=)...");
		menuIgual.addActionListener(this);
		
		menuDistinto=new JMenuItem("Distinto (<>)...");
		menuDistinto.addActionListener(this);
		
		menuMenor=new JMenuItem("Menor que (<)...");
		menuMenor.addActionListener(this);
		
		menuMenorIgual=new JMenuItem("Menor o igual que (<=)...");
		menuMenorIgual.addActionListener(this);
		
		menuMayor=new JMenuItem("Mayor que (>)...");
		menuMayor.addActionListener(this);
		
		menuMayorIgual=new JMenuItem("Mayor o igual que (>=)...");
		menuMayorIgual.addActionListener(this);
		
		menuY=new JMenuItem("Y (and)...");
		menuY.addActionListener(this);
		
		menuO=new JMenuItem("O (or)...");
		menuO.addActionListener(this);
		
		menuXO=new JMenuItem("XO (xor)...");
		menuXO.addActionListener(this);
		
		menuNegacion=new JMenuItem("No (not)...");
		menuNegacion.addActionListener(this);
		
		menuOperadoresLogicos.add(menuIgual);
		menuOperadoresLogicos.add(menuDistinto);
		menuOperadoresLogicos.add(menuMenor);
		menuOperadoresLogicos.add(menuMenorIgual);
		menuOperadoresLogicos.add(menuMayor);
		menuOperadoresLogicos.add(menuMayorIgual);
		menuOperadoresLogicos.add(new JSeparator());
		menuOperadoresLogicos.add(menuY);
		menuOperadoresLogicos.add(menuO);
		menuOperadoresLogicos.add(menuXO);
		menuOperadoresLogicos.add(menuNegacion);
		
		menuFuncionesAlgebraicas=new JMenu("Funciones Algebraicas");
		menuFuncionesAlgebraicas.setMnemonic(new Character('A'));
		
		menuValorAbsoluto=new JMenuItem("Valor Absoluto...");
		menuValorAbsoluto.addActionListener(this);
		
		menuRaiz=new JMenuItem("Raíz cuadrada...");
		menuRaiz.addActionListener(this);
		
		menuModulo=new JMenuItem("Resto de la división...");
		menuModulo.addActionListener(this);
		
		menuSigno=new JMenuItem("Signo...");
		menuSigno.addActionListener(this);
		
		menuLogaritmo=new JMenuItem("Logaritmo...");
		menuLogaritmo.addActionListener(this);
		
		menuRandom=new JMenuItem("Numero Aleatorio");
		menuRandom.addActionListener(this);
		
		menuFuncionesAlgebraicas.add(menuValorAbsoluto);
		menuFuncionesAlgebraicas.add(menuRaiz);
		menuFuncionesAlgebraicas.add(menuModulo);
		menuFuncionesAlgebraicas.add(menuSigno);
		menuFuncionesAlgebraicas.add(menuLogaritmo);
		menuFuncionesAlgebraicas.add(menuRandom);
		
		menuFuncionesTrigonometricas=new JMenu("Funciones Trigonométricas");
		menuFuncionesTrigonometricas.setMnemonic(new Character('T'));
		
		menuSen=new JMenuItem("Seno...");
		menuSen.addActionListener(this);
		
		menuCos=new JMenuItem("Coseno...");
		menuCos.addActionListener(this);
		
		menuTan=new JMenuItem("Tangente...");
		menuTan.addActionListener(this);
		
		menuSec=new JMenuItem("Secante...");
		menuSec.addActionListener(this);
		
		menuCsc=new JMenuItem("Cosecante...");
		menuCsc.addActionListener(this);
		
		menuCot=new JMenuItem("Cotangente...");
		menuCot.addActionListener(this);
		
		menuASen=new JMenuItem("Arcoseno...");
		menuASen.addActionListener(this);
		
		menuACos=new JMenuItem("Arcocoseno...");
		menuACos.addActionListener(this);
		
		menuATan=new JMenuItem("Arcotangente...");
		menuATan.addActionListener(this);
		
		menuASec=new JMenuItem("Arcosecante...");
		menuASec.addActionListener(this);
		
		menuACsc=new JMenuItem("Arcocosecante...");
		menuACsc.addActionListener(this);
		
		menuACot=new JMenuItem("Arcocotangente...");
		menuACot.addActionListener(this);
		
		menuSenH=new JMenuItem("Seno Hiperbólico...");
		menuSenH.addActionListener(this);
		
		menuCosH=new JMenuItem("Coseno Hiperbólico...");
		menuCosH.addActionListener(this);
		
		menuTanH=new JMenuItem("Tangente Hiperbólico...");
		menuTanH.addActionListener(this);
		
		menuSecH=new JMenuItem("Secante Hiperbólico...");
		menuSecH.addActionListener(this);
		
		menuCscH=new JMenuItem("Cosecante Hiperbólico...");
		menuCscH.addActionListener(this);
		
		menuCotH=new JMenuItem("Cotangente Hiperbólico...");
		menuCotH.addActionListener(this);
		
		menuFuncionesTrigonometricas.add(menuSen);
		menuFuncionesTrigonometricas.add(menuCos);
		menuFuncionesTrigonometricas.add(menuTan);
		menuFuncionesTrigonometricas.add(menuSec);
		menuFuncionesTrigonometricas.add(menuCsc);
		menuFuncionesTrigonometricas.add(menuCot);
		menuFuncionesTrigonometricas.add(new JSeparator());
		menuFuncionesTrigonometricas.add(menuASen);
		menuFuncionesTrigonometricas.add(menuACos);
		menuFuncionesTrigonometricas.add(menuATan);
		menuFuncionesTrigonometricas.add(menuASec);
		menuFuncionesTrigonometricas.add(menuACsc);
		menuFuncionesTrigonometricas.add(menuACot);
		menuFuncionesTrigonometricas.add(new JSeparator());
		menuFuncionesTrigonometricas.add(menuSenH);
		menuFuncionesTrigonometricas.add(menuCosH);
		menuFuncionesTrigonometricas.add(menuTanH);
		menuFuncionesTrigonometricas.add(menuSecH);
		menuFuncionesTrigonometricas.add(menuCscH);
		menuFuncionesTrigonometricas.add(menuCotH);
		
		menuMatematica=new JMenu("Matemática");
		menuMatematica.setMnemonic(new Character('M'));
		
		menuEvaluar=new JMenuItem("Evaluar...", 'E');
		menuEvaluar.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E, ActionEvent.CTRL_MASK+ActionEvent.ALT_MASK));
		menuEvaluar.addActionListener(this);
		
		menuDerivar=new JMenuItem("Derivar...", 'D');
		menuDerivar.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D, ActionEvent.CTRL_MASK+ActionEvent.ALT_MASK));
		menuDerivar.addActionListener(this);
		
		menuMatematica.add(menuEvaluar);
		menuMatematica.add(menuDerivar);
		
		menuTeoriaNumeros=new JMenu("Teoría de Números");
		
		menuEsPrimo=new JMenuItem("¿Es primo?...");
		menuEsPrimo.addActionListener(this);
		
		menuPrimo=new JMenuItem("Primo número...");
		menuPrimo.addActionListener(this);
		
		menuPrimos=new JMenuItem("Primos entre...");
		menuPrimos.addActionListener(this);
		
		menuFactorNumero=new JMenuItem("Factorizar número...");
		menuFactorNumero.addActionListener(this);
		
		menuFuncionPi=new JMenuItem("Pi...");
		menuFuncionPi.addActionListener(this);
		
		menuRestoChino=new JMenuItem("Teorema Resto Chino...");
		menuRestoChino.addActionListener(this);
		
		menuCoeficientes=new JMenuItem("Coeficientes del polinomio...");
		menuCoeficientes.addActionListener(this);
		
		menuTeoriaNumeros.add(menuEsPrimo);
		menuTeoriaNumeros.add(menuPrimo);
		menuTeoriaNumeros.add(menuPrimos);
		menuTeoriaNumeros.add(menuFactorNumero);
		menuTeoriaNumeros.add(menuFuncionPi);
		menuTeoriaNumeros.add(menuRestoChino);
		menuTeoriaNumeros.add(menuCoeficientes);
		
		menuMetodosNumericos=new JMenu("Métodos Numéricos");
		
		menuParteEntera=new JMenuItem("Parte entera...");
		menuParteEntera.addActionListener(this);
		
		menuRedondear=new JMenuItem("Redondear...");
		menuRedondear.addActionListener(this);
		
		menuTruncar=new JMenuItem("Truncar...");
		menuTruncar.addActionListener(this);
		
		menuMetodosNumericos.add(menuParteEntera);
		menuMetodosNumericos.add(menuRedondear);
		menuMetodosNumericos.add(menuTruncar);
		
		menuFuncionesLogicas=new JMenu("Funciones de Lógica");
		
		menuEsEntero=new JMenuItem("¿Es entero?...");
		menuEsEntero.addActionListener(this);
		
		menuEsNumero=new JMenuItem("¿Es un número?...");
		menuEsNumero.addActionListener(this);
		
		menuEsMonomio=new JMenuItem("¿Es un monomio?...");
		menuEsMonomio.addActionListener(this);
		
		menuEsPolinomio=new JMenuItem("¿Es un polinomio?...");
		menuEsPolinomio.addActionListener(this);
		
		menuSi=new JMenuItem("Si (if)...");
		menuSi.addActionListener(this);
		
		menuFuncionesLogicas.add(menuEsEntero);
		menuFuncionesLogicas.add(menuEsNumero);
		menuFuncionesLogicas.add(menuEsMonomio);
		menuFuncionesLogicas.add(menuEsPolinomio);
		menuFuncionesLogicas.add(new JSeparator());
		menuFuncionesLogicas.add(menuSi);
		
		menuMatricesConjuntos=new JMenu("Matrices y conjuntos");
		
		menuDeterminante=new JMenuItem("Determinante...");
		menuDeterminante.addActionListener(this);
		
		menuInversa=new JMenuItem("Inversa...");
		menuInversa.addActionListener(this);
		
		menuReduccionFilas=new JMenuItem("Reducción de filas...");
		menuReduccionFilas.addActionListener(this);
		
		menuTranspuesta=new JMenuItem("Transpuesta...");
		menuTranspuesta.addActionListener(this);
		
		menuUnion=new JMenuItem("Unión...");
		menuUnion.addActionListener(this);
		
		menuInterseccion=new JMenuItem("Intersección...");
		menuInterseccion.addActionListener(this);
		
		menufij=new JMenuItem("Intercambio de filas...");
		menufij.addActionListener(this);
		
		menufikfj=new JMenuItem("Fila i mas k por fila j...");
		menufikfj.addActionListener(this);
		
		menukfi=new JMenuItem("k por Fila i...");
		menukfi.addActionListener(this);
		
		menuNuevaMatriz=new JMenuItem("Nueva Matriz...");
		menuNuevaMatriz.addActionListener(this);
		
		menuMatricesConjuntos.add(menuDeterminante);
		menuMatricesConjuntos.add(menuTranspuesta);
		menuMatricesConjuntos.add(menuInversa);
		menuMatricesConjuntos.add(new JSeparator());
		menuMatricesConjuntos.add(menuReduccionFilas);
		menuMatricesConjuntos.add(menufij);
		menuMatricesConjuntos.add(menufikfj);
		menuMatricesConjuntos.add(menukfi);
		menuMatricesConjuntos.add(new JSeparator());
		menuMatricesConjuntos.add(menuUnion);
		menuMatricesConjuntos.add(menuInterseccion);
		menuMatricesConjuntos.add(new JSeparator());
		menuMatricesConjuntos.add(menuNuevaMatriz);
		
		menuDefinirConstante=new JMenuItem("Definir constante...");
		menuDefinirConstante.addActionListener(this);
		
		menuDefinirFuncion=new JMenuItem("Definir función...");
		menuDefinirFuncion.addActionListener(this);
		
		menuFunciones.add(menuConstantes);
		menuFunciones.add(new JSeparator());
		menuFunciones.add(menuOperadores);
		menuFunciones.add(menuOperadoresLogicos);
		menuFunciones.add(new JSeparator());
		menuFunciones.add(menuFuncionesAlgebraicas);
		menuFunciones.add(menuFuncionesTrigonometricas);
		menuFunciones.add(menuTeoriaNumeros);
		menuFunciones.add(menuMetodosNumericos);
		menuFunciones.add(menuFuncionesLogicas);
		menuFunciones.add(menuMatricesConjuntos);
		menuFunciones.add(new JSeparator());
		menuFunciones.add(menuMatematica);
		menuFunciones.add(new JSeparator());
		menuFunciones.add(menuDefinirConstante);
		menuFunciones.add(menuDefinirFuncion);
		
		//Menú de graficar
		
		menuGraficar=new JMenu("Graficar");
		menuGraficar.setMnemonic(new Character('G'));
		
		menuGraficarActual=new JMenuItem("Graficar", 'G');
		menuGraficarActual.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_G, ActionEvent.CTRL_MASK+ActionEvent.SHIFT_MASK));
		menuGraficarActual.addActionListener(this);
		
		menuBorraActual=new JMenuItem("Borrar", 'B');
		menuBorraActual.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_B, ActionEvent.CTRL_MASK+ActionEvent.SHIFT_MASK));
		menuBorraActual.addActionListener(this);
		
		menuBorraTodas=new JMenuItem("Borrar Todas", 'T');
		menuBorraTodas.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_T, ActionEvent.CTRL_MASK+ActionEvent.SHIFT_MASK));
		menuBorraTodas.addActionListener(this);
		
		menuZoomIn=new JMenuItem("Acercar", 'Z');
		menuZoomIn.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, ActionEvent.CTRL_MASK+ActionEvent.SHIFT_MASK));
		menuZoomIn.addActionListener(this);
		 
		menuZoomInV=new JMenuItem("Acercar Vertical", 'V');
		menuZoomInV.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V, ActionEvent.CTRL_MASK+ActionEvent.SHIFT_MASK));
		menuZoomInV.addActionListener(this);
		
		menuZoomInH=new JMenuItem("Acercar Horizontal", 'H');
		menuZoomInH.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_H, ActionEvent.CTRL_MASK+ActionEvent.SHIFT_MASK));
		menuZoomInH.addActionListener(this);
		
		menuZoomOut=new JMenuItem("Alejar", 'O');
		menuZoomOut.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.CTRL_MASK+ActionEvent.SHIFT_MASK));
		menuZoomOut.addActionListener(this);
		
		menuZoomOutV=new JMenuItem("Alejar Vertical", 'e');
		menuZoomOutV.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E, ActionEvent.CTRL_MASK+ActionEvent.SHIFT_MASK));
		menuZoomOutV.addActionListener(this);
		
		menuZoomOutH=new JMenuItem("Alejar Horizontal", 'r');
		menuZoomOutH.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R, ActionEvent.CTRL_MASK+ActionEvent.SHIFT_MASK));
		menuZoomOutH.addActionListener(this);
		
		menuGraficar.add(menuGraficarActual);
		menuGraficar.add(new JSeparator());
		menuGraficar.add(menuBorraActual);
		menuGraficar.add(menuBorraTodas);
		menuGraficar.add(new JSeparator());
		menuGraficar.add(menuZoomIn);
		menuGraficar.add(menuZoomInV);
		menuGraficar.add(menuZoomInH);
		menuGraficar.add(menuZoomOut);
		menuGraficar.add(menuZoomOutV);
		menuGraficar.add(menuZoomOutH);
		
		//Menú de graficar en 3D
		
		menuGraficar3D=new JMenu("Graficar 3D");
		menuGraficar3D.setMnemonic(new Character('3'));
		
		menuGraficar3DActual=new JMenuItem("Graficar", 'G');
		menuGraficar3DActual.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_G, ActionEvent.CTRL_MASK+ActionEvent.SHIFT_MASK));
		menuGraficar3DActual.addActionListener(this);
		
		menuGraficar3DBorraActual=new JMenuItem("Borrar", 'B');
		menuGraficar3DBorraActual.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_B, ActionEvent.CTRL_MASK+ActionEvent.SHIFT_MASK));
		menuGraficar3DBorraActual.addActionListener(this);
		
		menuGraficar3DBorraTodas=new JMenuItem("Borrar Todas", 'T');
		menuGraficar3DBorraTodas.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_T, ActionEvent.CTRL_MASK+ActionEvent.SHIFT_MASK));
		menuGraficar3DBorraTodas.addActionListener(this);
		
		menuGraficar3DZoomInPrincipal=new JMenu("Acercar");
		
		menuGraficar3DZoomIn=new JMenuItem("Acercar", 'Z');
		menuGraficar3DZoomIn.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, ActionEvent.CTRL_MASK+ActionEvent.SHIFT_MASK));
		menuGraficar3DZoomIn.addActionListener(this);
		
		menuGraficar3DZoomInX=new JMenuItem("Acercar en X");
		menuGraficar3DZoomInX.addActionListener(this);
		
		menuGraficar3DZoomInY=new JMenuItem("Acercar en Y");
		menuGraficar3DZoomInY.addActionListener(this);
		
		menuGraficar3DZoomInZ=new JMenuItem("Acercar en Z");
		menuGraficar3DZoomInZ.addActionListener(this);
		
		menuGraficar3DZoomInPrincipal.add(menuGraficar3DZoomIn);
		menuGraficar3DZoomInPrincipal.add(new JSeparator());
		menuGraficar3DZoomInPrincipal.add(menuGraficar3DZoomInX);
		menuGraficar3DZoomInPrincipal.add(menuGraficar3DZoomInY);
		menuGraficar3DZoomInPrincipal.add(menuGraficar3DZoomInZ);
		
		menuGraficar3DZoomOutPrincipal=new JMenu("Alejar");
		
		menuGraficar3DZoomOut=new JMenuItem("Alejar", 'Z');
		menuGraficar3DZoomOut.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.CTRL_MASK+ActionEvent.SHIFT_MASK));
		menuGraficar3DZoomOut.addActionListener(this);
		
		menuGraficar3DZoomOutX=new JMenuItem("Alejar X");
		menuGraficar3DZoomOutX.addActionListener(this);
		
		menuGraficar3DZoomOutY=new JMenuItem("Alejar Y");
		menuGraficar3DZoomOutY.addActionListener(this);
		
		menuGraficar3DZoomOutZ=new JMenuItem("Alejar Z");
		menuGraficar3DZoomOutZ.addActionListener(this);
		
		menuGraficar3DZoomOutPrincipal.add(menuGraficar3DZoomOut);
		menuGraficar3DZoomOutPrincipal.add(new JSeparator());
		menuGraficar3DZoomOutPrincipal.add(menuGraficar3DZoomOutX);
		menuGraficar3DZoomOutPrincipal.add(menuGraficar3DZoomOutY);
		menuGraficar3DZoomOutPrincipal.add(menuGraficar3DZoomOutZ);
		
		menuGraficar3D.add(menuGraficar3DActual);
		menuGraficar3D.add(new JSeparator());
		menuGraficar3D.add(menuGraficar3DBorraActual);
		menuGraficar3D.add(menuGraficar3DBorraTodas);
		menuGraficar3D.add(new JSeparator());
		menuGraficar3D.add(menuGraficar3DZoomInPrincipal);
		menuGraficar3D.add(menuGraficar3DZoomOutPrincipal);
		
		//Menu de Opciones
		
		menuOpciones=new JMenu("Opciones");
		menuOpciones.setMnemonic(new Character('O'));
		
		menuOpcionesGenerales=new JMenuItem(espacio + "Opciones generales...",'g');
		menuOpcionesGenerales.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.CTRL_MASK));
		menuOpcionesGenerales.addActionListener(this);
		
		menuOpciones.add(menuOpcionesGenerales);
		
		//Menú de ventanas
		
		menuVentanas=new JMenu("Ventanas");
		menuVentanas.setMnemonic(new Character('V'));
		
		menuCascada=new JMenuItem(espacio + "Cascada",'C');
		menuCascada.addActionListener(this);
		
		menuMosaicoHorizontal=new JMenuItem(espacio + "Mosaico Horizontal",'H');
		menuMosaicoHorizontal.addActionListener(this);
		
		menuMosaicoVertical=new JMenuItem(espacio + "Mosaico Vertical",'V');
		menuMosaicoVertical.addActionListener(this);
		
		menuIconos=new JMenuItem(espacio + "Iconos",'I');
		menuIconos.addActionListener(this);
		
		menuSiguiente=new JMenuItem(espacio + "Siguiente",'S');
		menuSiguiente.addActionListener(this);
		
		menuAnterior=new JMenuItem(espacio + "Anterior",'A');
		menuAnterior.addActionListener(this);
		
		menuVentanas.add(menuCascada);
		menuVentanas.add(menuMosaicoHorizontal);
		menuVentanas.add(menuMosaicoVertical);
		menuVentanas.add(menuIconos);
		menuVentanas.add(new JSeparator());
		menuVentanas.add(menuSiguiente);
		menuVentanas.add(menuAnterior);
		
		//Menú de ayuda

		menuAyuda=new JMenu("Ayuda");
		menuAyuda.setMnemonic(new Character('y'));
		
		menuIndice=new JMenuItem(espacio + "Indice",'I');
		menuIndice.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_I, ActionEvent.CTRL_MASK));
		menuIndice.addActionListener(this);
		menuIndice.setEnabled(false);
		
		menuContenido=new JMenuItem(espacio + "Contenido",'C');
		menuContenido.setAccelerator(KeyStroke.getKeyStroke("F1"));
		menuContenido.addActionListener(this);
		menuContenido.setEnabled(false);
		
		menuAcercaDe=new JMenuItem(espacio + "Acerca de...",'A');
		//menuAcercaDe.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F1, ActionEvent.CTRL_MASK));
		menuAcercaDe.addActionListener(this);
		menuAcercaDe.setEnabled(false);
		
		menuAyuda.add(menuIndice);
		menuAyuda.add(menuContenido);
		menuAyuda.add(new JSeparator());
		menuAyuda.add(menuAcercaDe);
		
		//Menú principal
		
		elMenu = new JMenuBar();
		
		elMenu.add(menuArchivo);
		elMenu.add(menuEdicion);
		elMenu.add(menuFunciones);
		elMenu.add(menuGraficar);
		elMenu.add(menuGraficar3D);
		elMenu.add(menuOpciones);
		elMenu.add(menuVentanas);
		elMenu.add(menuAyuda);
	}
	
	//Función que se encarga de llenar las tablas de funciones, constantes y operadores
	private void llenarTablas() {
		funciones=new TablaFunciones();
		constantes=new TablaConstantes();
		operadores=new TablaOperadores();
		funcionesUsuario=new TablaFuncionesUsuario();
		
		funciones.agregaFuncion("abs",new Abs());
		funciones.agregaFuncion("acos",new Acos());
		funciones.agregaFuncion("acot",new Acot());
		funciones.agregaFuncion("acr",new ACR());
		funciones.agregaFuncion("arc",new ACR());
		funciones.agregaFuncion("acsc",new Acsc());
		funciones.agregaFuncion("asec",new Asec());
		funciones.agregaFuncion("asen",new Asen());
		funciones.agregaFuncion("asin",new Asen());
		funciones.agregaFuncion("atan",new Atan());
		funciones.agregaFuncion("arccos",new Acos());
		funciones.agregaFuncion("arccot",new Acot());
		funciones.agregaFuncion("arccsc",new Acsc());
		funciones.agregaFuncion("arcsec",new Asec());
		funciones.agregaFuncion("arcsen",new Asen());
		funciones.agregaFuncion("arcsin",new Asen());
		funciones.agregaFuncion("arctan",new Atan());
		funciones.agregaFuncion("coef",new Coef());
		funciones.agregaFuncion("cos",new Cos());
		funciones.agregaFuncion("cosh",new Cosh());
		funciones.agregaFuncion("cot",new Cot());
		funciones.agregaFuncion("coth",new Coth());
		funciones.agregaFuncion("csc",new Csc());
		funciones.agregaFuncion("csch",new Csch());
		funciones.agregaFuncion("der",new Derivar(operadores, funciones, constantes, funcionesUsuario));
		funciones.agregaFuncion("det",new Determinante());
		funciones.agregaFuncion("esent",new EsEntero());
		funciones.agregaFuncion("esint",new EsEntero());
		funciones.agregaFuncion("esnum",new EsNumero());
		funciones.agregaFuncion("esmon",new EsMonomio());
		funciones.agregaFuncion("espol",new EsPolinomio());
		funciones.agregaFuncion("esprimo",new EsPrimo());
		funciones.agregaFuncion("isprime",new EsPrimo());
		funciones.agregaFuncion("eval",new Sustituir());
		funciones.agregaFuncion("factor",new Factor());
		funciones.agregaFuncion("fij",new Fij());
		funciones.agregaFuncion("fikfj",new FiKFj());
		funciones.agregaFuncion("inv",new Inversa());
		funciones.agregaFuncion("inter",new Interseccion());
		funciones.agregaFuncion("kfi",new KFi());
		funciones.agregaFuncion("ln",new Ln());
		funciones.agregaFuncion("log",new Log());
		funciones.agregaFuncion("mod",new Mod());
		funciones.agregaFuncion("parent",new ParteEntera());
		funciones.agregaFuncion("pi",new itcr.mathexplorer.funciones.Pi());
		funciones.agregaFuncion("primo",new Primo());
		funciones.agregaFuncion("prime",new Primo());
		funciones.agregaFuncion("primos",new Primos());
		funciones.agregaFuncion("primes",new Primos());
		funciones.agregaFuncion("reduce",new ReduccionFilas());
		funciones.agregaFuncion("rnd",new Rnd());
		funciones.agregaFuncion("round",new Round());
		funciones.agregaFuncion("truncar",new Truncar());
		funciones.agregaFuncion("si",new Si());
		funciones.agregaFuncion("if",new Si());
		funciones.agregaFuncion("signo",new Signo());
		funciones.agregaFuncion("sust",new Sustituir());
		funciones.agregaFuncion("sec",new Sec());
		funciones.agregaFuncion("sech",new Sech());
		funciones.agregaFuncion("sen",new Sen());
		funciones.agregaFuncion("sin",new Sen());
		funciones.agregaFuncion("senh",new Senh());
		funciones.agregaFuncion("sinh",new Senh());
		funciones.agregaFuncion("sqrt",new Sqrt());
		funciones.agregaFuncion("raiz",new Sqrt());
		funciones.agregaFuncion("tan",new Tan());
		funciones.agregaFuncion("tanh",new Tanh());
		funciones.agregaFuncion("trans",new Transpuesta());
		funciones.agregaFuncion("union",new Union());
		
		constantes.agregaConstante("#e",new E());
		constantes.agregaConstante("#pi",new itcr.mathexplorer.constantes.Pi());
		
		operadores.agregaOperador("/",new Division());
		operadores.agregaOperador("%",new Porcentaje());
		operadores.agregaOperador("*",new Multiplicacion());
		operadores.agregaOperador("·",new Multiplicacion());
		operadores.agregaOperador("^",new Potencia());
		operadores.agregaOperador("-",new Resta());
		operadores.agregaOperador("+",new Suma());
		operadores.agregaOperador("!",new Factorial());
		operadores.agregaOperador("=",new Igual());
		operadores.agregaOperador("<>",new Distinto());
		operadores.agregaOperador("<=",new MenorIgual());
		operadores.agregaOperador(">=",new MayorIgual());
		operadores.agregaOperador(">",new Mayor());
		operadores.agregaOperador("<",new Menor());
		operadores.agregaOperador("and",new And());
		operadores.agregaOperador("or",new Or());
		operadores.agregaOperador("xor",new XOr());
		operadores.agregaOperador("not",new Not());
	}
	
	//Funciones que se repiten en el menú y la barra de herramientas.
	private void ventanaNueva(){
		int i=1;
		
		try{
			VentanaInternaExpresiones nueva=new VentanaInternaExpresiones("Math Explorer", barraDeUtilidadesExpresion, operadores, funciones, constantes, funcionesUsuario);
			panelPrincipal.add(nueva);
			nueva.setVisible(true);
			nueva.setColorFondo(Opciones.getColorFondo());
			nueva.setSelected(true);
			nueva.addInternalFrameListener(this);
			panelPrincipal.setSelectedFrame(nueva);
			ventanaActual=nueva;
			ventanas.add(nueva);
			
			if(ventanas.size()==1){
				menuGuardar.setEnabled(true);
				menuGuardarComo.setEnabled(true);
				barraDeHerramientasGuardar.setEnabled(true);
				menuCascada.setEnabled(true);
				menuMosaicoHorizontal.setEnabled(true);
				menuMosaicoVertical.setEnabled(true);
				menuIconos.setEnabled(true);
				menuSiguiente.setEnabled(true);
				menuAnterior.setEnabled(true);
			}
			
			if(Graficos2D!=null && Graficos2D.isClosed()==false){
				Graficos2D.reshape(VentanaInternaExpresiones.xOffset*((VentanaInternaExpresiones.contadorFrames-i)%6 + (VentanaInternaExpresiones.contadorFrames-i)/6), VentanaInternaExpresiones.yOffset*((VentanaInternaExpresiones.contadorFrames-i)%6),VentanaInternaExpresiones.ancho,VentanaInternaExpresiones.alto);
				i--;
			}
			
			if(Graficos3D!=null && Graficos3D.isClosed()==false){
				Graficos3D.reshape(VentanaInternaExpresiones.xOffset*((VentanaInternaExpresiones.contadorFrames-i)%6 + (VentanaInternaExpresiones.contadorFrames-i)/6), VentanaInternaExpresiones.yOffset*((VentanaInternaExpresiones.contadorFrames-i)%6),VentanaInternaExpresiones.ancho,VentanaInternaExpresiones.alto);
			}
			
			barraDeGraficos.setVisible(false);
			barraDeGraficos3D.setVisible(false);
			
			barraDeUtilidadesExpresion.requestFocus();
		}catch(Exception e){
			e.printStackTrace();
			JOptionPane.showMessageDialog(this,"Se produjo un error al abrir una ventana nueva.","Error",JOptionPane.ERROR_MESSAGE);
		}
	}
	
	private void ventanaGuardar(){
		try{
			if(ventanaActual.getArchivoActual() !=null && !ventanaActual.getArchivoActual().toString().equals("")){
				ventanaActual.guardarArchivo(ventanaActual.getArchivoActual());
			}else{
				ventanaActual.guardarArchivo();
			}
		}catch(IOException e){
			e.printStackTrace();
			JOptionPane.showMessageDialog(this,"Se produjo un error al guardar el archivo: " + e.getMessage(),"Error al guardar el archivo",JOptionPane.ERROR_MESSAGE);
		}
	}
	
	private void ventanaGuardarComo(){
		try{
			ventanaActual.guardarArchivo();
		}catch(IOException e){
			e.printStackTrace();
			JOptionPane.showMessageDialog(this,"Se produjo un error al guardar el archivo: " + e.getMessage(),"Error al guardar el archivo",JOptionPane.ERROR_MESSAGE);
		}
	}
	
	private void ventanaAbrir(){
		try{
			if(ventanas.size()==1 && !((VentanaInternaExpresiones)ventanas.elementAt(0)).Cambios() && ((VentanaInternaExpresiones)ventanas.elementAt(0)).getArchivoActual()==null){
				ventanaActual=(VentanaInternaExpresiones)ventanas.elementAt(0);
			}else{
				ventanaNueva();
			}
			ventanaActual.abrirArchivo();
		}catch(IOException e){
			e.printStackTrace();
			JOptionPane.showMessageDialog(this,"Se produjo un error al abrir el archivo: " + e.getMessage(),"Error al guardar el archivo",JOptionPane.ERROR_MESSAGE);
		}
	}
	
	private void salirPrograma(){
		boolean cancelar=false;
		int i=0;
		
		while(i<ventanas.size() && !cancelar){
			ventanaActual=(VentanaInternaExpresiones)ventanas.elementAt(i);
			if(ventanaActual!=null){
	  			if(ventanaActual.Cambios()){
	  				ventanaActual.moveToFront();
					int respuesta = JOptionPane.showConfirmDialog(this,"¿Deseas guardar los cambios?","Salir de MathExplorer",JOptionPane.YES_NO_CANCEL_OPTION);
					if(respuesta == JOptionPane.YES_OPTION){
						try{
							if(ventanaActual.getArchivoActual()!=null){
								ventanaActual.guardarArchivo(ventanaActual.getArchivoActual());
							}else{
								cancelar=!ventanaActual.guardarArchivo();
							}
							if(!cancelar){
								ventanaActual.dispose();
							}
						}catch(IOException e){
							e.printStackTrace();
							JOptionPane.showMessageDialog(this,"Se produjo un error al guardar el archivo: " + e.getMessage(),"Error al guardar el archivo",JOptionPane.ERROR_MESSAGE);
						}
					}else if(respuesta==JOptionPane.NO_OPTION){
						ventanaActual.dispose();
					}else if(respuesta==JOptionPane.CANCEL_OPTION){
						cancelar=true;
					}
	  			}
			}
			i++;
		}
		if(!cancelar){
			System.exit(0);
		}
	}
	
	private void graficarActual(){
		try{
			if(Graficos2D==null || Graficos2D.isClosed()==true){
				Graficos2D=new VentanaGraficos2D(operadores, funciones, constantes, funcionesUsuario);
				if(ventanaActual.getExpresionActual()!=null){
					Graficos2D.agregaExpresion(ventanaActual.getExpresionActual().obtenExpresionPrefijo());
				}
				panelPrincipal.add(Graficos2D);
				Graficos2D.setVisible(true);
				Graficos2D.reshape(VentanaInternaExpresiones.xOffset*((VentanaInternaExpresiones.contadorFrames-1)%6 + (VentanaInternaExpresiones.contadorFrames)/6), VentanaInternaExpresiones.yOffset*((VentanaInternaExpresiones.contadorFrames-1)%6),VentanaInternaExpresiones.ancho,VentanaInternaExpresiones.alto);
				//Graficos2D.setDefaultCloseOperation(JInternalFrame.DO_NOTHING_ON_CLOSE);
				Graficos2D.addInternalFrameListener(this);
			}else{
				if(ventanaActual.getExpresionActual()!=null){
					Graficos2D.agregaExpresion(ventanaActual.getExpresionActual().obtenExpresionPrefijo());
				}
				Graficos2D.setSelected(true);
			}
			barraDeGraficos.setVisible(true);
			barraDeGraficos3D.setVisible(false);
		}catch(Exception e){
			e.printStackTrace();
			barraDeEstadoMensajes.setText("Error al manejar un evento: "+e.getMessage());
		}
	}
	
	private void graficar3DActual(){
		try{
			if(Graficos3D==null || Graficos3D.isClosed()==true){
				Graficos3D=new VentanaGraficos3D(operadores, funciones, constantes, funcionesUsuario);
				if(ventanaActual.getExpresionActual()!=null){
					Graficos3D.agregaExpresion(ventanaActual.getExpresionActual().obtenExpresionPrefijo());
				}
				panelPrincipal.add(Graficos3D);
				Graficos3D.setVisible(true);
				if(Graficos2D==null || Graficos2D.isClosed()==true){
					Graficos3D.reshape(VentanaInternaExpresiones.xOffset*((VentanaInternaExpresiones.contadorFrames-1)%6 + (VentanaInternaExpresiones.contadorFrames-1)/6), VentanaInternaExpresiones.yOffset*((VentanaInternaExpresiones.contadorFrames-1)%6),VentanaInternaExpresiones.ancho,VentanaInternaExpresiones.alto);
				}else{
					Graficos3D.reshape(VentanaInternaExpresiones.xOffset*((VentanaInternaExpresiones.contadorFrames)%6 + (VentanaInternaExpresiones.contadorFrames)/6), VentanaInternaExpresiones.yOffset*((VentanaInternaExpresiones.contadorFrames)%6),VentanaInternaExpresiones.ancho,VentanaInternaExpresiones.alto);
				}
				//Graficos3D.setDefaultCloseOperation(JInternalFrame.DO_NOTHING_ON_CLOSE);
				Graficos3D.addInternalFrameListener(this);
			}else{
				if(ventanaActual.getExpresionActual()!=null){
					Graficos3D.agregaExpresion(ventanaActual.getExpresionActual().obtenExpresionPrefijo());
				}
				Graficos3D.setSelected(true);
			}
			barraDeGraficos3D.setVisible(true);
			barraDeGraficos.setVisible(false);
		}catch(Exception e){
			e.printStackTrace();
			barraDeEstadoMensajes.setText("Error al manejar un evento: "+e.getMessage());
		}
	}
	
	/*
	 *Función que busca la primer variable de la expresión e
	 *Devuelve un String con la variable. 
	 */
	private String buscaVariable(Nodo e){
		String variable;
		
		if(e.getTipo().equals("Variable")){
			return e.getOperando(1).getTipo();
		}
		
		for(int i=1; i<=e.getNumeroOperandos();i++){
			variable=buscaVariable(e.getOperando(i));
			if(!variable.equals("")){
				return variable;
			}
		}
		
		return "";
	}
	
	//AUDITORES
	
	public void actionPerformed( ActionEvent evt ) {
		try{
			
			//Auditor para los eventos del menú
			
			if(evt.getSource()==menuSalir){
				salirPrograma();
	  			
			}else if(evt.getSource()==menuOpcionesGenerales){
				new PanelOpciones(operadores, funciones, constantes, funcionesUsuario);
				
				if(Graficos2D!=null){
					Graficos2D.retomarValores();
				}
				
				if(Graficos3D!=null){
					Graficos3D.retomarValores();
				}
				
				if(Opciones.getCambioAspecto()){
					String aspecto=Opciones.getAspecto();
					if(aspecto.equals("Native")){
						try{
							UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
							SwingUtilities.updateComponentTreeUI(esteJFrame);
						}catch(Exception e){
							JOptionPane.showMessageDialog(esteJFrame,"Se produjo un error al cambiar la apariencia del programa","Error",JOptionPane.ERROR_MESSAGE);
						}
					}else if(aspecto.equals("Metal")){
						try{
							UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
							SwingUtilities.updateComponentTreeUI(esteJFrame);
						}catch(Exception e){
							JOptionPane.showMessageDialog(esteJFrame,"Se produjo un error al cambiar la apariencia del programa","Error",JOptionPane.ERROR_MESSAGE);
						}
					}else{
						try{
							UIManager.setLookAndFeel("com.sun.java.swing.plaf.motif.MotifLookAndFeel");
							SwingUtilities.updateComponentTreeUI(esteJFrame);
						}catch(Exception e){
							e.printStackTrace();
							JOptionPane.showMessageDialog(esteJFrame,"Se produjo un error al cambiar la apariencia del programa","Error",JOptionPane.ERROR_MESSAGE);
						}
					}
				}
				
				if(Opciones.getCambioColorFondo()){
					try{
						for(int i=0; i<ventanas.size(); i++){
							((VentanaInternaExpresiones)ventanas.elementAt(i)).setColorFondo(Opciones.getColorFondo());
						}
					}catch(Exception e){
						e.printStackTrace();
						JOptionPane.showMessageDialog(esteJFrame,"Se produjo un error al cambiarle el color de fondo a las ventanas","Error",JOptionPane.ERROR_MESSAGE);
					}
				}
				
				if(Opciones.getCambioColorLetra()){
					try{
						for(int i=0; i<ventanas.size(); i++){
							((VentanaInternaExpresiones)ventanas.elementAt(i)).setColorLetra(Opciones.getColorLetra());
						}
					}catch(Exception e){
						e.printStackTrace();
						JOptionPane.showMessageDialog(esteJFrame,"Se produjo un error al cambiarle el color de la letra a las ventanas","Error",JOptionPane.ERROR_MESSAGE);
					}
				}
				
				if(Opciones.getCambioTipoLetra()){
					try{
						for(int i=0; i<ventanas.size(); i++){
							((VentanaInternaExpresiones)ventanas.elementAt(i)).setTipoLetra(Opciones.getTipoLetra());
						}
					}catch(Exception e){
						e.printStackTrace();
						JOptionPane.showMessageDialog(esteJFrame,"Se produjo un error al cambiarle el color de la letra a las ventanas","Error",JOptionPane.ERROR_MESSAGE);
					}
				}
				
				return;
			}else if(evt.getSource()==menuGuardar){
				ventanaGuardar();
				return;
			}else if(evt.getSource()==menuGuardarComo){
				ventanaGuardarComo();
				return;
			}else if(evt.getSource()==menuAbrir){
				ventanaAbrir();
				return;
			}else if(evt.getSource()==menuNuevo){
				ventanaNueva();
				return;
			}else if(evt.getSource()==menuPi){
				String texto=barraDeUtilidadesExpresion.getText();
				barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+ " #pi " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
				barraDeUtilidadesExpresion.requestFocus();
			}else if(evt.getSource()==menuE){
				String texto=barraDeUtilidadesExpresion.getText();
				barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart()) + " #e " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
				barraDeUtilidadesExpresion.requestFocus();
			}else if(evt.getSource()==menuSuma){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Adición de...", "Primer valor:", "Segundo valor:", "0" , "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Adición de...", "Primer valor:", "Segundo valor:",expresion.obtenExpresionInfijo(), "0");
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" ( " + panelEvaluar.getDato1() +" + " + panelEvaluar.getDato2() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" ( " + panelEvaluar.getDato1() +" + " + panelEvaluar.getDato2() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuResta){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Sustracción de...", "Primer valor:", "Segundo valor:", "0" , "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Sustracción de...", "Primer valor:", "Segundo valor:",expresion.obtenExpresionInfijo(), "0");
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" ( " + panelEvaluar.getDato1() +" - " + panelEvaluar.getDato2() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" ( " + panelEvaluar.getDato1() +" - " + panelEvaluar.getDato2() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuMultiplicacion){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Producto de...", "Primer valor:", "Segundo valor:", "0" , "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Producto de...", "Primer valor:", "Segundo valor:",expresion.obtenExpresionInfijo(), "0");
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" ( " + panelEvaluar.getDato1() +" · " + panelEvaluar.getDato2() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" ( " + panelEvaluar.getDato1() +" · " + panelEvaluar.getDato2() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuDivision){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("División de...", "Primer valor:", "Segundo valor:", "0" , "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("División de...", "Primer valor:", "Segundo valor:",expresion.obtenExpresionInfijo(), "0");
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" ( " + panelEvaluar.getDato1() +" / " + panelEvaluar.getDato2() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" ( " + panelEvaluar.getDato1() +" / " + panelEvaluar.getDato2() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuPotencia){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Potencia de...", "Base:", "Exponente:", "0" , "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Potencia de...", "Base:", "Exponente:",expresion.obtenExpresionInfijo(), "0");
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" ( " + panelEvaluar.getDato1() +" ^ " + panelEvaluar.getDato2() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" ( " + panelEvaluar.getDato1() +" ^ " + panelEvaluar.getDato2() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuFactorial){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Factorial de...", "Valor:", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Factorial de...", "Valor:", expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" ( " + panelEvaluar.getDato1() + " )! " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" ( " + panelEvaluar.getDato1() + " )! " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuIgual){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Igual que...", "Primer valor:", "Segundo valor:", "0" , "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Igual que...", "Primer valor:", "Segundo valor:",expresion.obtenExpresionInfijo(), "0");
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" ( " + panelEvaluar.getDato1() +" = " + panelEvaluar.getDato2() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" ( " + panelEvaluar.getDato1() +" = " + panelEvaluar.getDato2() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuDistinto){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Distinto que...", "Primer valor:", "Segundo valor:", "0" , "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Distinto que...", "Primer valor:", "Segundo valor:",expresion.obtenExpresionInfijo(), "0");
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" ( " + panelEvaluar.getDato1() +" <> " + panelEvaluar.getDato2() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" ( " + panelEvaluar.getDato1() +" <> " + panelEvaluar.getDato2() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuMenor){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Menor que...", "Primer valor:", "Segundo valor:", "0" , "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Menor que...", "Primer valor:", "Segundo valor:",expresion.obtenExpresionInfijo(), "0");
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" ( " + panelEvaluar.getDato1() +" < " + panelEvaluar.getDato2() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" ( " + panelEvaluar.getDato1() +" < " + panelEvaluar.getDato2() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuMenorIgual){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Menor o igual que...", "Primer valor:", "Segundo valor:", "0" , "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Menor o igual que...", "Primer valor:", "Segundo valor:",expresion.obtenExpresionInfijo(), "0");
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" ( " + panelEvaluar.getDato1() +" <= " + panelEvaluar.getDato2() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" ( " + panelEvaluar.getDato1() +" <= " + panelEvaluar.getDato2() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuMayor){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Mayor que...", "Primer valor:", "Segundo valor:", "0" , "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Mayor que...", "Primer valor:", "Segundo valor:",expresion.obtenExpresionInfijo(), "0");
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" ( " + panelEvaluar.getDato1() +" > " + panelEvaluar.getDato2() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" ( " + panelEvaluar.getDato1() +" > " + panelEvaluar.getDato2() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuMayorIgual){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Mayor o igual que...", "Primer valor:", "Segundo valor:", "0" , "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Mayor o igual que...", "Primer valor:", "Segundo valor:",expresion.obtenExpresionInfijo(), "0");
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" ( " + panelEvaluar.getDato1() +" >= " + panelEvaluar.getDato2() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" ( " + panelEvaluar.getDato1() +" >= " + panelEvaluar.getDato2() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuY){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Y...", "Primer valor:", "Segundo valor:", "0" , "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Y...", "Primer valor:", "Segundo valor:",expresion.obtenExpresionInfijo(), "0");
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" ( " + panelEvaluar.getDato1() +" and " + panelEvaluar.getDato2() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" ( " + panelEvaluar.getDato1() +" and " + panelEvaluar.getDato2() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuO){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("O...", "Primer valor:", "Segundo valor:", "0" , "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("O...", "Primer valor:", "Segundo valor:",expresion.obtenExpresionInfijo(), "0");
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" ( " + panelEvaluar.getDato1() +" or " + panelEvaluar.getDato2() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" ( " + panelEvaluar.getDato1() +" or " + panelEvaluar.getDato2() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuXO){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("XO...", "Primer valor:", "Segundo valor:", "0" , "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("XO...", "Primer valor:", "Segundo valor:",expresion.obtenExpresionInfijo(), "0");
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" ( " + panelEvaluar.getDato1() +" xor " + panelEvaluar.getDato2() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" ( " + panelEvaluar.getDato1() +" xor " + panelEvaluar.getDato2() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuNegacion){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Negación...", "Valor:", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Negación...", "Valor:",expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" not( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" not( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuValorAbsoluto){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Valor absoluto de...", "Valor:", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Valor absoluto de...", "Valor:",expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" abs( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" abs( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuRaiz){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Raíz cuadrada de...", "Valor:", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Raíz cuadrada de...", "Valor:",expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" sqrt( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" sqrt( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuModulo){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Módulo de...", "Primer valor:", "Segundo valor:", "0", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Módulo de...", "Primer valor:", "Segundo valor:",expresion.obtenExpresionInfijo(), "0");
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" mod( " + panelEvaluar.getDato1()+", " + panelEvaluar.getDato2() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" mod( " + panelEvaluar.getDato1()+", " + panelEvaluar.getDato2() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuSigno){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Signo de...", "Valor:", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Signo de...", "Valor:",expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" signo( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" signo( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuLogaritmo){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Logaritmo de...", "Valor:", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Logaritmo de...", "Valor:",expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" log( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" log( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuRandom){
				String texto=barraDeUtilidadesExpresion.getText();
				barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart()) + " rnd() " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
				barraDeUtilidadesExpresion.requestFocus();
			}else if(evt.getSource()==menuSen){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Seno de...", "Valor:", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Seno de...", "Valor:",expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" sen( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" sen( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuCos){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Coseno de...", "Valor:", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Coseno de...", "Valor:",expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" cos( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" cos( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuTan){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Tangente de...", "Valor:", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Tangente de...", "Valor:",expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" tan( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" tan( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuSec){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Secante de...", "Valor:", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Secante de...", "Valor:",expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" sec( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" sec( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuCsc){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Cosecante de...", "Valor:", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Cosecante de...", "Valor:",expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" csc( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" csc( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuCot){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Cotangente de...", "Valor:", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Cotangente de...", "Valor:",expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" cot( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" cot( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuASen){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Arcoseno de...", "Valor:", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Arcoseno de...", "Valor:",expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" asen( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" asen( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuACos){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Arcocoseno de...", "Valor:", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Arcocoseno de...", "Valor:",expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" acos( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" acos( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuATan){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Arcotangente de...", "Valor:", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Arcotangente de...", "Valor:",expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" atan( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" atan( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuASec){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Arcosecante de...", "Valor:", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Arcosecante de...", "Valor:",expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" asec( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" asec( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuACsc){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Arcocosecante de...", "Valor:", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Arcocosecante de...", "Valor:",expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" acsc( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" acsc( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuACot){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Arcocotangente de...", "Valor:", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Arcocotangente de...", "Valor:",expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" acot( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" acot( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuSenH){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Seno hiperbólico de...", "Valor:", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Seno hiperbólico de...", "Valor:",expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" senh( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" senh( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuCosH){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Coseno hiperbólico de...", "Valor:", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Coseno hiperbólico de...", "Valor:",expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" cosh( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" cosh( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuTanH){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Tangente hiperbólica de...", "Valor:", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Tangente hiperbólica de...", "Valor:",expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" tanh( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" tanh( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuSecH){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Secante hiperbólica de...", "Valor:", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Secante hiperbólica de...", "Valor:",expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" sech( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" sech( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuCscH){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Cosecante hiperbólica de...", "Valor:", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Cosecante hiperbólica de...", "Valor:",expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" csch( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" csch( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuCotH){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Cotangente hiperbólica de...", "Valor:", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Cotangente hiperbólica de...", "Valor:",expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" coth( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" coth( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuEsPrimo){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("¿Es primo?", "Valor:", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("¿Es primo?", "Valor:",expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" esprimo( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" esprimo( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuPrimo){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Primo número...", "Valor:", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Primo número...", "Valor:",expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" primo( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" primo( " + panelEvaluar.getDato1() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuPrimos){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Primos entre...", "Primer Valor:", "Segundo Valor:", "0", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Primos entre...", "Primer Valor:", "Segundo Valor:",expresion.obtenExpresionInfijo(), "0");
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" primos( " + panelEvaluar.getDato1()+", " + panelEvaluar.getDato2() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" primos( " + panelEvaluar.getDato1()+", " + panelEvaluar.getDato2() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuFactorNumero){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Factorizar número...", "Valor:", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Factorizar número...", "Valor:", expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" factor( " + panelEvaluar.getDato1()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" factor( " + panelEvaluar.getDato1()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuFuncionPi){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Pi de...", "Valor:", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Pi de...", "Valor:", expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" pi( " + panelEvaluar.getDato1()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" pi( " + panelEvaluar.getDato1()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuRestoChino){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Teorema Chino del Resto", "Primer Valor:", "Segundo Valor:", "0", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Teorema Chino del Resto", "Primer Valor:", "Segundo Valor:",expresion.obtenExpresionInfijo(), "0");
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" acr( " + panelEvaluar.getDato1()+", " + panelEvaluar.getDato2() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" acr( " + panelEvaluar.getDato1()+", " + panelEvaluar.getDato2() + " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuCoeficientes){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Coeficientes de...", "Polinomio:", "Variable:", "0", "x");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Coeficientes de...", "Polinomio:", "Variable:", expresion.obtenExpresionInfijo(), buscaVariable(expresion.obtenExpresionPrefijo()));
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" coef( " + panelEvaluar.getDato1()+", " + panelEvaluar.getDato2()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" coef( " + panelEvaluar.getDato1()+", " + panelEvaluar.getDato2()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuParteEntera){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Parte entera de...", "Valor:", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Parte entera de...", "Valor:", expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" parent( " + panelEvaluar.getDato1()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" parent( " + panelEvaluar.getDato1()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuRedondear){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Redondear...", "Valor:", "Decimales:", "0", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Redondear...", "Valor:", "Decimales:", expresion.obtenExpresionInfijo(), "0");
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" round( " + panelEvaluar.getDato1()+", " + panelEvaluar.getDato2()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" round( " + panelEvaluar.getDato1()+", " + panelEvaluar.getDato2()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuTruncar){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Truncar...", "Valor:", "Decimales:", "0", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Truncar...", "Valor:", "Decimales:", expresion.obtenExpresionInfijo(), "0");
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" truncar( " + panelEvaluar.getDato1()+", " + panelEvaluar.getDato2()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" truncar( " + panelEvaluar.getDato1()+", " + panelEvaluar.getDato2()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuEsEntero){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("¿Es entero?", "Valor:", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("¿Es entero?", "Valor:", expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" esent( " + panelEvaluar.getDato1()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" esent( " + panelEvaluar.getDato1()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuEsNumero){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("¿Es número?", "Valor:", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("¿Es número?", "Valor:", expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" esnum( " + panelEvaluar.getDato1()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" esnum( " + panelEvaluar.getDato1()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuEsMonomio){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("¿Es monomio?", "Expresión:", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("¿Es monomio?", "Expresión:", expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" esmon( " + panelEvaluar.getDato1()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" esmon( " + panelEvaluar.getDato1()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuEsPolinomio){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("¿Es polinomio?", "Expresión:", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("¿Es polinomio?", "Expresión:", expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" espol( " + panelEvaluar.getDato1()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" espol( " + panelEvaluar.getDato1()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuSi){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Si...", "Condición:", "Verdadero:", "Falso:", "0", "0", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Si...", "Condición:", "Verdadero:", "Falso:", expresion.obtenExpresionInfijo(), "0", "0");
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" si( " + panelEvaluar.getDato1()+", " + panelEvaluar.getDato2()+", " + panelEvaluar.getDato3()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" si( " + panelEvaluar.getDato1()+", " + panelEvaluar.getDato2()+", " + panelEvaluar.getDato3()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuDeterminante){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Determinante de...", "Matriz:", "[0]");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Determinante de...", "Matriz:", expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" det( " + panelEvaluar.getDato1()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" det( " + panelEvaluar.getDato1()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuTranspuesta){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Transpuesta de...", "Matriz:", "[0]");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Transpuesta de...", "Matriz:", expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" trans( " + panelEvaluar.getDato1()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" trans( " + panelEvaluar.getDato1()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuInversa){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Inversa de...", "Matriz:", "[0]");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Inversa de...", "Matriz:", expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" inv( " + panelEvaluar.getDato1()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" inv( " + panelEvaluar.getDato1()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuReduccionFilas){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Reducción por filas...", "Matriz:", "[0]");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Reducción por filas...", "Matriz:", expresion.obtenExpresionInfijo());
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" reduce( " + panelEvaluar.getDato1()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" reduce( " + panelEvaluar.getDato1()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menufij){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Intercambio de filas...", "Matriz:", "Primer fila:", "Segunda fila:", "[0]", "0", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Intercambio de filas...", "Matriz:", "Primer fila:", "Segunda fila:", expresion.obtenExpresionInfijo(), "0", "0");
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" fij( " + panelEvaluar.getDato1()+", " + panelEvaluar.getDato2()+", " + panelEvaluar.getDato3()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" fij( " + panelEvaluar.getDato1()+", " + panelEvaluar.getDato2()+", " + panelEvaluar.getDato3()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menufikfj){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Fila i mas k por fila j...", "Matriz:", "Fila i:", "k:", "Fila j:", "[0]", "0", "0", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Fila i mas k por fila j...", "Matriz:", "Fila i:", "k:", "Fila j:", expresion.obtenExpresionInfijo(), "0", "0", "0");
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" fikfj( " + panelEvaluar.getDato1()+", " + panelEvaluar.getDato2()+", " + panelEvaluar.getDato3()+", " + panelEvaluar.getDato4()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" fikfj( " + panelEvaluar.getDato1()+", " + panelEvaluar.getDato2()+", " + panelEvaluar.getDato3()+", " + panelEvaluar.getDato4()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menukfi){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("k por fila i...", "Matriz:", "k:", "Fila i:", "[0]", "0", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("k por fila i...", "Matriz:", "k:", "Fila i:", expresion.obtenExpresionInfijo(), "0", "0");
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" kfi( " + panelEvaluar.getDato1()+", " + panelEvaluar.getDato2()+", " + panelEvaluar.getDato3()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" kfi( " + panelEvaluar.getDato1()+", " + panelEvaluar.getDato2()+", " + panelEvaluar.getDato3()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuUnion){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Unión de...", "Primer conjunto:", "Segundo conjunto:", "{0}", "{0}");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Unión de...", "Primer conjunto:", "Segundo conjunto:", expresion.obtenExpresionInfijo(), "{0}");
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" union( " + panelEvaluar.getDato1()+", " + panelEvaluar.getDato2()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" union( " + panelEvaluar.getDato1()+", " + panelEvaluar.getDato2()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuInterseccion){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Intersección de...", "Primer conjunto:", "Segundo conjunto:", "{0}", "{0}");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Intersección de...", "Primer conjunto:", "Segundo conjunto:", expresion.obtenExpresionInfijo(), "{0}");
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" inter( " + panelEvaluar.getDato1()+", " + panelEvaluar.getDato2()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+" inter( " + panelEvaluar.getDato1()+", " + panelEvaluar.getDato2()+ " ) " + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuNuevaMatriz){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText(), comodin="[ ";

				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Nueva Matriz...", "Número Filas:", "Número Columnas:", "0", "0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Nueva Matriz...", "Número Filas:", "Número Columnas:", expresion.obtenExpresionInfijo(), "0");
				}
				
				try{
					if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
						int numFilas=Integer.parseInt(panelEvaluar.getDato1());
						int numColumnas=Integer.parseInt(panelEvaluar.getDato2());
						
						for(int i=1;i<=numFilas; i++){
							for(int j=1;j<numColumnas; j++){
								comodin=comodin.concat(" ,");
							}
							if(i!=numFilas){
								comodin=comodin.concat(" ;");
							}
						}
						comodin=comodin.concat(" ]");
						
						barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+comodin + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
						barraDeUtilidadesExpresion.requestFocus();
					}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
						int numFilas=Integer.parseInt(panelEvaluar.getDato1());
						int numColumnas=Integer.parseInt(panelEvaluar.getDato2());
						
						for(int i=1;i<numFilas; i++){
							for(int j=1;j<numColumnas; j++){
								comodin=comodin.concat(" ,");
							}
							comodin=comodin.concat(" ;");
						}
						comodin=comodin.concat(" ]");
						
						barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+comodin + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
						barraDeUtilidadesExpresion.postActionEvent();
					}
				}catch(Exception e){
					JOptionPane.showMessageDialog(this,"Alguno de los datos no es un número entero.","Error",JOptionPane.ERROR_MESSAGE);
					return;
				}
				return;
			}else if(evt.getSource()==menuEvaluar){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();
				
				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Evaluar una expresión...", "Expresión:", "Variable:", "Valor:", "x","x","0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Evaluar una expresión...", "Expresión:", "Variable:", "Valor:",expresion.obtenExpresionInfijo(), buscaVariable(expresion.obtenExpresionPrefijo()),"0");
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+"eval( " + panelEvaluar.getDato1() + ", " + panelEvaluar.getDato2() + ", " + panelEvaluar.getDato3() + " )" + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+"eval( " + panelEvaluar.getDato1() + ", " + panelEvaluar.getDato2() + ", " + panelEvaluar.getDato3() + " )" + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuDerivar){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();
				
				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Derivar una expresión...", "Expresión:", "Variable:", "Orden", "x","x", "1");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Evaluar una expresión...", "Expresión:", "Variable:", "Orden",expresion.obtenExpresionInfijo(), buscaVariable(expresion.obtenExpresionPrefijo()), "1");
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+"der( " + panelEvaluar.getDato1() + ", " + panelEvaluar.getDato2() + ", " + panelEvaluar.getDato3() + " )" + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+"der( " + panelEvaluar.getDato1() + ", " + panelEvaluar.getDato2() + ", " + panelEvaluar.getDato3() + " )" + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuDefinirFuncion){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();
				
				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Definir nueva función...", "Nombre:", "Variable(s):", "Expresión", "f","x", "1");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Definir nueva función...", "Nombre:", "Variable(s):", "Expresión", expresion.obtenExpresionInfijo(), buscaVariable(expresion.obtenExpresionPrefijo()), "0");
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+ panelEvaluar.getDato1() + "( " + panelEvaluar.getDato2() + " ):= " + panelEvaluar.getDato3() + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+ panelEvaluar.getDato1() + "( " + panelEvaluar.getDato2() + " ):= " + panelEvaluar.getDato3()  + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuDefinirConstante){
				PanelDatos panelEvaluar;
				String texto=barraDeUtilidadesExpresion.getText();
				
				if(ventanaActual==null || ventanaActual.getExpresionActual()==null){
					panelEvaluar=new PanelDatos("Definir nueva constante...", "Nombre:", "Valor:", "a","0");
				}else{
					Expresion expresion=ventanaActual.getExpresionActual();
					panelEvaluar=new PanelDatos("Definir nueva constante...", "Nombre:", "Valor:", expresion.obtenExpresionInfijo(), "0");
				}
				
				if(panelEvaluar.getBoton()==panelEvaluar.BTN_ACEPTAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+ panelEvaluar.getDato1() + " := " + panelEvaluar.getDato2() + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.requestFocus();
				}else if(panelEvaluar.getBoton()==panelEvaluar.BTN_EVALUAR){
					barraDeUtilidadesExpresion.setText(texto.substring(0, barraDeUtilidadesExpresion.getSelectionStart())+ panelEvaluar.getDato1() + " := " + panelEvaluar.getDato2()  + texto.substring(barraDeUtilidadesExpresion.getSelectionEnd()));
					barraDeUtilidadesExpresion.postActionEvent();
				}
				return;
			}else if(evt.getSource()==menuGraficarActual){
				graficarActual();
				return;
			}else if(evt.getSource()==menuBorraActual){
				if(Graficos2D!=null & Graficos2D.isClosed()==false){
					Graficos2D.borraExpresion();
				}
				return;
			}else if(evt.getSource()==menuBorraTodas){
				if(Graficos2D!=null & Graficos2D.isClosed()==false){
					Graficos2D.borraTodas();
				}
				return;
			}else if(evt.getSource()==menuZoomIn){
				if(Graficos2D!=null & Graficos2D.isClosed()==false){
					Graficos2D.zoomIn();
				}
				return;
			}else if(evt.getSource()==menuZoomInV){
				if(Graficos2D!=null & Graficos2D.isClosed()==false){
					Graficos2D.zoomInVertical();
				}
				return;
			}else if(evt.getSource()==menuZoomInH){
				if(Graficos2D!=null & Graficos2D.isClosed()==false){
					Graficos2D.zoomInHorizontal();
				}
				return;
			}else if(evt.getSource()==menuZoomOut){
				if(Graficos2D!=null & Graficos2D.isClosed()==false){
					Graficos2D.zoomIn();
				}
				return;
			}else if(evt.getSource()==menuZoomOutV){
				if(Graficos2D!=null & Graficos2D.isClosed()==false){
					Graficos2D.zoomInVertical();
				}
				return;
			}else if(evt.getSource()==menuZoomOutH){
				if(Graficos2D!=null & Graficos2D.isClosed()==false){
					Graficos2D.zoomInHorizontal();
				}
				return;
			}else if(evt.getSource()==menuGraficar3DActual){
				graficar3DActual();
				return;
			}else if(evt.getSource()==menuGraficar3DBorraActual){
				if(Graficos3D!=null & Graficos3D.isClosed()==false){
					Graficos3D.borraExpresion();
				}
				return;
			}else if(evt.getSource()==menuGraficar3DBorraTodas){
				if(Graficos3D!=null & Graficos3D.isClosed()==false){
					Graficos3D.borraTodas();
				}
				return;
			}else if(evt.getSource()==menuGraficar3DZoomIn){
				if(Graficos3D!=null & Graficos3D.isClosed()==false){
					Graficos3D.zoomIn();
				}
				return;
			}else if(evt.getSource()==menuGraficar3DZoomInX){
				if(Graficos3D!=null & Graficos3D.isClosed()==false){
					Graficos3D.zoomInX();
				}
				return;
			}else if(evt.getSource()==menuGraficar3DZoomInY){
				if(Graficos3D!=null & Graficos3D.isClosed()==false){
					Graficos3D.zoomInY();
				}
				return;
			}else if(evt.getSource()==menuGraficar3DZoomInZ){
				if(Graficos3D!=null & Graficos3D.isClosed()==false){
					Graficos3D.zoomInZ();
				}
				return;
			}else if(evt.getSource()==menuGraficar3DZoomOut){
				if(Graficos3D!=null & Graficos3D.isClosed()==false){
					Graficos3D.zoomOut();
				}
				return;
			}else if(evt.getSource()==menuGraficar3DZoomOutX){
				if(Graficos3D!=null & Graficos3D.isClosed()==false){
					Graficos3D.zoomOutX();
				}
				return;
			}else if(evt.getSource()==menuGraficar3DZoomOut){
				if(Graficos3D!=null & Graficos3D.isClosed()==false){
					Graficos3D.zoomOutY();
				}
				return;
			}else if(evt.getSource()==menuGraficar3DZoomOut){
				if(Graficos3D!=null & Graficos3D.isClosed()==false){
					Graficos3D.zoomOutZ();
				}
				return;
			}else if(evt.getSource()==menuIconos){
				try{
					for(int i=0; i<ventanas.size(); i++){
						((VentanaInternaExpresiones)ventanas.elementAt(i)).setIcon(true);
					}
					if(Graficos2D!=null){
						Graficos2D.setIcon(true);
					}
					if(Graficos3D!=null){
						Graficos3D.setIcon(true);
					}
				}catch(Exception e){
					e.printStackTrace();
					JOptionPane.showMessageDialog(esteJFrame,"Se produjo un error al iconizar las ventanas","Error",JOptionPane.ERROR_MESSAGE);
				}
				return;
			}else if(evt.getSource()==menuCascada){
				try{
					int i;
					int cantidad=(panelPrincipal.getHeight()-VentanaInternaExpresiones.alto)/VentanaInternaExpresiones.yOffset;
					if(cantidad<=0){
						cantidad=1;
					}
					if(Graficos2D!=null && Graficos2D.isClosed()==false){
						cantidad++;
					}
					if(Graficos3D!=null && Graficos3D.isClosed()==false){
						cantidad++;
					}
					
					for(i=0; i<ventanas.size(); i++){
						VentanaInternaExpresiones ventanaTrabajo=((VentanaInternaExpresiones)ventanas.elementAt(i));
						ventanaTrabajo.setIcon(false);
						ventanaTrabajo.reshape(VentanaInternaExpresiones.xOffset*(i%cantidad + i/cantidad),VentanaInternaExpresiones.yOffset*(i%cantidad),VentanaInternaExpresiones.ancho,VentanaInternaExpresiones.alto);
						ventanaTrabajo.moveToFront();
					}
					
					if(Graficos2D!=null && Graficos2D.isClosed()==false){
						Graficos2D.setIcon(false);
						Graficos2D.reshape(VentanaInternaExpresiones.xOffset*(i%cantidad + i/cantidad),VentanaInternaExpresiones.yOffset*(i%cantidad),VentanaInternaExpresiones.ancho,VentanaInternaExpresiones.alto);
						Graficos2D.moveToFront();
						i++;
					}
					
					if(Graficos3D!=null && Graficos3D.isClosed()==false){
						Graficos3D.setIcon(false);
						Graficos3D.reshape(VentanaInternaExpresiones.xOffset*(i%cantidad + i/cantidad),VentanaInternaExpresiones.yOffset*(i%cantidad),VentanaInternaExpresiones.ancho,VentanaInternaExpresiones.alto);
						Graficos3D.moveToFront();
					}
				}catch(Exception e){
					e.printStackTrace();
					JOptionPane.showMessageDialog(esteJFrame,"Se produjo un error al poner las ventanas en cascada","Error",JOptionPane.ERROR_MESSAGE);
				}
				return;
			}else if(evt.getSource()==menuSiguiente){
				try{
					int numActual=ventanas.indexOf(ventanaActual)+1;
					if(numActual==ventanas.size()){
						if(Graficos2D!=null && Graficos2D.isClosed()==false && Graficos3D!=null && Graficos3D.isClosed()==false){
							if(Graficos2D.isSelected()==true){
								Graficos3D.setIcon(false);
								Graficos3D.moveToFront();
								Graficos3D.setSelected(true);
							}else if(Graficos3D.isSelected()==true){
								VentanaInternaExpresiones ventanaTrabajo=((VentanaInternaExpresiones)ventanas.elementAt(0));
								ventanaTrabajo.setIcon(false);
								ventanaTrabajo.moveToFront();
								ventanaTrabajo.setSelected(true);
								ventanaActual=ventanaTrabajo;
							}else{
								Graficos2D.setIcon(false);
								Graficos2D.moveToFront();
								Graficos2D.setSelected(true);
							}
						}else if(Graficos2D!=null && Graficos2D.isClosed()==false){
							if(Graficos2D.isSelected()==true){
								VentanaInternaExpresiones ventanaTrabajo=((VentanaInternaExpresiones)ventanas.elementAt(0));
								ventanaTrabajo.setIcon(false);
								ventanaTrabajo.moveToFront();
								ventanaTrabajo.setSelected(true);
								ventanaActual=ventanaTrabajo;
							}else{
								Graficos2D.setIcon(false);
								Graficos2D.moveToFront();
								Graficos2D.setSelected(true);
							}
						}else if(Graficos3D!=null && Graficos3D.isClosed()==false){
							if(Graficos3D.isSelected()==true){
								VentanaInternaExpresiones ventanaTrabajo=((VentanaInternaExpresiones)ventanas.elementAt(0));
								ventanaTrabajo.setIcon(false);
								ventanaTrabajo.moveToFront();
								ventanaTrabajo.setSelected(true);
								ventanaActual=ventanaTrabajo;
							}else{
								Graficos3D.setIcon(false);
								Graficos3D.moveToFront();
								Graficos3D.setSelected(true);
							}
						}else{
							VentanaInternaExpresiones ventanaTrabajo=((VentanaInternaExpresiones)ventanas.elementAt(0));
							ventanaTrabajo.setIcon(false);
							ventanaTrabajo.moveToFront();
							ventanaTrabajo.setSelected(true);
							ventanaActual=ventanaTrabajo;
						}
					}else{
						VentanaInternaExpresiones ventanaTrabajo=((VentanaInternaExpresiones)ventanas.elementAt(numActual));
						ventanaTrabajo.setIcon(false);
						ventanaTrabajo.moveToFront();
						ventanaTrabajo.setSelected(true);
						ventanaActual=ventanaTrabajo;
					}
				}catch(Exception e){
					e.printStackTrace();
					JOptionPane.showMessageDialog(esteJFrame,"Se produjo un error al pasar a la siguiente ventana","Error",JOptionPane.ERROR_MESSAGE);
				}
				return;
			}else if(evt.getSource()==menuAnterior){
				try{
					int numActual=ventanas.indexOf(ventanaActual)-1;
					if(numActual==-1){
						if(Graficos2D!=null && Graficos2D.isClosed()==false && Graficos3D!=null && Graficos3D.isClosed()==false){
							if(Graficos3D.isSelected()==true){
								Graficos2D.setIcon(false);
								Graficos2D.moveToFront();
								Graficos2D.setSelected(true);
							}else if(Graficos2D.isSelected()==true){
								VentanaInternaExpresiones ventanaTrabajo=((VentanaInternaExpresiones)ventanas.elementAt(ventanas.size()-1));
								ventanaTrabajo.setIcon(false);
								ventanaTrabajo.moveToFront();
								ventanaTrabajo.setSelected(true);
								ventanaActual=ventanaTrabajo;
							}else{
								Graficos3D.setIcon(false);
								Graficos3D.moveToFront();
								Graficos3D.setSelected(true);
							}
						}else if(Graficos2D!=null && Graficos2D.isClosed()==false){
							if(Graficos2D.isSelected()==true){
								VentanaInternaExpresiones ventanaTrabajo=((VentanaInternaExpresiones)ventanas.elementAt(ventanas.size()-1));
								ventanaTrabajo.setIcon(false);
								ventanaTrabajo.moveToFront();
								ventanaTrabajo.setSelected(true);
								ventanaActual=ventanaTrabajo;
							}else{
								Graficos2D.setIcon(false);
								Graficos2D.moveToFront();
								Graficos2D.setSelected(true);
							}
						}else if(Graficos3D!=null && Graficos3D.isClosed()==false){
							if(Graficos3D.isSelected()==true){
								VentanaInternaExpresiones ventanaTrabajo=((VentanaInternaExpresiones)ventanas.elementAt(ventanas.size()-1));
								ventanaTrabajo.setIcon(false);
								ventanaTrabajo.moveToFront();
								ventanaTrabajo.setSelected(true);
								ventanaActual=ventanaTrabajo;
							}else{
								Graficos3D.setIcon(false);
								Graficos3D.moveToFront();
								Graficos3D.setSelected(true);
							}
						}else{
							VentanaInternaExpresiones ventanaTrabajo=((VentanaInternaExpresiones)ventanas.elementAt(ventanas.size()-1));
							ventanaTrabajo.setIcon(false);
							ventanaTrabajo.moveToFront();
							ventanaTrabajo.setSelected(true);
							ventanaActual=ventanaTrabajo;
						}
					}else{
						if(Graficos3D.isSelected()==true){
							Graficos2D.setIcon(false);
							Graficos2D.moveToFront();
							Graficos2D.setSelected(true);
						}else if(Graficos2D.isSelected()==true){
							VentanaInternaExpresiones ventanaTrabajo=((VentanaInternaExpresiones)ventanas.elementAt(ventanas.size()-1));
							ventanaTrabajo.setIcon(false);
							ventanaTrabajo.moveToFront();
							ventanaTrabajo.setSelected(true);
							ventanaActual=ventanaTrabajo;
						}else{
							VentanaInternaExpresiones ventanaTrabajo=((VentanaInternaExpresiones)ventanas.elementAt(numActual));
							ventanaTrabajo.setIcon(false);
							ventanaTrabajo.moveToFront();
							ventanaTrabajo.setSelected(true);
							ventanaActual=ventanaTrabajo;
						}
					}
				}catch(Exception e){
					e.printStackTrace();
					JOptionPane.showMessageDialog(esteJFrame,"Se produjo un error al pasar a la ventana anterior","Error",JOptionPane.ERROR_MESSAGE);
				}
				return;
			}else if(evt.getSource()==menuMosaicoHorizontal){
				try{
					int i;
					int cantidad=ventanas.size();
					if(Graficos2D!=null && Graficos2D.isClosed()==false){
						cantidad++;
					}
					if(Graficos3D!=null && Graficos3D.isClosed()==false){
						cantidad++;
					}
					int alto;
					int particiones=(cantidad-1)/4 + 1;
					int ancho=panelPrincipal.getWidth()/particiones;
					if (particiones>1){
						alto=panelPrincipal.getHeight()/4;
					}else{
						alto=panelPrincipal.getHeight()/cantidad;
					}
					
					for(i=0; i<ventanas.size(); i++){
						VentanaInternaExpresiones ventanaTrabajo=((VentanaInternaExpresiones)ventanas.elementAt(i));
						ventanaTrabajo.setIcon(false);
						ventanaTrabajo.reshape((i/4) *ancho,(i%4)*alto,ancho,alto);
					}
					
					if(Graficos2D!=null && Graficos2D.isClosed()==false){
						Graficos2D.setIcon(false);
						Graficos2D.reshape((i/4) *ancho,(i%4)*alto,ancho,alto);
						i++;
					}
					if(Graficos3D!=null && Graficos3D.isClosed()==false){
						Graficos3D.setIcon(false);
						Graficos3D.reshape((i/4) *ancho,(i%4)*alto,ancho,alto);
					}
				}catch(Exception e){
					e.printStackTrace();
					JOptionPane.showMessageDialog(esteJFrame,"Se produjo un error al pasar a la ventana anterior","Error",JOptionPane.ERROR_MESSAGE);
				}
				return;
			}else if(evt.getSource()==menuMosaicoVertical){
				try{
					int i;
					int cantidad=ventanas.size();
					if(Graficos2D!=null && Graficos2D.isClosed()==false){
						cantidad++;
					}
					if(Graficos3D!=null && Graficos3D.isClosed()==false){
						cantidad++;
					}
					int ancho;
					int particiones=(cantidad-1)/4 + 1;
					int alto=panelPrincipal.getHeight()/particiones;
					if (particiones>1){
						ancho=panelPrincipal.getWidth()/4;
					}else{
						ancho=panelPrincipal.getWidth()/cantidad;
					}
					
					for(i=0; i<ventanas.size(); i++){
						VentanaInternaExpresiones ventanaTrabajo=((VentanaInternaExpresiones)ventanas.elementAt(i));
						ventanaTrabajo.setIcon(false);
						ventanaTrabajo.reshape((i%4)*ancho,(i/4)*alto,ancho,alto);
					}
					
					if(Graficos2D!=null && Graficos2D.isClosed()==false){
						Graficos2D.setIcon(false);
						Graficos2D.reshape((i%4)*ancho,(i/4)*alto,ancho,alto);
						i++;
					}
					if(Graficos3D!=null && Graficos3D.isClosed()==false){
						Graficos3D.setIcon(false);
						Graficos3D.reshape((i%4)*ancho,(i/4)*alto,ancho,alto);
					}
				}catch(Exception e){
					e.printStackTrace();
					JOptionPane.showMessageDialog(esteJFrame,"Se produjo un error al pasar a la ventana anterior","Error",JOptionPane.ERROR_MESSAGE);
				}
				return;
			}
	
	
			//Auditor para los eventos de la barra de herramientas
			
			if (evt.getSource()==barraDeHerramientasGuardar){
				ventanaGuardar();
				return;
			}else if (evt.getSource()==barraDeHerramientasAbrir){
				ventanaAbrir();
				return;
			}else if (evt.getSource()==barraDeHerramientasNuevo){
				ventanaNueva();
				
			}else if (evt.getSource()==barraDeHerramientasGraficar || evt.getSource()==barraDeGraficosGraficar){
				graficarActual();
				return;
			}else if (evt.getSource()==barraDeHerramientasGraficar3D || evt.getSource()==barraDeGraficosGraficar3D){
				graficar3DActual();
				return;
			}
	
			//Auditor para los eventos de la barra de utilidades
						
			if (evt.getSource()==barraDeUtilidadesAceptar || evt.getSource()==barraDeUtilidadesExpresion){
				if (!barraDeUtilidadesExpresion.getText().trim().equals("")){
					try{
						
						if(ventanas.size()==0){
							ventanaNueva();
						}else{
							if(panelPrincipal.getSelectedFrame()!=Graficos2D && panelPrincipal.getSelectedFrame()!=Graficos3D){
								ventanaActual=(VentanaInternaExpresiones)panelPrincipal.getSelectedFrame();
								if(ventanaActual==null){
									ventanaActual=(VentanaInternaExpresiones)ventanas.elementAt(0);
								}
							}
						}
						
						ventanaActual.agregaExpresion(barraDeUtilidadesExpresion.getText());
	
						barraDeEstadoMensajes.setText("Expresion correcta");
						
						barraDeUtilidadesExpresion.selectAll();
						barraDeUtilidadesExpresion.requestFocus();
						
					}catch(SintaxException e){
						e.printStackTrace();
						//barraDeEstadoMensajes.setText("Error en la expresión");
						barraDeEstadoMensajes.setText(e.toString());
					}catch(Exception e) { 
						e.printStackTrace();
						barraDeEstadoMensajes.setText(e.toString());
					}
					
				}
				return;
			}else if (evt.getSource()==barraDeUtilidadesCancelar){
				barraDeUtilidadesExpresion.setText("");
				return;
			}
			
			//Auditor para los eventos de la barra de graficación
			
			if(evt.getSource()==barraDeGraficosBorraGrafica){
				Graficos2D.borraExpresion();
				return;
			}else if(evt.getSource()==barraDeGraficosBorraTodas){
				Graficos2D.borraTodas();
				return;
			}else if(evt.getSource()==barraDeGraficosZoomIn){
				Graficos2D.zoomIn();
				return;
			}else if(evt.getSource()==barraDeGraficosZoomInH){
				Graficos2D.zoomInHorizontal();
				return;
			}else if(evt.getSource()==barraDeGraficosZoomInV){
				Graficos2D.zoomInVertical();
				return;
			}else if(evt.getSource()==barraDeGraficosZoomOut){
				Graficos2D.zoomOut();
				return;
			}else if(evt.getSource()==barraDeGraficosZoomOutV){
				Graficos2D.zoomOutVertical();
				return;
			}else if(evt.getSource()==barraDeGraficosZoomOutH){
				Graficos2D.zoomOutHorizontal();
				return;
			}
			
			//Auditor para los eventos de la barra de graficación en 3D
			
			if(evt.getSource()==barraDeGraficosBorraGrafica3D){
				Graficos3D.borraExpresion();
				return;
			}else if(evt.getSource()==barraDeGraficosBorraTodas3D){
				Graficos3D.borraTodas();
				return;
			}else if(evt.getSource()==barraDeGraficosZoomIn3D){
				Graficos3D.zoomIn();
				return;
			}else if(evt.getSource()==barraDeGraficosZoomOut3D){
				Graficos3D.zoomOut();
				return;
			}
			
			barraDeEstadoMensajes.setText(evt.getSource().toString());
			barraDeUtilidadesExpresion.requestFocus();
		}catch(Exception e){
			e.printStackTrace();
			barraDeEstadoMensajes.setText("Error al manejar un evento: "+e.getMessage());
		}
	}
	
	
	//Auditor para los eventos de las ventanas internas
	
    public void internalFrameClosing(InternalFrameEvent e) {
    	
    	boolean cancelar=false;
    	
    	try{
    		if(e.getInternalFrame()==Graficos2D){
    			Graficos2D.dispose();
    			barraDeGraficos.setVisible(false);
    		}else if(e.getInternalFrame()==Graficos3D){
    			Graficos3D.dispose();
    			barraDeGraficos3D.setVisible(false);
    		}else{
		        VentanaInternaExpresiones ventana=(VentanaInternaExpresiones)e.getInternalFrame();
	
	  			if(ventana.Cambios()){
	  				ventana.moveToFront();
					int respuesta = JOptionPane.showConfirmDialog(esteJFrame,"¿Deseas guardar los cambios?","Salir de MathExplorer",JOptionPane.YES_NO_CANCEL_OPTION);
					if(respuesta == JOptionPane.YES_OPTION){
						if(ventana.getArchivoActual()!=null){
							ventana.guardarArchivo(ventana.getArchivoActual());
						}else{
							cancelar=!ventana.guardarArchivo();
						}
					}else if(respuesta == JOptionPane.CANCEL_OPTION){
						cancelar=true;
					}
	  			}
	  			
	  			if(!cancelar){
	  				ventanas.removeElement(ventana);
	  				ventana.dispose();
	  			}
	  			
	  			if(ventanas.size()==0){
	  				menuGuardar.setEnabled(false);
	  				menuGuardarComo.setEnabled(false);
	  				barraDeHerramientasGuardar.setEnabled(false);
	  				menuCascada.setEnabled(false);
	  				menuMosaicoHorizontal.setEnabled(false);
	  				menuMosaicoVertical.setEnabled(false);
	  				menuIconos.setEnabled(false);
	  				menuSiguiente.setEnabled(false);
	  				menuAnterior.setEnabled(false);
	  			}
    		}
	  			
	  		barraDeUtilidadesExpresion.requestFocus();
		}catch(Exception ex){
			ex.printStackTrace();
			JOptionPane.showMessageDialog(esteJFrame,"Se produjo un error al cerrar la ventana: " + ex.getMessage(),"Error al guardar el archivo",JOptionPane.ERROR_MESSAGE);
		}
    }

    public void internalFrameClosed(InternalFrameEvent e) {
        barraDeUtilidadesExpresion.requestFocus();
    }

    public void internalFrameOpened(InternalFrameEvent e) {
        barraDeUtilidadesExpresion.requestFocus();
    }

    public void internalFrameIconified(InternalFrameEvent e) {
        barraDeUtilidadesExpresion.requestFocus();
    }

    public void internalFrameDeiconified(InternalFrameEvent e) {
        barraDeUtilidadesExpresion.requestFocus();
    }

    public void internalFrameActivated(InternalFrameEvent e) {
    	if(e.getInternalFrame()==Graficos2D){
    		barraDeGraficos.setVisible(true);
    		barraDeGraficos3D.setVisible(false);
    	}else if(e.getInternalFrame()==Graficos3D){
    		barraDeGraficos3D.setVisible(true);
    		barraDeGraficos.setVisible(false);
    	}else{
        	barraDeGraficos.setVisible(false);
        	barraDeGraficos3D.setVisible(false);
    	}
        barraDeUtilidadesExpresion.requestFocus();
    }

    public void internalFrameDeactivated(InternalFrameEvent e) {
        barraDeUtilidadesExpresion.requestFocus();
    }
    
}//Fin de MathExplorer