package itcr.mathexplorer.constantes;

import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.funciones.*;
import itcr.mathexplorer.herramientas.Opciones;
import itcr.mathexplorer.herramientas.MetodosNumericos;
import java.math.BigDecimal;

/**
 * @(#)E.java
 * 
 * Esta es la implementación de la constante e.
 * 
 * @author Alexander Borbón Alpízar
 */
public class E extends Constante{
	/**
	 * Constructor vacío de la constante E.
	 */
	public E() {	}

	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		Nodo comodin;
		
		try{
			if (Opciones.getPresicion()){
				return expresionPrefijo;
			}else{
				int decimales=Opciones.getNumDecimales();
				if(decimales<=Opciones.getLongNumeros()){
					comodin=new Nodo("Decimal",1);
					comodin.agregaOperando(new Nodo("2.7182818284590452".substring(0,decimales+2)));
				}else{
					comodin=new Nodo("DecimalLargo",1);
					comodin.agregaOperando(new Nodo(MetodosNumericos.calculeE(BigDecimal.ONE,decimales).toString()));
				}
				return comodin;
			}
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar la constante e");
		}	
	}
}
