package itcr.mathexplorer.expresion;

import itcr.mathexplorer.exceptions.*;
import itcr.mathexplorer.tablas.*;
import itcr.mathexplorer.traductores.*;

/**
 *@(#)Expresion.java
 *
 *Esta es una clase que mantiene la expresión que el usuario digitó,
 *esta expresión la mantiene en tres formatos para rapidez del programa:
 *en formato prefijo, en infijo y en forma de imagen (JPanel)
 *
 *@author Alexander Borbón Alpízar
 */
public class Expresion{

	//VARIABLES PRIVADAS, solo se tiene acceso a travez de funciones
	
	private Nodo expresionEnPrefijo;
	private String expresionEnInfijo;
	
	private TraductorInfijoPrefijo traductorinfijoprefijo;
	private TraductorPrefijoInfijo traductorprefijoinfijo;
	private TraductorPrefijoImagen expresionEnImagen;
	
//	private SimplificacionAutomatica simplificacionautomatica;
	
	
	//CONSTRUCTORES
	/**
	 * Contructor de la Expresion
	 * @param tablaoperadores La tabla que contiene los operadores.
	 * @param tablafunciones La tabla que contiene las funciones.
	 * @param tablaconstantes La tabla que contiene las constantes.
	 * @param tablafuncionesusuario La tabla que contiene las funciones definidas por el usuario.
	 */
	public Expresion(TablaOperadores tablaoperadores, TablaFunciones tablafunciones, TablaConstantes tablaconstantes, TablaFuncionesUsuario tablafuncionesusuario){
		expresionEnPrefijo=new Nodo();
		expresionEnInfijo="";
		traductorinfijoprefijo=new TraductorInfijoPrefijo(tablaoperadores, tablafunciones, tablaconstantes, tablafuncionesusuario);
		traductorprefijoinfijo=new TraductorPrefijoInfijo(tablaoperadores, tablafunciones, tablaconstantes, tablafuncionesusuario);
		expresionEnImagen=new TraductorPrefijoImagen(tablaoperadores, tablafunciones, tablaconstantes, tablafuncionesusuario);
	}
	
	//FUNCIONES PUBLICAS

	/**
	 *Función que cambia la expresión que se guarda dando una nueva en formato infijo.
	 *@param exprEnInfijo La nueva expresión a guardar, esta debe estar en formato infijo.
	 *@param tipoGrafico Indica si el gráfico de la expresión es de la que da el estudiante (false) o es una ya simplificada (true).
	 *@return Se devuelve la misma expresión para recursividad.
	 *@exception SintaxException Se lanza si hay error de sintaxis en la expresión dada.
	 */
	public Expresion cambiaExpresionInfijo(String exprEnInfijo, boolean tipoGrafico) throws SintaxException{
		try{
			expresionEnPrefijo=traductorinfijoprefijo.traducir(exprEnInfijo);
			expresionEnInfijo=traductorprefijoinfijo.traducir(expresionEnPrefijo);
			expresionEnImagen.cambiaExpresion(expresionEnPrefijo, tipoGrafico);
			
			return this;
		}catch(SintaxException e){
			e.printStackTrace();
			throw e;
		}
		
	}
	
	/**
	 *Función que cambia la expresión que se guarda dando una nueva en formato prefijo.
	 *@param exprEnPrefijo La nueva expresión a guardar, esta debe estar en formato prefijo.
	 *@param tipoGrafico Indica si el gráfico de la expresión es de la que da el estudiante (false) o es una ya simplificada (true).
	 *@return La misma expresión para recursividad.
	 *@exception SintaxException Se lanza si hay error de sintaxis en la expresión dada.
	 */
	public Expresion cambiaExpresionPrefijo(Nodo exprEnPrefijo, boolean tipoGrafico) throws SintaxException{
		try{
			expresionEnPrefijo=exprEnPrefijo;
			expresionEnInfijo=traductorprefijoinfijo.traducir(expresionEnPrefijo);
			expresionEnImagen.cambiaExpresion(expresionEnPrefijo, tipoGrafico);
			
			return this;
		}catch(SintaxException e){
			e.printStackTrace();
			throw e;
		}
	}
	
	/**
	 *Función que devuelve la expresión que se guarda en formato infijo.
	 *@return La expresión que se guarda en formato infijo.
	 */
	public String obtenExpresionInfijo(){
		return expresionEnInfijo;
	}
	
	/**
	 *Función que devuelve la expresión que se guarda en formato postfijo.
	 *@return La expresión que se guarda en formato postfijo.
	 */
	public Nodo obtenExpresionPrefijo(){
		return expresionEnPrefijo;
	}
	
	/**
	 *Función que devuelve la expresión que se guarda en forma de imagen (un JPanel).
	 *@return La expresión que se guarda en forma de imagen (en un JPanel).
	 */
	public TraductorPrefijoImagen obtenExpresionImagen(){
		return expresionEnImagen;
	}
	
}//Fin de SintaxException