package itcr.mathexplorer.funciones;

import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.herramientas.Opciones;
import itcr.mathexplorer.herramientas.TeoriaNumeros;
import java.math.BigInteger;

/**
 *@(#)ACR.java
 *
 *Esta es la implementación de la función que calcula
 *el Algoritmo Chino del Resto
 *
 *@author Alexander Borbón Alpízar
 */
public class ACR extends Funcion{
	
	//CONSTRUCTOR
	/**
	 * Constructor vacío para la función ACR
	 */
	public ACR() {	}

	//FUNCIONES PUBLICAS
	
	/**
	 *ACR admite un número par de parámetros
	 */
	public boolean permiteNumeroParametros(int numeroParametros){
		return numeroParametros%2==0;
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		Nodo comodin;
		BigInteger Uis[], Ms[];
		String respuesta;
		
		try{
			if(expresionPrefijo.getOperando(1).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(1).getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
			}
			
			Uis= new BigInteger[expresionPrefijo.getNumeroOperandos()/2];
			Ms= new BigInteger[expresionPrefijo.getNumeroOperandos()/2];
			
			for(int i=1; i<=expresionPrefijo.getNumeroOperandos(); i++){
				if(!funcionSimplificarNumeros.esEntero(expresionPrefijo.getOperando(i))){
					return new Nodo("Indefinido");
				}else{
					if(i%2==1){
						Uis[i/2]=new BigInteger(expresionPrefijo.getOperando(i).getOperando(1).getTipo());
					}else{
						Ms[i/2-1]=new BigInteger(expresionPrefijo.getOperando(i).getOperando(1).getTipo());
					}
				}
			}
			
			try{
				respuesta=(TeoriaNumeros.Z_ACR(Uis, Ms)).toString();
				
				if(respuesta.length()<Opciones.getLongNumeros()){
					if (Opciones.getPresicion()){
						comodin=new Nodo("Entero",1);
					}else{
						comodin=new Nodo("Decimal",1);
					}
				}else{
					if (Opciones.getPresicion()){
						comodin=new Nodo("EnteroLargo",1);
					}else{
						comodin=new Nodo("DecimalLargo",1);
					}
				}
				
				comodin.agregaOperando(new Nodo(respuesta));
				
			}catch(Exception e){
				e.printStackTrace();
				return new Nodo("Indefinido");
			}

			return comodin;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar primos");
		}	
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		try{
			if(!expresionPrefijo.getTipo().equals("primos")){
				expresionPrefijo.cambiaTipo("primos");
			}
			
			if(expresionPrefijo.getOperando(1).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(1).getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
			}
			
			for(int i=1; i<=expresionPrefijo.getNumeroOperandos(); i++){
				if(!funcionSimplificar.esNumero(expresionPrefijo.getOperando(i))){
					return expresionPrefijo;
				}
			}
			
			return simplificarNumeros(expresionPrefijo, funcionSimplificar, funcionSimplificarNumeros);
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar primos");
		}
	}
	
	/**
	 *Función que deriva la expresión recibida.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		Nodo comodin;
		
		try{
			if (Opciones.getPresicion()){
				comodin=new Nodo("Entero",1);
			}else{
				comodin=new Nodo("Decimal",1);
			}
			comodin.agregaOperando(new Nodo("0"));
			
			return comodin;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al derivar primos");
		}
	}

}