package itcr.mathexplorer.funciones;

import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.herramientas.Opciones;

/**
 *@(#)Abs.java
 *
 *Esta es la implementación de la función valor absoluto
 *
 *@author Alexander Borbón Alpízar
 */
public class Abs extends Funcion{
	
	//CONSTRUCTOR
	/**
	 * Constructor vacío para la función Abs
	 */
	public Abs() {
	}

	//FUNCIONES PUBLICAS
	
	/**
	 *El valor absoluto sólo admite un parámetro
	 */
	public boolean permiteNumeroParametros(int numeroParametros){
		return numeroParametros==1;
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		Nodo signo, comodin, comodin2;
		//String tipo;

		try{
			if(expresionPrefijo.getOperando(1).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(1).getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
			}
			
			comodin=new Nodo("signo",1);
			comodin.agregaOperando(expresionPrefijo.getOperando(1).clonarNodo());
			
			signo=funcionSimplificarNumeros.simplificaNumeros(comodin, funcionSimplificar);
			
			if(signo.esIgual(SimplificarNumeros.UNO) || signo.esIgual(SimplificarNumeros.CERO)){
				return expresionPrefijo.getOperando(1).clonarNodo();
			}else{
				if (Opciones.getPresicion()){
					comodin=new Nodo("Entero",1);
				}else{
					comodin=new Nodo("Decimal",1);
				}
				comodin.agregaOperando(new Nodo("-1"));
				
				comodin2=new Nodo("·",2);
				comodin2.agregaOperando(comodin);
				comodin2.agregaOperando(expresionPrefijo.getOperando(1).clonarNodo());
				
				return comodin2;
			}
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar un valor absoluto");
		}	
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		try{
			if(!expresionPrefijo.getTipo().equals("abs")){
				expresionPrefijo.cambiaTipo("abs");
			}
			
			if(expresionPrefijo.getOperando(1).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(1).getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
			}
			
			if(funcionSimplificar.esNumero(expresionPrefijo.getOperando(1))){
				return funcionSimplificar.simplifica(simplificarNumeros(expresionPrefijo, funcionSimplificar, funcionSimplificarNumeros));
			}
			return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar un valor absoluto");
		}	
	}
	
	/**
	 *Función que deriva la expresión recibida.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		Nodo comodin, comodin2;
		
		try{
			comodin=new Nodo("/",2);
			
			comodin.agregaOperando(expresionPrefijo.clonarNodo());
			comodin.agregaOperando(expresionPrefijo.getOperando(1).clonarNodo());
			
			comodin2=new Nodo("·",2);
			
			comodin2.agregaOperando(comodin);
			comodin2.agregaOperando(funcionDerivar.deriva(expresionPrefijo.getOperando(1).clonarNodo(), variable));
			
			return comodin2;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al derivar un valor absoluto");
		}
	}

}
