package itcr.mathexplorer.funciones;

import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.herramientas.Opciones;
import itcr.mathexplorer.tipos.BigRational;
import java.math.BigDecimal;

/**
 *@(#)Acos.java
 *
 *Esta es la implementación de la función arcocoseno
 *
 *@author Alexander Borbón Alpízar
 */
public class Acos extends Funcion{
	
	//CONSTRUCTOR
	/**
	 * Constructor vacío de la función Acos
	 */
	public Acos() {
	}

	//FUNCIONES PUBLICAS
	
	/**
	 *El arcocoseno sólo admite un parámetro
	 */
	public boolean permiteNumeroParametros(int numeroParametros){
		return numeroParametros==1;
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		Nodo operador1, comodin, comodin2, comodin3;
		BigDecimal numero;
		double numero2;
		String tipo;
		
		try{
			if(expresionPrefijo.getOperando(1).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(1).getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
			}
			
			operador1=expresionPrefijo.getOperando(1).clonarNodo();
			tipo=operador1.getTipo();
			
			if(!Opciones.getPresicion()){
				if(Opciones.getNumDecimales()<=Opciones.getLongNumeros() && operador1.getOperando(1).getTipo().length()<=Opciones.getLongNumeros()){
					numero2=(new Double(operador1.getOperando(1).getTipo())).doubleValue();
					
					if (numero2>1 || numero2<-1){
						return new Nodo("Indefinido");
					}else{
						numero2=Math.acos(numero2);
					}
					
					comodin=new Nodo("Decimal",1);
					comodin.agregaOperando(new Nodo((new Double(numero2)).toString()));
					
					return funcionSimplificar.simplifica(comodin);
				}else{
					comodin=funcionSimplificarNumeros.piEntre(2);
					comodin2=new Nodo("asen",1);
					comodin2.agregaOperando(operador1);
					comodin3=new Nodo("-",2);
					comodin3.agregaOperando(comodin);
					comodin3.agregaOperando(comodin2);
					
					return funcionSimplificar.simplifica(comodin3);
				}
			}else{
				if(tipo.length()<=5 || "EnteroLargo DecimalLargo Racional".indexOf(tipo)==-1){ //Si no es número directo (es una operación de números)
					int numDecimales=Opciones.getNumDecimales();
					boolean presicion=Opciones.getPresicion();
					
					Opciones.setNumDecimales(Opciones.getNumDecimales());
					Opciones.setPresicion(false);
					
					operador1=funcionSimplificar.simplifica(operador1);
					numero=new BigDecimal(operador1.getOperando(1).getTipo());
					
					Opciones.setNumDecimales(numDecimales);
					Opciones.setPresicion(presicion);
				}else if(tipo.equals("Racional")){
					String numerador, denominador;
					
					numerador=operador1.getOperando(1).getTipo();
					denominador=operador1.getOperando(2).getTipo();
					
					numero=(new BigRational(numerador, denominador)).toBigDecimal(Math.max(numerador.length(),denominador.length())+Opciones.getNumDecimales());
				}else{
					numero=new BigDecimal(operador1.getOperando(1).getTipo());
				}
				
				if(numero.compareTo(BigDecimal.ONE)>0 || numero.compareTo(new BigDecimal(-1))<0){
					return new Nodo("Indefinido");
				}
				
				operador1=expresionPrefijo.getOperando(1);
				
				if(operador1.esIgual(SimplificarNumeros.UNO)){
					if(Opciones.getAngulo()){
						return SimplificarNumeros.CERO.clonarNodo();
					}else{
						comodin=new Nodo("Entero");
						comodin.agregaOperando(new Nodo("0"));
						return comodin;
					}
				}
				
				if(operador1.esIgual(SimplificarNumeros.MENOSUNO)){
					if(Opciones.getAngulo()){
						return funcionSimplificarNumeros.pi();
					}else{
						comodin=new Nodo("Entero");
						comodin.agregaOperando(new Nodo("180"));
						return comodin;
					}
				}
				
				if(operador1.esIgual(SimplificarNumeros.CERO)){
					if(Opciones.getAngulo()){
						return funcionSimplificarNumeros.piEntre(2);
					}else{
						comodin=new Nodo("Entero");
						comodin.agregaOperando(new Nodo("90"));
						return comodin;
					}
				}
				
				comodin=new Nodo("Racional",2);
				comodin.agregaOperando(new Nodo("1"));
				comodin.agregaOperando(new Nodo("2"));
				
				if(operador1.esIgual(comodin)){
					if(Opciones.getAngulo()){
						return funcionSimplificarNumeros.piEntre(3);
					}else{
						comodin=new Nodo("Entero");
						comodin.agregaOperando(new Nodo("60"));
						return comodin;
					}
				}
				
				comodin=new Nodo("Racional",2);
				comodin.agregaOperando(new Nodo("-1"));
				comodin.agregaOperando(new Nodo("2"));
				
				if(operador1.esIgual(comodin)){
					if(Opciones.getAngulo()){
						return funcionSimplificarNumeros.piPorEntre(2,3);
					}else{
						comodin=new Nodo("Entero");
						comodin.agregaOperando(new Nodo("120"));
						return comodin;
					}
				}
				
				comodin=new Nodo("Racional",2);
				comodin.agregaOperando(new Nodo("1"));
				comodin.agregaOperando(new Nodo("2"));
				
				comodin2=new Nodo("Entero",1);
				comodin2.agregaOperando(new Nodo("2"));
				
				comodin3=new Nodo("sqrt",1);
				comodin3.agregaOperando(comodin2);
				
				comodin2=new Nodo("·",2);
				comodin2.agregaOperando(comodin);
				comodin2.agregaOperando(comodin3);
				
				if(operador1.esIgual(comodin2)){
					if(Opciones.getAngulo()){
						return funcionSimplificarNumeros.piEntre(4);
					}else{
						comodin=new Nodo("Entero");
						comodin.agregaOperando(new Nodo("45"));
						return comodin;
					}
				}
				
				comodin=new Nodo("Racional",2);
				comodin.agregaOperando(new Nodo("-1"));
				comodin.agregaOperando(new Nodo("2"));
				
				comodin2=new Nodo("Entero",1);
				comodin2.agregaOperando(new Nodo("2"));
				
				comodin3=new Nodo("sqrt",1);
				comodin3.agregaOperando(comodin2);
				
				comodin2=new Nodo("·",2);
				comodin2.agregaOperando(comodin);
				comodin2.agregaOperando(comodin3);
				
				if(operador1.esIgual(comodin2)){
					if(Opciones.getAngulo()){
						return funcionSimplificarNumeros.piPorEntre(3,4);
					}else{
						comodin=new Nodo("Entero");
						comodin.agregaOperando(new Nodo("135"));
						return comodin;
					}
				}
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("5"));
				
				comodin2=new Nodo("sqrt",1);
				comodin2.agregaOperando(comodin);
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("1"));
				
				comodin3=new Nodo("+",2);
				comodin3.agregaOperando(comodin);
				comodin3.agregaOperando(comodin2);
				
				comodin2=new Nodo("Racional",2);
				comodin2.agregaOperando(new Nodo("1"));
				comodin2.agregaOperando(new Nodo("4"));
				
				comodin=new Nodo("·",2);
				comodin.agregaOperando(comodin2);
				comodin.agregaOperando(comodin3);
				
				if(operador1.esIgual(comodin)){
					if(Opciones.getAngulo()){
						return funcionSimplificarNumeros.piEntre(5);
					}else{
						comodin=new Nodo("Entero");
						comodin.agregaOperando(new Nodo("36"));
						return comodin;
					}
				}
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("5"));
				
				comodin2=new Nodo("sqrt",1);
				comodin2.agregaOperando(comodin);
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("-1"));
				
				comodin3=new Nodo("+",2);
				comodin3.agregaOperando(comodin);
				comodin3.agregaOperando(comodin2);
				
				comodin2=new Nodo("Racional",2);
				comodin2.agregaOperando(new Nodo("1"));
				comodin2.agregaOperando(new Nodo("4"));
				
				comodin=new Nodo("·",2);
				comodin.agregaOperando(comodin2);
				comodin.agregaOperando(comodin3);
				
				if(operador1.esIgual(comodin)){
					if(Opciones.getAngulo()){
						return funcionSimplificarNumeros.piPorEntre(2,5);
					}else{
						comodin=new Nodo("Entero");
						comodin.agregaOperando(new Nodo("72"));
						return comodin;
					}
				}
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("5"));
				
				comodin2=new Nodo("sqrt",1);
				comodin2.agregaOperando(comodin);
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("-1"));
				
				comodin3=new Nodo("+",2);
				comodin3.agregaOperando(comodin);
				comodin3.agregaOperando(comodin2);
				
				comodin2=new Nodo("Racional",2);
				comodin2.agregaOperando(new Nodo("-1"));
				comodin2.agregaOperando(new Nodo("4"));
				
				comodin=new Nodo("·",2);
				comodin.agregaOperando(comodin2);
				comodin.agregaOperando(comodin3);
				
				if(operador1.esIgual(comodin)){
					if(Opciones.getAngulo()){
						return funcionSimplificarNumeros.piPorEntre(3,5);
					}else{
						comodin=new Nodo("Entero");
						comodin.agregaOperando(new Nodo("108"));
						return comodin;
					}
				}
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("5"));
				
				comodin2=new Nodo("sqrt",1);
				comodin2.agregaOperando(comodin);
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("1"));
				
				comodin3=new Nodo("+",2);
				comodin3.agregaOperando(comodin);
				comodin3.agregaOperando(comodin2);
				
				comodin2=new Nodo("Racional",2);
				comodin2.agregaOperando(new Nodo("-1"));
				comodin2.agregaOperando(new Nodo("4"));
				
				comodin=new Nodo("·",2);
				comodin.agregaOperando(comodin2);
				comodin.agregaOperando(comodin3);
				
				if(operador1.esIgual(comodin)){
					if(Opciones.getAngulo()){
						return funcionSimplificarNumeros.piPorEntre(4,5);
					}else{
						comodin=new Nodo("Entero");
						comodin.agregaOperando(new Nodo("144"));
						return comodin;
					}
				}
				
				comodin=new Nodo("Racional",2);
				comodin.agregaOperando(new Nodo("1"));
				comodin.agregaOperando(new Nodo("2"));
				
				comodin2=new Nodo("Entero",1);
				comodin2.agregaOperando(new Nodo("3"));
				
				comodin3=new Nodo("sqrt",1);
				comodin3.agregaOperando(comodin2);
				
				comodin2=new Nodo("·",2);
				comodin2.agregaOperando(comodin);
				comodin2.agregaOperando(comodin3);
				
				if(operador1.esIgual(comodin2)){
					if(Opciones.getAngulo()){
						return funcionSimplificarNumeros.piEntre(6);
					}else{
						comodin=new Nodo("Entero");
						comodin.agregaOperando(new Nodo("30"));
						return comodin;
					}
				}
				
				comodin=new Nodo("Racional",2);
				comodin.agregaOperando(new Nodo("-1"));
				comodin.agregaOperando(new Nodo("2"));
				
				comodin2=new Nodo("Entero",1);
				comodin2.agregaOperando(new Nodo("3"));
				
				comodin3=new Nodo("sqrt",1);
				comodin3.agregaOperando(comodin2);
				
				comodin2=new Nodo("·",2);
				comodin2.agregaOperando(comodin);
				comodin2.agregaOperando(comodin3);
				
				if(operador1.esIgual(comodin2)){
					if(Opciones.getAngulo()){
						return funcionSimplificarNumeros.piPorEntre(5,6);
					}else{
						comodin=new Nodo("Entero");
						comodin.agregaOperando(new Nodo("150"));
						return comodin;
					}
				}
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("2"));
				
				comodin2=new Nodo("sqrt",1);
				comodin2.agregaOperando(comodin);
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("2"));
				
				comodin3=new Nodo("+",2);
				comodin3.agregaOperando(comodin);
				comodin3.agregaOperando(comodin2);
				
				comodin=new Nodo("sqrt",1);
				comodin.agregaOperando(comodin3);
				
				comodin2=new Nodo("Racional",2);
				comodin2.agregaOperando(new Nodo("1"));
				comodin2.agregaOperando(new Nodo("2"));
				
				comodin3=new Nodo("·",2);
				comodin3.agregaOperando(comodin2);
				comodin3.agregaOperando(comodin);
				
				if(operador1.esIgual(comodin3)){
					if(Opciones.getAngulo()){
						return funcionSimplificarNumeros.piEntre(8);
					}else{
						comodin=new Nodo("Decimal");
						comodin.agregaOperando(new Nodo("22.5"));
						return comodin;
					}
				}
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("2"));
				
				comodin2=new Nodo("sqrt",1);
				comodin2.agregaOperando(comodin);
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("-1"));
				
				comodin3=new Nodo("·",2);
				comodin3.agregaOperando(comodin);
				comodin3.agregaOperando(comodin2);
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("2"));
				
				comodin2=new Nodo("+",2);
				comodin2.agregaOperando(comodin);
				comodin2.agregaOperando(comodin3);
				
				comodin=new Nodo("sqrt",1);
				comodin.agregaOperando(comodin2);
				
				comodin2=new Nodo("Racional",2);
				comodin2.agregaOperando(new Nodo("1"));
				comodin2.agregaOperando(new Nodo("2"));
				
				comodin3=new Nodo("·",2);
				comodin3.agregaOperando(comodin2);
				comodin3.agregaOperando(comodin);
				
				if(operador1.esIgual(comodin3)){
					if(Opciones.getAngulo()){
						return funcionSimplificarNumeros.piPorEntre(3,8);
					}else{
						comodin=new Nodo("Decimal");
						comodin.agregaOperando(new Nodo("67.5"));
						return comodin;
					}
				}
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("2"));
				
				comodin2=new Nodo("sqrt",1);
				comodin2.agregaOperando(comodin);
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("-1"));
				
				comodin3=new Nodo("·",2);
				comodin3.agregaOperando(comodin);
				comodin3.agregaOperando(comodin2);
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("2"));
				
				comodin2=new Nodo("+",2);
				comodin2.agregaOperando(comodin);
				comodin2.agregaOperando(comodin3);
				
				comodin=new Nodo("sqrt",1);
				comodin.agregaOperando(comodin2);
				
				comodin2=new Nodo("Racional",2);
				comodin2.agregaOperando(new Nodo("-1"));
				comodin2.agregaOperando(new Nodo("2"));
				
				comodin3=new Nodo("·",2);
				comodin3.agregaOperando(comodin2);
				comodin3.agregaOperando(comodin);
				
				if(operador1.esIgual(comodin3)){
					if(Opciones.getAngulo()){
						return funcionSimplificarNumeros.piPorEntre(5,8);
					}else{
						comodin=new Nodo("Decimal");
						comodin.agregaOperando(new Nodo("112.5"));
						return comodin;
					}
				}
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("2"));
				
				comodin2=new Nodo("sqrt",1);
				comodin2.agregaOperando(comodin);
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("2"));
				
				comodin3=new Nodo("+",2);
				comodin3.agregaOperando(comodin);
				comodin3.agregaOperando(comodin2);
				
				comodin=new Nodo("sqrt",1);
				comodin.agregaOperando(comodin3);
				
				comodin2=new Nodo("Racional",2);
				comodin2.agregaOperando(new Nodo("-1"));
				comodin2.agregaOperando(new Nodo("2"));
				
				comodin3=new Nodo("·",2);
				comodin3.agregaOperando(comodin2);
				comodin3.agregaOperando(comodin);
				
				if(operador1.esIgual(comodin3)){
					if(Opciones.getAngulo()){
						return funcionSimplificarNumeros.piPorEntre(7,8);
					}else{
						comodin=new Nodo("Decimal");
						comodin.agregaOperando(new Nodo("157.5"));
						return comodin;
					}
				}
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("5"));
				
				comodin2=new Nodo("sqrt",1);
				comodin2.agregaOperando(comodin);
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("2"));
				
				comodin3=new Nodo("·",2);
				comodin3.agregaOperando(comodin);
				comodin3.agregaOperando(comodin2);
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("10"));
				
				comodin2=new Nodo("+",2);
				comodin2.agregaOperando(comodin);
				comodin2.agregaOperando(comodin3);
				
				comodin=new Nodo("sqrt",1);
				comodin.agregaOperando(comodin2);
				
				comodin2=new Nodo("Racional",2);
				comodin2.agregaOperando(new Nodo("1"));
				comodin2.agregaOperando(new Nodo("4"));
				
				comodin3=new Nodo("·",2);
				comodin3.agregaOperando(comodin2);
				comodin3.agregaOperando(comodin);
				
				if(operador1.esIgual(comodin3)){
					if(Opciones.getAngulo()){
						return funcionSimplificarNumeros.piEntre(10);
					}else{
						comodin=new Nodo("Entero");
						comodin.agregaOperando(new Nodo("18"));
						return comodin;
					}
				}
				
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("5"));
					
					comodin2=new Nodo("sqrt",1);
					comodin2.agregaOperando(comodin);
					
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("-2"));
					
					comodin3=new Nodo("·",2);
					comodin3.agregaOperando(comodin);
					comodin3.agregaOperando(comodin2);
					
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("10"));
					
					comodin2=new Nodo("+",2);
					comodin2.agregaOperando(comodin);
					comodin2.agregaOperando(comodin3);
					
					comodin=new Nodo("sqrt",1);
					comodin.agregaOperando(comodin2);
					
					comodin2=new Nodo("Racional",2);
					comodin2.agregaOperando(new Nodo("1"));
					comodin2.agregaOperando(new Nodo("4"));
					
					comodin3=new Nodo("·",2);
					comodin3.agregaOperando(comodin2);
					comodin3.agregaOperando(comodin);
				
				if(operador1.esIgual(comodin3)){
					if(Opciones.getAngulo()){
						return funcionSimplificarNumeros.piPorEntre(3,10);
					}else{
						comodin=new Nodo("Entero");
						comodin.agregaOperando(new Nodo("54"));
						return comodin;
					}
				}
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("5"));
				
				comodin2=new Nodo("sqrt",1);
				comodin2.agregaOperando(comodin);
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("-2"));
				
				comodin3=new Nodo("·",2);
				comodin3.agregaOperando(comodin);
				comodin3.agregaOperando(comodin2);
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("10"));
				
				comodin2=new Nodo("+",2);
				comodin2.agregaOperando(comodin);
				comodin2.agregaOperando(comodin3);
				
				comodin=new Nodo("sqrt",1);
				comodin.agregaOperando(comodin2);
				
				comodin2=new Nodo("Racional",2);
				comodin2.agregaOperando(new Nodo("-1"));
				comodin2.agregaOperando(new Nodo("4"));
				
				comodin3=new Nodo("·",2);
				comodin3.agregaOperando(comodin2);
				comodin3.agregaOperando(comodin);
				
				if(operador1.esIgual(comodin3)){
					if(Opciones.getAngulo()){
						return funcionSimplificarNumeros.piPorEntre(7,10);
					}else{
						comodin=new Nodo("Entero");
						comodin.agregaOperando(new Nodo("126"));
						return comodin;
					}
				}
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("5"));
				
				comodin2=new Nodo("sqrt",1);
				comodin2.agregaOperando(comodin);
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("2"));
				
				comodin3=new Nodo("·",2);
				comodin3.agregaOperando(comodin);
				comodin3.agregaOperando(comodin2);
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("10"));
				
				comodin2=new Nodo("+",2);
				comodin2.agregaOperando(comodin);
				comodin2.agregaOperando(comodin3);
				
				comodin=new Nodo("sqrt",1);
				comodin.agregaOperando(comodin2);
				
				comodin2=new Nodo("Racional",2);
				comodin2.agregaOperando(new Nodo("-1"));
				comodin2.agregaOperando(new Nodo("4"));
				
				comodin3=new Nodo("·",2);
				comodin3.agregaOperando(comodin2);
				comodin3.agregaOperando(comodin);
				
				if(operador1.esIgual(comodin3)){
					if(Opciones.getAngulo()){
						return funcionSimplificarNumeros.piPorEntre(9,10);
					}else{
						comodin=new Nodo("Entero");
						comodin.agregaOperando(new Nodo("162"));
						return comodin;
					}
				}
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("6"));
				
				comodin3=new Nodo("sqrt",1);
				comodin3.agregaOperando(comodin);
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("2"));
				
				comodin2=new Nodo("sqrt",1);
				comodin2.agregaOperando(comodin);
				
				comodin=new Nodo("+",2);
				comodin.agregaOperando(comodin2);
				comodin.agregaOperando(comodin3);
				
				comodin3=new Nodo("Racional",2);
				comodin3.agregaOperando(new Nodo("1"));
				comodin3.agregaOperando(new Nodo("4"));
				
				comodin2=new Nodo("·",2);
				comodin2.agregaOperando(comodin3);
				comodin2.agregaOperando(comodin);
				
				if(operador1.esIgual(comodin2)){
					if(Opciones.getAngulo()){
						return funcionSimplificarNumeros.piEntre(12);
					}else{
						comodin=new Nodo("Entero");
						comodin.agregaOperando(new Nodo("15"));
						return comodin;
					}
				}
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("2"));
				
				comodin3=new Nodo("sqrt",1);
				comodin3.agregaOperando(comodin);
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("-1"));
				
				comodin2=new Nodo("·",2);
				comodin2.agregaOperando(comodin);
				comodin2.agregaOperando(comodin3);
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("6"));
				
				comodin3=new Nodo("sqrt",1);
				comodin3.agregaOperando(comodin);
				
				comodin=new Nodo("+",2);
				comodin.agregaOperando(comodin2);
				comodin.agregaOperando(comodin3);
				
				comodin3=new Nodo("Racional",2);
				comodin3.agregaOperando(new Nodo("1"));
				comodin3.agregaOperando(new Nodo("4"));
				
				comodin2=new Nodo("·",2);
				comodin2.agregaOperando(comodin3);
				comodin2.agregaOperando(comodin);
				
				if(operador1.esIgual(comodin2)){
					if(Opciones.getAngulo()){
						return funcionSimplificarNumeros.piPorEntre(5,12);
					}else{
						comodin=new Nodo("Entero");
						comodin.agregaOperando(new Nodo("75"));
						return comodin;
					}
				}
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("6"));
				
				comodin3=new Nodo("sqrt",1);
				comodin3.agregaOperando(comodin);
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("-1"));
				
				comodin2=new Nodo("·",2);
				comodin2.agregaOperando(comodin);
				comodin2.agregaOperando(comodin3);
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("2"));
				
				comodin3=new Nodo("sqrt",1);
				comodin3.agregaOperando(comodin);
				
				comodin=new Nodo("+",2);
				comodin.agregaOperando(comodin3);
				comodin.agregaOperando(comodin2);
				
				comodin3=new Nodo("Racional",2);
				comodin3.agregaOperando(new Nodo("1"));
				comodin3.agregaOperando(new Nodo("4"));
				
				comodin2=new Nodo("·",2);
				comodin2.agregaOperando(comodin3);
				comodin2.agregaOperando(comodin);
				
				if(operador1.esIgual(comodin2)){
					if(Opciones.getAngulo()){
						return funcionSimplificarNumeros.piPorEntre(7,12);
					}else{
						comodin=new Nodo("Entero");
						comodin.agregaOperando(new Nodo("105"));
						return comodin;
					}
				}
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("6"));
				
				comodin3=new Nodo("sqrt",1);
				comodin3.agregaOperando(comodin);
				
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("2"));
				
				comodin2=new Nodo("sqrt",1);
				comodin2.agregaOperando(comodin);
				
				comodin=new Nodo("+",2);
				comodin.agregaOperando(comodin2);
				comodin.agregaOperando(comodin3);
				
				comodin3=new Nodo("Racional",2);
				comodin3.agregaOperando(new Nodo("-1"));
				comodin3.agregaOperando(new Nodo("4"));
				
				comodin2=new Nodo("·",2);
				comodin2.agregaOperando(comodin3);
				comodin2.agregaOperando(comodin);
				
				if(operador1.esIgual(comodin2)){
					if(Opciones.getAngulo()){
						return funcionSimplificarNumeros.piPorEntre(11,12);
					}else{
						comodin=new Nodo("Entero");
						comodin.agregaOperando(new Nodo("165"));
						return comodin;
					}
				}
				
			}
			
			return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar un arcocoseno");
		}
	}
	
	/**
	 *Función que simplifica la expresión recibida. Si esta función se evalúa aquí quiere decir que no encontró el operador
	 *donde se tenía que evaluar, lo que es un error.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		try{
			if(!expresionPrefijo.getTipo().equals("acos")){
				expresionPrefijo.cambiaTipo("acos");
			}
			
			if(expresionPrefijo.getOperando(1).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(1).getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
			}
			
			if(funcionSimplificar.esNumero(expresionPrefijo.getOperando(1))){
				return simplificarNumeros(expresionPrefijo, funcionSimplificar, funcionSimplificarNumeros);
			}
			
			return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar un arcocoseno");
		}
	}
	
	/**
	 *Función que deriva la expresión recibida.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		Nodo comodin, comodin2, comodin3;
		
		try{
			comodin=new Nodo("^",2);
			if (Opciones.getPresicion()){
				comodin2=new Nodo("Entero",1);
			}else{
				comodin2=new Nodo("Decimal",1);
			}
			comodin2.agregaOperando(new Nodo("2"));
			
			comodin.agregaOperando(expresionPrefijo.getOperando(1).clonarNodo());
			comodin.agregaOperando(comodin2);
			
			comodin3=new Nodo("-",2);
			
			if (Opciones.getPresicion()){
				comodin2=new Nodo("Entero",1);
			}else{
				comodin2=new Nodo("Decimal",1);
			}
			comodin2.agregaOperando(new Nodo("1"));
			
			comodin3.agregaOperando(comodin2);
			comodin3.agregaOperando(comodin);
			
			comodin=new Nodo("sqrt",1);
			comodin.agregaOperando(comodin3);
			
			if (Opciones.getPresicion()){
				comodin2=new Nodo("Entero",1);
			}else{
				comodin2=new Nodo("Decimal",1);
			}
			comodin2.agregaOperando(new Nodo("-1"));
			
			comodin3=new Nodo("/",2);
			comodin3.agregaOperando(comodin2);
			comodin3.agregaOperando(comodin);
			
			comodin=new Nodo("·",2);
			
			comodin.agregaOperando(comodin3);
			comodin.agregaOperando(funcionDerivar.deriva(expresionPrefijo.getOperando(1).clonarNodo(), variable));
			
			return comodin;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al derivar un arcocoseno");
		}
	}
}
