package itcr.mathexplorer.funciones;

import itcr.mathexplorer.expresion.*;

/**
 *@(#)AngPosEst.java
 *
 *Esta es la implementación de la función que pasa un ángulo cualquiera a su posición estándar.
 *
 *@author Alexander Borbón Alpízar
 */
public class AngPosEst extends Funcion{
	
	//CONSTRUCTOR
	/**
	 * Constructor vacío de la función AngPosEst
	 */
	public AngPosEst() {
	}

	//FUNCIONES PUBLICAS
	
	/**
	 *El seno sólo admite un parámetro
	 */
	public boolean permiteNumeroParametros(int numeroParametros){
		return numeroParametros==1;
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		try{
			if(expresionPrefijo.getOperando(1).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(1).getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
			}
			return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar un ángulo en posición estándar");
		}	
	}

	/**
	 *Función que simplifica la expresión recibida. Si esta función se evalúa aquí quiere decir que no encontró el operador
	 *donde se tenía que evaluar, lo que es un error.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		Nodo operando1;
		String tipo;
		
		try{
			if(!expresionPrefijo.getTipo().equals("angposest")){
				expresionPrefijo.cambiaTipo("angposest");
			}
			
			if(expresionPrefijo.getOperando(1).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(1).getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
			}
			
			operando1=expresionPrefijo.getOperando(1);
			tipo=operando1.getTipo();
			
			if(funcionSimplificar.esNumero(operando1)){
				if(tipo.equals("Entero")){
					return expresionPrefijo;
				}else if(tipo.equals("EnteroLargo")){
					return expresionPrefijo;
				}else if(tipo.equals("Decimal")){
					return expresionPrefijo;
				}else if(tipo.equals("DecimalLargo")){
					return expresionPrefijo;
				}else if(tipo.equals("Racional")){
					return expresionPrefijo;
				}else{ 
					return expresionPrefijo;
				}
			}else{
				return expresionPrefijo;
			}
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar un ángulo en posición estándar");
		}
	}
	
	/**
	 *Función que deriva la expresión recibida.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		try{
			return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al derivar un ángulo en posición estándar");
		}
	}
}
