package itcr.mathexplorer.funciones;

import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.herramientas.Opciones;

/**
 *@(#)Asec.java
 *
 *Esta es la implementación de la función arcosecante
 *
 *@author Alexander Borbón Alpízar
 */
public class Asec extends Funcion{
	
	//CONSTRUCTOR
	/**
	 * Constructor vacío de la función Asec
	 */
	public Asec() {	}

	//FUNCIONES PUBLICAS
	
	/**
	 *El arcosecante sólo admite un parámetro
	 */
	public boolean permiteNumeroParametros(int numeroParametros){
		return numeroParametros==1;
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		try{
			if(expresionPrefijo.getOperando(1).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(1).getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
			}
			return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar una arcosecante");
		}
	}

	/**
	 *Función que simplifica la expresión recibida. Si esta función se evalúa aquí quiere decir que no encontró el operador
	 *donde se tenía que evaluar, lo que es un error.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		try{
			if(!expresionPrefijo.getTipo().equals("asec")){
				expresionPrefijo.cambiaTipo("asec");
			}
			
			if(expresionPrefijo.getOperando(1).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(1).getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
			}
			return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar una arcosecante");
		}
	}
	
	/**
	 *Función que deriva la expresión recibida.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		Nodo comodin, comodin2, comodin3;
		
		try{
			comodin=new Nodo("^",2);
			if (Opciones.getPresicion()){
				comodin2=new Nodo("Entero",1);
			}else{
				comodin2=new Nodo("Decimal",1);
			}
			comodin2.agregaOperando(new Nodo("2"));
			
			comodin.agregaOperando(expresionPrefijo.getOperando(1).clonarNodo());
			comodin.agregaOperando(comodin2);
			
			comodin3=new Nodo("-",2);
			
			if (Opciones.getPresicion()){
				comodin2=new Nodo("Entero",1);
			}else{
				comodin2=new Nodo("Decimal",1);
			}
			comodin2.agregaOperando(new Nodo("1"));
			
			comodin3.agregaOperando(comodin);
			comodin3.agregaOperando(comodin2);
			
			comodin=new Nodo("sqrt",1);
			comodin.agregaOperando(comodin3);
			
			comodin2=new Nodo("abs",1);
			comodin2.agregaOperando(expresionPrefijo.getOperando(1).clonarNodo());
			
			comodin3=new Nodo("·",2);
			comodin3.agregaOperando(comodin2);
			comodin3.agregaOperando(comodin);
			
			if (Opciones.getPresicion()){
				comodin2=new Nodo("Entero",1);
			}else{
				comodin2=new Nodo("Decimal",1);
			}
			comodin2.agregaOperando(new Nodo("1"));
			
			comodin=new Nodo("/",2);
			comodin.agregaOperando(comodin2);
			comodin.agregaOperando(comodin3);
			
			comodin3=new Nodo("·",2);
			
			comodin3.agregaOperando(comodin);
			comodin3.agregaOperando(funcionDerivar.deriva(expresionPrefijo.getOperando(1).clonarNodo(), variable));
			
			return comodin3;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al derivar un arcosecante");
		}
	}
}
