package itcr.mathexplorer.funciones;

import itcr.mathexplorer.expresion.*;

/**
 *@(#)Coef.java
 *
 *Esta es la implementación de la función Coef
 *Esta función toma los coeficientes que tiene un polinomio
 *en orden con respecto a la variable dada.
 *
 *@author Alexander Borbón Alpízar
 */
public class Coef extends Funcion{
	
	private Nodo coeficientes;
	
	//CONSTRUCTOR
	/**
	 * Constructor vacío de la función EsPolinomio.
	 */
	public Coef() {
	}

	//FUNCIONES PUBLICAS
	
	/**
	 *El signo sólo admite dos parámetros: el polinomio y la variable
	 */
	public boolean permiteNumeroParametros(int numeroParametros){
		return numeroParametros==2;
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		Nodo comodin, comodin2;
		
		try{
			if(!expresionPrefijo.getTipo().equals("coef")){
				expresionPrefijo.cambiaTipo("coef");
			}
			
			if(expresionPrefijo.getOperando(1).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(1).getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
			}
			
			comodin=new Nodo("Fila",1);
			comodin.agregaOperando(expresionPrefijo.getOperando(1));
			
			comodin2=new Nodo("Matriz",1);
			comodin2.agregaOperando(comodin);
			return comodin2;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar Coef");
		}	
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		Nodo comodin;
		Nodo expresion=expresionPrefijo.getOperando(1);
		Nodo variable=expresionPrefijo.getOperando(2);
		
		try{
			if(!expresionPrefijo.getTipo().equals("coef")){
				expresionPrefijo.cambiaTipo("coef");
			}
			
			if(expresionPrefijo.getOperando(1).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(1).getTipo().equals("Boolean") || expresionPrefijo.getOperando(2).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(2).getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
			}
			
			if(!funcionSimplificar.esPolinomio(expresion) || !expresionPrefijo.getOperando(2).getTipo().equals("Variable")){
				return new Nodo("Indefinido");
			}
			
			if(funcionSimplificar.esNumero(expresionPrefijo.getOperando(1))){
				return simplificarNumeros(expresionPrefijo, funcionSimplificar, funcionSimplificarNumeros);
			}else{
				coeficientes=new Nodo("Fila");
				
				if(!expresion.getTipo().equals("+")){
					coeficientes=procesaMonomio(expresion, variable);
				}else{
					for(int i=1;i<=expresion.getNumeroOperandos();i++){
						coeficientes=procesaMonomio(expresion.getOperando(i), variable);
					}
				}
				
				for(int i=1; i<=coeficientes.getNumeroOperandos(); i++){
					if(coeficientes.getOperando(i).getTipo().equals("+") && coeficientes.getOperando(i).getNumeroOperandos()==0){
						coeficientes.eliminaOperando(i);
						coeficientes.agregaOperando(SimplificarNumeros.CERO, i);
					}else if(coeficientes.getOperando(i).getTipo().equals("+") && coeficientes.getOperando(i).getNumeroOperandos()==1){
						comodin=coeficientes.getOperando(i).getOperando(1);
						coeficientes.eliminaOperando(i);
						coeficientes.agregaOperando(comodin, i);
					}
				}
				
				comodin=new Nodo("Matriz",1);
				comodin.agregaOperando(coeficientes);
				return comodin;
			}
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar Coef");
		}
	}
	
	/**
	 *Función que deriva la expresión recibida.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		try{
			return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al derivar Coef");
		}
	}
	
	//FUNCIONES PRIVADAS
	
	private Nodo procesaMonomio(Nodo monomio, Nodo variable){
		Nodo comodin;
		int potencia;
		boolean encontro=false;
		
		comodin=new Nodo("Indefinido");
		if(!monomio.getTipo().equals("·")){
			if(monomio.esIgual(variable)){
				comodin=agregaCoef(SimplificarNumeros.UNO, 1);
			}else if(monomio.getTipo().equals("^") && monomio.getOperando(1).esIgual(variable)){
				potencia=new Integer(monomio.getOperando(2).getOperando(1).getTipo());
				comodin=agregaCoef(SimplificarNumeros.UNO, potencia);
			}else{
				comodin=agregaCoef(monomio, 0);
			}
		}else{
			for(int j=1;j<=monomio.getNumeroOperandos();j++){
				if(monomio.getOperando(j).esIgual(variable)){
					monomio.eliminaOperando(j);
					if(monomio.getNumeroOperandos()==1){
						comodin=agregaCoef(monomio.getOperando(1), 1);
					}else{
						comodin=agregaCoef(monomio, 1);
					}
					encontro=true;
				}else if(monomio.getOperando(j).getTipo().equals("^") && monomio.getOperando(j).getOperando(1).esIgual(variable)){
					potencia=new Integer(monomio.getOperando(j).getOperando(2).getOperando(1).getTipo());
					monomio.eliminaOperando(j);
					if(monomio.getNumeroOperandos()==1){
						comodin=agregaCoef(monomio.getOperando(1), potencia);
					}else{
						comodin=agregaCoef(monomio, potencia);
					}
					encontro=true;
				}
				
				if(j==monomio.getNumeroOperandos() && !encontro){ //Si llegó el final y no se encontró variable
					comodin=agregaCoef(monomio, 0);
				}
			}
		}
		
		return comodin;
	}
	
	private Nodo agregaCoef(Nodo coeficiente, int potencia){
		int num;
		
		if(potencia<coeficientes.getNumeroOperandos()){
			coeficientes.getOperando(potencia+1).agregaOperando(coeficiente);
		}else{
			num=coeficientes.getNumeroOperandos();
			for(int k=0;k<=potencia-num;k++){
				coeficientes.agregaOperando(new Nodo("+"));
			}
			coeficientes.getOperando(potencia+1).agregaOperando(coeficiente);
		}
		
		return coeficientes;
	}
}
