package itcr.mathexplorer.funciones;

import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.herramientas.Opciones;
import java.math.BigDecimal;

/**
 *@(#)Csc.java
 *
 *Esta es la implementación de la función cosecante
 *
 *@author Alexander Borbón Alpízar
 */
public class Csc extends Funcion{
	
	//CONSTRUCTOR
	/**
	 * Constructor vacío de la función Csc
	 */
	public Csc() {
	}

	//FUNCIONES PUBLICAS
	
	/**
	 *El cosecante sólo admite un parámetro
	 */
	public boolean permiteNumeroParametros(int numeroParametros){
		return numeroParametros==1;
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		Nodo operador1, comodin, comodin2, comodin3;
		
		try{
			if(expresionPrefijo.getOperando(1).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(1).getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
			}
			
			operador1=expresionPrefijo.getOperando(1).clonarNodo();
			
			//Si el ángulo está en grados se lo lleva a radianes
			if(!Opciones.getAngulo()){
				comodin=new Nodo("Constante",1);
				comodin.agregaOperando(new Nodo("#pi"));
				
				comodin3=new Nodo("Entero",1);
				comodin3.agregaOperando(new Nodo("180"));
				
				comodin2=new Nodo("/",2);
				comodin2.agregaOperando(comodin);
				comodin2.agregaOperando(comodin3);
				
				comodin=new Nodo("·",2);
				comodin.agregaOperando(operador1);
				comodin.agregaOperando(comodin2);
				
				operador1=funcionSimplificar.simplifica(comodin);
			}
			
			//Este procedimiento se lleva el ángulo entre 0 y 2pi
			comodin=new Nodo("Constante",1);
			comodin.agregaOperando(new Nodo("#pi"));
			
			comodin2=new Nodo("Entero",1);
			comodin2.agregaOperando(new Nodo("2"));
			
			comodin3=new Nodo("·",2);
			comodin3.agregaOperando(comodin2);
			comodin3.agregaOperando(comodin);
			
			comodin=new Nodo("/",2);
			comodin.agregaOperando(operador1.clonarNodo());
			comodin.agregaOperando(comodin3);
			
			comodin2=new Nodo("truncar",1);
			comodin2.agregaOperando(comodin);
			
			comodin2=funcionSimplificar.simplifica(comodin2);
			
			if(comodin2.getOperando(1).getTipo().charAt(0)=='-'){
				comodin=new Nodo("-",2);
				comodin.agregaOperando(comodin2);
				comodin.agregaOperando(SimplificarNumeros.UNO.clonarNodo());
				
				comodin2=comodin;
			}
			
			comodin=new Nodo("·",2);
			comodin.agregaOperando(comodin2);
			comodin.agregaOperando(comodin3);
			
			comodin2=new Nodo("-",2);
			comodin2.agregaOperando(operador1);
			comodin2.agregaOperando(comodin);
			
			operador1=funcionSimplificar.simplifica(comodin2);
			
			if(!Opciones.getPresicion()){
				if(Opciones.getNumDecimales()<=Opciones.getLongNumeros()){
					double numero=(new Double(operador1.getOperando(1).getTipo())).doubleValue();
					
					numero=Math.sin(numero);
					
					comodin=new Nodo("Decimal",1);
					comodin.agregaOperando(new Nodo((new Double(numero)).toString()));
					
					return funcionSimplificar.simplifica(comodin);
				}else{
					comodin=new Nodo("sen",1);
					comodin.agregaOperando(operador1.clonarNodo());
					
					comodin=funcionSimplificar.simplifica(comodin);
					
					BigDecimal numero=new BigDecimal(comodin.getOperando(1).getTipo());
					
					numero=BigDecimal.ONE.divide(numero, Opciones.getNumDecimales(), BigDecimal.ROUND_HALF_EVEN);
					
					comodin=new Nodo("DecimalLargo",1);
					comodin.agregaOperando(new Nodo(numero.toPlainString()));
					
					return funcionSimplificar.simplifica(comodin);
				}
			}else{
				if(operador1.esIgual(SimplificarNumeros.CERO) || operador1.esIgual(funcionSimplificarNumeros.pi())){
					return new Nodo("Indefinido");
				}else if(operador1.esIgual(funcionSimplificarNumeros.piEntre(2))){
					return SimplificarNumeros.UNO.clonarNodo();
				}else if(operador1.esIgual(funcionSimplificarNumeros.piPorEntre(3,2))){
					return SimplificarNumeros.MENOSUNO.clonarNodo();
				}else if(operador1.esIgual(funcionSimplificarNumeros.piEntre(3)) || operador1.esIgual(funcionSimplificarNumeros.piPorEntre(2,3))){
					comodin=new Nodo("Racional",2);
					comodin.agregaOperando(new Nodo("2"));
					comodin.agregaOperando(new Nodo("3"));
					
					comodin2=new Nodo("Entero",1);
					comodin2.agregaOperando(new Nodo("3"));
					
					comodin3=new Nodo("sqrt",1);
					comodin3.agregaOperando(comodin2);
					
					comodin2=new Nodo("·",2);
					comodin2.agregaOperando(comodin);
					comodin2.agregaOperando(comodin3);
					
					return comodin2;
				}else if(operador1.esIgual(funcionSimplificarNumeros.piPorEntre(4,3)) || operador1.esIgual(funcionSimplificarNumeros.piPorEntre(5,3))){
					comodin=new Nodo("Racional",2);
					comodin.agregaOperando(new Nodo("-2"));
					comodin.agregaOperando(new Nodo("3"));
					
					comodin2=new Nodo("Entero",1);
					comodin2.agregaOperando(new Nodo("3"));
					
					comodin3=new Nodo("sqrt",1);
					comodin3.agregaOperando(comodin2);
					
					comodin2=new Nodo("·",2);
					comodin2.agregaOperando(comodin);
					comodin2.agregaOperando(comodin3);
					
					return comodin2;
				}else if(operador1.esIgual(funcionSimplificarNumeros.piEntre(4)) || operador1.esIgual(funcionSimplificarNumeros.piPorEntre(3,4))){
					comodin2=new Nodo("Entero",1);
					comodin2.agregaOperando(new Nodo("2"));
					
					comodin3=new Nodo("sqrt",1);
					comodin3.agregaOperando(comodin2);
					
					return comodin3;
				}else if(operador1.esIgual(funcionSimplificarNumeros.piPorEntre(5,4)) || operador1.esIgual(funcionSimplificarNumeros.piPorEntre(7,4))){
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("-1"));
					
					comodin2=new Nodo("Entero",1);
					comodin2.agregaOperando(new Nodo("2"));
					
					comodin3=new Nodo("sqrt",1);
					comodin3.agregaOperando(comodin2);
					
					comodin2=new Nodo("·",2);
					comodin2.agregaOperando(comodin);
					comodin2.agregaOperando(comodin3);
					
					return comodin2;
				}else if(operador1.esIgual(funcionSimplificarNumeros.piEntre(5)) || operador1.esIgual(funcionSimplificarNumeros.piPorEntre(4,5))){
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("5"));
					
					comodin2=new Nodo("sqrt",1);
					comodin2.agregaOperando(comodin);
					
					comodin=new Nodo("Racional",2);
					comodin.agregaOperando(new Nodo("2"));
					comodin.agregaOperando(new Nodo("5"));
					
					comodin3=new Nodo("·",2);
					comodin3.agregaOperando(comodin);
					comodin3.agregaOperando(comodin2);
					
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("2"));
					
					comodin2=new Nodo("+",2);
					comodin2.agregaOperando(comodin);
					comodin2.agregaOperando(comodin3);
					
					comodin=new Nodo("sqrt",1);
					comodin.agregaOperando(comodin2);
					
					return comodin;
				}else if(operador1.esIgual(funcionSimplificarNumeros.piPorEntre(2,5)) || operador1.esIgual(funcionSimplificarNumeros.piPorEntre(3,5))){
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("5"));
					
					comodin2=new Nodo("sqrt",1);
					comodin2.agregaOperando(comodin);
					
					comodin=new Nodo("Racional",2);
					comodin.agregaOperando(new Nodo("-2"));
					comodin.agregaOperando(new Nodo("5"));
					
					comodin3=new Nodo("·",2);
					comodin3.agregaOperando(comodin);
					comodin3.agregaOperando(comodin2);
					
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("2"));
					
					comodin2=new Nodo("+",2);
					comodin2.agregaOperando(comodin);
					comodin2.agregaOperando(comodin3);
					
					comodin=new Nodo("sqrt",1);
					comodin.agregaOperando(comodin2);
					
					return comodin;
				}else if(operador1.esIgual(funcionSimplificarNumeros.piPorEntre(6,5)) || operador1.esIgual(funcionSimplificarNumeros.piPorEntre(9,5))){
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("5"));
					
					comodin2=new Nodo("sqrt",1);
					comodin2.agregaOperando(comodin);
					
					comodin=new Nodo("Racional",2);
					comodin.agregaOperando(new Nodo("2"));
					comodin.agregaOperando(new Nodo("5"));
					
					comodin3=new Nodo("·",2);
					comodin3.agregaOperando(comodin);
					comodin3.agregaOperando(comodin2);
					
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("2"));
					
					comodin2=new Nodo("+",2);
					comodin2.agregaOperando(comodin);
					comodin2.agregaOperando(comodin3);
					
					comodin=new Nodo("sqrt",1);
					comodin.agregaOperando(comodin2);
					
					comodin2=new Nodo("Entero",1);
					comodin2.agregaOperando(new Nodo("-1"));
					
					comodin3=new Nodo("·",2);
					comodin3.agregaOperando(comodin2);
					comodin3.agregaOperando(comodin);
					
					return comodin3;
				}else if(operador1.esIgual(funcionSimplificarNumeros.piPorEntre(7,5)) || operador1.esIgual(funcionSimplificarNumeros.piPorEntre(8,5))){
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("5"));
					
					comodin2=new Nodo("sqrt",1);
					comodin2.agregaOperando(comodin);
					
					comodin=new Nodo("Racional",2);
					comodin.agregaOperando(new Nodo("-2"));
					comodin.agregaOperando(new Nodo("5"));
					
					comodin3=new Nodo("·",2);
					comodin3.agregaOperando(comodin);
					comodin3.agregaOperando(comodin2);
					
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("2"));
					
					comodin2=new Nodo("+",2);
					comodin2.agregaOperando(comodin);
					comodin2.agregaOperando(comodin3);
					
					comodin=new Nodo("sqrt",1);
					comodin.agregaOperando(comodin2);
					
					comodin2=new Nodo("Entero",1);
					comodin2.agregaOperando(new Nodo("-1"));
					
					comodin3=new Nodo("·",2);
					comodin3.agregaOperando(comodin2);
					comodin3.agregaOperando(comodin);
					
					return comodin3;
				}else if(operador1.esIgual(funcionSimplificarNumeros.piEntre(6)) || operador1.esIgual(funcionSimplificarNumeros.piPorEntre(5,6))){
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("2"));
					
					return comodin;
				}else if(operador1.esIgual(funcionSimplificarNumeros.piPorEntre(7,6)) || operador1.esIgual(funcionSimplificarNumeros.piPorEntre(11,6))){
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("-2"));
					
					return comodin;
				}else if(operador1.esIgual(funcionSimplificarNumeros.piEntre(8)) || operador1.esIgual(funcionSimplificarNumeros.piPorEntre(7,8))){
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("2"));
					
					comodin2=new Nodo("sqrt",1);
					comodin2.agregaOperando(comodin);
					
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("2"));
					
					comodin3=new Nodo("·",2);
					comodin3.agregaOperando(comodin);
					comodin3.agregaOperando(comodin2);
					
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("4"));
					
					comodin2=new Nodo("+",2);
					comodin2.agregaOperando(comodin);
					comodin2.agregaOperando(comodin3);
					
					comodin=new Nodo("sqrt",1);
					comodin.agregaOperando(comodin2);
					
					return comodin;
				}else if(operador1.esIgual(funcionSimplificarNumeros.piPorEntre(9,8)) || operador1.esIgual(funcionSimplificarNumeros.piPorEntre(15,8))){
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("2"));
					
					comodin2=new Nodo("sqrt",1);
					comodin2.agregaOperando(comodin);
					
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("2"));
					
					comodin3=new Nodo("·",2);
					comodin3.agregaOperando(comodin);
					comodin3.agregaOperando(comodin2);
					
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("4"));
					
					comodin2=new Nodo("+",2);
					comodin2.agregaOperando(comodin);
					comodin2.agregaOperando(comodin3);
					
					comodin=new Nodo("sqrt",1);
					comodin.agregaOperando(comodin2);
					
					comodin2=new Nodo("Entero",1);
					comodin2.agregaOperando(new Nodo("-1"));
					
					comodin3=new Nodo("·",2);
					comodin3.agregaOperando(comodin2);
					comodin3.agregaOperando(comodin);
					
					return comodin3;
				}else if(operador1.esIgual(funcionSimplificarNumeros.piPorEntre(3,8)) || operador1.esIgual(funcionSimplificarNumeros.piPorEntre(5,8))){
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("2"));
					
					comodin2=new Nodo("sqrt",1);
					comodin2.agregaOperando(comodin);
					
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("-2"));
					
					comodin3=new Nodo("·",2);
					comodin3.agregaOperando(comodin);
					comodin3.agregaOperando(comodin2);
					
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("4"));
					
					comodin2=new Nodo("+",2);
					comodin2.agregaOperando(comodin);
					comodin2.agregaOperando(comodin3);
					
					comodin=new Nodo("sqrt",1);
					comodin.agregaOperando(comodin2);
					
					return comodin;
				}else if(operador1.esIgual(funcionSimplificarNumeros.piPorEntre(11,8)) || operador1.esIgual(funcionSimplificarNumeros.piPorEntre(13,8))){
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("2"));
					
					comodin2=new Nodo("sqrt",1);
					comodin2.agregaOperando(comodin);
					
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("-2"));
					
					comodin3=new Nodo("·",2);
					comodin3.agregaOperando(comodin);
					comodin3.agregaOperando(comodin2);
					
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("4"));
					
					comodin2=new Nodo("+",2);
					comodin2.agregaOperando(comodin);
					comodin2.agregaOperando(comodin3);
					
					comodin=new Nodo("sqrt",1);
					comodin.agregaOperando(comodin2);
					
					comodin2=new Nodo("Entero",1);
					comodin2.agregaOperando(new Nodo("-1"));
					
					comodin3=new Nodo("·",2);
					comodin3.agregaOperando(comodin2);
					comodin3.agregaOperando(comodin);
					
					return comodin3;
				}else if(operador1.esIgual(funcionSimplificarNumeros.piEntre(10)) || operador1.esIgual(funcionSimplificarNumeros.piPorEntre(9,10))){
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("5"));
					
					comodin2=new Nodo("sqrt",1);
					comodin2.agregaOperando(comodin);
					
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("1"));
					
					comodin3=new Nodo("+",2);
					comodin3.agregaOperando(comodin);
					comodin3.agregaOperando(comodin2);
					
					return comodin3;
				}else if(operador1.esIgual(funcionSimplificarNumeros.piPorEntre(11,10)) || operador1.esIgual(funcionSimplificarNumeros.piPorEntre(19,10))){
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("5"));
					
					comodin2=new Nodo("sqrt",1);
					comodin2.agregaOperando(comodin);
					
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("-1"));
					
					comodin3=new Nodo("·",2);
					comodin3.agregaOperando(comodin);
					comodin3.agregaOperando(comodin2);
					
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("-1"));
					
					comodin2=new Nodo("+",2);
					comodin2.agregaOperando(comodin);
					comodin2.agregaOperando(comodin3);
					
					return comodin2;
				}else if(operador1.esIgual(funcionSimplificarNumeros.piPorEntre(3,10)) || operador1.esIgual(funcionSimplificarNumeros.piPorEntre(7,10))){
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("5"));
					
					comodin2=new Nodo("sqrt",1);
					comodin2.agregaOperando(comodin);
					
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("-1"));
					
					comodin3=new Nodo("+",2);
					comodin3.agregaOperando(comodin);
					comodin3.agregaOperando(comodin2);
					
					return comodin3;
				}else if(operador1.esIgual(funcionSimplificarNumeros.piPorEntre(13,10)) || operador1.esIgual(funcionSimplificarNumeros.piPorEntre(17,10))){
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("5"));
					
					comodin2=new Nodo("sqrt",1);
					comodin2.agregaOperando(comodin);
					
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("-1"));
					
					comodin3=new Nodo("·",2);
					comodin3.agregaOperando(comodin);
					comodin3.agregaOperando(comodin2);
					
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("1"));
					
					comodin2=new Nodo("+",2);
					comodin2.agregaOperando(comodin);
					comodin2.agregaOperando(comodin3);
					
					return comodin2;
				}else if(operador1.esIgual(funcionSimplificarNumeros.piEntre(12)) || operador1.esIgual(funcionSimplificarNumeros.piPorEntre(11,12))){
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("6"));
					
					comodin3=new Nodo("sqrt",1);
					comodin3.agregaOperando(comodin);
					
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("2"));
					
					comodin2=new Nodo("sqrt",1);
					comodin2.agregaOperando(comodin);
					
					comodin=new Nodo("+",2);
					comodin.agregaOperando(comodin2);
					comodin.agregaOperando(comodin3);
					
					return comodin;
				}else if(operador1.esIgual(funcionSimplificarNumeros.piPorEntre(13,12)) || operador1.esIgual(funcionSimplificarNumeros.piPorEntre(23,12))){
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("6"));
					
					comodin3=new Nodo("sqrt",1);
					comodin3.agregaOperando(comodin);
					
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("-1"));
					
					comodin2=new Nodo("·",2);
					comodin2.agregaOperando(comodin);
					comodin2.agregaOperando(comodin3);
					
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("2"));
					
					comodin3=new Nodo("sqrt",1);
					comodin3.agregaOperando(comodin);
					
					Nodo comodin4=new Nodo("Entero",1);
					comodin4.agregaOperando(new Nodo("-1"));
					
					comodin=new Nodo("·",2);
					comodin.agregaOperando(comodin4);
					comodin.agregaOperando(comodin3);
					
					comodin3=new Nodo("+",2);
					comodin3.agregaOperando(comodin);
					comodin3.agregaOperando(comodin2);
					
					return comodin3;
				}else if(operador1.esIgual(funcionSimplificarNumeros.piPorEntre(5,12)) || operador1.esIgual(funcionSimplificarNumeros.piPorEntre(7,12))){
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("2"));
					
					comodin3=new Nodo("sqrt",1);
					comodin3.agregaOperando(comodin);
					
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("-1"));
					
					comodin2=new Nodo("·",2);
					comodin2.agregaOperando(comodin);
					comodin2.agregaOperando(comodin3);
					
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("6"));
					
					comodin3=new Nodo("sqrt",1);
					comodin3.agregaOperando(comodin);
					
					comodin=new Nodo("+",2);
					comodin.agregaOperando(comodin2);
					comodin.agregaOperando(comodin3);
					
					return comodin;
				}else if(operador1.esIgual(funcionSimplificarNumeros.piPorEntre(17,12)) || operador1.esIgual(funcionSimplificarNumeros.piPorEntre(19,12))){
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("6"));
					
					comodin3=new Nodo("sqrt",1);
					comodin3.agregaOperando(comodin);
					
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("-1"));
					
					comodin2=new Nodo("·",2);
					comodin2.agregaOperando(comodin);
					comodin2.agregaOperando(comodin3);
					
					comodin=new Nodo("Entero",1);
					comodin.agregaOperando(new Nodo("2"));
					
					comodin3=new Nodo("sqrt",1);
					comodin3.agregaOperando(comodin);
					
					comodin=new Nodo("+",2);
					comodin.agregaOperando(comodin2);
					comodin.agregaOperando(comodin3);
					
					return comodin;
				}
			}
			
			if(!Opciones.getAngulo()){
				comodin=new Nodo("Constante",1);
				comodin.agregaOperando(new Nodo("#pi"));
				
				comodin3=new Nodo("Entero",1);
				comodin3.agregaOperando(new Nodo("180"));
				
				comodin2=new Nodo("/",2);
				comodin2.agregaOperando(comodin3);
				comodin2.agregaOperando(comodin);
				
				comodin=new Nodo("·",2);
				comodin.agregaOperando(operador1);
				comodin.agregaOperando(comodin2);
				
				operador1=funcionSimplificar.simplifica(comodin);
			}
			
			comodin=new Nodo("sen",1);
			comodin.agregaOperando(operador1);
			return comodin;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar un seno");
		}
	}

	/**
	 *Función que simplifica la expresión recibida. Si esta función se evalúa aquí quiere decir que no encontró el operador
	 *donde se tenía que evaluar, lo que es un error.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		try{
			if(!expresionPrefijo.getTipo().equals("csc")){
				expresionPrefijo.cambiaTipo("csc");
			}
			
			if(expresionPrefijo.getOperando(1).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(1).getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
			}
			
			if(funcionSimplificar.esNumero(expresionPrefijo.getOperando(1))){
				return simplificarNumeros(expresionPrefijo, funcionSimplificar, funcionSimplificarNumeros);
			}
			
			return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar un seno");
		}
	}
	
	/**
	 *Función que deriva la expresión recibida.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		Nodo comodin, comodin2, comodin3, comodin4;
		
		try{
			comodin=new Nodo("csc",1);
			comodin.agregaOperando(expresionPrefijo.getOperando(1).clonarNodo());
			
			comodin2=new Nodo("cot",1);
			comodin2.agregaOperando(expresionPrefijo.getOperando(1).clonarNodo());
			
			if (Opciones.getPresicion()){
				comodin3=new Nodo("Entero",1);
			}else{
				comodin3=new Nodo("Decimal",1);
			}
			comodin3.agregaOperando(new Nodo("-1"));
			
			comodin4=new Nodo("·",4);
			
			comodin4.agregaOperando(comodin3);
			comodin4.agregaOperando(comodin);
			comodin4.agregaOperando(comodin2);
			comodin4.agregaOperando(funcionDerivar.deriva(expresionPrefijo.getOperando(1).clonarNodo(), variable));
			
			return comodin4;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al derivar una cosecante");
		}
	}
}
