package itcr.mathexplorer.funciones;

import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.herramientas.Opciones;
import itcr.mathexplorer.tablas.*;
import itcr.mathexplorer.operadores.*;
import itcr.mathexplorer.tipos.FuncionUsuario;

/**
 *@(#)Derivar.java
 *
 *Esta es la implementación de la función derivar
 *
 *@author Alexander Borbón Alpízar
 */
public class Derivar extends Funcion{
	
	private TablaOperadores operadores;
	private TablaFunciones funciones;
	private TablaConstantes constantes;
	private TablaFuncionesUsuario funcionesUsuario;
	
	//CONSTRUCTOR
	
	/**
	 * Constructor de la función Derivar
	 * @param tablaoperadores La tabla que contiene los operadores.
	 * @param tablafunciones La tabla que contiene las funciones.
	 * @param tablaconstantes La tabla que contiene las constantes.
	 * @param tablafuncionesusuario La tabla que contiene las funciones definidas por el usuario.
	 */
	public Derivar(TablaOperadores tablaoperadores, TablaFunciones tablafunciones, TablaConstantes tablaconstantes, TablaFuncionesUsuario tablafuncionesusuario){
		operadores=tablaoperadores;
		funciones=tablafunciones;
		constantes=tablaconstantes;
		funcionesUsuario=tablafuncionesusuario;
	}

	//FUNCIONES PUBLICAS
	
	/**
	 *La derivada admite entre uno y tres parámetros
	 */
	public boolean permiteNumeroParametros(int numeroParametros){
		return (numeroParametros>=1 && numeroParametros<=3);
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		try{
			if(expresionPrefijo.getOperando(1).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(1).getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
			}
			return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar una derivada");
		}	
	}

	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		int num=1;
		Nodo variable, expresion;
		Nodo comodin, comodin2, comodin3;
		
		try{
			if(!expresionPrefijo.getTipo().equals("der")){
				expresionPrefijo.cambiaTipo("der");
			}
			
			for(int i=1; i<=expresionPrefijo.getNumeroOperandos(); i++){
				if(expresionPrefijo.getOperando(i).getTipo().equals("Indefinido")){
					return new Nodo("Indefinido");
				}
			}
			
			expresion=expresionPrefijo.clonarNodo();
			
			if (expresion.getNumeroOperandos()>1){
				if(expresion.getOperando(2).getTipo().equals("Variable")){
					variable=expresion.getOperando(2);
				}else{
					return expresion;
				}
			}else{
				variable=buscaVariable(expresion);
				
				if(variable==null){
					return SimplificarNumeros.CERO.clonarNodo();
				}
				expresionPrefijo.agregaOperando(variable);
			}
			
			if(expresion.getOperando(1).getTipo().equals("der")){ //derivada de una derivada
				if(variable.esIgual(expresion.getOperando(1).getOperando(2))){ //Si tienen la misma variable
					
					if(expresion.getNumeroOperandos()<=2){
						if (Opciones.getPresicion()){
							comodin2=new Nodo("Entero",1);
						}else{
							comodin2=new Nodo("Decimal",1);
						}
						comodin2.agregaOperando(new Nodo("1"));
					}else{
						comodin2=expresion.getOperando(3).clonarNodo();
					}
					
					if(expresion.getOperando(1).getNumeroOperandos()<=2){
						if (Opciones.getPresicion()){
							comodin3=new Nodo("Entero",1);
						}else{
							comodin3=new Nodo("Decimal",1);
						}
						comodin3.agregaOperando(new Nodo("1"));
					}else{
						comodin3=expresion.getOperando(1).getOperando(3).clonarNodo();
					}
					
					comodin=new Nodo("+", 2);
					comodin.agregaOperando(comodin2);
					comodin.agregaOperando(comodin3);
					
					comodin=funcionSimplificar.simplifica(comodin.clonarNodo());
					
					comodin2=new Nodo("der",3);
					comodin2.agregaOperando(expresion.getOperando(1).getOperando(1).clonarNodo());
					comodin2.agregaOperando(variable.clonarNodo());
					comodin2.agregaOperando(comodin);
					
					return comodin2;
					
				}else{
					return expresion;
				}
			}
			
			if (expresion.getNumeroOperandos()>2){
				String tipo=expresion.getOperando(3).getTipo();
				if(tipo.equals("Entero")){
					num=Integer.parseInt(expresion.getOperando(3).getOperando(1).getTipo());
				}else if(tipo.equals("Decimal")){
					double comparar=Double.parseDouble(expresion.getOperando(3).getOperando(1).getTipo());
					double comparar2=Math.ceil(comparar);
					
					if(comparar==comparar2){
						num=(new Double(comparar)).intValue();
					}
				}else if(tipo.equals("EnteroLargo") || tipo.equals("DecimalLargo")){
					throw new ArithmeticException("El número de derivadas es demasiado grande");
				}else{
					return expresion;
				}
			}
			
			expresion=expresion.getOperando(1);
			
			for(int i=1; i<=num; i++){
				expresion=funcionSimplificar.simplifica(deriva(expresion, variable));
			}
			
			return expresion;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar una derivada");
		}
	}
	
	private Nodo buscaVariable(Nodo expresionPrefijo){
		String tipo=expresionPrefijo.getTipo();
		Nodo encontrado;
		
		if(tipo.equals("Variable")){
			return expresionPrefijo;
		}else if(tipo.length()>5 && ("EnteroLargo DecimalLargo Constante Racional").indexOf(tipo)!=-1){
			return null;
		}else{
			for(int i=1; i<=expresionPrefijo.getNumeroOperandos(); i++){
				encontrado=buscaVariable(expresionPrefijo.getOperando(i));
				
				if(encontrado!=null){
					return encontrado;
				}
			}
		}
			
		return null;
	}
	
	/**
	 * Función que busca si la expresión posee la variable dada.
	 * @param expresionPrefijo La expresión dónde se buscará la variable.
	 * @param variable La variable a buscar.
	 * @return True si encontró la variable o false en caso contrario.
	 */
	public boolean buscaVariable(Nodo expresionPrefijo, Nodo variable){
		String tipo=expresionPrefijo.getTipo();
		boolean encontrado;
		
		if(tipo.equals("Variable")){
			if(expresionPrefijo.esIgual(variable)){
				return true;
			}else{
				return false;
			}
		}else if(tipo.length()>5 && ("EnteroLargo DecimalLargo Constante Racional").indexOf(tipo)!=-1){
			return false;
		}else{
			for(int i=1; i<=expresionPrefijo.getNumeroOperandos(); i++){
				encontrado=buscaVariable(expresionPrefijo.getOperando(i), variable);
				
				if(encontrado){
					return encontrado;
				}
			}
		}
			
		return false;
	}
	
	/**
	 *Función que deriva la expresión recibida.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		Nodo comodin, comodin2, comodin3;
		
		try{
			if(variable.esIgual(expresionPrefijo.getOperando(2))){
				if(expresionPrefijo.getNumeroOperandos()==2){
					
					comodin=expresionPrefijo.clonarNodo();
					
					if (Opciones.getPresicion()){
						comodin2=new Nodo("Entero",1);
					}else{
						comodin2=new Nodo("Decimal",1);
					}
					comodin2.agregaOperando(new Nodo("2"));
					
					comodin.agregaOperando(comodin2);
					
					return comodin;
				}else{
					comodin=expresionPrefijo.getOperando(3).clonarNodo();
					
					if (Opciones.getPresicion()){
						comodin2=new Nodo("Entero",1);
					}else{
						comodin2=new Nodo("Decimal",1);
					}
					comodin2.agregaOperando(new Nodo("1"));
					
					comodin3=new Nodo("+",2);
					comodin3.agregaOperando(comodin);
					comodin3.agregaOperando(comodin2);
					
					comodin=new Nodo("der",3);
					comodin.agregaOperando(expresionPrefijo.getOperando(1).clonarNodo());
					comodin.agregaOperando(expresionPrefijo.getOperando(2).clonarNodo());
					comodin.agregaOperando(comodin3);
					
					return comodin;
				}
			}else{
				comodin=new Nodo("der",3);
				comodin.agregaOperando(expresionPrefijo.clonarNodo());
				comodin.agregaOperando(variable.clonarNodo());
				
				return comodin;
			}
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al derivar una derivada");
		}
	}
	
	/**
	 *Función que determina la derivada de una expresión.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto de la cuál se calculará la derivada.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error en la expresión.
	 */
	public Nodo deriva(Nodo expresionPrefijo, Nodo variable) throws ArithmeticException{
		String tipo=expresionPrefijo.getTipo();
		Nodo comodin;
		
		try{
			if(tipo.equals("Variable")){
				if(Opciones.getPresicion()){
					comodin=new Nodo("Entero",1);
				}else{
					comodin=new Nodo("Decimal",1);
				}
				
				if(expresionPrefijo.esIgual(variable)){
					comodin.agregaOperando(new Nodo("1"));
				}else{
					comodin.agregaOperando(new Nodo("0"));
				}
				
				return comodin;
			}else if (tipo.length()>5 && ("EnteroLargo DecimalLargo Constante Racional").indexOf(tipo)!=-1){
				if(Opciones.getPresicion()){
					comodin=new Nodo("Entero",1);
				}else{
					comodin=new Nodo("Decimal",1);
				}
				comodin.agregaOperando(new Nodo("0"));
				
				return comodin;
			}else if(tipo.equals("Matriz")){
				for(int i=1; i<=expresionPrefijo.getNumeroOperandos(); i++){
					for(int j=1; j<=expresionPrefijo.getOperando(1).getNumeroOperandos(); j++){
						comodin=deriva(expresionPrefijo.getOperando(i).getOperando(j), variable);
						
						if(comodin.getTipo().equals("Indefinido")){
							return new Nodo("Indefinido");
						}
						
						expresionPrefijo.getOperando(i).eliminaOperando(j);
						expresionPrefijo.getOperando(i).agregaOperando(comodin,j);
	
					}
				}
				
				return expresionPrefijo;
			}else if(tipo.equals("Lista")){
				comodin=new Nodo("der",3);
				comodin.agregaOperando(expresionPrefijo.clonarNodo());
				comodin.agregaOperando(variable.clonarNodo());
				
				return comodin;
			}else if(operadores.apareceLlave(tipo)){
				return ((Operador)operadores.devuelveOperador(tipo)).derivar(expresionPrefijo, variable, this);
			}else if(constantes.apareceLlave(tipo)){
				if(Opciones.getPresicion()){
					comodin=new Nodo("Entero",1);
				}else{
					comodin=new Nodo("Decimal",1);
				}
				comodin.agregaOperando(new Nodo("0"));
				
				return comodin;
			}else if(funcionesUsuario.apareceLlave(tipo)){
				return deriva(((FuncionUsuario)funcionesUsuario.devuelveFuncion(tipo)).obtenExpresion(), variable);
			}else{ //Tiene que ser una función
				return ((Funcion)funciones.devuelveFuncion(tipo)).derivar(expresionPrefijo, variable, this);
			}
			
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error en la expresión al derivar");
		}
	}
}