package itcr.mathexplorer.funciones;

import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.herramientas.Opciones;

/**
 *@(#)Determinante.java
 *
 *Esta es la implementación de la función que toma una matriz
 *y determina su inversa.
 *
 *@author Alexander Borbón Alpízar
 */
public class Determinante extends Funcion{
	
	//CONSTRUCTOR
	/**
	 * Constructor vacío de la función Determinante
	 */
	public Determinante() {
	}

	//FUNCIONES PUBLICAS
	
	/**
	 *Determinante admite sólo un parámetros: la matriz
	 */
	public boolean permiteNumeroParametros(int numeroParametros){
		return numeroParametros==1;
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		int numFilas;
		Nodo matriz, comodin, comodin2, comodin3;
		Nodo solucion;
		
		try{
			if(!expresionPrefijo.getTipo().equals("det")){
				expresionPrefijo.cambiaTipo("det");
			}
			
			matriz=expresionPrefijo.getOperando(1);
			
			if(matriz.getTipo().equals("Indefinido")){
				return new Nodo("Indefinido");
			}else if(!matriz.getTipo().equals("Matriz") ){
				return expresionPrefijo;
			}else if(matriz.getNumeroOperandos()!=matriz.getOperando(1).getNumeroOperandos()){ //Si la matriz no es cuadrada
				return new Nodo("Indefinido");
			}else{
				
				solucion=new Nodo("·");
				
				//Se realiza reducción de filas
				
				numFilas=matriz.getNumeroOperandos();
				
				for(int i=1; i<=numFilas; i++){
					
					//Si el elemento que sigue no es un cero se busca alguno para intercambiar las filas
					
					if(matriz.getOperando(i).getOperando(i).esIgual(SimplificarNumeros.CERO)){
						int j=i;
						
						while(j<=numFilas && matriz.getOperando(j).getOperando(i).esIgual(SimplificarNumeros.CERO)){
							j++;
						}
						
						if(j>numFilas){
							return SimplificarNumeros.CERO.clonarNodo();
						}else{
							comodin=new Nodo("fij",3);
							comodin.agregaOperando(matriz);
							
							if (Opciones.getPresicion()){
								comodin2=new Nodo("Entero",1);
							}else{
								comodin2=new Nodo("Decimal",1);
							}
							comodin2.agregaOperando(new Nodo((new Integer(i)).toString()));
							
							comodin.agregaOperando(comodin2);
							
							if (Opciones.getPresicion()){
								comodin2=new Nodo("Entero",1);
							}else{
								comodin2=new Nodo("Decimal",1);
							}
							comodin2.agregaOperando(new Nodo((new Integer(j)).toString()));
							
							comodin.agregaOperando(comodin2);
							
							matriz=funcionSimplificar.simplifica(comodin);
							
							solucion.agregaOperando(SimplificarNumeros.MENOSUNO.clonarNodo());
						}
					}
					
					//Se hace el uno y se utiliza para cancelar la columna
					
					if (Opciones.getPresicion()){
						comodin3=new Nodo("Entero",1);
					}else{
						comodin3=new Nodo("Decimal",1);
					}
					comodin3.agregaOperando(new Nodo((new Integer(i)).toString()));
					
					comodin2=new Nodo("^");
					comodin2.agregaOperando(matriz.getOperando(i).getOperando(i).clonarNodo());
					comodin2.agregaOperando(SimplificarNumeros.MENOSUNO.clonarNodo());
					
					solucion.agregaOperando(matriz.getOperando(i).getOperando(i).clonarNodo());
					
					comodin=new Nodo("kfi");
					comodin.agregaOperando(matriz);
					comodin.agregaOperando(comodin2);
					comodin.agregaOperando(comodin3);
					
					matriz=funcionSimplificar.simplifica(comodin);
					
					//Se cancelan los de la columna para hacerla triangular
					
					for(int j=i+1; j<=numFilas; j++){
						if (Opciones.getPresicion()){
							comodin3=new Nodo("Entero",1);
						}else{
							comodin3=new Nodo("Decimal",1);
						}
						comodin3.agregaOperando(new Nodo((new Integer(j)).toString()));
						
						comodin2=new Nodo("·");
						comodin2.agregaOperando(SimplificarNumeros.MENOSUNO.clonarNodo());
						comodin2.agregaOperando(matriz.getOperando(j).getOperando(i).clonarNodo());
						
						comodin=new Nodo("fikfj");
						comodin.agregaOperando(matriz);
						comodin.agregaOperando(comodin3);
						comodin.agregaOperando(comodin2);
						
						if (Opciones.getPresicion()){
							comodin3=new Nodo("Entero",1);
						}else{
							comodin3=new Nodo("Decimal",1);
						}
						comodin3.agregaOperando(new Nodo((new Integer(i)).toString()));
						
						comodin.agregaOperando(comodin3);
						
						matriz=funcionSimplificar.simplifica(comodin);
					}
				}
			}
			
			return funcionSimplificar.simplifica(solucion);
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar Determinante");
		}	
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		try{
			if(!expresionPrefijo.getTipo().equals("det")){
				expresionPrefijo.cambiaTipo("det");
			}
			
			if(!expresionPrefijo.getOperando(1).getTipo().equals("Matriz")){
				return expresionPrefijo;
			}
			
			return simplificarNumeros(expresionPrefijo, funcionSimplificar, funcionSimplificarNumeros);
			
			//return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar Determinante");
		}
	}
	
	/**
	 *Función que deriva la expresión recibida.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		try{
			return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al derivar Determinante");
		}
	}

}