package itcr.mathexplorer.funciones;

import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.herramientas.Opciones;
import itcr.mathexplorer.herramientas.TeoriaNumeros;
import java.math.BigInteger;

/**
 *@(#)EsPrimo.java
 *
 *Esta es la implementación de la función EsPrimo
 *Esta función identifica si el número dado es primo o no.
 *
 *@author Alexander Borbón Alpízar
 */
public class EsPrimo extends Funcion{
	
	//CONSTRUCTOR
	/**
	 * Constructor vacío de la función EsPrimo
	 */
	public EsPrimo() {
	}

	//FUNCIONES PUBLICAS
	
	/**
	 *El signo sólo admite un parámetro
	 */
	public boolean permiteNumeroParametros(int numeroParametros){
		return numeroParametros==1;
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		Nodo comodin;
		
		try{
			if(expresionPrefijo.getOperando(1).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(1).getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
			}
			
			comodin=new Nodo("Boolean",1);
			
			if(funcionSimplificarNumeros.esEntero(expresionPrefijo.getOperando(1))){
				try{
					if(TeoriaNumeros.esPrimo(new BigInteger(expresionPrefijo.getOperando(1).getOperando(1).getTipo()))){
						comodin.agregaOperando(new Nodo("Verdadero"));
					}else{
						comodin.agregaOperando(new Nodo("Falso"));
					}
				}catch(Exception e){
					comodin.agregaOperando(new Nodo("Falso"));
					return comodin;
				}
			}else{
				comodin.agregaOperando(new Nodo("Falso"));
			}
			return comodin;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar si es primo");
		}	
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		try{
			if(!expresionPrefijo.getTipo().equals("esprimo")){
				expresionPrefijo.cambiaTipo("esprimo");
			}
			
			if(expresionPrefijo.getOperando(1).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(1).getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
			}
			
			if(funcionSimplificar.esNumero(expresionPrefijo.getOperando(1))){
				return simplificarNumeros(expresionPrefijo, funcionSimplificar, funcionSimplificarNumeros);
			}
			
			return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar si es primo");
		}
	}
	
	/**
	 *Función que deriva la expresión recibida.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		Nodo comodin;
		
		try{
			if (Opciones.getPresicion()){
				comodin=new Nodo("Entero",1);
			}else{
				comodin=new Nodo("Decimal",1);
			}
			comodin.agregaOperando(new Nodo("0"));
			
			return comodin;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al derivar un EsPrimo");
		}
	}

}