package itcr.mathexplorer.funciones;

import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.herramientas.Opciones;
import itcr.mathexplorer.herramientas.TeoriaNumeros;
import java.math.BigInteger;
import java.util.Vector;

/**
 *@(#)Factor.java
 *
 *Esta es la implementación de la función Factor
 *Esta función factoriza un número entero.
 *
 *@author Alexander Borbón Alpízar
 */
public class Factor extends Funcion{
	
	//CONSTRUCTOR
	/**
	 * Constructor vacío de la función Factor
	 */
	public Factor() {
	}

	//FUNCIONES PUBLICAS
	
	/**
	 *El signo sólo admite un parámetro
	 */
	public boolean permiteNumeroParametros(int numeroParametros){
		return numeroParametros==1;
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		Nodo comodin, comodin2, comodin3;
		Vector respuesta;
		String numero;
		boolean negativo=false;
		
		try{
			if(expresionPrefijo.getOperando(1).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(1).getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
			}
			
			if(funcionSimplificarNumeros.esEntero(expresionPrefijo.getOperando(1))){
				try{
					numero=expresionPrefijo.getOperando(1).getOperando(1).getTipo();
					
					if(numero.indexOf("-")!=-1){
						negativo=true;
						numero=numero.substring(1);
					}
					
					respuesta=TeoriaNumeros.Factor((new BigInteger(numero)));
					if (respuesta.size()==0){
						return expresionPrefijo;
					}
					
					if(respuesta.size()==2){
						if (Opciones.getPresicion()){
							comodin2=new Nodo("Entero",1);
						}else{
							comodin2=new Nodo("Decimal",1);
						}
						comodin2.agregaOperando(new Nodo(((BigInteger)respuesta.elementAt(0)).toString()));
						
						comodin=new Nodo("^",2);
						comodin.agregaOperando(comodin2);
						
						if (Opciones.getPresicion()){
							comodin2=new Nodo("Entero",1);
						}else{
							comodin2=new Nodo("Decimal",1);
						}
						comodin2.agregaOperando(new Nodo(((BigInteger)respuesta.elementAt(1)).toString()));

						comodin.agregaOperando(comodin2);
					}else{
						comodin3=new Nodo("·");
						
						for(int i=0; i<respuesta.size()-1; i=i+2){
							if (Opciones.getPresicion()){
								comodin2=new Nodo("Entero",1);
							}else{
								comodin2=new Nodo("Decimal",1);
							}
							comodin2.agregaOperando(new Nodo(((BigInteger)respuesta.elementAt(i)).toString()));
							
							comodin=new Nodo("^",2);
							comodin.agregaOperando(comodin2);
							
							if (Opciones.getPresicion()){
								comodin2=new Nodo("Entero",1);
							}else{
								comodin2=new Nodo("Decimal",1);
							}
							comodin2.agregaOperando(new Nodo(((BigInteger)respuesta.elementAt(i+1)).toString()));
	
							comodin.agregaOperando(comodin2);
							
							comodin3.agregaOperando(comodin);
						}
						
						comodin=comodin3;
					}
					
					if(negativo){
						if (Opciones.getPresicion()){
							comodin2=new Nodo("Entero",1);
						}else{
							comodin2=new Nodo("Decimal",1);
						}
						comodin2.agregaOperando(new Nodo("-1"));
						
						if(comodin.getTipo().equals("·")){
							comodin.agregaOperando(comodin2,1);
						}else{
							comodin3=new Nodo("·", 2);
							
							comodin3.agregaOperando(comodin2);
							comodin3.agregaOperando(comodin);
							comodin=comodin3;
						}
					}
					return comodin;
				}catch(Exception e){
					e.printStackTrace();
					return new Nodo("Indefinido");
				}
			}else{
				return new Nodo("Indefinido");
			}
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar si es primo");
		}	
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		try{
			if(!expresionPrefijo.getTipo().equals("primo")){
				expresionPrefijo.cambiaTipo("primo");
			}
			
			if(expresionPrefijo.getOperando(1).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(1).getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
			}
			
			if(funcionSimplificar.esNumero(expresionPrefijo.getOperando(1))){
				return simplificarNumeros(expresionPrefijo, funcionSimplificar, funcionSimplificarNumeros);
			}
			
			return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar si es primo");
		}
	}
	
	/**
	 *Función que deriva la expresión recibida.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		Nodo comodin;
		
		try{
			if (Opciones.getPresicion()){
				comodin=new Nodo("Entero",1);
			}else{
				comodin=new Nodo("Decimal",1);
			}
			comodin.agregaOperando(new Nodo("0"));
			
			return comodin;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al derivar un EsPrimo");
		}
	}

}