package itcr.mathexplorer.funciones;

import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.herramientas.Opciones;

/**
 *@(#)FiKFj.java
 *
 *Esta es la implementación de la función que toma una matriz
 *y multiplica la fila i por k.
 *
 *@author Alexander Borbón Alpízar
 */
public class FiKFj extends Funcion{
	
	//CONSTRUCTOR
	/**
	 * Constructor vacío de la función FiKFj.
	 */
	public FiKFj() {
	}

	//FUNCIONES PUBLICAS
	
	/**
	 *FiKFj admite sólo cuatro parámetros: la matriz, la fila i, el k y la fila j
	 */
	public boolean permiteNumeroParametros(int numeroParametros){
		return numeroParametros==4;
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		int i, j;
		Nodo matriz, k, comodin, comodin2;
		
		try{
			if(!expresionPrefijo.getTipo().equals("fikfj")){
				expresionPrefijo.cambiaTipo("fikfj");
			}
			
			for(int l=1; l<=4; l++){
				if(expresionPrefijo.getOperando(l).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(l).getTipo().equals("Boolean")){
					return new Nodo("Indefinido");
				}
			}
			
			if(!expresionPrefijo.getOperando(1).getTipo().equals("Matriz")){
				return expresionPrefijo;
			}else if(!funcionSimplificarNumeros.esEntero(expresionPrefijo.getOperando(2)) && !funcionSimplificarNumeros.esEntero(expresionPrefijo.getOperando(4))){
				return new Nodo("Indefinido");
			}else{
				i=new Integer(expresionPrefijo.getOperando(2).getOperando(1).getTipo());
				j=new Integer(expresionPrefijo.getOperando(4).getOperando(1).getTipo());
				k=expresionPrefijo.getOperando(3);
				matriz=expresionPrefijo.getOperando(1);
				
				if(i>matriz.getNumeroOperandos() || j>matriz.getNumeroOperandos()){
					return new Nodo("Indefinido");
				}else{
					for(int l=1; l<=matriz.getOperando(i).getNumeroOperandos(); l++){
						comodin2=new Nodo("·");
						comodin2.agregaOperando(k);
						comodin2.agregaOperando(matriz.getOperando(j).getOperando(l));
						comodin=new Nodo("+");
						comodin.agregaOperando(matriz.getOperando(i).getOperando(l));
						comodin.agregaOperando(comodin2);
						matriz.getOperando(i).eliminaOperando(l);
						matriz.getOperando(i).agregaOperando(funcionSimplificar.simplifica(comodin),l);
					}
				}
			}
			
			return matriz;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar fikfj");
		}	
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		try{
			if(!expresionPrefijo.getTipo().equals("fikfj")){
				expresionPrefijo.cambiaTipo("fikfj");
			}
			
			for(int k=1; k<=4; k++){
				if(expresionPrefijo.getOperando(k).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(k).getTipo().equals("Boolean")){
					return new Nodo("Indefinido");
				}
			}
			
			if(!expresionPrefijo.getOperando(1).getTipo().equals("Matriz")){
				return expresionPrefijo;
			}
			
			if(funcionSimplificar.esNumero(expresionPrefijo.getOperando(2)) && funcionSimplificar.esNumero(expresionPrefijo.getOperando(4))){
				return simplificarNumeros(expresionPrefijo, funcionSimplificar, funcionSimplificarNumeros);
			}
			
			return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar fikfj");
		}
	}
	
	/**
	 *Función que deriva la expresión recibida.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		Nodo comodin;
		
		try{
			if (Opciones.getPresicion()){
				comodin=new Nodo("Entero",1);
			}else{
				comodin=new Nodo("Decimal",1);
			}
			comodin.agregaOperando(new Nodo("0"));
			
			return comodin;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al derivar fikfj");
		}
	}

}