package itcr.mathexplorer.funciones;

import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.herramientas.Opciones;

/**
 *@(#)Fij.java
 *
 *Esta es la implementación de la función que intercambia
 *dos filas en una matriz
 *
 *@author Alexander Borbón Alpízar
 */
public class Fij extends Funcion{
	
	//CONSTRUCTOR
	/**
	 * Constructor vacío de la función Fij.
	 */
	public Fij() {
	}

	//FUNCIONES PUBLICAS
	
	/**
	 *Fij admite sólo tres parámetros: la matriz y las filas a intercambiar
	 */
	public boolean permiteNumeroParametros(int numeroParametros){
		return numeroParametros==3;
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		int i, j;
		Nodo matriz;
		
		try{
			if(!expresionPrefijo.getTipo().equals("fij")){
				expresionPrefijo.cambiaTipo("fij");
			}
			
			for(int k=1; k<=3; k++){
				if(expresionPrefijo.getOperando(k).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(k).getTipo().equals("Boolean")){
					return new Nodo("Indefinido");
				}
			}
			
			if(!expresionPrefijo.getOperando(1).getTipo().equals("Matriz")){
				return expresionPrefijo;
			}else if(!funcionSimplificarNumeros.esEntero(expresionPrefijo.getOperando(2)) || !funcionSimplificarNumeros.esEntero(expresionPrefijo.getOperando(3))){
				return new Nodo("Indefinido");
			}else{
				i=new Integer(expresionPrefijo.getOperando(2).getOperando(1).getTipo());
				j=new Integer(expresionPrefijo.getOperando(3).getOperando(1).getTipo());
				matriz=expresionPrefijo.getOperando(1);
				
				if(i>matriz.getNumeroOperandos() || j>matriz.getNumeroOperandos()){
					return new Nodo("Indefinido");
				}else{
					matriz.intercambiaOperandos(i,j);
				}
			}
			
			return matriz;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar fij");
		}	
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		try{
			if(!expresionPrefijo.getTipo().equals("fij")){
				expresionPrefijo.cambiaTipo("fij");
			}
			
			for(int k=1; k<=3; k++){
				if(expresionPrefijo.getOperando(k).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(k).getTipo().equals("Boolean")){
					return new Nodo("Indefinido");
				}
			}
			
			if(!expresionPrefijo.getOperando(1).getTipo().equals("Matriz")){
				return expresionPrefijo;
			}
			
			if(funcionSimplificar.esNumero(expresionPrefijo.getOperando(2)) && funcionSimplificar.esNumero(expresionPrefijo.getOperando(3))){
				return simplificarNumeros(expresionPrefijo, funcionSimplificar, funcionSimplificarNumeros);
			}
			
			return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar fij");
		}
	}
	
	/**
	 *Función que deriva la expresión recibida.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		Nodo comodin;
		
		try{
			if (Opciones.getPresicion()){
				comodin=new Nodo("Entero",1);
			}else{
				comodin=new Nodo("Decimal",1);
			}
			comodin.agregaOperando(new Nodo("0"));
			
			return comodin;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al derivar fij");
		}
	}

}