package itcr.mathexplorer.funciones;

import itcr.mathexplorer.expresion.*;

/**
 *@(#)Funcion.java
 *
 *Esta es una interface que define el cazcarón que debe tener cualquier función
 *utilizada en el programa.
 *
 *@author Alexander Borbón Alpízar
 */
public class Funcion implements Funciones{
	
	//CONSTRUCTORES
	
	/**
	 * Constructor vacío de la interfaz para la Funcion
	 */
	public Funcion(){ }

	//FUNCIONES PUBLICAS
	
	/**
	 *Si esta función se corrió aquí es porque la función no es reconocida.
	 */
	public boolean permiteNumeroParametros(int numeroParametros){
		return false;
	}

	/**
	 *Función que simplifica la expresión recibida. Si esta función se evalúa aquí quiere decir que no encontró el operador
	 *donde se tenía que evaluar, lo que es un error.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		throw new ArithmeticException("Error al simplificar la función");
	}
	
	/**
	 *Función que simplifica la expresión recibida. Si esta función se evalúa aquí quiere decir que no encontró el operador
	 *donde se tenía que evaluar, lo que es un error.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		throw new ArithmeticException("Error al simplificar la función");
	}
	
	/**
	 *Función que deriva la expresión recibida. Si esta función se evalúa aquí quiere decir que no encontró el operador
	 *donde se tenía que evaluar, lo que es un error.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		throw new ArithmeticException("Error al derivar la función");
	}

	
}//Fin de SintaxException