package itcr.mathexplorer.funciones;

import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.herramientas.Opciones;

/**
 *@(#)KFi.java
 *
 *Esta es la implementación de la función que toma una matriz
 *y multiplica la fila i por k.
 *
 *@author Alexander Borbón Alpízar
 */
public class KFi extends Funcion{
	
	//CONSTRUCTOR
	/**
	 * Constructor vacío de la función KFi.
	 */
	public KFi() {
	}

	//FUNCIONES PUBLICAS
	
	/**
	 *KFi admite sólo tres parámetros: la matriz, el k y la fila 
	 */
	public boolean permiteNumeroParametros(int numeroParametros){
		return numeroParametros==3;
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		int i;
		Nodo matriz, k, comodin;
		
		try{
			if(!expresionPrefijo.getTipo().equals("kfi")){
				expresionPrefijo.cambiaTipo("kfi");
			}
			
			for(int l=1; l<=3; l++){
				if(expresionPrefijo.getOperando(l).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(l).getTipo().equals("Boolean")){
					return new Nodo("Indefinido");
				}
			}
			
			if(!expresionPrefijo.getOperando(1).getTipo().equals("Matriz")){
				return expresionPrefijo;
			}else if(!funcionSimplificarNumeros.esEntero(expresionPrefijo.getOperando(3))){
				return new Nodo("Indefinido");
			}else{
				i=new Integer(expresionPrefijo.getOperando(3).getOperando(1).getTipo());
				k=expresionPrefijo.getOperando(2);
				matriz=expresionPrefijo.getOperando(1);
				
				if(i>matriz.getNumeroOperandos()){
					return new Nodo("Indefinido");
				}else{
					for(int j=1; j<=matriz.getOperando(i).getNumeroOperandos(); j++){
						comodin=new Nodo("·");
						comodin.agregaOperando(k);
						comodin.agregaOperando(matriz.getOperando(i).getOperando(j));
						matriz.getOperando(i).eliminaOperando(j);
						matriz.getOperando(i).agregaOperando(funcionSimplificar.simplifica(comodin),j);
					}
				}
			}
			
			return matriz;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar Kfij");
		}	
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		try{
			if(!expresionPrefijo.getTipo().equals("kfi")){
				expresionPrefijo.cambiaTipo("kfi");
			}
			
			for(int k=1; k<=3; k++){
				if(expresionPrefijo.getOperando(k).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(k).getTipo().equals("Boolean")){
					return new Nodo("Indefinido");
				}
			}
			
			if(!expresionPrefijo.getOperando(1).getTipo().equals("Matriz")){
				return expresionPrefijo;
			}
			
			if(funcionSimplificar.esNumero(expresionPrefijo.getOperando(3))){
				return simplificarNumeros(expresionPrefijo, funcionSimplificar, funcionSimplificarNumeros);
			}
			
			return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar Kfij");
		}
	}
	
	/**
	 *Función que deriva la expresión recibida.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		Nodo comodin;
		
		try{
			if (Opciones.getPresicion()){
				comodin=new Nodo("Entero",1);
			}else{
				comodin=new Nodo("Decimal",1);
			}
			comodin.agregaOperando(new Nodo("0"));
			
			return comodin;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al derivar Kfij");
		}
	}

}