package itcr.mathexplorer.funciones;

import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.herramientas.Opciones;
import itcr.mathexplorer.herramientas.TeoriaNumeros;

/**
 *@(#)Primo.java
 *
 *Esta es la implementación de la función Primo
 *Esta función devuelve el n-ésimo primo.
 *
 *@author Alexander Borbón Alpízar
 */
public class Primo extends Funcion{
	
	//CONSTRUCTOR
	/**
	 * Constructor vacío de la función Primo
	 */
	public Primo() {	}

	//FUNCIONES PUBLICAS
	
	/**
	 *El signo sólo admite un parámetro
	 */
	public boolean permiteNumeroParametros(int numeroParametros){
		return numeroParametros==1;
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		Nodo comodin;
		int respuesta;
		
		try{
			if(expresionPrefijo.getOperando(1).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(1).getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
			}
			
			if(funcionSimplificarNumeros.esEntero(expresionPrefijo.getOperando(1))){
				try{
					respuesta=TeoriaNumeros.Primo((new Integer(expresionPrefijo.getOperando(1).getOperando(1).getTipo())).intValue());
					if (respuesta==0){
						return expresionPrefijo;
					}else if(respuesta==-1){
						return new Nodo("Indefinido");
					}
					
					if (Opciones.getPresicion()){
						comodin=new Nodo("Entero",1);
					}else{
						comodin=new Nodo("Decimal",1);
					}
					comodin.agregaOperando(new Nodo((new Long(respuesta)).toString()));
				}catch(Exception e){
					e.printStackTrace();
					return new Nodo("Indefinido");
				}
			}else{
				return new Nodo("Indefinido");
			}
			return comodin;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar si es primo");
		}	
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		try{
			if(!expresionPrefijo.getTipo().equals("primo")){
				expresionPrefijo.cambiaTipo("primo");
			}
			
			if(expresionPrefijo.getOperando(1).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(1).getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
			}
			
			if(funcionSimplificar.esNumero(expresionPrefijo.getOperando(1))){
				return simplificarNumeros(expresionPrefijo, funcionSimplificar, funcionSimplificarNumeros);
			}
			
			return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar si es primo");
		}
	}
	
	/**
	 *Función que deriva la expresión recibida.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		Nodo comodin;
		
		try{
			if (Opciones.getPresicion()){
				comodin=new Nodo("Entero",1);
			}else{
				comodin=new Nodo("Decimal",1);
			}
			comodin.agregaOperando(new Nodo("0"));
			
			return comodin;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al derivar si es primo");
		}
	}

}