package itcr.mathexplorer.funciones;

import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.herramientas.Opciones;
import itcr.mathexplorer.herramientas.TeoriaNumeros;
import java.util.Vector;

/**
 *@(#)Primos.java
 *
 *Esta es la implementación de la función Primos
 *Esta función devuelve una lista de los primos menores a n
 *cuando se da un solo parámetro o todos los primos comprendidos
 *entre m y n si se dan dos parámetros.
 *
 *@author Alexander Borbón Alpízar
 */
public class Primos extends Funcion{
	
	//CONSTRUCTOR
	/**
	 * Constructor vacío de la función Primos
	 */
	public Primos() {	}

	//FUNCIONES PUBLICAS
	
	/**
	 *primos admite uno o dos parámetros
	 */
	public boolean permiteNumeroParametros(int numeroParametros){
		return numeroParametros==1 || numeroParametros==2;
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		Nodo comodin, comodin2;
		Vector respuesta;
		
		try{
			if(expresionPrefijo.getOperando(1).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(1).getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
			}
			
			if(expresionPrefijo.getNumeroOperandos()==1){
				if(funcionSimplificarNumeros.esEntero(expresionPrefijo.getOperando(1))){
					try{
						respuesta=TeoriaNumeros.HacerListaPrimos((new Integer(expresionPrefijo.getOperando(1).getOperando(1).getTipo())).intValue());
						
						comodin=new Nodo("Lista");
						
						for(int i=0; i<respuesta.size(); i++){
							if (Opciones.getPresicion()){
								comodin2=new Nodo("Entero",1);
							}else{
								comodin2=new Nodo("Decimal",1);
							}
							comodin2.agregaOperando(new Nodo(((Integer)respuesta.elementAt(i)).toString()));
							
							comodin.agregaOperando(comodin2);
						}
					}catch(Exception e){
						e.printStackTrace();
						return new Nodo("Indefinido");
					}
				}else{
					return new Nodo("Indefinido");
				}
			}else{
				if(funcionSimplificarNumeros.esEntero(expresionPrefijo.getOperando(1)) && funcionSimplificarNumeros.esEntero(expresionPrefijo.getOperando(2))){
					try{
						respuesta=TeoriaNumeros.PrimosEntre((new Integer(expresionPrefijo.getOperando(1).getOperando(1).getTipo())).intValue(), (new Integer(expresionPrefijo.getOperando(2).getOperando(1).getTipo())).intValue());
						
						comodin=new Nodo("Lista");
						
						for(int i=0; i<respuesta.size(); i++){
							if (Opciones.getPresicion()){
								comodin2=new Nodo("Entero",1);
							}else{
								comodin2=new Nodo("Decimal",1);
							}
							comodin2.agregaOperando(new Nodo(((Integer)respuesta.elementAt(i)).toString()));
							
							comodin.agregaOperando(comodin2);
						}
					}catch(Exception e){
						e.printStackTrace();
						return new Nodo("Indefinido");
					}
				}else{
					return new Nodo("Indefinido");
				}
			}
			return comodin;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar primos");
		}	
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		try{
			if(!expresionPrefijo.getTipo().equals("primos")){
				expresionPrefijo.cambiaTipo("primos");
			}
			
			if(expresionPrefijo.getOperando(1).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(1).getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
			}
			
			if(expresionPrefijo.getNumeroOperandos()==1){
				if(funcionSimplificar.esNumero(expresionPrefijo.getOperando(1))){
					return simplificarNumeros(expresionPrefijo, funcionSimplificar, funcionSimplificarNumeros);
				}
			}else{
				if(funcionSimplificar.esNumero(expresionPrefijo.getOperando(1)) && funcionSimplificar.esNumero(expresionPrefijo.getOperando(2))){
					return simplificarNumeros(expresionPrefijo, funcionSimplificar, funcionSimplificarNumeros);
				}
			}
			
			return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar primos");
		}
	}
	
	/**
	 *Función que deriva la expresión recibida.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		Nodo comodin;
		
		try{
			if (Opciones.getPresicion()){
				comodin=new Nodo("Entero",1);
			}else{
				comodin=new Nodo("Decimal",1);
			}
			comodin.agregaOperando(new Nodo("0"));
			
			return comodin;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al derivar primos");
		}
	}

}