package itcr.mathexplorer.funciones;

import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.herramientas.Opciones;

/**
 *@(#)ReduccionFilas.java
 *
 *Esta es la implementación de la función que toma una matriz
 *y determina su inversa.
 *
 *@author Alexander Borbón Alpízar
 */
public class ReduccionFilas extends Funcion{
	
	//CONSTRUCTOR
	/**
	 * Constructor vacío de la función ReduccionFilas
	 */
	public ReduccionFilas() {	}

	//FUNCIONES PUBLICAS
	
	/**
	 *ReduccionFilas admite sólo un parámetros: la matriz
	 */
	public boolean permiteNumeroParametros(int numeroParametros){
		return numeroParametros==1;
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		int numFilas, numColumnas, fila, columna;
		Nodo matriz, comodin, comodin2, comodin3;
		
		try{
			if(!expresionPrefijo.getTipo().equals("reduce")){
				expresionPrefijo.cambiaTipo("reduce");
			}
			
			matriz=expresionPrefijo.getOperando(1);
			
			if(matriz.getTipo().equals("Indefinido")){
				return new Nodo("Indefinido");
			}else if(!matriz.getTipo().equals("Matriz") ){
				return expresionPrefijo;
			}else{
				//Se realiza reducción de filas
				
				numFilas=matriz.getNumeroOperandos();
				if(numFilas==0){
					return matriz;
				}
				numColumnas=matriz.getOperando(1).getNumeroOperandos();
				columna=1;
				
				for(fila=1; fila<=numFilas; fila++){
					
					//Si el elemento que sigue no es un cero se busca alguno para intercambiar las filas
					
					while(columna<=numColumnas && matriz.getOperando(fila).getOperando(columna).esIgual(SimplificarNumeros.CERO)){
						int j=fila;
						
						while(j<=numFilas && matriz.getOperando(j).getOperando(columna).esIgual(SimplificarNumeros.CERO)){
							j++;
						}
						
						if(j>numFilas){
							columna++;
						}else{
							comodin=new Nodo("fij",3);
							comodin.agregaOperando(matriz);
							
							if (Opciones.getPresicion()){
								comodin2=new Nodo("Entero",1);
							}else{
								comodin2=new Nodo("Decimal",1);
							}
							comodin2.agregaOperando(new Nodo((new Integer(fila)).toString()));
							
							comodin.agregaOperando(comodin2);
							
							if (Opciones.getPresicion()){
								comodin2=new Nodo("Entero",1);
							}else{
								comodin2=new Nodo("Decimal",1);
							}
							comodin2.agregaOperando(new Nodo((new Integer(j)).toString()));
							
							comodin.agregaOperando(comodin2);
							
							matriz=funcionSimplificar.simplifica(comodin);
						}
					}
					
					if(columna>numColumnas){
						return matriz;
					}
					
					//Se hace el uno y se utiliza para cancelar la columna
					
					if (Opciones.getPresicion()){
						comodin3=new Nodo("Entero",1);
					}else{
						comodin3=new Nodo("Decimal",1);
					}
					comodin3.agregaOperando(new Nodo((new Integer(fila)).toString()));
					
					comodin2=new Nodo("^");
					comodin2.agregaOperando(matriz.getOperando(fila).getOperando(columna).clonarNodo());
					comodin2.agregaOperando(SimplificarNumeros.MENOSUNO.clonarNodo());
					
					comodin=new Nodo("kfi");
					comodin.agregaOperando(matriz);
					comodin.agregaOperando(comodin2);
					comodin.agregaOperando(comodin3);
					
					matriz=funcionSimplificar.simplifica(comodin);
					
					for(int j=1; j<=numFilas; j++){
						if(j!=fila){
							if (Opciones.getPresicion()){
								comodin3=new Nodo("Entero",1);
							}else{
								comodin3=new Nodo("Decimal",1);
							}
							comodin3.agregaOperando(new Nodo((new Integer(j)).toString()));
							
							comodin2=new Nodo("·");
							comodin2.agregaOperando(SimplificarNumeros.MENOSUNO.clonarNodo());
							comodin2.agregaOperando(matriz.getOperando(j).getOperando(columna).clonarNodo());
							
							comodin=new Nodo("fikfj");
							comodin.agregaOperando(matriz);
							comodin.agregaOperando(comodin3);
							comodin.agregaOperando(comodin2);
							
							if (Opciones.getPresicion()){
								comodin3=new Nodo("Entero",1);
							}else{
								comodin3=new Nodo("Decimal",1);
							}
							comodin3.agregaOperando(new Nodo((new Integer(fila)).toString()));
							
							comodin.agregaOperando(comodin3);
							
							matriz=funcionSimplificar.simplifica(comodin);
						}
					}
				}
			}
			
			return matriz;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar ReduccionFilas");
		}	
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		try{
			if(!expresionPrefijo.getTipo().equals("reduce")){
				expresionPrefijo.cambiaTipo("reduce");
			}
			
			if(!expresionPrefijo.getOperando(1).getTipo().equals("Matriz")){
				return expresionPrefijo;
			}
			
			return simplificarNumeros(expresionPrefijo, funcionSimplificar, funcionSimplificarNumeros);
			
			//return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar Inversa");
		}
	}
	
	/**
	 *Función que deriva la expresión recibida.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		try{
			return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al derivar ReduccionFilas");
		}
	}

}