package itcr.mathexplorer.funciones;

import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.herramientas.Opciones;

/**
 *@(#)Rnd.java
 *
 *Esta es la implementación de la función número aleatorio
 *
 *@author Alexander Borbón Alpízar
 */
public class Rnd extends Funcion{
	
	//CONSTRUCTOR
	/**
	 * Constructor vacío de la función Rnd
	 */
	public Rnd() {	}

	//FUNCIONES PUBLICAS
	
	/**
	 *El número aleatorio no admite parámetros
	 */
	public boolean permiteNumeroParametros(int numeroParametros){
		return numeroParametros==0;
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		try{
			return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar un número aleatorio");
		}
	}

	/**
	 *Función que simplifica la expresión recibida. Si esta función se evalúa aquí quiere decir que no encontró el operador
	 *donde se tenía que evaluar, lo que es un error.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		try{
			if(!expresionPrefijo.getTipo().equals("rnd")){
				expresionPrefijo.cambiaTipo("rnd");
			}
			
			return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar un número aleatorio");
		}
	}
	
	/**
	 *Función que deriva la expresión recibida.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		Nodo comodin;
		
		try{
			if (Opciones.getPresicion()){
				comodin=new Nodo("Entero",1);
			}else{
				comodin=new Nodo("Decimal",1);
			}
			comodin.agregaOperando(new Nodo("0"));
			
			return comodin;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al derivar un número aleatorio");
		}
	}
}