package itcr.mathexplorer.funciones;

import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.herramientas.Opciones;

/**
 *@(#)Si.java
 *
 *Esta es la implementación de la función Si
 *Esta función tiene tres parámetros, is
 *el primero es true devuelve el segundo y si
 *es false devuelve el tercero.
 *
 *@author Alexander Borbón Alpízar
 */
public class Si extends Funcion{
	
	//CONSTRUCTOR
	/**
	 * Constructor vacío de la función Si
	 */
	public Si() {	}

	//FUNCIONES PUBLICAS
	
	/**
	 *El signo sólo admite un parámetro
	 */
	public boolean permiteNumeroParametros(int numeroParametros){
		return (numeroParametros==2 || numeroParametros==3);
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		try{
			return new Nodo("Indefinido");
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar un signo");
		}	
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		try{
			if(!expresionPrefijo.getTipo().equals("si")){
				expresionPrefijo.cambiaTipo("si");
			}
			
			if(expresionPrefijo.getOperando(1).getTipo().equals("Indefinido")){
				return new Nodo("Indefinido");
			}
			
			if(expresionPrefijo.getOperando(1).getTipo().equals("Boolean")){
				if(expresionPrefijo.getOperando(1).getOperando(1).getTipo().equals("True")){
					return expresionPrefijo.getOperando(2);
				}else{
					if(expresionPrefijo.getNumeroOperandos()>=3){
						return expresionPrefijo.getOperando(3);
					}else{
						return new Nodo("Indefinido");
					}
				}
			}else if(funcionSimplificar.esNumero(expresionPrefijo.getOperando(1))){
				return new Nodo("Indefinido");
			}else{
				return expresionPrefijo;	
			}
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar un signo");
		}
	}
	
	/**
	 *Función que deriva la expresión recibida.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		Nodo comodin;
		
		try{
			if (Opciones.getPresicion()){
				comodin=new Nodo("Entero",1);
			}else{
				comodin=new Nodo("Decimal",1);
			}
			comodin.agregaOperando(new Nodo("0"));
			
			return comodin;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al derivar un EsNumero");
		}
	}

}