package itcr.mathexplorer.funciones;

import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.herramientas.Opciones;

/**
 *@(#)Signo.java
 *
 *Esta es la implementación de la función signo
 *
 *@author Alexander Borbón Alpízar
 */
public class Signo extends Funcion{
	
	//CONSTRUCTOR
	/**
	 * Constructor vacío de la función Signo
	 */
	public Signo() {	}

	//FUNCIONES PUBLICAS
	
	/**
	 *El signo sólo admite un parámetro
	 */
	public boolean permiteNumeroParametros(int numeroParametros){
		return numeroParametros==1;
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		Nodo operando1, comodin;
		String tipo;

		try{
			if(expresionPrefijo.getOperando(1).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(1).getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
			}
			
			operando1=expresionPrefijo.getOperando(1).clonarNodo();
			tipo=operando1.getTipo();
			
			if(tipo.length()<=5 || "EnteroLargo DecimalLargo Racional".indexOf(tipo)==-1){
				boolean presicion=Opciones.getPresicion();
				
				Opciones.setPresicion(false);
				
				operando1=funcionSimplificar.simplifica(operando1);
				
				Opciones.setPresicion(presicion);
			}
			
			if(operando1.getOperando(1).getTipo().indexOf("-")!=-1){
				if (Opciones.getPresicion()){
					comodin=new Nodo("Entero",1);
				}else{
					comodin=new Nodo("Decimal",1);
				}
				comodin.agregaOperando(new Nodo("-1"));
				
				return comodin;
			}else if(operando1.esIgual(SimplificarNumeros.CERO)){
				return SimplificarNumeros.CERO.clonarNodo();
			}else{
				return SimplificarNumeros.UNO.clonarNodo();
			}
				
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar un signo");
		}	
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		try{
			if(!expresionPrefijo.getTipo().equals("signo")){
				expresionPrefijo.cambiaTipo("signo");
			}
			
			if(expresionPrefijo.getOperando(1).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(1).getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
			}
			
			if(funcionSimplificar.esNumero(expresionPrefijo.getOperando(1))){
				return simplificarNumeros(expresionPrefijo, funcionSimplificar, funcionSimplificarNumeros);
			}
			return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar un signo");
		}	
	}
	
	/**
	 *Función que deriva la expresión recibida.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		Nodo comodin;
		
		try{
			if (Opciones.getPresicion()){
				comodin=new Nodo("Entero",1);
			}else{
				comodin=new Nodo("Decimal",1);
			}
			comodin.agregaOperando(new Nodo("0"));
			
			return comodin;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al derivar un signo");
		}
	}

}
