package itcr.mathexplorer.funciones;

import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.tablas.*;
import itcr.mathexplorer.tipos.*;
import itcr.mathexplorer.herramientas.Opciones;
import itcr.mathexplorer.operadores.*;
import java.math.BigDecimal;

/**
 *@(#)Simplificar.java
 *
 *Esta es la implementación de la función que realiza 
 *la simplificación de una expresión dada en prefijo.
 *
 *Este nodo está pensado para utilizarse en conjunto con expresion
 *y los traductores del paquete.
 *
 *@author Alexander Borbón Alpízar
 */
public class Simplificar extends Funcion{
	//VARIABLES PRIVADAS
	
	private TablaOperadores operadores;
	private TablaFunciones funciones;
	private TablaConstantes constantes;
	private TablaFuncionesUsuario funcionesUsuario;
	
	private SimplificarNumeros simplificarnumeros;
	
	
	/**
	 *Constructor que recibe las tablas de operadores, funciones y constantes para comparar.
	 *@param tablaoperadores La tabla de operadores.
	 *@param tablafunciones La tabla de funciones.
	 *@param tablaconstantes La tabla de constantes.
	 *@param tablafuncionesusuario La tabla de funciones definidas por el usuario.
	 */
	public Simplificar(TablaOperadores tablaoperadores, TablaFunciones tablafunciones, TablaConstantes tablaconstantes, TablaFuncionesUsuario tablafuncionesusuario){
		operadores=tablaoperadores;
		funciones=tablafunciones;
		constantes=tablaconstantes;
		funcionesUsuario=tablafuncionesusuario;
		
		simplificarnumeros=new SimplificarNumeros(operadores, funciones, constantes, funcionesUsuario);
	}

	//FUNCIONES PUBLICAS
	
	/**
	 *Simplificar sólo admite un parámetro
	 */
	public boolean permiteNumeroParametros(int numeroParametros){
		return numeroParametros==1;
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		try{
			if(expresionPrefijo.getOperando(1).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(1).getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
			}
			return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar una simplificación");
		}	
	}
	
	/**
	 *Función que simplifica la expresión recibida. Si esta función se evalúa aquí quiere decir que no encontró el operador
	 *donde se tenía que evaluar, lo que es un error.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		try{
			if(expresionPrefijo.getOperando(1).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(1).getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
			}
			return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar una simplificación");
		}
	}

	/**
	 * Función que realiza la simplificación automática de una expresión en prefijo.
	 * @param expresionPrefijo La expresión a simplificar.
	 * @return Un Nodo con la expresión simplificada. 
	 * @exception ArithmeticException Se lanza si hay error en la expresión.
	 */
	public Nodo simplifica(Nodo expresionPrefijo) throws ArithmeticException{
		String tipo=expresionPrefijo.getTipo();
		Nodo comodin, comodin1, comodin2;
		//String numero="";
		
		try{
			if(tipo.equals("Indefinido")){
				return new Nodo("Indefinido");
			}else if(tipo.equals("Variable") || tipo.equals("Boolean")){
				return expresionPrefijo;
			}else if (tipo.length()>5 && ("EnteroLargo DecimalLargo Constante Racional").indexOf(tipo)!=-1){
				comodin=simplificarnumeros.simplificaNumeros(expresionPrefijo, this);
				
				return comodin;
			}else if(tipo.equals(":=")){
				comodin=simplifica(expresionPrefijo.getOperando(2));
				expresionPrefijo.eliminaOperando(2);
				expresionPrefijo.agregaOperando(comodin,2);
				funcionesUsuario.agregaFuncion(expresionPrefijo.getOperando(1).getTipo(),new FuncionUsuario(expresionPrefijo.getOperando(1).clonarNodo(), expresionPrefijo.getOperando(2).clonarNodo()));
				
				return expresionPrefijo;
			}else{
				
				for(int i=1; i<=expresionPrefijo.getNumeroOperandos(); i++){
					comodin=simplifica(expresionPrefijo.getOperando(i));
					if(comodin.getTipo().equals("Indefinido")){
						return new Nodo("Indefinido");
					}
					expresionPrefijo.eliminaOperando(i);
					expresionPrefijo.agregaOperando(comodin,i);
				}
				
				if(operadores.apareceLlave(tipo)){
					return ((Operador)operadores.devuelveOperador(tipo)).simplificar(expresionPrefijo, this, simplificarnumeros);
/*				}else if(tipo.equals(":=")){
					funcionesUsuario.agregaFuncion(expresionPrefijo.getOperando(1).getTipo(),new FuncionUsuario(expresionPrefijo.getOperando(1).clonarNodo(), expresionPrefijo.getOperando(2).clonarNodo()));
					
					return expresionPrefijo;*/
				}else if(funcionesUsuario.apareceLlave(tipo)){
					comodin1=((FuncionUsuario)funcionesUsuario.devuelveFuncion(tipo)).obtenFuncion();
					comodin2=((FuncionUsuario)funcionesUsuario.devuelveFuncion(tipo)).obtenExpresion().clonarNodo();
					
					for(int i=1; i<=expresionPrefijo.getNumeroOperandos(); i++){
						if(!comodin1.getOperando(i).esIgual(expresionPrefijo.getOperando(i))){
							comodin=new Nodo("eval");
							comodin.agregaOperando(comodin2);
							comodin.agregaOperando(comodin1.getOperando(i));
							comodin.agregaOperando(expresionPrefijo.getOperando(i));
							comodin2=comodin;
						}
					}
					
					return simplifica(comodin2);
				}else if (tipo.equals("Lista")){
					for(int i=2; i<=expresionPrefijo.getNumeroOperandos(); i++){
						comodin=expresionPrefijo.getOperando(i);
						int j=i-1;
						//boolean salir=false;
						while(j>0 && comparar(comodin, expresionPrefijo.getOperando(j))){
							j--;
						}
						if(j+1!=i){
							expresionPrefijo.eliminaOperando(i);
							expresionPrefijo.agregaOperando(comodin, j+1);
						}
					}
					
					for(int i=2; i<=expresionPrefijo.getNumeroOperandos(); i++){
						if(expresionPrefijo.getOperando(i-1).esIgual(expresionPrefijo.getOperando(i))){
							expresionPrefijo.eliminaOperando(i);
							i--;
						}
					}
					
					return expresionPrefijo;
				}else if (tipo.equals("Fila")){
					return expresionPrefijo;
				}else if (tipo.equals("Matriz")){
					return expresionPrefijo;
				}else{ //Tiene que ser una función
					return ((Funcion)funciones.devuelveFuncion(tipo)).simplificar(expresionPrefijo, this, simplificarnumeros);
				}
			}
			
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error en la expresión al simplificar:" + e.getMessage());
		}
	}
	
	/**
	 *Función que indica si la expresión recibida es un número entero
	 *@param expresionPrefijo La expresión a evaluar.
	 *@return True si es un número entero, false en caso contrario.
	 */
	public boolean esEntero(Nodo expresionPrefijo){
		String tipo;
		
		tipo=expresionPrefijo.getTipo();
		
		if(tipo.length()>5 && "EnteroLargo".indexOf(tipo)!=-1){
			return true;
		}else if(tipo.length()>5 && "DecimalLargo".indexOf(tipo)!=-1 && expresionPrefijo.getOperando(1).getTipo().indexOf(".")==-1){
			return true;
		}
		
		return false;
	}
	
	/**
	 *Función que indica si la expresión recibida es un número o operaciones con números (números y constantes)
	 *@param expresionPrefijo La expresión a evaluar.
	 *@return True si es un número, false en caso contrario.
	 */
	public boolean esNumero(Nodo expresionPrefijo){
		String tipo;
		
		tipo=expresionPrefijo.getTipo();
		
		if(tipo.length()>5 && "EnteroLargo DecimalLargo Racional Constante".indexOf(tipo)!=-1){
			return true;
		}else if (tipo.equals("Variable") || tipo.equals("Boolean")){
			return false;
		}else if(operadores.apareceLlave(tipo)){
			for(int i=1; i<=expresionPrefijo.getNumeroOperandos();i++){
				if(!esNumero(expresionPrefijo.getOperando(i))){
					return false;
				}
			}
		}else if(funciones.apareceLlave(tipo)){
			for(int i=1; i<=expresionPrefijo.getNumeroOperandos();i++){
				if(!esNumero(expresionPrefijo.getOperando(i))){
					return false;
				}
			}
		}
		return true;
	}
	
	/**
	 *Función que indica si la expresión recibida es un monomio (a*x0^n0*x1^n1*...*xm^nm)
	 *@param expresionPrefijo La expresión a evaluar.
	 *@return True si es un monomio, false en caso contrario.
	 */
	public boolean esMonomio(Nodo expresionPrefijo){
		String tipo;
		
		tipo=expresionPrefijo.getTipo();
		
		if(esNumero(expresionPrefijo)){
			return true;
		}else if (tipo.equals("Variable")){
			return true;
		}else if(tipo.equals("^")){
			if(expresionPrefijo.getOperando(1).getTipo().equals("Variable") && esEntero(expresionPrefijo.getOperando(2))){
				tipo=expresionPrefijo.getOperando(2).getOperando(1).getTipo();
				if(tipo.indexOf("-")==-1){
					return true;
				}
			}
			return false;
		}else if(tipo.equals("·")){
			for(int i=1; i<=expresionPrefijo.getNumeroOperandos(); i++){
				if(!esMonomio(expresionPrefijo.getOperando(i))){
					return false;
				}
			}
			return true;
		}
		
		return false;
	}
	
	/**
	 *Función que indica si la expresión recibida es un polinomio (suma de monomios)
	 *@param expresionPrefijo La expresión a evaluar.
	 *@return True si es un polinomio, false en caso contrario.
	 */
	public boolean esPolinomio(Nodo expresionPrefijo){
		String tipo;
		
		tipo=expresionPrefijo.getTipo();
		
		if(esMonomio(expresionPrefijo)){
			return true;
		}else if(tipo.equals("+")){
			for(int i=1; i<=expresionPrefijo.getNumeroOperandos(); i++){
				if(!esMonomio(expresionPrefijo.getOperando(i))){
					return false;
				}
			}
			return true;
		}
		
		return false;
	}
	
	/**
	 * Compara dos expresiones para decidir cuál tiene mayor prioridad en la simplificación automática.
	 * @param a La primera expresión a comparar.
	 * @param b La segunda expresión a comparar.
	 * @return True si a tiene mayor prioridad que b, false en caso contrario.
	 */
	public boolean comparar(Nodo a, Nodo b){
		String tipoA, tipoB;
		int m, n;
		Nodo comodin, comodin2;
		
		tipoA=a.getTipo();
		tipoB=b.getTipo();
		
		if(tipoA.length()>5 && ("EnteroLargo DecimalLargo Racional").indexOf(tipoA)!=-1){ //Si es un número
			if(("EnteroLargo DecimalLargo Racional").indexOf(tipoB)!=-1){ //Si ambos son constantes
				return comparaNumeros(a, b);
			}else{
				return true;
			}
		}else if(tipoA.equals("Constante")){
			if(tipoB.equals("Constante")){
				return (a.getOperando(1).getTipo()).compareTo(b.getOperando(1).getTipo())<0;
			}else if(("EnteroLargo DecimalLargo Racional").indexOf(tipoB)!=-1){ //Si ambos son constantes
				return false;
			}else{
				return true;
			}
		}else if(tipoA.equals("Variable") && tipoB.equals("Variable")){
			return (a.getOperando(1).getTipo()).compareTo(b.getOperando(1).getTipo())<0;
		}else if((tipoA.equals("+") && tipoB.equals("+")) || (tipoA.equals("·") && tipoB.equals("·"))){
			m=a.getNumeroOperandos();
			n=b.getNumeroOperandos();
			
			while(m>0 && n>0){
				if(!a.getOperando(m).esIgual(b.getOperando(n))){
					return comparar(a.getOperando(m),b.getOperando(n));
				}
				m--;
				n--;
			}
			
			if(m==0){
				return true;
			}else{
				return false;
			}
		}else if(tipoA.equals("^") && tipoB.equals("^")){
			if(!base(a).esIgual(base(b))){
				return comparar(base(a),base(b));
			}else{
				return comparar(exponente(a),exponente(b));
			}
		}else if(tipoA.equals("!") && tipoB.equals("!")){
			return comparar(a.getOperando(1),b.getOperando(1));
		}else if(funciones.apareceLlave(tipoA) && funciones.apareceLlave(tipoB)){
			if(!tipoA.equals(tipoB)){
				return tipoA.compareTo(tipoB)<0;
			}else{
				m=1;
				
				while(m<=a.getNumeroOperandos() && m<=b.getNumeroOperandos()){
					if(!a.getOperando(m).esIgual(b.getOperando(m))){
						return comparar(a.getOperando(m),b.getOperando(m));
					}
					m++;
				}
				if(m==a.getNumeroOperandos()+1){
					return true;
				}else{
					return false;
				}
			}
		}else if(tipoA.equals("·") && ("^ + ! Variable".indexOf(tipoB)!=-1 || funciones.apareceLlave(tipoB))){
			comodin=new Nodo("·",1);
			comodin.agregaOperando(b);
			
			return comparar(a,comodin);
		}else if(tipoA.equals("^") && ("+ ! Variable".indexOf(tipoB)!=-1 || funciones.apareceLlave(tipoB))){
			comodin2=new Nodo("Entero",1);
			comodin2.agregaOperando(new Nodo("1"));
			comodin=new Nodo("^",2);
			comodin.agregaOperando(b);
			comodin.agregaOperando(comodin2);
			
			return comparar(a,comodin);
		}else if(tipoA.equals("+") && ("! Variable".indexOf(tipoB)!=-1 || funciones.apareceLlave(tipoB))){
			comodin=new Nodo("+",1);
			comodin.agregaOperando(b);
			
			return comparar(a,comodin);
		}else if(tipoA.equals("!") && (tipoB.equals("Variable") || funciones.apareceLlave(tipoB))){
			if(a.getOperando(1).esIgual(b)){
				return false;
			}else{
				comodin=new Nodo("!",1);
				comodin.agregaOperando(b);
				
				return comparar(a,comodin);
			}
		}else if(funciones.apareceLlave(tipoA) && tipoB.equals("Variable")){
			if(tipoA.equals(b.getOperando(1).getTipo())){
				return false;
			}else{
				comodin=new Nodo("Variable",1);
				comodin.agregaOperando(new Nodo(tipoA));
				
				return comparar(comodin,b);
			}
		}else if(tipoA.equals("Matriz")){
			if(tipoB.equals("Matriz")){
				return true;
				/*if(a.getNumeroOperandos()<b.getNumeroOperandos()){
					return true;
				}else if(a.getNumeroOperandos()>b.getNumeroOperandos()){
					return false;
				}
				
				if(a.getOperando(1).getNumeroOperandos()<b.getOperando(1).getNumeroOperandos()){
					return true;
				}else if(a.getNumeroOperandos()>b.getNumeroOperandos()){
					return false;
				}
				
				n=1;
				m=1;
				while(n<=a.getNumeroOperandos()){
					while(m<=a.getOperando(n).getNumeroOperandos() && m<=b.getOperando(n).getNumeroOperandos()){
						if(!a.getOperando(n).getOperando(m).esIgual(b.getOperando(n).getOperando(m))){
							return comparar(a.getOperando(n).getOperando(m),b.getOperando(n).getOperando(m));
						}
						m++;
					}
					n++;
				}
				if(m==a.getNumeroOperandos()+1){
					return true;
				}else{
					return false;
				}else if(tipoB.equals("Lista")){
					return true;
				}*/
			}
			return true;
		}else{
			return !comparar(b,a);
		}
	}

	//FUNCIONES PRIVADAS

	/*
	 *Compara dos números para decidir cuál es menor
	 */
	private boolean comparaNumeros(Nodo a, Nodo b) throws ArithmeticException{
		String tipoA, tipoB, numerador, denominador;
		BigDecimal A, B;
		
		try{
			
			tipoA=a.getTipo();
			tipoB=b.getTipo();
			
			if(tipoA.length()>5 && ("EnteroLargo DecimalLargo").indexOf(tipoA)!=-1){
				A=new BigDecimal(a.getOperando(1).getTipo());
			}else if(tipoA.equals("Constante")){
				boolean presicion=Opciones.getPresicion();
				
				Opciones.setPresicion(false);
				
				Nodo comodin=simplificarnumeros.simplificaNumeros(a, this);
				
				Opciones.setPresicion(presicion);
				
				A=new BigDecimal(comodin.getOperando(1).getTipo());
			}else{ //Es un racional
				numerador=a.getOperando(1).getTipo();
				denominador=a.getOperando(2).getTipo();
			
				A=(new BigRational(numerador,denominador)).toBigDecimal(Math.max(numerador.length(),denominador.length())+Opciones.getNumDecimales());
			}
			
			if(tipoB.length()>5 && ("EnteroLargo DecimalLargo").indexOf(tipoB)!=-1){
				B=new BigDecimal(b.getOperando(1).getTipo());
			}else if(tipoB.equals("Constante")){
				boolean presicion=Opciones.getPresicion();
				
				Opciones.setPresicion(false);
				
				Nodo comodin=simplificarnumeros.simplificaNumeros(b, this);
				
				Opciones.setPresicion(presicion);
				
				B=new BigDecimal(comodin.getOperando(1).getTipo());
			}else{ //Es un racional
				numerador=b.getOperando(1).getTipo();
				denominador=b.getOperando(2).getTipo();
			
				B=(new BigRational(numerador,denominador)).toBigDecimal(Math.max(numerador.length(),denominador.length())+Opciones.getNumDecimales());
			}
			
			return (A.compareTo(B))<0;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("El formato de los números no es correcto");
		}
	}
	
	/*
	 *Devuelve la base de la expresión vista como una potencia.
	 */
	private Nodo base(Nodo expresionPrefijo){
		String tipo;
		
		tipo=expresionPrefijo.getTipo();
		//if(("EnteroLargo DecimalLargo Constante Racional").indexOf(tipo)!=-1){ //Si es un número
		if(tipo.length()>5 && ("EnteroLargo DecimalLargo Racional").indexOf(tipo)!=-1){ //Si es un número
			return new Nodo("Indefinido");
		}else if (tipo.equals("^")){
			return expresionPrefijo.getOperando(1);
		}else{
			return expresionPrefijo;
		}
	}
	
	/*
	 *Devuelve el exponente de la expresión vista como una potencia.
	 */
	private Nodo exponente(Nodo expresionPrefijo){
		String tipo;
		Nodo comodin;
		
		tipo=expresionPrefijo.getTipo();
		//if(("EnteroLargo DecimalLargo Constante Racional").indexOf(tipo)!=-1){ //Si es un número
		if(tipo.length()>5 && ("EnteroLargo DecimalLargo Racional").indexOf(tipo)!=-1){ //Si es un número
			return new Nodo("Indefinido");
		}else if (tipo.equals("^")){
			return expresionPrefijo.getOperando(2);
		}else{
			comodin=new Nodo("Entero");
			comodin.agregaOperando(new Nodo("1"));
			return comodin;
		}
	}
	
	/**
	 *Función que deriva la expresión recibida.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		try{
			return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al derivar un simplificar");
		}
	}
}
