package itcr.mathexplorer.funciones;

import itcr.mathexplorer.expresion.*;

/**
 *@(#)Sustituir.java
 *
 *Esta es la implementación de la función sustituir.
 *Esta función busca una expresión determinada dentro de otra para
 *sustituirla por una tercera.
 *Esta función recibe tres parámetros: la expresión donde se sustituirá,
 *la expresión que se buscara para ser sustituida y la expresión por la
 *que se sustituirá.
 *
 *@author Alexander Borbón Alpízar
 */
public class Sustituir extends Funcion{
	
	//CONSTRUCTOR
	/**
	 * Constructor vacío de la función Sustituir.
	 */
	public Sustituir() {	}

	//FUNCIONES PUBLICAS
	
	/**
	 *La función sustituir recibe tres parámetros.
	 */
	public boolean permiteNumeroParametros(int numeroParametros){
		return numeroParametros==3;
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		try{
			//Esta función no tiene que ser corrida aquí ya que no importa si hay números o no se simplifica en simplificar.
			return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar una sustitución");
		}	
	}

	/**
	 *Función que simplifica la expresión recibida. Si esta función se evalúa aquí quiere decir que no encontró el operador
	 *donde se tenía que evaluar, lo que es un error.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		Nodo sustituido;
		
		try{
			if(!expresionPrefijo.getTipo().equals("eval")){
				expresionPrefijo.cambiaTipo("eval");
			}
			
			for(int i=1; i<=expresionPrefijo.getNumeroOperandos(); i++){
				if(expresionPrefijo.getOperando(i).getTipo().equals("Indefinido")){
					return new Nodo("Indefinido");
				}
			}
			
			sustituido=funcionSimplificar.simplifica(sustituir(expresionPrefijo.getOperando(1).clonarNodo(), expresionPrefijo.getOperando(2).clonarNodo(), expresionPrefijo.getOperando(3).clonarNodo()));
			
			return sustituido;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar una sustitución");
		}
	}
	
	/*
	 *Función que sustituye en la expresión dada, el término porSustituir y lo cambia
	 *por expParaSustituir
	 */
	private Nodo sustituir(Nodo expresion, Nodo porSustituir, Nodo expParaSustituir){
		Nodo comodin;
		
		if(expresion.esIgual(porSustituir)){
			return expParaSustituir.clonarNodo();
		}else{
			for(int i=1; i<=expresion.getNumeroOperandos(); i++){
				comodin=sustituir(expresion.getOperando(i), porSustituir, expParaSustituir);
				expresion.eliminaOperando(i);
				expresion.agregaOperando(comodin,i);
			}
			return expresion;
		}
	}
	
	/**
	 *Función que deriva la expresión recibida.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		try{
			return new Nodo("Indefinido");
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al derivar una raíz");
		}
	}
}