package itcr.mathexplorer.funciones;

import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.herramientas.Opciones;

/**
 *@(#)Transpuesta.java
 *
 *Esta es la implementación de la función que toma una matriz
 *y multiplica la fila i por k.
 *
 *@author Alexander Borbón Alpízar
 */
public class Transpuesta extends Funcion{
	
	//CONSTRUCTOR
	/**
	 * Constructor vacío de la función Transpuesta.
	 */
	public Transpuesta() {	}

	//FUNCIONES PUBLICAS
	
	/**
	 *Transpuesta admite sólo un parámetro: la matriz.
	 */
	public boolean permiteNumeroParametros(int numeroParametros){
		return numeroParametros==1;
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		int i, j;
		Nodo matriz, comodin;
		
		try{
			if(!expresionPrefijo.getTipo().equals("trans")){
				expresionPrefijo.cambiaTipo("trans");
			}
			
			if(expresionPrefijo.getOperando(1).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(1).getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
			}
			
			if(!expresionPrefijo.getOperando(1).getTipo().equals("Matriz")){
				return expresionPrefijo;
			}else if(expresionPrefijo.getNumeroOperandos()==0){
				return expresionPrefijo;
			}else{
				matriz=expresionPrefijo.getOperando(1);
				i=new Integer(matriz.getNumeroOperandos());
				j=new Integer(matriz.getOperando(1).getNumeroOperandos());
				
				comodin=new Nodo("Matriz");
				for(int k=1; k<=j; k++){
					comodin.agregaOperando(new Nodo("Fila"));
				}
				
				for(int k=1; k<=i; k++){
					for(int l=1; l<=j; l++){
						comodin.getOperando(l).agregaOperando(matriz.getOperando(k).getOperando(l));
					}
				}
			}
			
			return comodin;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar trans");
		}	
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		try{
			if(!expresionPrefijo.getTipo().equals("trans")){
				expresionPrefijo.cambiaTipo("trans");
			}
			
			if(expresionPrefijo.getOperando(1).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(1).getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
			}
			
			if(!expresionPrefijo.getOperando(1).getTipo().equals("Matriz")){
				return expresionPrefijo;
			}else{
				return simplificarNumeros(expresionPrefijo, funcionSimplificar, funcionSimplificarNumeros);
			}
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar trans");
		}
	}
	
	/**
	 *Función que deriva la expresión recibida.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		Nodo comodin;
		
		try{
			if (Opciones.getPresicion()){
				comodin=new Nodo("Entero",1);
			}else{
				comodin=new Nodo("Decimal",1);
			}
			comodin.agregaOperando(new Nodo("0"));
			
			return comodin;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al derivar trans");
		}
	}

}