package itcr.mathexplorer.funciones;

import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.tipos.*;
import itcr.mathexplorer.herramientas.Opciones;
import java.math.BigDecimal;

/**
 *@(#)Truncar.java
 *
 *Esta es la implementación de la función truncar
 *
 *@author Alexander Borbón Alpízar
 */
public class Truncar extends Funcion{
	
	//CONSTRUCTOR
	/**
	 * Constructor vacío de la función Truncar
	 */
	public Truncar() {	}

	//FUNCIONES PUBLICAS
	
	/**
	 *El redondeo sólo admite un parámetro
	 */
	public boolean permiteNumeroParametros(int numeroParametros){
		return (numeroParametros==1 || numeroParametros==2);
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		Nodo operando1;
		String tipo;
		int numop, decimales=0, pospunto;
		String numero;
		boolean negativo=false;
		
		try{
			numop=expresionPrefijo.getNumeroOperandos();
			for(int i=1; i<=numop; i++){
				if(expresionPrefijo.getOperando(i).getTipo().equals("Indefinido")){
					return new Nodo("Indefinido");
				}
			}
			
			if(numop==2){
				decimales=(new Integer(expresionPrefijo.getOperando(2).getOperando(1).getTipo())).intValue();
			}
			
			operando1=expresionPrefijo.getOperando(1).clonarNodo();
			tipo=operando1.getTipo();
			
			if(tipo.length()<=5 || "EnteroLargo DecimalLargo Racional".indexOf(tipo)==-1){ //Si no es número directo (es una operación de números)
				int numDecimales=Opciones.getNumDecimales();
				boolean presicion=Opciones.getPresicion();
				
				Opciones.setNumDecimales(Opciones.getNumDecimales());
				Opciones.setPresicion(false);
				
				operando1=funcionSimplificar.simplifica(operando1);
				numero=operando1.getOperando(1).getTipo();
				
				Opciones.setNumDecimales(numDecimales);
				Opciones.setPresicion(presicion);
			}else if(tipo.equals("Racional")){
				String numerador, denominador;
				
				numerador=operando1.getOperando(1).getTipo();
				denominador=operando1.getOperando(2).getTipo();
				
				numero=(new BigRational(numerador, denominador)).toBigDecimal(Math.max(numerador.length(),denominador.length())+Opciones.getNumDecimales()).toString();
			}else{
				numero=(new BigDecimal(operando1.getOperando(1).getTipo()).toString());
			}
			
			if(numero.toString().indexOf("-")!=-1){
				negativo=true;
				numero=numero.substring(1);
			}
			
			pospunto=numero.indexOf(".");
			if(pospunto==-1){
				pospunto=numero.length();
			}
			
			if(decimales>0){
				if(decimales+pospunto>=numero.length()-1){
					return expresionPrefijo.getOperando(1).clonarNodo();
				}else{
					numero=numero.substring(0,decimales+pospunto+1);
				}
			}else if(decimales==0){
				if(pospunto>=numero.length()){
					return expresionPrefijo.getOperando(1).clonarNodo();
				}else{
					numero=numero.substring(0,pospunto);
				}
			}else{ //decimales es negativo
				if(-1*decimales>=pospunto){
					return SimplificarNumeros.CERO;
				}else{
					numero=numero.substring(0,decimales+pospunto);
					
					for(int i=0;i<-1*decimales;i++){
						numero=numero.concat("0");
					}
				}
			}
			
			if(negativo){
				numero="-".concat(numero);
			}
			
			if(numero.length()>18){
				operando1=new Nodo("DecimalLargo",1);
			}else{
				operando1=new Nodo("Decimal",1);
			}
			operando1.agregaOperando(new Nodo(numero));

			return funcionSimplificarNumeros.simplificaNumeros(operando1, funcionSimplificar);
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar un truncamiento");
		}
	}

	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		int numop;
		
		try{
			if(!expresionPrefijo.getTipo().equals("truncar")){
				expresionPrefijo.cambiaTipo("truncar");
			}
			
			numop=expresionPrefijo.getNumeroOperandos();
			for(int i=1; i<=numop; i++){
				if(expresionPrefijo.getOperando(i).getTipo().equals("Indefinido")){
					return new Nodo("Indefinido");
				}
			}
			
			if(funcionSimplificar.esNumero(expresionPrefijo.getOperando(1))){
				//Si hay dos operadores el segundo tiene que ser un entero (o un decimal sin punto)
				if(numop==1 || (numop==2 && (expresionPrefijo.getOperando(2).getTipo().equals("Entero") || expresionPrefijo.getOperando(2).getTipo().equals("Decimal") && expresionPrefijo.getOperando(2).getOperando(1).getTipo().indexOf(".")==-1))){
					return simplificarNumeros(expresionPrefijo, funcionSimplificar, funcionSimplificarNumeros);
				}else{
					return new Nodo("Indefinido");
				}
			}
			
			return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar un truncamiento");
		}
	}
	
	/**
	 *Función que deriva la expresión recibida.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		Nodo comodin;
		
		try{
			if (Opciones.getPresicion()){
				comodin=new Nodo("Entero",1);
			}else{
				comodin=new Nodo("Decimal",1);
			}
			comodin.agregaOperando(new Nodo("0"));
			
			return comodin;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al derivar un truncar");
		}
	}
}
