package itcr.mathexplorer.funciones;

import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.herramientas.Opciones;
//import itcr.mathexplorer.herramientas.TeoriaNumeros;
//import java.util.Vector;

/**
 *@(#)Primos.java
 *
 *Esta es la implementación de la función Union
 *Esta función devuelve la union de dos conjuntos
 *
 *@author Alexander Borbón Alpízar
 */
public class Union extends Funcion{
	
	//CONSTRUCTOR
	/**
	 * Constructor vacío de la función Union
	 */
	public Union() {	}

	//FUNCIONES PUBLICAS
	
	/**
	 *union admite dos parámetros
	 */
	public boolean permiteNumeroParametros(int numeroParametros){
		return numeroParametros==2;
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		try{
			return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar unión");
		}	
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		Nodo solucion, operando1=expresionPrefijo.getOperando(1), operando2=expresionPrefijo.getOperando(2);
		
		try{
			if(!expresionPrefijo.getTipo().equals("union")){
				expresionPrefijo.cambiaTipo("union");
			}
			
			if(operando1.getTipo().equals("Indefinido") || operando2.getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
			}
			
			if(operando1.getTipo().equals("Lista") && operando2.getTipo().equals("Lista")){
				solucion=operando1.clonarNodo();
				for(int i=1; i<=operando2.getNumeroOperandos(); i++){
					solucion.agregaOperando(operando2.getOperando(i));
				}
				
				return funcionSimplificar.simplifica(solucion);
			}
			
			return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar unión");
		}
	}
	
	/**
	 *Función que deriva la expresión recibida.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		Nodo comodin;
		
		try{
			if (Opciones.getPresicion()){
				comodin=new Nodo("Entero",1);
			}else{
				comodin=new Nodo("Decimal",1);
			}
			comodin.agregaOperando(new Nodo("0"));
			
			return comodin;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al derivar unión");
		}
	}

}