package itcr.mathexplorer.herramientas;

import java.awt.*;

/**
 *@(#)Opciones.java
 *
 * Clase para el manejo de las opciones del programa
 *
 * @author Alexander Borbón Alpízar
 */
public class Opciones{
	
	//VARIABLES
	
	/*
	 *Variables estáticas para ser accesadas por cualquier clase del programa.
	 */
	static private boolean exacto=true, radianes=true, alternarColorGrafica=true, alternarColorGrafica3D=true;
	static private int numDecimales=16, longNumeros=16, 
		pixelesMarcasX=40, pixelesMarcasY=40, numPuntosGrafica=10, 
			pixelesMarcasX3D=40, pixelesMarcasY3D=40, pixelesMarcasZ3D=40, 
				numPuntosGrafica3D=10;
	static private Color colorFondo=new Color(238,238,238), colorLetra=Color.BLACK, 
		colorFondoGrafica=Color.WHITE, colorEjesGrafica=Color.GRAY, 
			colorEjesAtenuadosGrafica=Color.LIGHT_GRAY, colorGrafica=Color.BLUE,
				colorFondoGrafica3D=Color.WHITE, colorEjesGrafica3D=Color.GRAY, 
					colorEjesAtenuadosGrafica3D=Color.LIGHT_GRAY, colorGrafica3D=Color.BLUE;
	static private String aspectoGeneral="Metal";
	static private boolean cambioAspecto, cambioColorFondo, cambioColorLetra, cambioTipoLetra, 
		cambioAlternarColoresGrafica, cambioColorFondoGrafica, cambioColorEjesAtenuadosGrafica, 
			cambioColorGrafica, cambioColorEjesGrafica, cambioTipoLetraEjesGrafica,
				cambioAlternarColoresGrafica3D, cambioColorFondoGrafica3D, cambioColorEjesAtenuadosGrafica3D, 
					cambioColorGrafica3D, cambioColorEjesGrafica3D, cambioTipoLetraEjesGrafica3D;
	static private Font tipoLetra=new Font(null,Font.PLAIN, 12), tipoLetraEjesGrafica=new Font(null,Font.PLAIN, 12),
		tipoLetraEjesGrafica3D=new Font(null,Font.PLAIN, 12);


	//CONSTRUCTORES
	
	/**
	 *Constructor por defecto
	 */
    public Opciones() {
    }
    
    /**
     *Función que retorna el aspecto en que trabaja el programa.
     *@return Metal, Native o Motif de acuerdo al aspecto en que trabajará el programa.
     */
	static public String getAspecto(){
		return aspectoGeneral;
	}
	
    /**
     *Función que cambia el aspecto en que trabaja el programa.
     *@param aspecto Metal, Native o Motif de acuerdo al aspecto en que trabajará el programa.
     */
	static public void setAspecto(String aspecto){
		if("Metal Native Motif".indexOf(aspecto)!=-1 && !aspecto.equals(aspectoGeneral)){
			aspectoGeneral=aspecto;
			cambioAspecto=true;
		}
	}
    
    /**
     *Función que retorna el modo en que trabaja el programa.
     *@return True si se trabaja en modo exacto o false en modo aproximado.
     */
	static public boolean getPresicion(){
		return exacto;
	}
	
    /**
     *Función que cambia la presición en la que trabaja el programa.
     *@param presicion True si se trabaja en modo exacto o false en modo aproximado.
     */
	static public void setPresicion(boolean presicion){
		exacto=presicion;
	}
	
    /**
     *Función que retorna si se debe alternar los colores de las gráficas o usar uno fijo.
     *@return True si se debe alternar el color de las gráficas o false si es un color fijo.
     */
	static public boolean getAlternarColorGrafica(){
		return alternarColorGrafica;
	}
	
    /**
     *Función que define si se debe alternar el color de las gráficas o se utiliza un color fijo.
     *@param alternar True si se alterna el color de las gráficas o false si se usa un color fijo.
     */
	static public void setAlternarColorGrafica(boolean alternar){
		alternarColorGrafica=alternar;
	}
	
    /**
     *Función que retorna si se debe alternar los colores de las gráficas en 3D o usar uno fijo.
     *@return True si se debe alternar el color de las gráficas o false si es un color fijo.
     */
	static public boolean getAlternarColorGrafica3D(){
		return alternarColorGrafica3D;
	}
	
    /**
     *Función que define si se debe alternar el color de las gráficas en 3D o se utiliza un color fijo.
     *@param alternar True si se alterna el color de las gráficas o false si se usa un color fijo.
     */
	static public void setAlternarColorGrafica3D(boolean alternar){
		alternarColorGrafica3D=alternar;
	}
	
    /**
     *Función que retorna el número de decimales con el que trabaja el programa.
     *@return El número de decimales para hacer los cálculos.
     */
	static public int getNumDecimales(){
		return numDecimales;
	}
	
    /**
     *Función que cambia el número de decimales con los que se trabajará en el programa.
     *@param pixeles El nuevo número de decimales.
     */
	static public void setNumDecimales(int pixeles){
		numDecimales=pixeles;
	}
	
    /**
     *Función que retorna el número de pixeles aproximados a los cuales
     *se ponen las marcas del eje x en el graficador.
     *@return El número de pixeles a los cuales se ponen las marcas del eje x en el graficador.
     */
	static public int getPixelesMarcasX(){
		return pixelesMarcasX;
	}
	
    /**
     *Función que cambia el número de pixeles aproximadas a los cuales
     *se ponen las marcas del eje x en el graficador.
     *@param pixeles El número de pixeles a los cuales se ponen las marcas del eje x en el graficador.
     */
	static public void setPixelesMarcasX(int pixeles){
		pixelesMarcasX=pixeles;
	}
	
    /**
     *Función que retorna el número de pixeles aproximados a los cuales
     *se ponen las marcas del eje x en el graficador.
     *@return El número de pixeles a los cuales se ponen las marcas del eje y en el graficador.
     */
	static public int getPixelesMarcasY(){
		return pixelesMarcasY;
	}
	
    /**
     *Función que cambia el número de pixeles aproximadas a los cuales
     *se ponen las marcas del eje y en el graficador.
     *@param pixeles El número de pixeles a los cuales se ponen las marcas del eje y en el graficador.
     */
	static public void setPixelesMarcasY(int pixeles){
		pixelesMarcasY=pixeles;
	}
	
    /**
     *Función que retorna el número de puntos con los que se realizará la gráfica.
     *@return El número de puntos con los que se realizará la gráfica.
     */
	static public int getNumPuntosGrafica(){
		return numPuntosGrafica;
	}
	
    /**
     *Función que define el número de puntos que se utilizan para realizar la gráfica.
     *@param puntos El número de puntos para hacer las gráficas.
     */
	static public void setNumPuntosGrafica(int puntos){
		numPuntosGrafica=puntos;
	}
	
	   /**
     *Función que retorna el número de pixeles aproximados a los cuales
     *se ponen las marcas del eje x en el graficador en 3D.
     *@return El número de pixeles a los cuales se ponen las marcas del eje x en el graficador.
     */
	static public int getPixelesMarcasX3D(){
		return pixelesMarcasX3D;
	}
	
    /**
     *Función que cambia el número de pixeles aproximadas a los cuales
     *se ponen las marcas del eje x en el graficadoren 3D.
     *@param pixeles El número de pixeles a los cuales se ponen las marcas del eje x en el graficador.
     */
	static public void setPixelesMarcasX3D(int pixeles){
		pixelesMarcasX3D=pixeles;
	}
	
    /**
     *Función que retorna el número de pixeles aproximados a los cuales
     *se ponen las marcas del eje x en el graficador en 3D.
     *@return El número de pixeles a los cuales se ponen las marcas del eje y en el graficador.
     */
	static public int getPixelesMarcasY3D(){
		return pixelesMarcasY3D;
	}
	
    /**
     *Función que cambia el número de pixeles aproximadas a los cuales
     *se ponen las marcas del eje y en el graficador en 3D.
     *@param pixeles El número de pixeles a los cuales se ponen las marcas del eje y en el graficador.
     */
	static public void setPixelesMarcasY3D(int pixeles){
		pixelesMarcasY3D=pixeles;
	}
	
    /**
     *Función que retorna el número de pixeles aproximados a los cuales
     *se ponen las marcas del eje z en el graficador en 3D.
     *@return El número de pixeles a los cuales se ponen las marcas del eje z en el graficador.
     */
	static public int getPixelesMarcasZ3D(){
		return pixelesMarcasZ3D;
	}
	
    /**
     *Función que cambia el número de pixeles aproximadas a los cuales
     *se ponen las marcas del eje z en el graficador en 3D.
     *@param pixeles El número de pixeles a los cuales se ponen las marcas del eje z en el graficador.
     */
	static public void setPixelesMarcasZ3D(int pixeles){
		pixelesMarcasZ3D=pixeles;
	}
	
    /**
     *Función que retorna el número de puntos con los que se realizará la gráfica en 3D.
     *@return El número de puntos con los que se realizará la gráfica.
     */
	static public int getNumPuntosGrafica3D(){
		return numPuntosGrafica3D;
	}
	
    /**
     *Función que define el número de puntos que se utilizan para realizar la gráfica en 3D.
     *@param puntos El número de puntos para hacer las gráficas.
     */
	static public void setNumPuntosGrafica3D(int puntos){
		numPuntosGrafica3D=puntos;
	}
	
    /**
     *Función que retorna el tipo de ángulos en que trabajará el programa.
     *@return True si se trabajará en radianes y false si se trabajará en grados.
     */
	static public boolean getAngulo(){
		return radianes;
	}
	
    /**
     *Función que cambia el tipo de ángulos en que trabajará el programa.
     *@param angulo True si se trabajará en radianes y false si se trabajará en grados.
     */
	static public void setAngulo(boolean angulo){
		radianes=angulo;
	}
    
    /**
     *Función que retorna el color del fondo del programa.
     *@return El color para el fondo del programa.
     */
	static public Color getColorFondo(){
		return colorFondo;
	}
	
    /**
     *Función que cambia el color del fondo del programa.
     *@param colorfondo El color para el fondo del programa.
     */
	static public void setColorFondo(Color colorfondo){
		if(colorfondo!=null && !colorfondo.equals(colorFondo)){
			colorFondo=colorfondo;
			cambioColorFondo=true;
		}
	}
	
    /**
     *Función que retorna el color de la letra de las expresiones matemáticas.
     *@return El color de la letra de las expresiones matemáticas.
     */
	static public Color getColorLetra(){
		return colorLetra;
	}
	
    /**
     *Función que cambia el color de la letra de las expresiones matemáticas.
     *@param colorletra El color de la letra de las expresiones matemáticas.
     */
	static public void setColorLetra(Color colorletra){
		if(colorletra!=null && !colorletra.equals(colorLetra)){
			colorLetra=colorletra;
			cambioColorLetra=true;
		}
	}
	
    /**
     *Función que retorna el color de fondo de la gráfica.
     *@return El color de fondo de la gráfica.
     */
	static public Color getColorFondoGrafica(){
		return colorFondoGrafica;
	}
	
    /**
     *Función que cambia el color de fondo de la gráfica.
     *@param color El color de fondo para la gráfica.
     */
	static public void setColorFondoGrafica(Color color){
		if(color!=null && !color.equals(colorFondoGrafica)){
			colorFondoGrafica=color;
			cambioColorFondoGrafica=true;
		}
	}
	
    /**
     *Función que retorna el color de los ejes de la gráfica.
     *@return El color de los ejes de la gráfica.
     */
	static public Color getColorEjesGrafica(){
		return colorEjesGrafica;
	}
	
    /**
     *Función que cambia el color de los ejes de la gráfica.
     *@param color El color de los ejes para la gráfica.
     */
	static public void setColorEjesGrafica(Color color){
		if(color!=null && !color.equals(colorEjesGrafica)){
			colorEjesGrafica=color;
			cambioColorEjesGrafica=true;
		}
	}
	
    /**
     *Función que retorna el color de los ejes atenuados de la gráfica.
     *@return El color de los ejes atenuados de la gráfica.
     */
	static public Color getColorEjesAtenuadosGrafica(){
		return colorEjesAtenuadosGrafica;
	}
	
    /**
     *Función que cambia el color de los ejes atenuados de la gráfica.
     *@param color El color de los ejes atenuados para la gráfica.
     */
	static public void setColorEjesAtenuadosGrafica(Color color){
		if(color!=null && !color.equals(colorEjesAtenuadosGrafica)){
			colorEjesAtenuadosGrafica=color;
			cambioColorEjesAtenuadosGrafica=true;
		}
	}
	
    /**
     *Función que retorna el color de la gráfica.
     *@return El color de la gráfica.
     */
	static public Color getColorGrafica(){
		return colorGrafica;
	}
	
    /**
     *Función que cambia el color de la gráfica.
     *@param color El color de la gráfica.
     */
	static public void setColorGrafica(Color color){
		if(color!=null && !color.equals(colorGrafica)){
			colorGrafica=color;
			cambioColorGrafica=true;
		}
	}
	
	   /**
     *Función que retorna el color de fondo de la gráfica en 3D.
     *@return El color de fondo de la gráfica.
     */
	static public Color getColorFondoGrafica3D(){
		return colorFondoGrafica3D;
	}
	
    /**
     *Función que cambia el color de fondo de la gráfica en 3D.
     *@param color El color de fondo para la gráfica.
     */
	static public void setColorFondoGrafica3D(Color color){
		if(color!=null && !color.equals(colorFondoGrafica3D)){
			colorFondoGrafica3D=color;
			cambioColorFondoGrafica3D=true;
		}
	}
	
    /**
     *Función que retorna el color de los ejes de la gráfica en 3D.
     *@return El color de los ejes de la gráfica.
     */
	static public Color getColorEjesGrafica3D(){
		return colorEjesGrafica3D;
	}
	
    /**
     *Función que cambia el color de los ejes de la gráfica en 3D.
     *@param color El color de los ejes para la gráfica.
     */
	static public void setColorEjesGrafica3D(Color color){
		if(color!=null && !color.equals(colorEjesGrafica3D)){
			colorEjesGrafica3D=color;
			cambioColorEjesGrafica3D=true;
		}
	}
	
    /**
     *Función que retorna el color de los ejes atenuados de la gráfica en 3D.
     *@return El color de los ejes atenuados de la gráfica.
     */
	static public Color getColorEjesAtenuadosGrafica3D(){
		return colorEjesAtenuadosGrafica3D;
	}
	
    /**
     *Función que cambia el color de los ejes atenuados de la gráfica en 3D.
     *@param color El color de los ejes atenuados para la gráfica.
     */
	static public void setColorEjesAtenuadosGrafica3D(Color color){
		if(color!=null && !color.equals(colorEjesAtenuadosGrafica3D)){
			colorEjesAtenuadosGrafica3D=color;
			cambioColorEjesAtenuadosGrafica3D=true;
		}
	}
	
    /**
     *Función que retorna el color de la gráfica en 3D.
     *@return El color de la gráfica.
     */
	static public Color getColorGrafica3D(){
		return colorGrafica3D;
	}
	
    /**
     *Función que cambia el color de la gráfica en 3D.
     *@param color El color de la gráfica.
     */
	static public void setColorGrafica3D(Color color){
		if(color!=null && !color.equals(colorGrafica3D)){
			colorGrafica3D=color;
			cambioColorGrafica3D=true;
		}
	}
	
    /**
     *Función que retorna el tipo de la letra de las expresiones matemáticas.
     *@return El tipo de la letra de las expresiones matemáticas.
     */
	static public Font getTipoLetra(){
		return tipoLetra;
	}
	
    /**
     *Función que cambia el tipo de la letra de las expresiones matemáticas.
     *@param tipoletra El tipo de la letra de las expresiones matemáticas.
     */
	static public void setTipoLetra(Font tipoletra){
		if(tipoletra!=null && !tipoLetra.equals(tipoletra)){
			tipoLetra=tipoletra;
			cambioTipoLetra=true;
		}
	}
	
    /**
     *Función que retorna el tipo de la letra de los ejes para la gráfica.
     *@return El tipo de la letra de las expresiones matemáticas.
     */
	static public Font getTipoLetraEjesGrafica(){
		return tipoLetraEjesGrafica;
	}
	
    /**
     *Función que cambia el tipo de la letra de los ejes de la gráfica.
     *@param tipoletra El tipo de la letra de las expresiones matemáticas.
     */
	static public void setTipoLetraEjesGrafica(Font tipoletra){
		if(tipoletra!=null && !tipoLetraEjesGrafica.equals(tipoletra)){
			tipoLetraEjesGrafica=tipoletra;
			cambioTipoLetraEjesGrafica=true;
		}
	}
	
    /**
     *Función que retorna el tipo de la letra de los ejes para la gráfica en 3D.
     *@return El tipo de la letra de las expresiones matemáticas.
     */
	static public Font getTipoLetraEjesGrafica3D(){
		return tipoLetraEjesGrafica3D;
	}
	
    /**
     *Función que cambia el tipo de la letra de los ejes de la gráfica en 3D.
     *@param tipoletra El tipo de la letra de las expresiones matemáticas.
     */
	static public void setTipoLetraEjesGrafica3D(Font tipoletra){
		if(tipoletra!=null && !tipoLetraEjesGrafica3D.equals(tipoletra)){
			tipoLetraEjesGrafica3D=tipoletra;
			cambioTipoLetraEjesGrafica3D=true;
		}
	}
	
    /**
     *Función que indica si el aspecto del programa cambió.
     *@return True si el aspecto cambió o false si no ha cambiado.
     */
	static public boolean getCambioAspecto(){
		if(cambioAspecto){
			cambioAspecto=false;
			return true;
		}else{
			return false;
		}
	}
	
    /**
     *Función que indica si hubo un cambio en el color de fondo del programa.
     *@return True si el color de fondo cambió o false si no ha cambiado.
     */
	static public boolean getCambioColorFondo(){
		if(cambioColorFondo){
			cambioColorFondo=false;
			return true;
		}else{
			return false;
		}
	}
	
    /**
     *Función que indica si hubo un cambio en el color de letra del programa.
     *@return True si el color de letra cambió o false si no ha cambiado.
     */
	static public boolean getCambioColorLetra(){
		if(cambioColorLetra){
			cambioColorLetra=false;
			return true;
		}else{
			return false;
		}
	}
	
    /**
     *Función que indica si hubo un cambio en el tipo de letra del programa.
     *@return True si el tipo de letra cambió o false si no ha cambiado.
     */
	static public boolean getCambioTipoLetra(){
		if(cambioTipoLetra){
			cambioTipoLetra=false;
			return true;
		}else{
			return false;
		}
	}
	
    /**
     *Función que indica si hubo un cambio en el tipo de letra de los ejes para las gráficas.
     *@return True si el tipo de letra cambió o false si no ha cambiado.
     */
	static public boolean getCambioTipoLetraEjesGrafica(){
		if(cambioTipoLetraEjesGrafica){
			cambioTipoLetraEjesGrafica=false;
			return true;
		}else{
			return false;
		}
	}
	
    /**
     *Función que indica si hubo un cambio en alternar los colores de la gráfica o mantener un color fijo.
     *@return True si alternar los colores cambió o false si no ha cambiado.
     */
	static public boolean getCambioAlternarColoresGrafica(){
		if(cambioAlternarColoresGrafica){
			cambioAlternarColoresGrafica=false;
			return true;
		}else{
			return false;
		}
	}
	
    /**
     *Función que indica si hubo un cambio en el color de fondo de la gráfica.
     *@return True si el color de fondo cambió o false si no ha cambiado.
     */
	static public boolean getCambioColorFondoGrafica(){
		if(cambioColorFondoGrafica){
			cambioColorFondoGrafica=false;
			return true;
		}else{
			return false;
		}
	}
	
    /**
     *Función que indica si hubo un cambio en el color de los ejes atenuados de la gráfica.
     *@return True si el color de los ejes atenuados cambió o false si no ha cambiado.
     */
	static public boolean getCambioColorEjesAtenuados(){
		if(cambioColorEjesAtenuadosGrafica){
			cambioColorEjesAtenuadosGrafica=false;
			return true;
		}else{
			return false;
		}
	}
	
    /**
     *Función que indica si hubo un cambio en el color de la gráfica.
     *@return True si el color de la gráfica cambió o false si no ha cambiado.
     */
	static public boolean getCambioColorGrafica(){
		if(cambioColorGrafica){
			cambioColorGrafica=false;
			return true;
		}else{
			return false;
		}
	}
	
    /**
     *Función que indica si hubo un cambio en el color de los ejes de la gráfica.
     *@return True si el color de los ejes de la gráfica cambió o false si no ha cambiado.
     */
	static public boolean getCambioColorEjesGrafica(){
		if(cambioColorEjesGrafica){
			cambioColorEjesGrafica=false;
			return true;
		}else{
			return false;
		}
	}
	
    /**
     *Función que indica si hubo un cambio en el tipo de letra de los ejes para las gráficas en 3D.
     *@return True si el tipo de letra cambió o false si no ha cambiado.
     */
	static public boolean getCambioTipoLetraEjesGrafica3D(){
		if(cambioTipoLetraEjesGrafica3D){
			cambioTipoLetraEjesGrafica3D=false;
			return true;
		}else{
			return false;
		}
	}
	
    /**
     *Función que indica si hubo un cambio en alternar los colores de la gráfica en 3D o mantener un color fijo.
     *@return True si alternar los colores cambió o false si no ha cambiado.
     */
	static public boolean getCambioAlternarColoresGrafica3D(){
		if(cambioAlternarColoresGrafica3D){
			cambioAlternarColoresGrafica3D=false;
			return true;
		}else{
			return false;
		}
	}
	
    /**
     *Función que indica si hubo un cambio en el color de fondo de la gráfica en 3D.
     *@return True si el color de fondo cambió o false si no ha cambiado.
     */
	static public boolean getCambioColorFondoGrafica3D(){
		if(cambioColorFondoGrafica3D){
			cambioColorFondoGrafica3D=false;
			return true;
		}else{
			return false;
		}
	}
	
    /**
     *Función que indica si hubo un cambio en el color de los ejes atenuados de la gráfica en 3D.
     *@return True si el color de los ejes atenuados cambió o false si no ha cambiado.
     */
	static public boolean getCambioColorEjesAtenuados3D(){
		if(cambioColorEjesAtenuadosGrafica3D){
			cambioColorEjesAtenuadosGrafica3D=false;
			return true;
		}else{
			return false;
		}
	}
	
    /**
     *Función que indica si hubo un cambio en el color de la gráfica en 3D.
     *@return True si el color de la gráfica cambió o false si no ha cambiado.
     */
	static public boolean getCambioColorGrafica3D(){
		if(cambioColorGrafica3D){
			cambioColorGrafica3D=false;
			return true;
		}else{
			return false;
		}
	}
	
    /**
     *Función que indica si hubo un cambio en el color de los ejes de la gráfica en 3D.
     *@return True si el color de los ejes de la gráfica cambió o false si no ha cambiado.
     */
	static public boolean getCambioColorEjesGrafica3D(){
		if(cambioColorEjesGrafica3D){
			cambioColorEjesGrafica3D=false;
			return true;
		}else{
			return false;
		}
	}
	
    /**
     *Función que devuelve la longitud de donde un número pasa a ser largo.
     *@return La longitud de los números antes de pasar a ser largos.
     */
	static public int getLongNumeros(){
		return longNumeros;
	}
    
}//Opciones