package itcr.mathexplorer.herramientas;

import java.math.BigInteger;
import java.util.Vector;

/**
 *@(#)TeoriaNumeros.java
 *
 * Clase con los algoritmos para la teoría de números.
 * Las funciones de esta clase son estáticas, por lo que no hace falta
 * crear un objeto de tipo TeoriaNumeros. Para usar una función de esta
 * clase simplemente se debe poner TeoriaNumeros.funcion().
 *
 * @author Walter Mora Flores
 *
 *Métodos principales:
 *
 *BigInteger raiz(BigInteger n) >> raíz cuadrada entera de n
 *boolean 	EsPrimo(BigInteger N)
 *Vector  	HacerlistaPrimos(int N)  >> primos menores que N
 *void  	PrintListaPrimos(int N)
 *int  		Pi(double x)             >> cantidad de primos <= x
 *int   	Primo(int N)             >> Nth primo
 *Vector  	Factzar_Ensayo_Error(BigInteger Ne, BigInteger limG) >>Busca factores hasta Ge=10^7
 *Vector  	Factzar_Ensayo_Error(BigInteger Ne) >>Busca factores hasta Min{sqrt{N},Ge}, Ge=10^7
 *BigInteger FactorPollard(BigInteger N,BigInteger a) >> busca un factor usando pollard, a=random(2,N-1) (a no 0 ni -2)
 *Vector  	Factor(BigInteger N) >> factoriza usando trial division+pollard
 *String  	printFactorizacion(Vector lista) >>Imprime factorización: printFactorizacion(Factor(N))
 *BigInteger Z_ACR(BigInteger Uis[], BigInteger Ms[])>> PROBLEMA CHINO DEL RESTO
 */
public class TeoriaNumeros{
	//CONSTANTES PRIVADAS
    private static final int MAXN=200000000;
    private static final BigInteger Ge = new BigInteger("10000000");//10^7
    private static final BigInteger UNO = BigInteger.ONE;
    private static final BigInteger DOS = new BigInteger("2");
    private static final BigInteger TRES = new BigInteger("3");
    private static final BigInteger SEIS = new BigInteger("4");
    
    //VARIABLES PRIVADAS
    private static Vector<Number> salida = new Vector<Number>(1);
    private static BigInteger Nf = new BigInteger("1");
    private static int pos = 1; //posición del exponente del factor en factorización
	
	//CONSTRUCTOR
    /**
     * Constructor vacío de la clase TeoriaNumeros.
     * Las funciones de esta clase son estáticas, por lo que no hace falta
     * crear un objeto de tipo TeoriaNumeros. Para usar una función de esta
     * clase simplemente se debe poner TeoriaNumeros.funcion().
     */
    public TeoriaNumeros(){ }


    //-------------------------- PRIMALIDAD -----------------------------------------------------
    /**
     *Función que indica si un número es primo, probablemente primo o compuesto.
     *@param N El número a revisar.
     *@return True si es primo o probablemente primo, false si es compuesto.
     */
    static public boolean esPrimoMR(BigInteger N)
    {
		BigInteger  N1      = N.subtract(BigInteger.ONE);//N-1
		BigInteger  DOS     = new BigInteger("2");
		int[]       primo   = {2,3,5,7,11,13,17,19};
	    int         s       = 0;
		boolean     esPrimo = true;
		BigInteger  a,r,y;
		int         j;

		//n-1 = 2^s r
		while(N1.remainder(DOS).compareTo(BigInteger.ZERO)==0)
		{    N1=N1.divide(DOS);
		     s=s+1;
		}
		r  = N1;
		N1 = N.subtract(BigInteger.ONE);

		for(int i=0; i<=7; i++)
		{
            a = new BigInteger(""+primo[i]);
            y = a.modPow(r, N);
            if( y.compareTo(BigInteger.ONE)!=0 && y.compareTo(N1)!=0)
            {
			   j=1;
			   while(j<= s-1 && y.compareTo(N1)!=0 )
			   {
                  y = y.modPow(DOS, N);
                  if(y.compareTo(BigInteger.ONE)==0) esPrimo=false;
                  j++;
		       }
		       if(y.compareTo(N1)!=0) esPrimo = false;
			}
		}
       return esPrimo;
	}
	
	/**
     *Función que indica si un número es primo, probablemente primo o compuesto.
     *@param N El número a revisar.
     *@return True si es primo o probablemente primo, false si es compuesto.
     */
    static public boolean esPrimo(BigInteger N)
	{
		//n>3 e impar. Respuesta 100% segura si N <341 550 071 728 321
		BigInteger  NMAX    = new BigInteger("341550071728321");
		boolean salida = false;

		if(N.compareTo(NMAX)==-1 && N.compareTo(new BigInteger("19"))==1)
		{
		 salida = esPrimoMR(N);
	    }else salida = N.isProbablePrime(200);
      return salida;
	}

	//------------------------------- ERATOSTENES -----------------------------------------------------
	/**
     *Función que realiza una lista de primos hasta el primo n-ésimo.
     *@param n El número de primos que se va a generar.
     *@return Un vector con todos los primos.
     */
	static public Vector HacerListaPrimos(int n)
	{
        Vector<Integer> salida = new Vector<Integer>(1);
        int k   = 1;
        int max = (n-3)/2;
        boolean[] esPrimo  = new boolean[max+1];

        for(int i = 0; i <= max; i++)
            esPrimo[i]=true;

        for(int i = 0; (2*i+3)*(2*i+3) <= n; i++)
        {
          k = i+1;

          while( ((2*k+1)*(2*i+3)) <= n)
          {
            esPrimo[((2*k+1)*(2*i+3)-3)/2]=false;
            k++;
          }
         }

         salida.addElement(new Integer(2));

         for(int i = 0; i <=max; i++)
         {
         	if(esPrimo[i])
         		salida.addElement(new Integer(2*i+3));
         }
         salida.trimToSize();
         return salida;
     }
     
	/**
     *Función que realiza una lista de primos hasta el primo n-ésimo.
     *@param n El número de primos que se va a generar.
     *@return Un vector con todos los primos.
     */
	static public Vector HacerListaPrimos(long n)
	{
		Vector<Long> salida;
		if(n<=4*MAXN){
	        salida = new Vector<Long>(1);
	        int k   = 1;
	        int max = (new Long((n-3)/2)).intValue();
	        boolean[] esPrimo  = new boolean[max+1];
	
	        for(int i = 0; i <= max; i++)
	            esPrimo[i]=true;
	
	        for(int i = 0; (2*i+3)*(2*i+3) <= n; i++)
	        {
	          k = i+1;
	
	          while( ((2*k+1)*(2*i+3)) <= n)
	          {
	            esPrimo[((2*k+1)*(2*i+3)-3)/2]=false;
	            k++;
	          }
	         }
	
	         salida.addElement(new Long(2));
	
	         for(int i = 0; i <=max; i++)
	         {
	         	if(esPrimo[i])
	         		salida.addElement(new Long(2*i+3));
	         }
	         salida.trimToSize();
		}else{
			salida=new Vector<Long>(0);
		}
        return salida;
     }

     //N < 1000000 (el teórico es N < 2 147 483 647)
     /**
      *Función que imprime la lista de primos hasta el n-ésimo en la salida de JAVA.
      *@param n La lista se imprime hasta el n-esimo primo
      */
    static public void PrintListaPrimos(int n)
	{
		Vector lista_de_primos;
		lista_de_primos = HacerListaPrimos(n);
		
		for(int p = 0; p < lista_de_primos.size(); p++)
		{
			System.out.print(""+lista_de_primos.elementAt(p)+ " , ");
		}
	}

	 //cantidad de primos <= x < MAXN
	 /**
	  *Función que calcula la cantidad de primos que hay hasta x.
	  *@param x El valor hasta el cual se calcula la cantidad de primos.
	  *@return La cantidad de primos.
	  */
	static public int Pi(double x)
	{
		 int nmax = MAXN;
		 if(Math.round(x) <= nmax)
		    nmax=(int)Math.round(x);

		 Vector lista_de_primos;
		 lista_de_primos = HacerListaPrimos(nmax);
         return lista_de_primos.size();
	}
	
	/**
	 *Función que determina todos los primos entre m y n.
	 *@param m El número a partir del cual se calcularán los primos.
	 *@param n El número hasta el cual se cualcularán los primos.
	 *@return Un vector con todos los primos que hay entre m y n.
	 */
	static public Vector PrimosEntre(int m, int n)
	{
    	salida  = new Vector<Number>(1);
	    boolean[]   esPrimo   = new boolean[n-m+2];
	    int           rN      = (int)Math.round(Math.sqrt(n));
	    Vector        Primos  = new Vector(1);   //objetos Integer
	    int           np, r, k=1,pi,k1;

	    Primos = HacerListaPrimos(rN);
	    np = Primos.size();

	    for(int j = 0; j <= n-m; j++)
			esPrimo[j]=true; //[0][1]...[n-m]
	                        //[0+m][1+m]...[n]: aumentando m se obtiene m,m+1,...,n
	                        // Para cada m-1-r+kpi, -1-r+k*pi deberá estar en {0,1,...,n-m}
	    for(int i = 0; i < np; i++){ 
			k  = 1;
			pi = ((Integer)Primos.elementAt(i)).intValue();
			r  = (m-1)% pi;
			if(m-1-r==0)
				k=2;
			k1 =(int)(1-m+Math.pow(pi,2)+r)/pi;
			if(m <= m-1-r+k1*pi && m-1-r+k1*pi<= n) 
				k=k1;
			
			while(-1-r+k*pi<= n-m){ 
				esPrimo[-1-r+k*pi]=false;//
				k++;
			}
	    }

	    for(int j = 0; j <= n-m; j++){
	    	if(esPrimo[j]){ 
	    		//System.out.print(" , "+(j+m)+ " , ");
	       		salida.addElement(new Integer(j+m));
	     	}
	    }
	    return salida;
	}


	//Primo n-ésimo
	/**
	  *Función que encuentra el primo n-ésimo.
	  *@param n El número del primo buscado.
	  *@return El primo buscado.
	  */
    static public int Primo(int n)
    {
		salida = new Vector<Number>(1);
	    boolean[] esPrimo;
        int k=1, max, pi=0;
   		int cota=100000, conta=0;
   		double xcota=1;

		if(n<0){
   			return -1;
		}else if(n>= 9999){
		      xcota = n*Math.log(n)+n*(Math.log(Math.log(n))-0.8395);
	    }else{
	    	xcota = 104730;
	    }

		if(xcota <= MAXN)
		{
			cota     = (int)Math.round(xcota);
			max      = (cota-3)/2;
			esPrimo  = new boolean[max+1];
			
			for(int i = 0; i <= max; i++)
				esPrimo[i]=true;
			
			for(int i = 0; (2*i+3)*(2*i+3) <= cota; i++){
				k = i+1;
				while( ((2*k+1)*(2*i+3)) <= cota)
				{
					esPrimo[((2*k+1)*(2*i+3)-3)/2]=false;
					k++;
				}
			}
			
			salida.addElement(new Integer(2));//conta=0 primo
			
			while(conta<= n){
				if(esPrimo[pi]){
					conta++;
					salida.addElement(new Integer(2*pi+3));
				}
				pi++;
			}
		 }else{
			return 0;
		 }

	    salida.trimToSize();
		return ((Integer)salida.elementAt(n-1)).intValue();
	}

	//--------------------------------------------------------------------------------------------------
	//-----------------------------------------  FACTORIZACION  ----------------------------------------
	/**
	 *Función que reduce el número Ne con el factor p.
	 *@param Ne El número que se va a reducir.
	 *@param p El factor para ser reducido.
	 *@return El número reducido.
	 */
	static public BigInteger reducir(BigInteger Ne, BigInteger p){
        int exp = 0, posAct = pos;
	    BigInteger residuo;
	    residuo =  Ne.mod(p);

	    if(residuo.compareTo(BigInteger.ZERO)==0)
	    {
	      salida.addElement(p); //p es objeto BigInteger
	      salida.addElement(BigInteger.ONE); //exponente
	      pos = pos+2; //posición del siguiente exponente (si hubiera)
	    }

	    while(residuo.compareTo(BigInteger.ZERO)==0)
	    {
		 Ne      =  Ne.divide(p); // Ne = Ne/p
		 residuo =  Ne.mod(p);
		 exp=exp+1;
		 salida.set(posAct, new BigInteger(""+exp)); //p es objeto BigInteger
	    }
    	return Ne;

    }//

	/**
	 *Función que factoriza el número Ne usando ensayo y error con factores menores a limG.
	 *@param Ne El número al que se le buscará la factorización.
	 *@param limG El límite hasta el cual se buscan los factores.
	 *@return Un vector con los factores y los exponentes (las multiplicidades).
	 */
    static public Vector<Number> Factzar_Ensayo_Error(BigInteger Ne, BigInteger limG)
    {
    	BigInteger p    = new BigInteger("5");

    	Nf = Ne;
    	Nf = reducir(Nf, DOS);
    	Nf = reducir(Nf, TRES);

    	while(p.compareTo(limG)<=0)
    	{
          Nf= reducir(Nf, p);          //dividir por p
          Nf= reducir(Nf, p.add(DOS)); //dividir por p+2
          p = p.add(SEIS); //p=p+6
    	}


    	if(Nf.compareTo(BigInteger.ONE)>0)
    	{
          salida.addElement(Nf); //p es objeto BigInteger
	      salida.addElement(BigInteger.ONE); //exponente
	    }
    	return salida;
    }
	    
	// Solo un argumento. Busca factores 
	/**
	 *Función que factoriza el número Ne usando ensayo y error con factores hasta Min{sqrt{N},10^7}.
	 *@param Ne El número al que se le buscará la factorización.
	 *@return Un vector con los factores y los exponentes (las multiplicidades).
	 */
    static public Vector Factzar_Ensayo_Error(BigInteger Ne)
    {
	    BigInteger limG = Ge.min(raiz(Ne));
		return Factzar_Ensayo_Error(Ne, limG);
    }
	    
	/**
	 *Función que calcula la raíz cuadrada entera de un entero.
	 *@param n El número al que se le calculará la raíz cuadrada entera.
	 *@return La raíz cuadrada entera de n.
	 */
	static public BigInteger raiz(BigInteger n)
	{
	    BigInteger xkm1 = n.divide(DOS);
	    BigInteger xk   = n;
	
	      if(n.compareTo(BigInteger.ONE)< 0)
	        return xkm1=n;
	    while(xk.add(xkm1.negate()).compareTo(BigInteger.ONE)>0)
	    {
	        xk=xkm1;
	        xkm1=xkm1.add(n.divide(xkm1));
	        xkm1=xkm1.divide(DOS);
	    }
	    return xkm1;
	}

	//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	/*static private BigInteger f(BigInteger x, BigInteger a){
		return x.multiply(x).add(a);//x^2+1
	}*/
	
	/**
	 *Función que encuentra un factor del número Ne usando Pollard con la constante a.
	 *@param N El número al que se le buscará un factor.
	 *@param a La constante con la que se aplicará el método de Pollard.
	 *@return El factor del número N.
	 */
	static public BigInteger FactorPollard(BigInteger N, BigInteger a)
	{
	     int i, k;
	     BigInteger xi,xj;
	     BigInteger g   = BigInteger.ONE;
	     BigInteger x0  = new BigInteger(""+2);;
	     boolean salir  = false;

	     k = 0;
	     xi= x0;
	     xj= x0;
	     while(salir==false)
	     { i=(int)(Math.pow(2,k)-1);
	       for(int j=i+1; j<=2*i+1; j++)
	       {
			   xj=x0.multiply(x0).add(a).mod(N);
			   //xj=f(x0,a).mod(N);
			   if(xi.compareTo(xj)==0)//son iguales
			   {salir=true;
			    //System.out.print("Falló"+"\n\n");
			    break;
			   }

			   g= N.gcd(xi.subtract(xj));
			   if(g.compareTo(BigInteger.ONE)==1 && g.compareTo(N)==-1)//1<g<N
			   {salir=true;
			    break;
			   }
			   x0=xj;
		   }
		   xi=xj;
		   k++;
		 }
		return g;
	}
	
	/**
	 *Función que factoriza el número N usando ensayo y error y Pollard.
	 *@param N El número al que se le buscará la factorización.
	 *@return Un vector con los factores y los exponentes (las multiplicidades).
	 */
	static public Vector Factor(BigInteger N)
	{
		BigInteger  rN,fN;
		salida=new Vector<Number>();
		pos=1;

		rN=raiz(N);
		if(rN.compareTo(Ge)<0){ 
			salida = Factzar_Ensayo_Error(N, rN);//busca factores hasta rN
		}else{
			   Factzar_Ensayo_Error(N);//busca factores hasta G
			   //llamamos a rho
			   if(esPrimo(Nf))
			   {//nada
			   }else{
					   fN= FactorPollard(Nf, BigInteger.ONE);
					   //Otro intento?
					   if(fN.compareTo(BigInteger.ONE)==0 || fN.compareTo(Nf)==0 )
						  fN= FactorPollard(Nf, (BigInteger.ONE).negate());

					   if(fN.compareTo(BigInteger.ONE)==1 && fN.compareTo(Nf)==-1)
					   {
						 salida.set(pos-1, fN); //pos
						 salida.set(pos, UNO);
						 salida.addElement( Nf.divide(fN));
						 salida.addElement( UNO);
					   }
				   }
			 }
		return salida;
	}

	//Imprimir
	/**
	 *Función que imprime un vector que representa un número factorizado.
	 *@param lista El vector con el número factorizado.
	 *@return Un string con la representación del número factorizado.
	 */
	static public String printFactorizacion(Vector lista)
	{
	     	String tira="";
	     	for(int p = 0; p < lista.size(); p++)
	     	{
	     		if(p%2==0)     tira= tira+lista.elementAt(p);
	     		else           tira= tira+"^"+lista.elementAt(p)+" * ";
	     	}
	        return tira.substring(0,tira.length()-3);
	}


	//------------------------------------------------------------------------------------------------------------
	//------------------------------------  ALGORITMO CHINO DEL RESTO --------------------------------------------
	
	/**
	 *Función que calcula el algoritmo Chino del Resto,
	 *resuelve el problema de encontrar un número que sea
	 *equivalente con U1 mod M1, U2 mod M2, ..., UN mod MN.
	 *@param Uis Las u's del Problema Chino del Resto.
	 *@param Ms Las m's del Problema Chino del Resto.
	 *@return El número que resuelve el Problema Chino del Resto.
	 */
	static public BigInteger Z_ACR(BigInteger Uis[], BigInteger Ms[]){
		//Requiere Ms[i]>2.
	    int        n        = Ms.length-1; //Ms[0],...,Ms[n]
	    BigInteger u        = BigInteger.ZERO;
	    BigInteger producto = BigInteger.ONE;
	    BigInteger temp;
	    BigInteger gamma[]  = new BigInteger[n+1]; //gamma[1],...,gamma[n]
	    BigInteger v[]      = new BigInteger[n+1]; //gamma[1],...,gamma[n]

	    //para k=1,2,...,n, gamma_k = (Prod mi_{i=0}^{k-1})^{-1} Mod m_k.
	    for(int k=1; k<=n; k++)
	    {   producto = Ms[0].mod(Ms[k]);
	        for(int i=1; i<= k-1; i++)
	           producto = (producto.multiply(Ms[i])).mod(Ms[k]);
	        gamma[k] = producto.modInverse(Ms[k]);
	    }
	    int j;
	    v[0]=Uis[0];
	    for(int k=1; k<=n; k++)
	    {   temp = v[k-1];
	        j=k-2;
	        while(j>=0)
	        {temp = ((temp.multiply(Ms[j])).add(v[j])).mod(Ms[k]);
	         j=j-1;
	        }
	        v[k]= (Uis[k].subtract(temp)).multiply(gamma[k]).mod(Ms[k]);
	    }
	   u = v[n];
	   j = n-1;
	   while(j >= 0)
	   {u = (u.multiply(Ms[j])).add(v[j]);
	    j = j-1;
	   }
	   return u;
	}
}