package itcr.mathexplorer.operadores;

import itcr.mathexplorer.funciones.*;
import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.herramientas.Opciones;

/**
 *@(#)Division.java
 *
 *Esta es la implementación de la función division: /
 *
 *@author Alexander Borbón Alpízar
 */
public class Division extends Operador{
	
	/**
	 * Constructor vacío del operador de división.
	 *
	 */
	public Division() {	}

	/**
	 *La division tiene una prioridad de 3.
	 *@return La prioridad de la division: 3.
	 *@exception ArithmeticException Se lanza si hay error al evaluar en la expresión.
	 */
	public int prioridad() throws ArithmeticException{
		return 3;
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		Nodo a=expresionPrefijo.getOperando(1);//funcionSimplificarNumeros.simplificaNumeros(expresionPrefijo.getOperando(1));
		Nodo b=expresionPrefijo.getOperando(2);//funcionSimplificarNumeros.simplificaNumeros(expresionPrefijo.getOperando(2));
		Nodo comodin, comodin2;
		
		try{
			if(expresionPrefijo.getOperando(1).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(1).getTipo().equals("Boolean") || expresionPrefijo.getOperando(2).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(2).getTipo().equals("Boolean")){
					return new Nodo("Indefinido");
			}
			
			comodin=new Nodo("Entero");
			comodin.agregaOperando(new Nodo("-1"));
			comodin2=new Nodo("^");
			comodin2.agregaOperando(b);
			comodin2.agregaOperando(comodin);
			comodin2=funcionSimplificarNumeros.simplificaNumeros(comodin2, funcionSimplificar);
			comodin=new Nodo("·");
			comodin.agregaOperando(a);
			comodin.agregaOperando(comodin2);
			
			return funcionSimplificarNumeros.simplificaNumeros(comodin, funcionSimplificar);
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar una división");
		}	
	}
	
	/**
	 *Función que simplifica la expresión recibida. Si esta función se evalúa aquí quiere decir que no encontró el operador
	 *donde se tenía que evaluar, lo que es un error.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		Nodo comodin, comodin2;
		try{
			if(expresionPrefijo.getOperando(1).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(2).getTipo().equals("Indefinido") 
				|| expresionPrefijo.getOperando(1).getTipo().equals("Boolean") || expresionPrefijo.getOperando(2).getTipo().equals("Boolean")){
					//|| expresionPrefijo.getOperando(1).getTipo().equals("Matriz") || expresionPrefijo.getOperando(2).getTipo().equals("Matriz")){
					return new Nodo("Indefinido");
			}
			
			comodin=new Nodo("^");
			comodin2=new Nodo("Entero");
			comodin2.agregaOperando(new Nodo("-1"));
			comodin.agregaOperando(expresionPrefijo.getOperando(2));
			comodin.agregaOperando(comodin2);
			comodin=funcionSimplificar.simplifica(comodin);
			comodin2=new Nodo("·");
			comodin2.agregaOperando(expresionPrefijo.getOperando(1));
			comodin2.agregaOperando(comodin);
			/*if(expresionPrefijo.getOperando(2).getTipo().equals("Matriz")){
				return comodin2;
			}else{*/
				return funcionSimplificar.simplifica(comodin2);
			//}
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar una división");
		}		
	}

	/**
	 *Función que devuelve si la división es binaria.
	 *@return True si la división es binaria, false en caso contrario.
	 *@exception ArithmeticException Se lanza si el operador no existe.
	 */
	public boolean esBinario() throws ArithmeticException{
		return true;
	}
	
	/**
	 *Función que deriva la expresión recibida.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		Nodo comodin, comodin2, comodin3;
		
		try{
			comodin=new Nodo("·",2);
			comodin.agregaOperando(funcionDerivar.deriva(expresionPrefijo.getOperando(1).clonarNodo(), variable));
			comodin.agregaOperando(expresionPrefijo.getOperando(2).clonarNodo());
			
			comodin2=new Nodo("·",2);
			comodin2.agregaOperando(expresionPrefijo.getOperando(1).clonarNodo());
			comodin2.agregaOperando(funcionDerivar.deriva(expresionPrefijo.getOperando(2).clonarNodo(), variable));
			
			comodin3=new Nodo("-",2);
			comodin3.agregaOperando(comodin);
			comodin3.agregaOperando(comodin2);
			
			if (Opciones.getPresicion()){
				comodin2=new Nodo("Entero",1);
			}else{
				comodin2=new Nodo("Decimal",1);
			}
			comodin2.agregaOperando(new Nodo("2"));
			
			comodin=new Nodo("^",2);
			comodin.agregaOperando(expresionPrefijo.getOperando(2).clonarNodo());
			comodin.agregaOperando(comodin2);
			
			comodin2=new Nodo("/",2);
			comodin2.agregaOperando(comodin3);
			comodin2.agregaOperando(comodin);

			return comodin2;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al derivar una división");
		}
	}

}
