package itcr.mathexplorer.operadores;

import itcr.mathexplorer.funciones.*;
import itcr.mathexplorer.expresion.*;
import java.math.BigInteger;

/**
 *@(#)Factorial.java
 *
 *Esta es la implementación de la función factorial: !
 *
 *@author Alexander Borbón Alpízar
 */
public class Factorial extends Operador{
	
	/**
	 * Constructor vacío del operador Factorial.
	 *
	 */
	public Factorial() {	}

	/**
	 *En factorial en realidad sólo saca un valor, su prioridad es de nivel 5.
	 *@return La prioridad del factorial: 5.
	 *@exception ArithmeticException Se lanza si hay error al evaluar en la expresión.
	 */
	public int prioridad() throws ArithmeticException{
		return 5;
	}

	/**
	 *Función que simplifica la expresión recibida. 
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		Nodo a=expresionPrefijo.getOperando(1);//funcionSimplificarNumeros.simplificaNumeros(expresionPrefijo.getOperando(1));
		String solucion;
		String tipo=a.getTipo();
		BigInteger Solucion=new BigInteger("1");
		Nodo comodin;
		
		try{
			if(tipo.equals("Entero")){
				long valor=new Long(a.getOperando(1).getTipo()).longValue();
				for(long i=1; i<=valor; i++){
					Solucion=Solucion.multiply(new BigInteger((new Long(i)).toString()));
				}
				
				solucion=Solucion.toString();
				
				if(solucion.length()<=18){
					comodin=new Nodo("Entero",1);
				}else{
					comodin=new Nodo("EnteroLargo",1);
				}
				comodin.agregaOperando(new Nodo(solucion));
				
				return comodin;
			}else{
				return new Nodo("Indefinido");
			}
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar un factorial");
		}	
	}

	/**
	 *Función que simplifica la expresión recibida. Si esta función se evalúa aquí quiere decir que no encontró el operador
	 *donde se tenía que evaluar, lo que es un error.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		String tipo;
		Nodo operando;
		
		try{
			operando=expresionPrefijo.getOperando(1);
			tipo=operando.getTipo();
			
			if (tipo.equals("Entero")){
				return simplificarNumeros(expresionPrefijo, funcionSimplificar, funcionSimplificarNumeros);
			}else if(("EnteroLargo DecimalLargo Racional Indefinido Boolean").indexOf(tipo)!=-1){
				return new Nodo("Indefinido");
			}else{
				return expresionPrefijo;
			}
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar un factorial");
		}
	}


	/**
	 *Función que devuelve si el factorial es binario.
	 *@return True si el factorial es binario, false en caso contrario.
	 *@exception ArithmeticException Se lanza si el operador no existe.
	 */
	public boolean esBinario() throws ArithmeticException{
		return false;
	}
	
	/**
	 *Función que deriva la expresión recibida.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		try{
			return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al derivar un factorial");
		}
	}
}