package itcr.mathexplorer.operadores;

import itcr.mathexplorer.funciones.*;
import itcr.mathexplorer.expresion.*;

/**
 *@(#)Igual.java
 *
 *Esta es la implementación del operador igual: =
 *
 *@author Alexander Borbón Alpízar
 */
public class Igual extends Operador{
	
	/**
	 * Constructor vacío del operador Igual.
	 */
	public Igual() {	}

	/**
	 *La igualdad tiene una prioridad de 1.
	 *@return La prioridad del igual: 1.
	 *@exception ArithmeticException Se lanza si hay error al evaluar en la expresión.
	 */
	public int prioridad() throws ArithmeticException{
		return 1;
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		Nodo a=expresionPrefijo.getOperando(1);//funcionSimplificarNumeros.simplificaNumeros(expresionPrefijo.getOperando(1));
		Nodo b=expresionPrefijo.getOperando(2);//funcionSimplificarNumeros.simplificaNumeros(expresionPrefijo.getOperando(2));
		
		try{
			if(a.equals("Indefinido") || b.getTipo().equals("Indefinido")  || a.getTipo().equals("Boolean") || b.getTipo().equals("Boolean")){
					return new Nodo("Indefinido");
			}
			if(a.esIgual(b)){
				return SimplificarNumeros.TRUE.clonarNodo();
			}else{
				return SimplificarNumeros.FALSE.clonarNodo();
			}
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar una división");
		}	
	}
	
	/**
	 *Función que simplifica la expresión recibida. Si esta función se evalúa aquí quiere decir que no encontró el operador
	 *donde se tenía que evaluar, lo que es un error.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		try{
			Nodo a=expresionPrefijo.getOperando(1), b=expresionPrefijo.getOperando(2);
			
			if(a.getTipo().equals("Indefinido") || b.getTipo().equals("Indefinido") || a.getTipo().equals("Boolean") || b.getTipo().equals("Boolean")){
					return new Nodo("Indefinido");
			}
			
			if(funcionSimplificar.esNumero(a) && funcionSimplificar.esNumero(b)){
				return simplificarNumeros(expresionPrefijo, funcionSimplificar, funcionSimplificarNumeros);
			}else if(a.esIgual(b)){
				return SimplificarNumeros.TRUE.clonarNodo();
			}else{
				return expresionPrefijo;
			}
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar una división");
		}		
	}

	/**
	 *Función que devuelve si el igual es binario.
	 *@return True si la igualdad es binaria, false en caso contrario.
	 *@exception ArithmeticException Se lanza si el operador no existe.
	 */
	public boolean esBinario() throws ArithmeticException{
		return true;
	}
	
	/**
	 *Función que deriva la expresión recibida.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		try{
			return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al derivar una división");
		}
	}

}
