package itcr.mathexplorer.operadores;

import itcr.mathexplorer.funciones.*;
import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.tipos.*;
import itcr.mathexplorer.herramientas.Opciones;
import java.math.BigDecimal;

/**
 *@(#)Mayor.java
 *
 *Esta es la implementación del operador mayor: >
 *
 *@author Alexander Borbón Alpízar
 */
public class Mayor extends Operador{
	
	/**
	 * Constructor vacío del operador Mayor que.
	 */
	public Mayor() {	}

	/**
	 *El mayor tiene una prioridad de 1.
	 *@return La prioridad del mayor: 1.
	 *@exception ArithmeticException Se lanza si hay error al evaluar en la expresión.
	 */
	public int prioridad() throws ArithmeticException{
		return 1;
	}
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		Nodo a=expresionPrefijo.getOperando(1);//funcionSimplificarNumeros.simplificaNumeros(expresionPrefijo.getOperando(1));
		Nodo b=expresionPrefijo.getOperando(2);//funcionSimplificarNumeros.simplificaNumeros(expresionPrefijo.getOperando(2));
		String tipoa=a.getTipo(), tipob=b.getTipo();
		BigDecimal numa, numb;
		
		try{
			if(a.equals("Indefinido") || a.getTipo().equals("Boolean") || b.getTipo().equals("Indefinido") || b.getTipo().equals("Boolean")){
					return new Nodo("Indefinido");
			}
			
			if(tipoa.length()<=5 || "EnteroLargo DecimalLargo Racional".indexOf(tipoa)==-1){ //Si no es número directo (es una operación de números)
				int numDecimales=Opciones.getNumDecimales();
				boolean presicion=Opciones.getPresicion();
				
				Opciones.setNumDecimales(Opciones.getNumDecimales());
				Opciones.setPresicion(false);
				
				a=funcionSimplificar.simplifica(a);
				numa=new BigDecimal(a.getOperando(1).getTipo());
				
				Opciones.setNumDecimales(numDecimales);
				Opciones.setPresicion(presicion);
			}else if(tipoa.equals("Racional")){
				String numerador, denominador;
				
				numerador=a.getOperando(1).getTipo();
				denominador=a.getOperando(2).getTipo();
				
				numa=(new BigRational(numerador, denominador)).toBigDecimal(Math.max(numerador.length(),denominador.length())+Opciones.getNumDecimales());
			}else{
				numa=new BigDecimal(a.getOperando(1).getTipo());
			}
			
			if(tipob.length()<=5 || "EnteroLargo DecimalLargo Racional".indexOf(tipob)==-1){ //Si no es número directo (es una operación de números)
				int numDecimales=Opciones.getNumDecimales();
				boolean presicion=Opciones.getPresicion();
				
				Opciones.setNumDecimales(Opciones.getNumDecimales());
				Opciones.setPresicion(false);
				
				b=funcionSimplificar.simplifica(b);
				numb=new BigDecimal(b.getOperando(1).getTipo());
				
				Opciones.setNumDecimales(numDecimales);
				Opciones.setPresicion(presicion);
			}else if(tipob.equals("Racional")){
				String numerador, denominador;
				
				numerador=b.getOperando(1).getTipo();
				denominador=b.getOperando(2).getTipo();
				
				numb=(new BigRational(numerador, denominador)).toBigDecimal(Math.max(numerador.length(),denominador.length())+Opciones.getNumDecimales());
			}else{
				numb=new BigDecimal(b.getOperando(1).getTipo());
			}
			
			if(numa.compareTo(numb)>0){
				return SimplificarNumeros.TRUE.clonarNodo();
			}else{
				return SimplificarNumeros.FALSE.clonarNodo();
			}
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar un mayor que");
		}	
	}
	
	/**
	 *Función que simplifica la expresión recibida. Si esta función se evalúa aquí quiere decir que no encontró el operador
	 *donde se tenía que evaluar, lo que es un error.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		try{
			Nodo a=expresionPrefijo.getOperando(1), b=expresionPrefijo.getOperando(2);
			
			if(a.getTipo().equals("Indefinido") || b.getTipo().equals("Indefinido") || a.getTipo().equals("Boolean") || b.getTipo().equals("Boolean")){
					return new Nodo("Indefinido");
			}
			
			if(funcionSimplificar.esNumero(a) && funcionSimplificar.esNumero(b)){
				return simplificarNumeros(expresionPrefijo, funcionSimplificar, funcionSimplificarNumeros);
			}else{
				return expresionPrefijo;
			}
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar un mayor que");
		}		
	}

	/**
	 *Función que devuelve si el igual es binario.
	 *@return True si la igualdad es binaria, false en caso contrario.
	 *@exception ArithmeticException Se lanza si el operador no existe.
	 */
	public boolean esBinario() throws ArithmeticException{
		return true;
	}
	
	/**
	 *Función que deriva la expresión recibida.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		try{
			return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al derivar un mayor que");
		}
	}

}
