package itcr.mathexplorer.operadores;

import itcr.mathexplorer.funciones.*;
import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.tipos.*;
import itcr.mathexplorer.herramientas.Opciones;
import java.math.BigInteger;
import java.math.BigDecimal;
import java.math.MathContext;

/**
 *@(#)Multiplicacion.java
 *
 *Esta es la implementación de la función multiplicacion: *
 *
 *@author Alexander Borbón Alpízar
 */
public class Multiplicacion extends Operador{
	
	/**
	 * Constructor vacío del operador de multiplicación.
	 */
	public Multiplicacion() {	}

	/**
	 *La multiplicacion tiene una prioridad de 3.
	 *@return La prioridad de la multiplicacion: 3.
	 *@exception ArithmeticException Se lanza si hay error al evaluar en la expresión.
	 */
	public int prioridad() throws ArithmeticException{
		return 3;
	}

	/**
	 *Función que simplifica la expresión recibida. Si esta función se evalúa aquí quiere decir que no encontró el operador
	 *donde se tenía que evaluar, lo que es un error.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		Nodo a=expresionPrefijo.getOperando(1);//funcionSimplificarNumeros.simplificaNumeros(expresionPrefijo.getOperando(1));
		Nodo b=expresionPrefijo.getOperando(2);//funcionSimplificarNumeros.simplificaNumeros(expresionPrefijo.getOperando(2));
		String tipoA=a.getTipo(), tipoB=b.getTipo();
		String solucion, numerador, denominador;
		Nodo comodin;
		
		try{
		
			if(a.getTipo().equals("Indefinido") || b.getTipo().equals("Indefinido") || a.getTipo().equals("Boolean") || b.getTipo().equals("Boolean")){
					return new Nodo("Indefinido");
			}else if("EnteroLargo".indexOf(tipoA)!=-1 && "EnteroLargo".indexOf(tipoB)!=-1){
				solucion=((new BigInteger(a.getOperando(1).getTipo())).multiply(new BigInteger(b.getOperando(1).getTipo()))).toString();
				if(solucion.length()<=18){
					comodin=new Nodo("Entero",1);
				}else{
					comodin=new Nodo("EnteroLargo",1);
				}
				comodin.agregaOperando(new Nodo(solucion));
				return comodin;
			}else if("DecimalLargo".indexOf(tipoA)!=-1 || "DecimalLargo".indexOf(tipoB)!=-1){
				BigDecimal Solucion;
				
				if(tipoA.equals("Racional")){
					numerador=a.getOperando(1).getTipo();
					denominador=a.getOperando(2).getTipo();
					
					Solucion=(new BigRational(numerador,denominador)).toBigDecimal(Math.max(numerador.length(),denominador.length())+Opciones.getNumDecimales()).multiply(new BigDecimal(b.getOperando(1).getTipo()));
				}else if(tipoB.equals("Racional")){
					numerador=b.getOperando(1).getTipo();
					denominador=b.getOperando(2).getTipo();
					
					Solucion=(new BigDecimal(a.getOperando(1).getTipo())).multiply((new BigRational(numerador,denominador)).toBigDecimal(Math.max(numerador.length(),denominador.length())+Opciones.getNumDecimales()));
				}else{
					Solucion=(new BigDecimal(a.getOperando(1).getTipo())).multiply(new BigDecimal(b.getOperando(1).getTipo()));
				}
				Solucion=Solucion.round(new MathContext(Opciones.getNumDecimales()+Solucion.toString().indexOf(".")+1));
				
				try{
					BigInteger SolucionInteger=Solucion.toBigIntegerExact();
					
					solucion=SolucionInteger.toString();
					
					if(solucion.length()<=18 && solucion.indexOf("E")==-1){
						if(Opciones.getPresicion()){
							comodin=new Nodo("Entero",1);
						}else{
							comodin=new Nodo("Decimal",1);
						}
					}else{
						if(Opciones.getPresicion()){
							comodin=new Nodo("EnteroLargo",1);
						}else{
							comodin=new Nodo("DecimalLargo",1);
						}
					}
					comodin.agregaOperando(new Nodo(solucion));
					return funcionSimplificarNumeros.simplificaNumeros(comodin, funcionSimplificar);
				}catch(ArithmeticException e){
					solucion=Solucion.toString();
					
					if(solucion.length()<=18 && solucion.indexOf("E")==-1){
						comodin=new Nodo("Decimal",1);
					}else{
						comodin=new Nodo("DecimalLargo",1);
					}
					comodin.agregaOperando(new Nodo(solucion));
					return funcionSimplificarNumeros.simplificaNumeros(comodin, funcionSimplificar);
				}
			}else{ //Alguno de los dos es racional
				BigRational Solucion;
				
				if("EnteroLargo".indexOf(tipoA)!=-1){
					Solucion=(new BigRational(a.getOperando(1).getTipo(),"1")).multiply(new BigRational(b.getOperando(1).getTipo(),b.getOperando(2).getTipo()));
				}else if("EnteroLargo".indexOf(tipoB)!=-1){
					Solucion=(new BigRational(a.getOperando(1).getTipo(),a.getOperando(2).getTipo())).multiply(new BigRational(b.getOperando(1).getTipo(),"1"));
				}else{
					Solucion=(new BigRational(a.getOperando(1).getTipo(),a.getOperando(2).getTipo())).multiply(new BigRational(b.getOperando(1).getTipo(),b.getOperando(2).getTipo()));
				}
				
				if(Solucion.getDenominator().toString().equals("1")){
					solucion=Solucion.getNumerator().toString();
					
					if(solucion.length()<=18 && solucion.indexOf("E")==-1){
						if(Opciones.getPresicion()){
							comodin=new Nodo("Entero",1);
						}else{
							comodin=new Nodo("Decimal",1);
						}
					}else{
						if(Opciones.getPresicion()){
							comodin=new Nodo("EnteroLargo",1);
						}else{
							comodin=new Nodo("DecimalLargo",1);
						}
					}
					comodin.agregaOperando(new Nodo(solucion));
				}else{
					comodin=new Nodo("Racional",2);
					comodin.agregaOperando(new Nodo(Solucion.getNumerator().toString()));
					comodin.agregaOperando(new Nodo(Solucion.getDenominator().toString()));
				}
				return comodin;
			}
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar una multiplicación");
		}	
	}


	/**
	 *Función que simplifica la expresión recibida. Si esta función se evalúa aquí quiere decir que no encontró el operador
	 *donde se tenía que evaluar, lo que es un error.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		boolean esCero=false;
		Nodo comodin;
		
		try{
			for(int i=1; i<=expresionPrefijo.getNumeroOperandos(); i++){
				comodin=expresionPrefijo.getOperando(i);
				if (comodin.getTipo().equals("Indefinido") || comodin.getTipo().equals("Boolean")){
					return new Nodo("Indefinido");
				}else if(comodin.esIgual(SimplificarNumeros.CERO)){
					esCero=true;
				}
			}
			
			if(esCero){
				return SimplificarNumeros.CERO.clonarNodo();
			}else if(expresionPrefijo.getNumeroOperandos()==1){
				return expresionPrefijo.getOperando(1);
			}else{
				comodin=simplificaMultiplicacionRecursiva(expresionPrefijo, funcionSimplificar, funcionSimplificarNumeros);
				
				if(comodin.getTipo().equals("Indefinido")){
					return new Nodo("Indefinido");
				}
				
				int num=comodin.getNumeroOperandos();
				if (num==0){
					return SimplificarNumeros.UNO.clonarNodo();
				}else if (num==1){
					return comodin.getOperando(1);
				}else{
					return comodin;
				}
			}
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar una multiplicación");
		}
	}
	
	/*
	 *Función que simplifica las multiplicaciones de manera recursiva.
	 */
	private Nodo simplificaMultiplicacionRecursiva(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		String tipo1, tipo2;
		Nodo comodin, comodin2, comodin3;
		
		try{
			if(expresionPrefijo.getNumeroOperandos()==2){
				Nodo operando1=expresionPrefijo.getOperando(1);
				Nodo operando2=expresionPrefijo.getOperando(2);
				tipo1=operando1.getTipo();
				tipo2=operando2.getTipo();
				
				if(!tipo1.equals("·") && !tipo2.equals("·")){
					if (("EnteroLargo DecimalLargo Racional").indexOf(tipo1)!=-1 && ("EnteroLargo DecimalLargo Racional").indexOf(tipo2)!=-1){
						comodin=simplificarNumeros(expresionPrefijo, funcionSimplificar, funcionSimplificarNumeros);
						if(comodin.esIgual(SimplificarNumeros.UNO)){
							return new Nodo("·");
						}else{
							comodin2=new Nodo("·",1);
							comodin2.agregaOperando(comodin);
							return comodin2;
						}
					}else if(operando1.esIgual(SimplificarNumeros.UNO)){
						comodin=new Nodo("·",1);
						comodin.agregaOperando(operando2);
						return comodin;
					}else if(operando2.esIgual(SimplificarNumeros.UNO)){
						comodin=new Nodo("·",1);
						comodin.agregaOperando(operando1);
						return comodin;
					/*}else if(("EnteroLargo DecimalLargo Racional").indexOf(tipo1)!=-1 && tipo2.equals("Constante")){
						return expresionPrefijo;
					}else if(tipo1.equals("Constante") && ("EnteroLargo DecimalLargo Racional").indexOf(tipo2)!=-1){
						comodin=new Nodo("·",2);
						comodin.agregaOperando(operando2);
						comodin.agregaOperando(operando1);
						return comodin;
					}else if(tipo1.equals("Constante") && tipo2.equals("Constante")){
						comodin=new Nodo("Variable", 1);
						comodin.agregaOperando(operando1.getOperando(1));
						comodin2=new Nodo("Variable", 1);
						comodin2.agregaOperando(operando2.getOperando(1));
						if(base(comodin).esIgual(base(comodin2))){
							comodin=new Nodo("+");
							comodin.agregaOperando(exponente(operando1));
							comodin.agregaOperando(exponente(operando2));
							comodin=funcionSimplificar.simplifica(comodin);
							comodin2=new Nodo("^",2);
							comodin2.agregaOperando(base(operando1));
							comodin2.agregaOperando(comodin);
							comodin2=funcionSimplificar.simplifica(comodin2);
							if(comodin2.esIgual(uno)){
								return new Nodo("·");
							}else{
								comodin=new Nodo("·");
								comodin.agregaOperando(comodin2);
								return comodin;
							}	
						}else if(funcionSimplificar.comparar(operando2,operando1)){
							comodin=new Nodo("·",2);
							comodin.agregaOperando(operando2);
							comodin.agregaOperando(operando1);
							return comodin;
						}*/
					}else if(tipo1.equals("Matriz") || tipo2.equals("Matriz")){
						//Si ambas no son matrices o si las matrices no tienen el mismo tamaño
						if(!tipo1.equals(tipo2)){
							if(!tipo2.equals("Matriz")){
								comodin=operando1;
								operando1=operando2;
								operando2=comodin;
							}
							
							comodin=new Nodo("Matriz");
							
							for(int i=1; i<=operando2.getNumeroOperandos(); i++){//Pasa por las filas del primero
								comodin.agregaOperando(new Nodo("Fila"));
								for(int j=1; j<=operando2.getOperando(1).getNumeroOperandos(); j++){//Pasa por las columnas del segundo
									comodin2=new Nodo("·");
									comodin2.agregaOperando(operando1);
									comodin2.agregaOperando(operando2.getOperando(i).getOperando(j));
									comodin2=funcionSimplificar.simplifica(comodin2);
									comodin.getOperando(i).agregaOperando(comodin2);
								}
							}
							
							comodin2=new Nodo("·",1);
							comodin2.agregaOperando(comodin);
							
							return comodin2;
						}else if(operando1.getOperando(1).getNumeroOperandos()!=operando2.getNumeroOperandos()){
							return new Nodo("Indefinido");
						}
						
						comodin=new Nodo("Matriz");
						
						for(int i=1; i<=operando1.getNumeroOperandos(); i++){//Pasa por las filas del primero
							comodin.agregaOperando(new Nodo("Fila"));
							for(int j=1; j<=operando2.getOperando(1).getNumeroOperandos(); j++){//Pasa por las columnas del segundo
								comodin2=new Nodo("+",2);
								for(int k=1;k<=operando1.getOperando(1).getNumeroOperandos(); k++){
									comodin3=new Nodo("·");
									comodin3.agregaOperando(operando1.getOperando(i).getOperando(k));
									comodin3.agregaOperando(operando2.getOperando(k).getOperando(j));
									comodin2.agregaOperando(comodin3);
								}
								comodin2=funcionSimplificar.simplifica(comodin2);
								comodin.getOperando(i).agregaOperando(comodin2);
							}
						}
						
						comodin2=new Nodo("·",1);
						comodin2.agregaOperando(comodin);
						
						return comodin2;
					}else if(tipo1.equals("Lista") || tipo2.equals("Lista")){
						//Si ambas no son listas
						if(!tipo1.equals(tipo2)){
							return new Nodo("Indefinido");
						}
						
						comodin=new Nodo("Lista");
						
						for(int i=1; i<=operando1.getNumeroOperandos(); i++){
							for(int j=1; j<=operando2.getNumeroOperandos(); j++){
								comodin2=new Nodo("Matriz");
								comodin3=new Nodo("Fila");
								comodin3.agregaOperando(operando1.getOperando(i));
								comodin3.agregaOperando(operando2.getOperando(j));
								comodin2.agregaOperando(comodin3);
								comodin.agregaOperando(comodin2);
							}
						}
						
						comodin2=new Nodo("·",1);
						comodin2.agregaOperando(comodin);
						
						return comodin2;
					}else if(base(operando1).esIgual(base(operando2))){
						comodin=new Nodo("+");
						comodin.agregaOperando(exponente(operando1));
						comodin.agregaOperando(exponente(operando2));
						comodin=funcionSimplificar.simplifica(comodin);
						comodin2=new Nodo("^",2);
						comodin2.agregaOperando(base(operando1));
						comodin2.agregaOperando(comodin);
						comodin2=funcionSimplificar.simplifica(comodin2);
						if(comodin2.esIgual(SimplificarNumeros.UNO)){
							return new Nodo("·");
						}else{
							comodin=new Nodo("·");
							comodin.agregaOperando(comodin2);
							return comodin;
						}
					}else if(funcionSimplificar.comparar(operando2,operando1)){
						comodin=new Nodo("·",2);
						comodin.agregaOperando(operando2);
						comodin.agregaOperando(operando1);
						return comodin;
					}else{
						return expresionPrefijo;
					}
				}else{
					return UnirMultiplicaciones(operando1,operando2, funcionSimplificar, funcionSimplificarNumeros);
				}
			}else{
				comodin=expresionPrefijo.clonarNodo();
				comodin.eliminaOperando(1);
				comodin=simplificaMultiplicacionRecursiva(comodin, funcionSimplificar, funcionSimplificarNumeros);
				return UnirMultiplicaciones(expresionPrefijo.getOperando(1),comodin, funcionSimplificar, funcionSimplificarNumeros);
			}
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar una multiplicación");
		}
	}
	
	/*
	 *Función que toma dos expresiones donde alguna de ellas es un producto y devuelve el unión de ambos como producto.
	 */
	private Nodo UnirMultiplicaciones(Nodo operando1, Nodo operando2, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		String tipo1=operando1.getTipo(), tipo2=operando2.getTipo();
		Nodo retornar, comodin;

		try{
		
			if(!tipo1.equals("·")){
				comodin=new Nodo("·",1);
				comodin.agregaOperando(operando1);
				operando1=comodin;
			}else if(!tipo2.equals("·")){
				comodin=new Nodo("·",1);
				comodin.agregaOperando(operando2);
				operando2=comodin;
			}
			
			retornar=new Nodo("·");
			int i=1;
			int j=1;
			while(i<=operando1.getNumeroOperandos() && j<=operando2.getNumeroOperandos()){
				comodin=new Nodo("·",1);
				comodin.agregaOperando(operando1.getOperando(i));
				comodin.agregaOperando(operando2.getOperando(j));
				comodin=simplificaMultiplicacionRecursiva(comodin, funcionSimplificar, funcionSimplificarNumeros);
				
				if(comodin.getNumeroOperandos()==0){
					i++;
					j++;
				}else if(comodin.getNumeroOperandos()==1){
					retornar.agregaOperando(comodin.getOperando(1));
					i++;
					j++;
				}else if(comodin.getOperando(1).esIgual(operando1.getOperando(i))){
					retornar.agregaOperando(comodin.getOperando(1));
					i++;
				}else{
					retornar.agregaOperando(comodin.getOperando(1));
					j++;
				}
			}
			
			if(i<=operando1.getNumeroOperandos()){
				for(; i<=operando1.getNumeroOperandos(); i++){
					retornar.agregaOperando(operando1.getOperando(i));
				}
			}else if(j<=operando2.getNumeroOperandos()){
				for(; j<=operando2.getNumeroOperandos(); j++){
					retornar.agregaOperando(operando2.getOperando(j));
				}
			}
			
			return retornar;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar una multiplicación");
		}
	}
	
	/*
	 *Devuelve la base de la expresión vista como una potencia.
	 */
	private Nodo base(Nodo expresionPrefijo){
		String tipo;
		
		tipo=expresionPrefijo.getTipo();
		//if(("EnteroLargo DecimalLargo Constante Racional").indexOf(tipo)!=-1){ //Si es un número
		if(("EnteroLargo DecimalLargo Racional").indexOf(tipo)!=-1){ //Si es un número
			return new Nodo("Indefinido");
		}else if (tipo.equals("^")){
			return expresionPrefijo.getOperando(1);
		}else{
			return expresionPrefijo;
		}
	}
	
	/*
	 *Devuelve el exponente de la expresión vista como una potencia.
	 */
	private Nodo exponente(Nodo expresionPrefijo){
		String tipo;
		Nodo comodin;
		
		tipo=expresionPrefijo.getTipo();
		//if(("EnteroLargo DecimalLargo Constante Racional").indexOf(tipo)!=-1){ //Si es un número
		if(("EnteroLargo DecimalLargo Racional").indexOf(tipo)!=-1){ //Si es un número
			return new Nodo("Indefinido");
		}else if (tipo.equals("^")){
			return expresionPrefijo.getOperando(2);
		}else{
			comodin=new Nodo("Entero");
			comodin.agregaOperando(new Nodo("1"));
			return comodin;
		}
	}


	/**
	 *Función que devuelve si la multiplicación es binaria.
	 *@return True si la multiplicación es binaria, false en caso contrario.
	 *@exception ArithmeticException Se lanza si el operador no existe.
	 */
	public boolean esBinario() throws ArithmeticException{
		return true;
	}
	
	/**
	 *Función que deriva la expresión recibida.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		Nodo comodin, comodin2, comodin3;
		
		try{
			comodin=new Nodo("·",2);
			
			comodin.agregaOperando(funcionDerivar.deriva(expresionPrefijo.getOperando(1).clonarNodo(), variable));
			if(expresionPrefijo.getNumeroOperandos()==2){
				comodin.agregaOperando(expresionPrefijo.getOperando(2).clonarNodo());
			}else{
				comodin.agregaOperando(expresionPrefijo.clonarNodo().eliminaOperando(1));
			}
			
			comodin2=new Nodo("·",2);
			
			comodin2.agregaOperando(expresionPrefijo.getOperando(1).clonarNodo());
			if(expresionPrefijo.getNumeroOperandos()==2){
				comodin2.agregaOperando(funcionDerivar.deriva(expresionPrefijo.getOperando(2).clonarNodo(), variable));
			}else{
				comodin2.agregaOperando(funcionDerivar.deriva(expresionPrefijo.clonarNodo().eliminaOperando(1), variable));
			}
			comodin3=new Nodo("+",2);
			
			comodin3.agregaOperando(comodin);
			comodin3.agregaOperando(comodin2);
			
			return comodin3;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al derivar una multiplicación");
		}
	}

}
