/**
 *@(#)Operador.java
 *
 *Esta es una interface que define el cazcarón que debe tener cualquier operador
 *utilizada en el programa.
 *
 *@author Alexander Borbón Alpízar
 */

package itcr.mathexplorer.operadores;

//import java.util.*;
import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.funciones.*;
//import itcr.mathexplorer.tipos.*;
//import java.math.BigInteger;
//import java.math.BigDecimal;

/**
 *Esta es una interface que define el cazcarón que debe tener cualquier operador
 *utilizada en el programa para que todas posean el método evaluar.
 */
public class Operador implements Operadores{

	
	//CONSTRUCTORES
	
	/**
	 *Constructor vacío.
	 */
	public Operador(){
		
	}

	/**
	 *Función que devuelve la prioridad que tiene el operador dado. Si esta función se evalúa aquí quiere decir que no encontró el operador
	 *donde se tenía que evaluar, lo que es un error.
	 *@return La prioridad del operador.
	 *@exception ArithmeticException Se lanza si hay error al evaluar en la expresión.
	 */
	public int prioridad() throws ArithmeticException{
		throw new ArithmeticException("Error al devolver la prioridad del operador");
	}
	
	/**
	 *Función que simplifica la expresión recibida. Si esta función se evalúa aquí quiere decir que no encontró el operador
	 *donde se tenía que evaluar, lo que es un error.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		throw new ArithmeticException("Error al simplificar el operador");
	}
	
	/**
	 *Función que simplifica la expresión recibida. Si esta función se evalúa aquí quiere decir que no encontró el operador
	 *donde se tenía que evaluar, lo que es un error.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		throw new ArithmeticException("Error al simplificar el operador");
	}
	
	/**
	 *Función que devuelve si el operador es binario. Si esta función se evalúa aquí es porque no se encontró la función requerida.
	 *@return True si el operador es binario, false en caso contrario.
	 *@exception ArithmeticException Se lanza si el operador no existe.
	 */
	public boolean esBinario() throws ArithmeticException{
		throw new ArithmeticException("El operador buscado no se encontró");
	}
	
	/**
	 *Función que deriva la expresión recibida. Si esta función se evalúa aquí quiere decir que no encontró el operador
	 *donde se tenía que evaluar, lo que es un error.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		throw new ArithmeticException("Error al derivar un operador");
	}
	
}//Fin de SintaxException