package itcr.mathexplorer.operadores;

import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.funciones.*;


/**
 *@(#)Operadores.java
 *
 *Esta es una interface que define el cazcarón que debe tener cualquier operador
 *utilizada en el programa.
 *
 *Por operadores se entienden en este programa los operadores binarios: +, -, *, /, ^, %.
 *
 *@author Alexander Borbón Alpízar
 */
public interface Operadores{

	/**
	 *Función que devuelve la prioridad que tiene el operador dado.
	 *@return La prioridad del operador.
	 *@exception ArithmeticException Se lanza si hay error al buscar una prioridad.
	 */
	public int prioridad() throws ArithmeticException;
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException;
	
	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException;

	/**
	 *Función que devuelve si el operador es binario (en caso contrario se diría que es unaria).
	 *El caso del - unario y binario se maneja por separado dentro del código, esta función está pensada
	 *para manejar operadores como el factorial.
	 *@return True si el operador es binario, false en caso contrario.
	 *@exception ArithmeticException Se lanza si el operador no existe.
	 */
	public boolean esBinario() throws ArithmeticException;
	
	/**
	 *Función que deriva la expresión recibida.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException;

	
}//Fin de SintaxException