package itcr.mathexplorer.operadores;

import itcr.mathexplorer.funciones.*;
import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.herramientas.Opciones;

/**
 *@(#)Porcentaje.java
 *
 *Esta es la implementación de la función porcentaje: /
 *
 *@author Alexander Borbón Alpízar
 */
public class Porcentaje extends Operador{
	
	/**
	 * Constructor vacío del operador Porcentaje.
	 */
	public Porcentaje() {	}

	/**
	 *El porcentaje tiene una prioridad de 3.
	 *@return La prioridad del porcentaje: 3.
	 *@exception ArithmeticException Se lanza si hay error al evaluar en la expresión.
	 */
	public int prioridad() throws ArithmeticException{
		return 3;
	}
	
	/**
	 *Función que simplifica la expresión recibida. 
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		try{
			if(expresionPrefijo.getOperando(1).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(1).getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
			}
			
			return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar un porcentaje");
		}	
	}

	/**
	 *Función que simplifica la expresión recibida. Si esta función se evalúa aquí quiere decir que no encontró el operador
	 *donde se tenía que evaluar, lo que es un error.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		Nodo comodin, comodin2;
		
		try{
			if(expresionPrefijo.getOperando(1).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(1).getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
			}
			
			comodin=new Nodo("/");
			
			if (Opciones.getPresicion()){
				comodin2=new Nodo("Entero",1);
			}else{
				comodin2=new Nodo("Decimal",1);
			}
			comodin2.agregaOperando(new Nodo("1"));
			
			comodin.agregaOperando(comodin2);
			
			if (Opciones.getPresicion()){
				comodin2=new Nodo("Entero",1);
			}else{
				comodin2=new Nodo("Decimal",1);
			}
			comodin2.agregaOperando(new Nodo("100"));
			
			comodin.agregaOperando(comodin2);
			
			comodin2=new Nodo("·",2);
			comodin2.agregaOperando(comodin);
			comodin2.agregaOperando(expresionPrefijo.getOperando(1).clonarNodo());
			
			return funcionSimplificar.simplifica(comodin2);
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar un porcentaje");
		}
	}

	/**
	 *Función que devuelve si el módulo es binario.
	 *@return True si el módulo es binario, false en caso contrario.
	 *@exception ArithmeticException Se lanza si el operador no existe.
	 */
	public boolean esBinario() throws ArithmeticException{
		return false;
	}
	
	/**
	 *Función que deriva la expresión recibida.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		try{
			return expresionPrefijo;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al derivar un porcentaje");
		}
	}

}