package itcr.mathexplorer.operadores;

import itcr.mathexplorer.funciones.*;
import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.tipos.*;
import itcr.mathexplorer.herramientas.Opciones;
import java.math.BigInteger;
import java.math.BigDecimal;
import java.math.MathContext;

/**
 *@(#)Potencia.java
 *
 *Esta es la implementación de la función potencia: ^
 *
 *@author Alexander Borbón Alpízar
 */
public class Potencia extends Operador{
	
	/**
	 * Constructor vacío del operador de Potencia.
	 */
	public Potencia() {	}

	/**
	 *La potencia tiene una prioridad de 4.
	 *@return La prioridad de la potencia: 4.
	 *@exception ArithmeticException Se lanza si hay error al evaluar en la expresión.
	 */
	public int prioridad() throws ArithmeticException{
		return 4;
	}

	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		Nodo a=expresionPrefijo.getOperando(1);//funcionSimplificarNumeros.simplificaNumeros(expresionPrefijo.getOperando(1));
		Nodo b=expresionPrefijo.getOperando(2);//funcionSimplificarNumeros.simplificaNumeros(expresionPrefijo.getOperando(2));
		String tipoA=a.getTipo(), tipoB=b.getTipo();
		String solucion;
		int potencia;
		Nodo comodin;
		
		try{
			if(((Opciones.getPresicion() && tipoB.equals("Entero")) || (!Opciones.getPresicion() && tipoB.equals("Decimal") && b.getOperando(1).getTipo().indexOf(".")==-1)) && b.getOperando(1).getTipo().length()<10){ //¿Cuánto tiene un entero?
				if("EnteroLargo".indexOf(tipoA)!=-1){
					potencia=(new Integer(b.getOperando(1).getTipo())).intValue();
					if(potencia>0){
						solucion=((new BigInteger(a.getOperando(1).getTipo())).pow(potencia)).toString();
						if(solucion.length()<=18){
							comodin=new Nodo("Entero",1);
						}else{
							comodin=new Nodo("EnteroLargo",1);
						}
						comodin.agregaOperando(new Nodo(solucion));
					}else if(potencia==0){
						if(a.esIgual(SimplificarNumeros.CERO)){
							return new Nodo("Indefinido");
						}else{
							return SimplificarNumeros.UNO.clonarNodo();
						}
					}else{
						if(a.esIgual(SimplificarNumeros.CERO)){
							return new Nodo("Indefinido");
						}else if(a.esIgual(SimplificarNumeros.UNO)){
							return SimplificarNumeros.UNO.clonarNodo();
						}else if(a.esIgual(SimplificarNumeros.MENOSUNO)){
							return SimplificarNumeros.MENOSUNO.clonarNodo();
						}else{
							try{
								BigRational solucionRacional=new BigRational(new BigInteger("1"),(new BigInteger(a.getOperando(1).getTipo())).pow(-1*potencia));
								comodin=new Nodo("Racional");
								comodin.agregaOperando(new Nodo(solucionRacional.getNumerator().toString()));
								comodin.agregaOperando(new Nodo(solucionRacional.getDenominator().toString()));
							}catch(Exception e){
								return new Nodo("Indefinido");
							}
						}
					}
					return comodin;
				}else if("DecimalLargo".indexOf(tipoA)!=-1){
					BigDecimal Solucion;
					
					potencia=(new Integer(b.getOperando(1).getTipo())).intValue();
					if(potencia>=0){
						Solucion=(new BigDecimal(a.getOperando(1).getTipo())).pow(potencia);
					}else{
						try{
							Solucion=(new BigDecimal(a.getOperando(1).getTipo())).pow(-1*potencia);
							Solucion=BigDecimal.ONE.divide(Solucion, new MathContext(Opciones.getNumDecimales()+Solucion.toString().indexOf(".")+1));
						}catch(Exception e){
								return new Nodo("Indefinido");
						}
					}
					
					try{
						BigInteger SolucionInteger=Solucion.toBigIntegerExact();
						
						solucion=SolucionInteger.toString();
						
						if(solucion.length()<=18){
							comodin=new Nodo("Entero",1);
							comodin.agregaOperando(new Nodo(solucion));
						}else{
							comodin=new Nodo("EnteroLargo",1);
							comodin.agregaOperando(new Nodo(solucion));
						}
						return comodin;
					}catch(ArithmeticException e){
						solucion=Solucion.toString();
						
						if(solucion.length()<=18 && solucion.indexOf("E")==-1){
							comodin=new Nodo("Decimal",1);
							comodin.agregaOperando(new Nodo(solucion));
						}else{
							comodin=new Nodo("DecimalLargo",1);
							comodin.agregaOperando(new Nodo(solucion));
						}
						return comodin;
					}
				}else{ //La base es racional
					BigRational Solucion=(new BigRational(a.getOperando(1).getTipo(),a.getOperando(2).getTipo())).pow((new Integer(b.getOperando(1).getTipo())).intValue());
					if(Solucion.getDenominator().toString().equals("1")){
						solucion=Solucion.getNumerator().toString();
						
						if(solucion.length()<=18){
							comodin=new Nodo("Entero",1);
						}else{
							comodin=new Nodo("EnteroLargo",1);
						}
						comodin.agregaOperando(new Nodo(solucion));
					}else{
						comodin=new Nodo("Racional",2);
						comodin.agregaOperando(new Nodo(Solucion.getNumerator().toString()));
						comodin.agregaOperando(new Nodo(Solucion.getDenominator().toString()));
					}
					return comodin;
				}
			}else{
				return new Nodo("Indefinido");
			}
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar una división");
		}	
	}


	/**
	 *Función que simplifica la expresión recibida. Si esta función se evalúa aquí quiere decir que no encontró el operador
	 *donde se tenía que evaluar, lo que es un error.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		Nodo base=expresionPrefijo.getOperando(1);
		Nodo exponente=expresionPrefijo.getOperando(2);
		String tipoExponente=exponente.getTipo();
		
		try{
			if(base.getTipo().equals("Indefinido") || base.getTipo().equals("Boolean") || exponente.getTipo().equals("Indefinido") || exponente.getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
/*			}else if(base.esIgual(funcionSimplificarNumeros.CERO)){
				if("EnteroLargo Racional".indexOf(tipoExponente)!=-1 && ){
					return cero.clonarNodo();
				}else{
					return new Nodo("Indefinido");
				}*/
			}else if(base.esIgual(SimplificarNumeros.UNO)){
				return SimplificarNumeros.UNO.clonarNodo();
			}else if(tipoExponente.length()>5 && ("EnteroLargo".indexOf(tipoExponente)!=-1 || ("DecimalLargo".indexOf(tipoExponente)!=-1 && exponente.getOperando(1).getTipo().indexOf(".")==-1))){
				return simplificaPotenciaEntera(base, exponente, funcionSimplificar, funcionSimplificarNumeros);
			}else{
				return expresionPrefijo;
			}
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar una potencia");
		}
	}
	
	/*
	 *Función que simplifica las potencias con exponente entero.
	 */
	private Nodo simplificaPotenciaEntera(Nodo base, Nodo exponente, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		String tipo=base.getTipo();
		Nodo comodin;
		
		try{
			if (tipo.length()>5 && ("EnteroLargo DecimalLargo Racional").indexOf(tipo)!=-1){
				comodin=new Nodo("^",2);
				comodin.agregaOperando(base);
				comodin.agregaOperando(exponente);
				return simplificarNumeros(comodin, funcionSimplificar, funcionSimplificarNumeros);
			}else if(exponente.esIgual(SimplificarNumeros.CERO)){
				return SimplificarNumeros.UNO.clonarNodo();
			}else if(exponente.esIgual(SimplificarNumeros.UNO)){
				return base;
			}else if(tipo.equals("^")){
				Nodo r=base.getOperando(1);
				Nodo s=base.getOperando(2);
				comodin=new Nodo("·",2);
				comodin.agregaOperando(s);
				comodin.agregaOperando(exponente);
				Nodo p=funcionSimplificar.simplifica(comodin);
				if (("EnteroLargo").indexOf(p.getTipo())!=-1){
					return simplificaPotenciaEntera(r, p, funcionSimplificar, funcionSimplificarNumeros);
				}else{
					comodin = new Nodo("^", 2);
					comodin.agregaOperando(r);
					comodin.agregaOperando(p);
					return comodin;
				}
			}else if(tipo.equals("·")){
				comodin = new Nodo("·", base.getNumeroOperandos());
				for(int i=1; i<=base.getNumeroOperandos(); i++){
					comodin.agregaOperando(simplificaPotenciaEntera(base.getOperando(i),exponente, funcionSimplificar, funcionSimplificarNumeros));
				}
				return funcionSimplificar.simplifica(comodin);
			}else{
				comodin=new Nodo("^",2);
				comodin.agregaOperando(base);
				comodin.agregaOperando(exponente);
				
				return comodin;
			}
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar una potencia");
		}
	}

	/**
	 *Función que devuelve si la potencia es binaria.
	 *@return True si la potencia es binario, false en caso contrario.
	 *@exception ArithmeticException Se lanza si el operador no existe.
	 */
	public boolean esBinario() throws ArithmeticException{
		return true;
	}
	
	/**
	 *Función que deriva la expresión recibida.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		Nodo comodin, comodin2;
		
		try{
			if(!funcionDerivar.buscaVariable(expresionPrefijo.getOperando(2), variable)){
				if (Opciones.getPresicion()){
					comodin2=new Nodo("Entero",1);
				}else{
					comodin2=new Nodo("Decimal",1);
				}
				comodin2.agregaOperando(new Nodo("1"));
				
				comodin=new Nodo("-",2);
				comodin.agregaOperando(expresionPrefijo.getOperando(2).clonarNodo());
				comodin.agregaOperando(comodin2);
				
				comodin2=new Nodo("^",2);
				comodin2.agregaOperando(expresionPrefijo.getOperando(1));
				comodin2.agregaOperando(comodin);
				
				comodin=new Nodo("·",3);
				comodin.agregaOperando(expresionPrefijo.getOperando(2).clonarNodo());
				comodin.agregaOperando(comodin2);
				comodin.agregaOperando(funcionDerivar.deriva(expresionPrefijo.getOperando(1).clonarNodo(),variable));
				
				return comodin;
			}else if(!funcionDerivar.buscaVariable(expresionPrefijo.getOperando(1), variable)){
				comodin=new Nodo("ln",1);
				comodin.agregaOperando(expresionPrefijo.getOperando(1).clonarNodo());
				
				comodin2=new Nodo("·",3);
				comodin2.agregaOperando(expresionPrefijo.clonarNodo());
				comodin2.agregaOperando(comodin);
				comodin2.agregaOperando(funcionDerivar.deriva(expresionPrefijo.getOperando(2).clonarNodo(), variable));
				
				return comodin2;
			}else{
				comodin=new Nodo("ln",1);
				comodin.agregaOperando(expresionPrefijo.getOperando(1).clonarNodo());
				
				comodin2=new Nodo("·",2);
				comodin2.agregaOperando(expresionPrefijo.getOperando(2).clonarNodo());
				comodin2.agregaOperando(comodin);
				
				comodin=funcionDerivar.deriva(comodin2, variable);
				
				comodin2=new Nodo("·",2);
				comodin2.agregaOperando(expresionPrefijo.clonarNodo());
				comodin2.agregaOperando(comodin);
				
				return comodin2;
			}
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al derivar una potencia");
		}
	}

}
