package itcr.mathexplorer.operadores;

import itcr.mathexplorer.funciones.*;
import itcr.mathexplorer.expresion.*;

/**
 *@(#)Resta.java
 *
 *Esta es la implementación de la función resta: -
 *
 *@author Alexander Borbón Alpízar
 */
public class Resta extends Operador{
	
	/**
	 * Constructor vacío del operador de Resta.
	 */
	public Resta() {	}

	/**
	 *La resta tiene una prioridad de 2.
	 *@return La prioridad de la resta: 2.
	 *@exception ArithmeticException Se lanza si hay error al evaluar en la expresión.
	 */
	public int prioridad() throws ArithmeticException{
		return 2;
	}

	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		Nodo a=expresionPrefijo.getOperando(1);//funcionSimplificarNumeros.simplificaNumeros(expresionPrefijo.getOperando(1));
		Nodo b=expresionPrefijo.getOperando(2);//funcionSimplificarNumeros.simplificaNumeros(expresionPrefijo.getOperando(2));
		Nodo comodin, comodin2;
		
		try{
			if(a.getTipo().equals("Indefinido") || b.getTipo().equals("Indefinido") || a.getTipo().equals("Boolean") || b.getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
			}else{
				comodin=new Nodo("Entero");
				comodin.agregaOperando(new Nodo("-1"));
				comodin2=new Nodo("·");
				comodin2.agregaOperando(comodin);
				comodin2.agregaOperando(b);
				comodin2=funcionSimplificarNumeros.simplificaNumeros(comodin2, funcionSimplificar);
				comodin=new Nodo("+");
				comodin.agregaOperando(a);
				comodin.agregaOperando(comodin2);
				
				return funcionSimplificarNumeros.simplificaNumeros(comodin, funcionSimplificar);
			}
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar una división");
		}	
	}


	/**
	 *Función que simplifica la expresión recibida. Si esta función se evalúa aquí quiere decir que no encontró el operador
	 *donde se tenía que evaluar, lo que es un error.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		Nodo comodin, comodin2;
		
		try{
			if(expresionPrefijo.getOperando(1).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(2).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(1).getTipo().equals("Boolean") || expresionPrefijo.getOperando(2).getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
			}else{
				comodin=new Nodo("·");
				comodin2=new Nodo("Entero");
				comodin2.agregaOperando(new Nodo("-1"));
				comodin.agregaOperando(comodin2);
				comodin.agregaOperando(expresionPrefijo.getOperando(2));
				comodin=funcionSimplificar.simplifica(comodin);
				comodin2=new Nodo("+");
				comodin2.agregaOperando(expresionPrefijo.getOperando(1));
				comodin2.agregaOperando(comodin);
				comodin2=funcionSimplificar.simplifica(comodin2);
			
				return comodin2;
			}
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar una resta");
		}
	}


	/**
	 *Función que devuelve si la resta es binaria.
	 *@return True si la resta es binaria, false en caso contrario.
	 *@exception ArithmeticException Se lanza si el operador no existe.
	 */
	public boolean esBinario() throws ArithmeticException{
		return true;
	}
	
	/**
	 *Función que deriva la expresión recibida.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		Nodo comodin;
		
		try{
			comodin=new Nodo("-",expresionPrefijo.getNumeroOperandos());
			
			for(int i=1; i<=expresionPrefijo.getNumeroOperandos(); i++){
				comodin.agregaOperando(funcionDerivar.deriva(expresionPrefijo.getOperando(i).clonarNodo(), variable));
			}

			return comodin;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al derivar una resta");
		}
	}
}
