package itcr.mathexplorer.operadores;

import itcr.mathexplorer.funciones.*;
import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.tipos.*;
import itcr.mathexplorer.herramientas.Opciones;
import java.math.BigInteger;
import java.math.BigDecimal;

/**
 *@(#)Suma.java
 *
 *Esta es la implementación de la función suma: +
 *
 *@author Alexander Borbón Alpízar
 */
public class Suma extends Operador{
	
	/**
	 * Constructor vacío del operador de Suma.
	 */
	public Suma() {	}

	/**
	 *La suma tiene una prioridad de 2.
	 *@return La prioridad de la suma: 2.
	 *@exception ArithmeticException Se lanza si hay error al evaluar en la expresión.
	 */
	public int prioridad() throws ArithmeticException{
		return 2;
	}

	/**
	 *Función que simplifica la expresión recibida.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return El resultado de la operación con números.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificarNumeros(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		Nodo a=expresionPrefijo.getOperando(1);//funcionSimplificarNumeros.simplificaNumeros(expresionPrefijo.getOperando(1));
		Nodo b=expresionPrefijo.getOperando(2);//funcionSimplificarNumeros.simplificaNumeros(expresionPrefijo.getOperando(2));
		String tipoA=a.getTipo(), tipoB=b.getTipo();
		String solucion, numerador, denominador;
		Nodo comodin;
		
		try{
			if(a.getTipo().equals("Indefinido") || b.getTipo().equals("Indefinido") || a.getTipo().equals("Boolean") || b.getTipo().equals("Boolean")){
				return new Nodo("Indefinido");
			}else if("EnteroLargo".indexOf(tipoA)!=-1 && "EnteroLargo".indexOf(tipoB)!=-1){
				solucion=((new BigInteger(a.getOperando(1).getTipo())).add(new BigInteger(b.getOperando(1).getTipo()))).toString();
				if(solucion.length()<=18){
					comodin=new Nodo("Entero",1);
				}else{
					comodin=new Nodo("EnteroLargo",1);
				}
				comodin.agregaOperando(new Nodo(solucion));
				return comodin;
			}else if("DecimalLargo".indexOf(tipoA)!=-1 || "DecimalLargo".indexOf(tipoB)!=-1){
				BigDecimal Solucion;
				
				if(tipoA.equals("Racional")){
					numerador=a.getOperando(1).getTipo();
					denominador=a.getOperando(2).getTipo();
					
					Solucion=(new BigRational(numerador,denominador)).toBigDecimal(Math.max(numerador.length(),denominador.length())+Opciones.getNumDecimales()).add(new BigDecimal(b.getOperando(1).getTipo()));
				}else if(tipoB.equals("Racional")){
					numerador=b.getOperando(1).getTipo();
					denominador=b.getOperando(2).getTipo();
					
					Solucion=(new BigDecimal(a.getOperando(1).getTipo())).add((new BigRational(numerador,denominador)).toBigDecimal(Math.max(numerador.length(),denominador.length())+Opciones.getNumDecimales()));
				}else{
					Solucion=(new BigDecimal(a.getOperando(1).getTipo())).add(new BigDecimal(b.getOperando(1).getTipo()));
				}
				
				try{
					BigInteger SolucionInteger=Solucion.toBigIntegerExact();
					
					solucion=SolucionInteger.toString();
					
					if(solucion.length()<=18 && solucion.indexOf("E")==-1){
						if(Opciones.getPresicion()){
							comodin=new Nodo("Entero",1);
						}else{
							comodin=new Nodo("Decimal",1);
						}
					}else{
						if(Opciones.getPresicion()){
							comodin=new Nodo("EnteroLargo",1);
						}else{
							comodin=new Nodo("DecimalLargo",1);
						}
					}
					comodin.agregaOperando(new Nodo(solucion));
					return comodin;
				}catch(ArithmeticException e){
					solucion=Solucion.toString();
					
					if(solucion.length()<=18 && solucion.indexOf("E")==-1){
						comodin=new Nodo("Decimal",1);
					}else{
						comodin=new Nodo("DecimalLargo",1);
					}
					comodin.agregaOperando(new Nodo(solucion));
					return comodin;
				}
			}else{ //Alguno de los dos es racional
				BigRational Solucion;
				
				if("EnteroLargo".indexOf(tipoA)!=-1){
					Solucion=(new BigRational(a.getOperando(1).getTipo(),"1")).add(new BigRational(b.getOperando(1).getTipo(),b.getOperando(2).getTipo()));
				}else if("EnteroLargo".indexOf(tipoB)!=-1){
					Solucion=(new BigRational(a.getOperando(1).getTipo(),a.getOperando(2).getTipo())).add(new BigRational(b.getOperando(1).getTipo(),"1"));
				}else{
					Solucion=(new BigRational(a.getOperando(1).getTipo(),a.getOperando(2).getTipo())).add(new BigRational(b.getOperando(1).getTipo(),b.getOperando(2).getTipo()));
				}
				
				if(Solucion.getDenominator().toString().equals("1")){
					solucion=Solucion.getNumerator().toString();
					
					if(solucion.length()<=18 && solucion.indexOf("E")==-1){
						if(Opciones.getPresicion()){
							comodin=new Nodo("Entero",1);
						}else{
							comodin=new Nodo("Decimal",1);
						}
					}else{
						if(Opciones.getPresicion()){
							comodin=new Nodo("EnteroLargo",1);
						}else{
							comodin=new Nodo("DecimalLargo",1);
						}
					}
					comodin.agregaOperando(new Nodo(solucion));
				}else{
					comodin=new Nodo("Racional",2);
					comodin.agregaOperando(new Nodo(Solucion.getNumerator().toString()));
					comodin.agregaOperando(new Nodo(Solucion.getDenominator().toString()));
				}
				return comodin;
			}
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar una suma");
		}	
	}


	/**
	 *Función que simplifica la expresión recibida. Si esta función se evalúa aquí quiere decir que no encontró el operador
	 *donde se tenía que evaluar, lo que es un error.
	 *@param expresionPrefijo La expresión a simplificar.
	 *@param funcionSimplificar La función que se encarga de simplificar. Se necesita para la recursividad.
	 *@param funcionSimplificarNumeros La función que se encarga de simplificar operaciones numéricas. Se necesita para la recursividad.
	 *@return La expresión simplificada.
	 *@exception ArithmeticException Se lanza si hay error al simplificar en la expresión.
	 */
	public Nodo simplificar(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		Nodo comodin;
		
		try{
			for(int i=1; i<=expresionPrefijo.getNumeroOperandos(); i++){
				if (expresionPrefijo.getOperando(i).getTipo().equals("Indefinido") || expresionPrefijo.getOperando(i).getTipo().equals("Boolean")){
					return new Nodo("Indefinido");
				}
			}
			
			if(expresionPrefijo.getNumeroOperandos()==1){
				return expresionPrefijo.getOperando(1);
			}else{
				comodin=simplificaSumaRecursiva(expresionPrefijo, funcionSimplificar, funcionSimplificarNumeros);
				if(comodin.getTipo().equals("Indefinido")){
					return new Nodo("Indefinido");
				}
				int num=comodin.getNumeroOperandos();
				if (num==0){
					return SimplificarNumeros.CERO.clonarNodo();
				}else if (num==1){
					return comodin.getOperando(1);
				}else{
					return comodin;
				}
			}
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar una suma");
		}
	}
	
	/*
	 *Función que simplifica las sumas de manera recursiva.
	 */
	private Nodo simplificaSumaRecursiva(Nodo expresionPrefijo, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		String tipo1, tipo2;
		Nodo comodin, comodin2;
		
		try{
			if(expresionPrefijo.getNumeroOperandos()==2){
				Nodo operando1=expresionPrefijo.getOperando(1);
				Nodo operando2=expresionPrefijo.getOperando(2);
				tipo1=operando1.getTipo();
				tipo2=operando2.getTipo();
				
				if(!tipo1.equals("+") && !tipo2.equals("+")){
					if (tipo1.length()>5 && ("EnteroLargo DecimalLargo Racional").indexOf(tipo1)!=-1 && tipo2.length()>5 && ("EnteroLargo DecimalLargo Racional").indexOf(tipo2)!=-1){
						comodin=simplificarNumeros(expresionPrefijo, funcionSimplificar, funcionSimplificarNumeros);
						if(comodin.esIgual(SimplificarNumeros.CERO)){
							return new Nodo("+");
						}else{
							comodin2=new Nodo("+",1);
							comodin2.agregaOperando(comodin);
							return comodin2;
						}
					}else if(operando1.esIgual(SimplificarNumeros.CERO)){
						comodin=new Nodo("+",1);
						comodin.agregaOperando(operando2);
						return comodin;
					}else if(operando2.esIgual(SimplificarNumeros.CERO)){
						comodin=new Nodo("+",1);
						comodin.agregaOperando(operando1);
						return comodin;
					}else if(tipo1.equals("Matriz") || tipo2.equals("Matriz")){
						//Si ambas no son matrices o si las matrices no tienen el mismo tamaño
						if(!tipo1.equals(tipo2) || operando1.getNumeroOperandos()!=operando2.getNumeroOperandos() || operando1.getOperando(1).getNumeroOperandos()!=operando2.getOperando(1).getNumeroOperandos()){
							return new Nodo("Indefinido");
						}
						
						comodin=new Nodo("Matriz");
						
						for(int i=1; i<=operando1.getNumeroOperandos(); i++){
							comodin.agregaOperando(new Nodo("Fila"));
							for(int j=1; j<=operando1.getOperando(1).getNumeroOperandos(); j++){
								comodin2=new Nodo("+",2);
								comodin2.agregaOperando(operando1.getOperando(i).getOperando(j));
								comodin2.agregaOperando(operando2.getOperando(i).getOperando(j));
								comodin2=funcionSimplificar.simplifica(comodin2);
								comodin.getOperando(i).agregaOperando(comodin2);
							}
						}
						
						comodin2=new Nodo("+",1);
						comodin2.agregaOperando(comodin);
						
						return comodin2;
					}else if(tipo1.equals("Matriz") || tipo2.equals("Matriz")){
						return new Nodo("Indefinido");
					}else if(termino(operando1).esIgual(termino(operando2))){
						comodin=new Nodo("+");
						comodin.agregaOperando(constante(operando1));
						comodin.agregaOperando(constante(operando2));
						comodin=simplificar(comodin, funcionSimplificar, funcionSimplificarNumeros);
						if(comodin.esIgual(SimplificarNumeros.CERO)){
							return new Nodo("+");
						}else if(comodin.esIgual(SimplificarNumeros.UNO)){
							comodin=new Nodo("+");
							comodin.agregaOperando(termino(operando1));
							return comodin;
						}else{
							comodin2=termino(operando1);
							comodin2.agregaOperando(comodin,1);
							comodin=new Nodo("+");
							comodin.agregaOperando(comodin2);
							return comodin;
						}
					}else if(funcionSimplificar.comparar(operando2,operando1)){
						comodin=new Nodo("+",2);
						comodin.agregaOperando(operando2);
						comodin.agregaOperando(operando1);
						return comodin;
					}else{
						return expresionPrefijo;
					}
				}else{
					return UnirSumas(operando1,operando2, funcionSimplificar, funcionSimplificarNumeros);
				}
			}else{
				if(expresionPrefijo.getTipo().equals("Indefinido")){
					return new Nodo("Indefinido");
				}
				comodin=expresionPrefijo.clonarNodo();
				comodin.eliminaOperando(1);
				comodin=simplificaSumaRecursiva(comodin, funcionSimplificar, funcionSimplificarNumeros);
				return UnirSumas(expresionPrefijo.getOperando(1),comodin, funcionSimplificar, funcionSimplificarNumeros);
			}
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar una suma");
		}
	}
	
	/*
	 *Función que toma dos expresiones donde alguna de ellas es una suma y devuelve el unión de ambos como suma.
	 */
	private Nodo UnirSumas(Nodo operando1, Nodo operando2, Simplificar funcionSimplificar, SimplificarNumeros funcionSimplificarNumeros) throws ArithmeticException{
		String tipo1=operando1.getTipo(), tipo2=operando2.getTipo();
		Nodo retornar, comodin;

		try{
		
			if(!tipo1.equals("+")){
				comodin=new Nodo("+",1);
				comodin.agregaOperando(operando1);
				operando1=comodin;
			}else if(!tipo2.equals("+")){
				comodin=new Nodo("+",1);
				comodin.agregaOperando(operando2);
				operando2=comodin;
			}
			
			retornar=new Nodo("+");
			int i=1;
			int j=1;
			while(i<=operando1.getNumeroOperandos() && j<=operando2.getNumeroOperandos()){
				comodin=new Nodo("+",1);
				comodin.agregaOperando(operando1.getOperando(i));
				comodin.agregaOperando(operando2.getOperando(j));
				comodin=simplificaSumaRecursiva(comodin, funcionSimplificar, funcionSimplificarNumeros);
				
				if(comodin.getNumeroOperandos()==0){
					i++;
					j++;
				}else if(comodin.getNumeroOperandos()==1){
					retornar.agregaOperando(comodin.getOperando(1));
					i++;
					j++;
				}else if(comodin.getOperando(1).esIgual(operando1.getOperando(i))){
					retornar.agregaOperando(comodin.getOperando(1));
					i++;
				}else{
					retornar.agregaOperando(comodin.getOperando(1));
					j++;
				}
			}
			
			if(i<=operando1.getNumeroOperandos()){
				for(; i<=operando1.getNumeroOperandos(); i++){
					retornar.agregaOperando(operando1.getOperando(i));
				}
			}else if(j<=operando2.getNumeroOperandos()){
				for(; j<=operando2.getNumeroOperandos(); j++){
					retornar.agregaOperando(operando2.getOperando(j));
				}
			}
			
			return retornar;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al simplificar una suma");
		}
	}
	
	/*
	 *Devuelve el término de la expresión vista como una multiplicación.
	 */
	private Nodo termino(Nodo expresionPrefijo){
		String tipo;
		Nodo comodin, comodin2;
		
		tipo=expresionPrefijo.getTipo();
		//if(("EnteroLargo DecimalLargo Constante Racional").indexOf(tipo)!=-1){ //Si es un número
		if(("EnteroLargo DecimalLargo Racional").indexOf(tipo)!=-1){ //Si es un número
			return new Nodo("Indefinido");
		}else if (tipo.equals("·")){
			//if(("EnteroLargo DecimalLargo Constante Racional").indexOf(expresionPrefijo.getOperando(1).getTipo())!=-1){ //Si el primer operando es un número
			if(("EnteroLargo DecimalLargo Racional").indexOf(expresionPrefijo.getOperando(1).getTipo())!=-1){ //Si el primer operando es un número
				comodin2=expresionPrefijo.clonarNodo();
				comodin2.eliminaOperando(1);

				return comodin2;
			}else{
				return expresionPrefijo;
			}
		}else{
			comodin=new Nodo("·");
			comodin.agregaOperando(expresionPrefijo);
			return comodin;
		}
	}
	
	/*
	 *Devuelve la constante de la expresión vista como una multiplicación.
	 */
	private Nodo constante(Nodo expresionPrefijo){
		String tipo;
		Nodo comodin;
		
		tipo=expresionPrefijo.getTipo();
		//if(("EnteroLargo DecimalLargo Constante Racional").indexOf(tipo)!=-1){ //Si es un número
		if(("EnteroLargo DecimalLargo Racional").indexOf(tipo)!=-1){ //Si es un número
			return new Nodo("Indefinido");
		}else if (tipo.equals("·")){
			comodin=expresionPrefijo.getOperando(1);
			//if(("EnteroLargo DecimalLargo Constante Racional").indexOf(comodin.getTipo())!=-1){ //Si el primer operando es un número
			if(("EnteroLargo DecimalLargo Racional").indexOf(comodin.getTipo())!=-1){ //Si el primer operando es un número
				return comodin;
			}else{
				comodin=new Nodo("Entero",1);
				comodin.agregaOperando(new Nodo("1"));
				return comodin;
			}
		}else{
			comodin=new Nodo("Entero",1);
			comodin.agregaOperando(new Nodo("1"));
			return comodin;
		}
	}


	/**
	 *Función que devuelve si la suma es binaria.
	 *@return True si la suma es binaria, false en caso contrario.
	 *@exception ArithmeticException Se lanza si el operador no existe.
	 */
	public boolean esBinario() throws ArithmeticException{
		return true;
	}
	
	/**
	 *Función que deriva la expresión recibida.
	 *@param expresionPrefijo La expresión a derivar.
	 *@param variable La variable con respecto a la que se va a derivar.
	 *@param funcionDerivar La función que se encarga de derivar.
	 *@return La expresión derivada.
	 *@exception ArithmeticException Se lanza si hay error al derivar en la expresión.
	 */
	public Nodo derivar(Nodo expresionPrefijo, Nodo variable, Derivar funcionDerivar) throws ArithmeticException{
		Nodo comodin;
		
		try{
			comodin=new Nodo("+",expresionPrefijo.getNumeroOperandos());
			
			for(int i=1; i<=expresionPrefijo.getNumeroOperandos(); i++){
				comodin.agregaOperando(funcionDerivar.deriva(expresionPrefijo.getOperando(i).clonarNodo(), variable));
			}

			return comodin;
		}catch(Exception e){
			e.printStackTrace();
			throw new ArithmeticException("Error al derivar una suma");
		}
	}
}
