package itcr.mathexplorer.paneles;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.TitledBorder;

/**
 *@(#)PanelTipoLetra.java
 *
 *Este es el Panel que se encarga de la elección de la fuente.
 *
 *@author Alexander Borbón Alpízar
 */
public class FontChooser extends JDialog implements ActionListener, ListSelectionListener, WindowListener, KeyListener {
	static final long serialVersionUID=10001;
	
	private JTextField fuente, estilo, tamanho;
	private JButton aceptar, cancelar;
	private JList tipoLetra, estiloLetra, tamanhoLetra;
	//private DefaultListModel modoLetra, modoEstilo, modoTamanho;
	private JLabel vistaPrevia;
	//private JPanel colorLabel;
	private Font font = new Font(null,Font.PLAIN,12);
	//private Color color = Color.black;
	private String[] fonts;
	private String[] styles = {"Normal","Cursiva","Negrita","Negrita Cursiva"};
	private String[] sizes= {"8","9","10","11","12","14","16","18","20","22","24","28","36","48","72"};
	private JDialog frame = this;
	private boolean eventoLista = false, eventoCampo = false, ButtonOK = false;

	// CONSTRUCTORES
	/**
	 *Constructor que recibe la ventana dueña de este FontChooser
	 *@param ventana La ventana que es dueña del FontChooser
	 *@param titulo El título de la ventana
	 */
	public FontChooser(Dialog ventana, String titulo){
		super(ventana, titulo);
		crearGUI();
	}

	/**
	 *Constructor que recibe la ventana dueña de este FontChooser
	 *@param ventana La ventana que es dueña del FontChooser
	 *@param titulo El título de la ventana
	 *@param font La fuente inicial del FontChooser
	 */
	public FontChooser(Dialog ventana, String titulo, Font font){
		super(ventana, titulo);
		this.font = font;
		crearGUI();
	}

	// Método constructor de la interfaz
	private void crearGUI(){
		setLayout(null);
		
		fonts = GraphicsEnvironment.getLocalGraphicsEnvironment().getAvailableFontFamilyNames();

		JLabel label1 = new JLabel("Fuente:");
		label1.setBounds(7,8,45,15);
		add(label1);

		fuente = new JTextField(font.getName(), 10);
		fuente.setBounds(7,25,214,19);
		fuente.selectAll();
		fuente.addKeyListener(this);
		add(fuente);

		JLabel label2 = new JLabel("Estilo de fuente:");
		label2.setBounds(226,8,90,15);
		add(label2);

		estilo = new JTextField(10);
		estilo.setBounds(226,25,116,20);
		estilo.addKeyListener(this);
		add(estilo);

		JLabel label3 = new JLabel("Tamaño:");
		label3.setBounds(347,8,50,15);
		add(label3);

		tamanho = new JTextField(font.getSize()+"", 10);
		tamanho.setBounds(347,25,75,20);
		tamanho.selectAll();
		tamanho.addKeyListener(this);
		add(tamanho);

		tipoLetra = new JList(fonts);
		tipoLetra.addListSelectionListener(this);
		tipoLetra.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

		JScrollPane scroll1 = new JScrollPane(tipoLetra);
		scroll1.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		scroll1.setBounds(14,45,205,73);
		add(scroll1);
		tipoLetra.ensureIndexIsVisible(obtIndiceFont(font.getName()));
		tipoLetra.setSelectedIndex(obtIndiceFont(font.getName()));
		
		estiloLetra = new JList(styles);
		estiloLetra.addListSelectionListener(this);
		estiloLetra.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

		JScrollPane scroll2 = new JScrollPane(estiloLetra);
		scroll2.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		scroll2.setBounds(232,45,110,73);
		add(scroll2);
		int num = font.getStyle();
		estilo.setText(styles[num]);
		estilo.selectAll();
		estiloLetra.setSelectedIndex(num);
		
		tamanhoLetra = new JList(sizes);
		tamanhoLetra.addListSelectionListener(this);
		tamanhoLetra.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

		JScrollPane scroll3 = new JScrollPane(tamanhoLetra);
		scroll3.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		scroll3.setBounds(355,45,67,73);
		add(scroll3);
		tamanhoLetra.ensureIndexIsVisible(obtIndiceSize(font.getSize()));
		tamanhoLetra.setSelectedIndex(obtIndiceSize(font.getSize()));

		JPanel panel = new JPanel();
		TitledBorder titledborder = BorderFactory.createTitledBorder("Vista previa:");
		panel.setBorder(titledborder);
		panel.setLayout(new BorderLayout());
		panel.setBounds(10,150,415,75);
		panel.setBackground(Color.white);
		add(panel);

		panel.add(new JLabel("_________      "), BorderLayout.WEST);

		vistaPrevia = new JLabel(font.getName());
		vistaPrevia.setHorizontalAlignment(SwingConstants.CENTER);
		vistaPrevia.setFont(font);
		panel.add(vistaPrevia, BorderLayout.CENTER);

		panel.add(new JLabel("      _________"), BorderLayout.EAST);

		aceptar = new JButton("Aceptar");
		aceptar.setBounds(208,235,105,22);
		aceptar.setMnemonic('A');
		aceptar.addActionListener(this);
		add(aceptar);

		cancelar = new JButton("Cancelar");
		cancelar.setBounds(320,235,105,22);
		cancelar.setMnemonic('C');
		cancelar.addActionListener(this);
		add(cancelar);

		KeyStroke escape = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE,0);
		JRootPane rootPane = frame.getRootPane();
		rootPane.registerKeyboardAction(this,"ESC",escape,JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
		this.addWindowListener(this);
		eventoLista = true;
		eventoCampo = true;
	}//CONSTRUCTORES


	//METODOS PUBLICOS

	/**
	 * Método para mostrar el JPanel
	 * @param titulo El título del JPanel 
	 */
	public void showDialog(String titulo){
		//frame.getContentPane().add(this, BorderLayout.CENTER);
		frame.setTitle(titulo);
		frame.setSize(445,300);
		frame.setLocationRelativeTo(null);
		frame.setResizable(false);
		frame.setModal(true);
		frame.setVisible(true);

		//Reiniciar la ventana
		eventoCampo = false;
		Font fuent = vistaPrevia.getFont();
		fuente.setText(fuent.getName());
		fuente.selectAll();
		int num = fuent.getStyle();
		if(num == 0){
			estilo.setText(styles[0]);
			estilo.selectAll();
		}else if(num == 1){
			estilo.setText(styles[1]);
			estilo.selectAll();
		}else if(num == 2){
			estilo.setText(styles[2]);
			estilo.selectAll();
		}else{
			estilo.setText(styles[3]);
			estilo.selectAll();
		}
		tamanho.setText(fuent.getSize()+"");
		tamanho.selectAll();
		eventoCampo = true;
	}// fin de show

	/**
	 * Método para obtener el font elegido.
	 * @return El Font elegido.
	 */
	public Font getSelectedFont(){
		String nom = fonts[tipoLetra.getSelectedIndex()];
		int styleInt = obtStyle(styles[estiloLetra.getSelectedIndex()]);
		int n = (new Integer(sizes[tamanhoLetra.getSelectedIndex()])).intValue();
		try{
			n = Integer.parseInt(tamanho.getText());
		}catch(NumberFormatException e){}
		return new Font(nom, styleInt, n);
	}// fin de getSelectedFont
	
	/**
	 * Método para obtener el boton elegido
	 * @return True si fue elegido el botón de OK o false si se presionó otro botón.
	 */
	public boolean getBotonPresionado(){
		return ButtonOK;
	}// fin de getSelectedFont


	// METODOS PRIVADOS

	// Método para obtener el indice de un font
	private int obtIndiceFont(String cadena){
		for(int i=0; i<fonts.length; i++){
			if(fonts[i].equalsIgnoreCase(cadena)){
				return i;
			}
		}
		return 0;
	}// fin de obtIndiceFont

	// Método para obtener el indice de un tamaño
	private int obtIndiceSize(int num){
		for(int i=0; i<sizes.length; i++){
			if((new Integer(sizes[i])).intValue() == num){
				return i;
			}
		}
		return 0;
	}// fin de obtIndiceSize

	// Método para obtener el estilo
	private int obtStyle(String cadena){
		if(cadena.equalsIgnoreCase("Normal")){
			return Font.PLAIN;
		}else if(cadena.equalsIgnoreCase("Cursiva")){
			return Font.ITALIC;
		}else if(cadena.equalsIgnoreCase("Negrita")){
			return Font.BOLD;
		}else if(cadena.equalsIgnoreCase("Negrita Cursiva")){
			return Font.BOLD + Font.ITALIC;
		}
		return 0;
	}// fin de obtStyle

	//Método para saber si una cadena empieza con otra
	private boolean beginWidth(String cadena, String subCadena){
		cadena = cadena.trim().toLowerCase();
		if(!cadena.equals("")){
			subCadena = subCadena.trim().toLowerCase();
			int n = cadena.indexOf(subCadena);
			if(n == 0){ return true;}
		}
		return false;
	}// fin de beginWidth


	//AUDITORES

	//Auditor para los botones y campos de texto
	public void actionPerformed(ActionEvent event){
		if(event.getActionCommand().equals("ESC")){
			event.setSource(cancelar);
		}
		
		if(event.getSource() == aceptar){
			ButtonOK = true;
			this.dispose();
		}else if(event.getSource() == cancelar){
			ButtonOK = false;
			this.dispose();
		}
	}// fin del actionPerformed

	// Auditor de los JList
	public void valueChanged(ListSelectionEvent event){
		if(!eventoLista){ return; }
		Font fuent = vistaPrevia.getFont();
		eventoCampo = false;
		if(event.getSource() == tipoLetra){
			fuente.setText(fonts[tipoLetra.getSelectedIndex()]);
			fuente.selectAll();
			//fuente.requestFocus();
			int styleInt = fuent.getStyle();
			int n = fuent.getSize();
			fuent = new Font(fonts[tipoLetra.getSelectedIndex()],styleInt,n);
			vistaPrevia.setText(fonts[tipoLetra.getSelectedIndex()]);
		}else if(event.getSource() == estiloLetra){
			estilo.setText(styles[estiloLetra.getSelectedIndex()]);
			estilo.selectAll();
			//estilo.requestFocus();
			int styleInt = obtStyle(styles[estiloLetra.getSelectedIndex()]);
			int n = fuent.getSize();
			fuent = new Font(fuent.getName(),styleInt,n);
		}else{
			tamanho.setText(sizes[tamanhoLetra.getSelectedIndex()]+"");
			tamanho.selectAll();
			//tamanho.requestFocus();
			int n = (new Integer(sizes[tamanhoLetra.getSelectedIndex()])).intValue();
			int styleInt = fuent.getStyle();
			fuent = new Font(fuent.getName(),styleInt,n);
		}
		vistaPrevia.setFont(fuent);
		eventoCampo = true;
	}

	//Auditor para la ventana
	public void windowActivated(WindowEvent e){ }
	public void windowClosed(WindowEvent e){ }
	public void windowClosing(WindowEvent event){ cancelar.doClick(); }
	public void windowDeactivated(WindowEvent e){ }
	public void windowDeiconified(WindowEvent e){ }
	public void windowIconified(WindowEvent e){ }
	public void windowOpened(WindowEvent e){ }


	//Auditor para los campos de texto (es un auditor del teclado)
	public void keyReleased(KeyEvent e) {
		if(e.getSource() == fuente){
			String cadena = fuente.getText().trim();
			if(!cadena.equals("") && eventoCampo){
				for(int i=0; i<fonts.length; i++){
					if(beginWidth(fonts[i], cadena)){
						tipoLetra.ensureIndexIsVisible(i);
						if(fonts[i].equalsIgnoreCase(cadena)){
							eventoLista = false;
							tipoLetra.setSelectedIndex(i);
							eventoLista = true;
							Font fuent = vistaPrevia.getFont();
							int styleInt = fuent.getStyle();
							int n = fuent.getSize();
							fuent = new Font(fonts[i],styleInt,n);
							vistaPrevia.setText(fonts[tipoLetra.getSelectedIndex()]);
							vistaPrevia.setFont(fuent);
							fuente.selectAll();
						}
						i = fonts.length;
					}
				}
			}
		}else if(e.getSource() == estilo){
			String cadena = estilo.getText().trim();
			if(!cadena.equals("") && eventoCampo){
				for(int i=0; i<styles.length; i++){
					if(beginWidth(styles[i], cadena)){
						estiloLetra.ensureIndexIsVisible(i);
						if(styles[i].equalsIgnoreCase(cadena)){
							eventoLista = false;
							estiloLetra.setSelectedIndex(i);
							eventoLista = true;
							Font fuent = vistaPrevia.getFont();
							int styleInt = obtStyle(styles[i]);
							int n = fuent.getSize();
							fuent = new Font(fuent.getName(),styleInt,n);
							vistaPrevia.setFont(fuent);
							estilo.selectAll();
						}
						i = styles.length;
					}
				}
			}
		}else if(e.getSource() == tamanho){
			String cadena = tamanho.getText().trim();
			if(!cadena.equals("") && eventoCampo){
				for(int i=0; i<sizes.length; i++){
					if(beginWidth(sizes[i]+"", cadena)){
						tamanhoLetra.ensureIndexIsVisible(i);
						if(cadena.equals(sizes[i]+"")){
							eventoLista = false;
							tamanhoLetra.setSelectedIndex(i);
							eventoLista = true;
							Font fuent = vistaPrevia.getFont();
							int n = (new Integer(sizes[i])).intValue();
							int styleInt = fuent.getStyle();
							fuent = new Font(fuent.getName(),styleInt,n);
							vistaPrevia.setFont(fuent);
							tamanho.selectAll();
						}
						i = sizes.length;
					}
				}
			}
		}
		//eventoCampo=false;
	}
	public void keyPressed(KeyEvent e) { }
	
	public void keyTyped(KeyEvent e) {
		//eventoCampo=true;
	}

}//PanelTipoLetra