package itcr.mathexplorer.paneles;
 
import java.awt.*;
import javax.swing.*;
import java.math.*;
import java.awt.event.*;
import java.awt.image.*;
import java.util.*;
import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.herramientas.Opciones;
import itcr.mathexplorer.tablas.*;
import itcr.mathexplorer.funciones.Simplificar;

/**
 * @(#)PanelGraficos2D.java
 *
 * Panel para la graficación de funciones en dos dimensiones
 *
 * @author Alexander Borbón Alpízar
 * @version 1.00 2007/11/1
 */
public class PanelGraficos2D extends JPanel implements MouseListener, MouseMotionListener, MouseWheelListener{
	static final long serialVersionUID=10004;
	
	//VARIABLES PRIVADAS
	//Esta es la tabla que contiene a todas las funciones permitidas
	TablaFunciones funciones;
	
	//Esta es la tabla que contiene a todas las constantes permitidas
	TablaConstantes constantes;
	
	//Esta es la tabla que contiene a todos los operadores permitidos
	TablaOperadores operadores;
	
	//Esta es la tabla que contiene a todas las funciones permitidas
	TablaFuncionesUsuario funcionesUsuario;
	
	//El tamaño a graficar en coordenadas reales, se graficará de (xmin, ymin) a (xmax, ymax)
	private BigDecimal xmin, xmax, ymin, ymax; 
	private int ancho, alto, anchoAnterior, altoAnterior; //tamaño de la pantalla en pixeles
	private BigDecimal anchoEjeX, anchoEjeY;
	private int presicion=40, pixelesMarcasX=40, pixelesMarcasY=40;
	//Variables para los movimientos del mouse
	private int xmouse, ymouse, xmouse2, ymouse2, boton;
	
	//Arreglo donde se guardar todas las expresiones a graficar
	private Nodo[] Expresiones;
	private Nodo[] Variables;
	private int expresionUltima;//, total;
	private Vector<Nodo> variables; //variable donde se guarda la variable de la expresión ingresada.
	
	//Arreglo que contiene los colores para realizar las gráficas
	private Color[] colores, todosColores;
	private int numColores, numTotalColores;
	
	private boolean exacto, cuadro, alternar, repintar=true;
	
	//Función para simplificar las expresiones
	Simplificar simplificar;
	
	//Opciones
	Color colorFondo, colorEjesAtenuados, colorEjes, colorGrafica;
	Font tipoLetra;
	int numPuntos;
	
	//Se mantiene una copia anterior del gráfico para aumentar la velocidad del repintado
	BufferedImage graficoAnterior;
	
	//CONSTRUCTORES
	
	/**
	 *Constructor que define el área a grafica.
	 *Por defecto se grafica desde -5 a 5 en el eje
	 *X y de -3 a 3 en el eje Y.
	 *@param tablaOperadores Tabla con todos los operadores del programa.
	 *@param tablaFunciones Tabla con todas las funciones del programa.
	 *@param tablaConstantes Tabla con todas las constantes del programa.
	 *@param tablaFuncionesUsuario Tabla con todas las funciones definidas por el usuario en el programa.
	 */
	public PanelGraficos2D(TablaOperadores tablaOperadores, TablaFunciones tablaFunciones, TablaConstantes tablaConstantes, TablaFuncionesUsuario tablaFuncionesUsuario){
		constructor(new BigDecimal(-5), new BigDecimal(5), new BigDecimal(-3), new BigDecimal(3), tablaOperadores, tablaFunciones, tablaConstantes, tablaFuncionesUsuario);
	}
	
	/**
	 *Constructor vacío que define el área a grafica.
	 *Por defecto se grafica desde -5 a 5 en el eje
	 *X y de -3 a 3 en el eje Y.
	 *@param e La expresión inicial a graficar
	 *@param tablaOperadores Tabla con todos los operadores del programa.
	 *@param tablaFunciones Tabla con todas las funciones del programa.
	 *@param tablaConstantes Tabla con todas las constantes del programa.
	 *@param tablaFuncionesUsuario Tabla con todas las funciones definidas por el usuario en el programa.
	 */
	public PanelGraficos2D(Nodo e, TablaOperadores tablaOperadores, TablaFunciones tablaFunciones, TablaConstantes tablaConstantes, TablaFuncionesUsuario tablaFuncionesUsuario){
		constructor(new BigDecimal(-5), new BigDecimal(5), new BigDecimal(-3), new BigDecimal(3), tablaOperadores, tablaFunciones, tablaConstantes, tablaFuncionesUsuario);
		agregaExpresion(e);
	}
	
	/**
	 *Constructor que define el área a grafica.
	 *En este caso se grafica desde minX a maxX en el eje
	 *X y de minY a maxY en el eje Y.
	 *Si los valores mínimos son mayores a los máximos
	 *entonces se define de -3 a 3 en el eje correspondiente.
	 *@param minX El valor mínimo en el eje x a graficar.
	 *@param maxX El valor máximo en el eje x a graficar.
	 *@param minY El valor mínimo en el eje y a graficar.
	 *@param maxY El valor máximo en el eje y a graficar.
	 *@param tablaOperadores Tabla con todos los operadores del programa.
	 *@param tablaFunciones Tabla con todas las funciones del programa.
	 *@param tablaConstantes Tabla con todas las constantes del programa.
	 *@param tablaFuncionesUsuario Tabla con todas las funciones definidas por el usuario en el programa.
	 */
	public PanelGraficos2D(double minX, double maxX, double minY, double maxY, TablaOperadores tablaOperadores, TablaFunciones tablaFunciones, TablaConstantes tablaConstantes, TablaFuncionesUsuario tablaFuncionesUsuario){
		constructor(new BigDecimal(minX), new BigDecimal(maxX), new BigDecimal(minY), new BigDecimal(maxY), tablaOperadores, tablaFunciones, tablaConstantes, tablaFuncionesUsuario);
	}
	
	/**
	 *Constructor que define el área a grafica.
	 *En este caso se grafica desde minX a maxX en el eje
	 *X y de minY a maxY en el eje Y.
	 *Si los valores mínimos son mayores a los máximos
	 *entonces se define de -3 a 3 en el eje correspondiente.
	 *@param e La expresión inicial a graficar
	 *@param minX El valor mínimo en el eje x a graficar.
	 *@param maxX El valor máximo en el eje x a graficar.
	 *@param minY El valor mínimo en el eje y a graficar.
	 *@param maxY El valor máximo en el eje y a graficar.
	 *@param tablaOperadores Tabla con todos los operadores del programa.
	 *@param tablaFunciones Tabla con todas las funciones del programa.
	 *@param tablaConstantes Tabla con todas las constantes del programa.
	 *@param tablaFuncionesUsuario Tabla con todas las funciones definidas por el usuario en el programa.
	 */
	public PanelGraficos2D(Nodo e, double minX, double maxX, double minY, double maxY, TablaOperadores tablaOperadores, TablaFunciones tablaFunciones, TablaConstantes tablaConstantes, TablaFuncionesUsuario tablaFuncionesUsuario){
		constructor(new BigDecimal(minX), new BigDecimal(maxX), new BigDecimal(minY), new BigDecimal(maxY), tablaOperadores, tablaFunciones, tablaConstantes, tablaFuncionesUsuario);
		agregaExpresion(e);
	}
	
	/**
	 *Constructor que define el área a grafica.
	 *En este caso se grafica desde minX a maxX en el eje
	 *X y de minY a maxY en el eje Y.
	 *Si los valores mínimos son mayores a los máximos
	 *entonces se define de -3 a 3 en el eje correspondiente.
	 *@param minX El valor mínimo en el eje x a graficar.
	 *@param maxX El valor máximo en el eje x a graficar.
	 *@param minY El valor mínimo en el eje y a graficar.
	 *@param maxY El valor máximo en el eje y a graficar.
	 *@param tablaOperadores Tabla con todos los operadores del programa.
	 *@param tablaFunciones Tabla con todas las funciones del programa.
	 *@param tablaConstantes Tabla con todas las constantes del programa.
	 *@param tablaFuncionesUsuario Tabla con todas las funciones definidas por el usuario en el programa.
	 */
	public PanelGraficos2D(BigDecimal minX, BigDecimal maxX, BigDecimal minY, BigDecimal maxY, TablaOperadores tablaOperadores, TablaFunciones tablaFunciones, TablaConstantes tablaConstantes, TablaFuncionesUsuario tablaFuncionesUsuario){
		constructor(minX, maxX, minY, maxY, tablaOperadores, tablaFunciones, tablaConstantes, tablaFuncionesUsuario);
	}
	
	/**
	 *Constructor que define el área a grafica.
	 *En este caso se grafica desde minX a maxX en el eje
	 *X y de minY a maxY en el eje Y.
	 *Si los valores mínimos son mayores a los máximos
	 *entonces se define de -3 a 3 en el eje correspondiente.
	 *@param e La expresión inicial a graficar
	 *@param minX El valor mínimo en el eje x a graficar.
	 *@param maxX El valor máximo en el eje x a graficar.
	 *@param minY El valor mínimo en el eje y a graficar.
	 *@param maxY El valor máximo en el eje y a graficar.
	 *@param tablaOperadores Tabla con todos los operadores del programa.
	 *@param tablaFunciones Tabla con todas las funciones del programa.
	 *@param tablaConstantes Tabla con todas las constantes del programa.
	 *@param tablaFuncionesUsuario Tabla con todas las funciones definidas por el usuario en el programa.
	 */
	public PanelGraficos2D(Nodo e, BigDecimal minX, BigDecimal maxX, BigDecimal minY, BigDecimal maxY, TablaOperadores tablaOperadores, TablaFunciones tablaFunciones, TablaConstantes tablaConstantes, TablaFuncionesUsuario tablaFuncionesUsuario){
		constructor(minX, maxX, minY, maxY, tablaOperadores, tablaFunciones, tablaConstantes, tablaFuncionesUsuario);
		agregaExpresion(e);
	}
	
	private void constructor(BigDecimal minX, BigDecimal maxX, BigDecimal minY, BigDecimal maxY, TablaOperadores tablaOperadores, TablaFunciones tablaFunciones, TablaConstantes tablaConstantes, TablaFuncionesUsuario tablaFuncionesUsuario){
		colorFondo=Opciones.getColorFondoGrafica();
		colorEjesAtenuados=Opciones.getColorEjesAtenuadosGrafica();
		colorEjes=Opciones.getColorEjesGrafica();
		colorGrafica=Opciones.getColorGrafica();
		alternar=Opciones.getAlternarColorGrafica();
		tipoLetra=Opciones.getTipoLetraEjesGrafica();
		numPuntos=Opciones.getNumPuntosGrafica();
		pixelesMarcasX=Opciones.getPixelesMarcasX();
		pixelesMarcasY=Opciones.getPixelesMarcasY();
		
		this.setBackground(colorFondo);
		
		if(minX.compareTo(maxX)==-1){
			xmin=minX;
			xmax=maxX;
		}else{
			xmin=new BigDecimal(-3);
			xmax=new BigDecimal(3);	
		}
		
		if(minY.compareTo(maxY)==-1){
			ymin=minY;
			ymax=maxY;
		}else{
			ymin=new BigDecimal(-3);
			ymax=new BigDecimal(3);
		}
		
		Expresiones=new Nodo[10];
		Variables=new Nodo[10];
		
		numTotalColores=10;
		todosColores=new Color[numTotalColores];
		todosColores[0]=Color.BLACK;
		todosColores[1]=Color.BLUE;
		todosColores[2]=Color.GREEN;
		todosColores[3]=Color.RED;
		todosColores[4]=Color.YELLOW;
		todosColores[5]=Color.ORANGE;
		todosColores[6]=Color.CYAN;
		todosColores[7]=Color.MAGENTA;
		todosColores[8]=Color.PINK;
		todosColores[9]=Color.GRAY;
		numColores=0;
		
		colores=new Color[10];
		
		expresionUltima=0;
		//total=1;
		
		operadores=tablaOperadores;
		funciones=tablaFunciones;
		constantes=tablaConstantes;
		funcionesUsuario=tablaFuncionesUsuario;
		
		simplificar=new Simplificar(operadores, funciones, constantes, funcionesUsuario);
		
		this.addMouseListener(this);
		this.addMouseMotionListener(this);
		this.addMouseWheelListener(this);
	}
	
	//Método que repinta

	public void paintComponent(Graphics ge) {
		this.setBackground(colorFondo);
		super.paintComponent(ge);
		
		anchoAnterior=ancho;
		altoAnterior=alto;
		ancho=this.getWidth();
		alto=this.getHeight();
		
		if(anchoAnterior!=ancho || altoAnterior!=alto){
			repintar=true;
		}
		
		if(repintar){
			
			graficoAnterior= (BufferedImage)(this.createImage(ancho,alto));
            Graphics2D ggraficoAnterior = graficoAnterior.createGraphics();
            ggraficoAnterior.setBackground(colorFondo);
            ggraficoAnterior.clearRect(0,0,ancho,alto);
			ggraficoAnterior.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
			
			BigInteger centroX, centroY;
			int numMarcas, coordenadaPant, cont;
			int cifra, potencia, presicion2;
			BigDecimal anchoMarcas, inicio;
			BigDecimal coordenada;
			int altotexto, anchotexto;
			String coord, coordcomodin, stringAnchoMarcas;
			FontMetrics tamanoletra;
			boolean negativo, exponente;
			
			anchoEjeX=xmax.subtract(xmin).stripTrailingZeros();
			anchoEjeY=ymax.subtract(ymin).stripTrailingZeros();
			
			//Este procedimiento calcula la presicion necesaria para hacer un buen gráfico
			
			stringAnchoMarcas=anchoEjeY.toPlainString();
			if(stringAnchoMarcas.indexOf(".")!=-1){
				stringAnchoMarcas=stringAnchoMarcas.substring(stringAnchoMarcas.indexOf(".") + 1);
				presicion=1;
				while(stringAnchoMarcas.substring(0,1).equals("0")){
					stringAnchoMarcas=stringAnchoMarcas.substring(1);
					presicion++;
				}
			}else{
				presicion=1;
				/*while(coord.substring(coord.length()-1).equals("0")){
					coord=coord.substring(0,coord.length()-1);
					presicion++;
				}*/
			}
			
			stringAnchoMarcas=anchoEjeX.toPlainString();
			if(stringAnchoMarcas.indexOf(".")!=-1){
				stringAnchoMarcas=stringAnchoMarcas.substring(stringAnchoMarcas.indexOf(".") + 1);
				presicion2=1;
				while(stringAnchoMarcas.substring(0,1).equals("0")){
					stringAnchoMarcas=stringAnchoMarcas.substring(1);
					presicion2++;
				}
			}else{
				presicion2=1;
				/*while(coord.substring(coord.length()-1).equals("0")){
					coord=coord.substring(0,coord.length()-1);
					presicion++;
				}*/
			}
			
			presicion = Math.max(presicion,presicion2)+5;
			//Ejes Coordenados
			
			//Eje X
			centroY=convCoordPantY(BigDecimal.ZERO);
			if(centroY.compareTo(new BigInteger("30"))<0){
				centroY=new BigInteger((new Long(30)).toString());
				ggraficoAnterior.setColor(colorEjesAtenuados);
			}else if(centroY.compareTo(new BigInteger((new Integer(alto-30)).toString()))>0){ //Si se sale de la pantalla
				centroY=new BigInteger((new Long(alto-30)).toString());
				ggraficoAnterior.setColor(colorEjesAtenuados);
			}else{
				ggraficoAnterior.setColor(colorEjes);
			}
			ggraficoAnterior.setFont(tipoLetra); //Opciones de fuente para las marcas
			tamanoletra=ggraficoAnterior.getFontMetrics();
			altotexto=tamanoletra.getAscent();
	
			ggraficoAnterior.drawLine(0, centroY.intValue(), ancho, centroY.intValue());
			
			//Lo que sigue reliza las marcas en los ejes
			numMarcas=(new Double(ancho/pixelesMarcasX)).intValue(); //Cantidad de marcas que se realizarán
			numMarcas= numMarcas==0 ? 1 : numMarcas;
			anchoMarcas=anchoEjeX.divide(new BigDecimal(numMarcas), presicion, BigDecimal.ROUND_HALF_EVEN); //El ancho que tendrá cada marca
			if(anchoMarcas.compareTo(BigDecimal.ONE)>=0){
				stringAnchoMarcas=anchoMarcas.toPlainString();
				cifra=new Integer(stringAnchoMarcas.substring(0,1)); //El ancho de cada marca
				potencia=stringAnchoMarcas.indexOf(".") - 1;
			}else if(anchoMarcas.compareTo(new BigDecimal(0.7))>=0){
				cifra=1;
				potencia=0;
			}else if(anchoMarcas.compareTo(new BigDecimal(0.3))>=0){
				cifra=5;
				potencia=-1;
			}else if(anchoMarcas.compareTo(new BigDecimal(0.15))>=0){
				cifra=2;
				potencia=-1;
			}else if(anchoMarcas.compareTo(new BigDecimal(0.09))>=0){
				cifra=1;
				potencia=-1;
			}else{
				stringAnchoMarcas=anchoMarcas.toPlainString();
				
				if(stringAnchoMarcas.indexOf("E")!=-1){
					potencia=new Integer(stringAnchoMarcas.substring(stringAnchoMarcas.indexOf("E") + 1));
				}else{
					stringAnchoMarcas=stringAnchoMarcas.substring(stringAnchoMarcas.indexOf(".") + 1);
					potencia=-1;
					while(stringAnchoMarcas.substring(0,1).equals("0")){
						stringAnchoMarcas=stringAnchoMarcas.substring(1);
						potencia--;
					}
				}
				cifra=new Integer(stringAnchoMarcas.substring(0,1)); //El ancho de cada marca
			}
			
			if(potencia>=0){
				anchoMarcas=(new BigDecimal(cifra)).multiply((new BigDecimal(10)).pow(potencia)).stripTrailingZeros();
			}else{
				anchoMarcas=(new BigDecimal(cifra)).divide((new BigDecimal(10)).pow(-1*potencia), presicion, BigDecimal.ROUND_HALF_EVEN).stripTrailingZeros();
			}
			//inicio=xmin.divide(anchoMarcas, xmin.toString().indexOf(".")-Math.abs(potencia), BigDecimal.ROUND_CEILING).multiply(anchoMarcas).round(new MathContext(Math.abs(potencia), RoundingMode.HALF_EVEN)); //El valor donde se pone la primera marca
			inicio=xmin.divideToIntegralValue(anchoMarcas).multiply(anchoMarcas);//.round(new MathContext(Math.abs(potencia), RoundingMode.HALF_EVEN)); //El valor donde se pone la primera marca
			numMarcas=anchoEjeX.divide(anchoMarcas, presicion, BigDecimal.ROUND_HALF_EVEN).round(new MathContext(presicion, RoundingMode.HALF_EVEN)).intValue();
			
			exponente=false;
			for(int i=0; i<=numMarcas; i++){ //Dibujo de las marcas
				coordenada=inicio.add(new BigDecimal(i).multiply(anchoMarcas));
				coordenadaPant=convCoordPantX(coordenada).intValue();
				ggraficoAnterior.drawLine(coordenadaPant, centroY.intValue(), coordenadaPant, centroY.intValue()+5);
	
				coord=coordenada.stripTrailingZeros().toPlainString(); //new Double(coordenada - (coordenada%anchoMarcas)).toString(); //(new Long(Math.round(coordenada))).toString();
				
				if(coord.indexOf(".")!=-1){
					while(coord.substring(coord.length()-1).equals("0")){
						coord=coord.substring(0,coord.length()-1);
					}
					if(coord.substring(coord.length()-1).equals(".")){
						coord=coord.substring(0,coord.length()-1);
					}
					
					if(coord.substring(0,1).equals("-")){
						negativo=true;
						coordcomodin=coord.substring(1);
					}else{
						negativo=false;
						coordcomodin=coord;
					}
					
					cont=-1;
					while(!coordcomodin.equals("") && (coordcomodin.substring(0,1).equals("0") || coordcomodin.substring(0,1).equals("."))){
						coordcomodin=coordcomodin.substring(1);
						cont++;
					}
					if(coordcomodin.equals("")){
						coord="0";
					}
					
					if(coordcomodin.indexOf(".")==-1){
						cont+=coordcomodin.length()-1;
					}
					
					if(cont>=3 && cont+coordcomodin.length()>=5){
						if(negativo){
							coord="-".concat(coordcomodin).concat("·10^").concat((new Integer(-1*cont)).toString());
						}else{
							coord=coordcomodin.concat("·10^").concat((new Integer(-1*cont)).toString());
						}
					}
	
				}else if(exponente || coord.length()>=7){ //Si se debe poner en forma exponencial
					exponente=true;
					int j=0;
					while(coord.substring(coord.length()-1).equals("0")){
						coord=coord.substring(0,coord.length()-1);
						j++;
					}
					
					coord=coord.concat("·10^").concat((new Integer(j)).toString());
				}
				
				anchotexto=tamanoletra.stringWidth(coord);
				if(!coord.equals("0")){
					ggraficoAnterior.drawString(coord, coordenadaPant-anchotexto/2, centroY.intValue()+8+altotexto);
				}
			}
			
			//Eje Y
			centroX=convCoordPantX(BigDecimal.ZERO);
			if(centroX.compareTo(new BigInteger("30"))<0){
				centroX=new BigInteger((new Integer(30)).toString());
				ggraficoAnterior.setColor(colorEjesAtenuados);
			}else if(centroX.compareTo(new BigInteger((new Integer(ancho-30)).toString()))>0){ //Si se sale de la pantalla
				centroX=new BigInteger((new Integer(ancho-30)).toString());
				ggraficoAnterior.setColor(colorEjesAtenuados);
			}else{
				ggraficoAnterior.setColor(colorEjes);
			}
			ggraficoAnterior.setFont(tipoLetra); //Opciones de fuente para las marcas
			tamanoletra=ggraficoAnterior.getFontMetrics();
			altotexto=tamanoletra.getAscent();
	
			ggraficoAnterior.drawLine(centroX.intValue(), 0, centroX.intValue(), alto);
			
			//Lo que sigue reliza las marcas en los ejes
			numMarcas=(new Double(alto/pixelesMarcasY)).intValue(); //Cantidad de marcas que se realizarán
			numMarcas= numMarcas==0 ? 1 : numMarcas;
			anchoMarcas=anchoEjeY.divide(new BigDecimal(numMarcas), presicion, BigDecimal.ROUND_HALF_EVEN); //El ancho que tendrá cada marca
			if(anchoMarcas.compareTo(BigDecimal.ONE)>=0){
				stringAnchoMarcas=anchoMarcas.toPlainString();
				cifra=new Integer(stringAnchoMarcas.substring(0,1)); //El ancho de cada marca
				potencia=stringAnchoMarcas.indexOf(".") - 1;
			}else if(anchoMarcas.compareTo(new BigDecimal(0.7))>=0){
				cifra=1;
				potencia=0;
			}else if(anchoMarcas.compareTo(new BigDecimal(0.3))>=0){
				cifra=5;
				potencia=-1;
			}else if(anchoMarcas.compareTo(new BigDecimal(0.15))>=0){
				cifra=2;
				potencia=-1;
			}else if(anchoMarcas.compareTo(new BigDecimal(0.09))>=0){
				cifra=1;
				potencia=-1;
			}else{
				stringAnchoMarcas=anchoMarcas.toPlainString();
				
				if(stringAnchoMarcas.indexOf("E")!=-1){
					potencia=new Integer(stringAnchoMarcas.substring(stringAnchoMarcas.indexOf("E") + 1));
				}else{
					stringAnchoMarcas=stringAnchoMarcas.substring(stringAnchoMarcas.indexOf(".") + 1);
					potencia=-1;
					while(stringAnchoMarcas.substring(0,1).equals("0")){
						stringAnchoMarcas=stringAnchoMarcas.substring(1);
						potencia--;
					}
				}
				cifra=new Integer(stringAnchoMarcas.substring(0,1)); //El ancho de cada marca
			}
			
			if(potencia>=0){
				anchoMarcas=(new BigDecimal(cifra)).multiply((new BigDecimal(10)).pow(potencia)).stripTrailingZeros();
			}else{
				anchoMarcas=(new BigDecimal(cifra)).divide((new BigDecimal(10)).pow(-1*potencia), presicion, BigDecimal.ROUND_HALF_EVEN).stripTrailingZeros();
			}
			//inicio=xmin.divide(anchoMarcas, xmin.toString().indexOf(".")-Math.abs(potencia), BigDecimal.ROUND_CEILING).multiply(anchoMarcas).round(new MathContext(Math.abs(potencia), RoundingMode.HALF_EVEN)); //El valor donde se pone la primera marca
			inicio=ymin.divideToIntegralValue(anchoMarcas).multiply(anchoMarcas);//.round(new MathContext(Math.abs(potencia), RoundingMode.HALF_EVEN)); //El valor donde se pone la primera marca
			numMarcas=anchoEjeY.divide(anchoMarcas, presicion, BigDecimal.ROUND_HALF_EVEN).round(new MathContext(presicion, RoundingMode.HALF_EVEN)).intValue();
			
			exponente=false;
			for(int i=0; i<=numMarcas; i++){ //Dibujo de las marcas
				coordenada=inicio.add(new BigDecimal(i).multiply(anchoMarcas));
				coordenadaPant=convCoordPantY(coordenada).intValue();
				ggraficoAnterior.drawLine(centroX.intValue(), coordenadaPant, centroX.intValue()+5, coordenadaPant);
	
				coord=coordenada.stripTrailingZeros().toPlainString(); //new Double(coordenada - (coordenada%anchoMarcas)).toString(); //(new Long(Math.round(coordenada))).toString();
				
				if(coord.indexOf(".")!=-1){
					while(coord.substring(coord.length()-1).equals("0")){
						coord=coord.substring(0,coord.length()-1);
					}
					if(coord.substring(coord.length()-1).equals(".")){
						coord=coord.substring(0,coord.length()-1);
					}
					
					if(coord.substring(0,1).equals("-")){
						negativo=true;
						coordcomodin=coord.substring(1);
					}else{
						negativo=false;
						coordcomodin=coord;
					}
					
					cont=-1;
					while(!coordcomodin.equals("") && (coordcomodin.substring(0,1).equals("0") || coordcomodin.substring(0,1).equals("."))){
						coordcomodin=coordcomodin.substring(1);
						cont++;
					}
					if(coordcomodin.equals("")){
						coord="0";
					}
					
					if(coordcomodin.indexOf(".")==-1){
						cont+=coordcomodin.length()-1;
					}
					
					if(cont>=3 && cont+coordcomodin.length()>=5){
						if(negativo){
							coord="-".concat(coordcomodin).concat("·10^").concat((new Integer(-1*cont)).toString());
						}else{
							coord=coordcomodin.concat("·10^").concat((new Integer(-1*cont)).toString());
						}
					}
	
				}else if(exponente || coord.length()>=7){ //Si se debe poner en forma exponencial
					exponente=true;
					int j=0;
					while(coord.substring(coord.length()-1).equals("0")){
						coord=coord.substring(0,coord.length()-1);
						j++;
					}
					
					coord=coord.concat("·10^").concat((new Integer(j)).toString());
				}
				
				altotexto=tamanoletra.getAscent();
				if(!coord.equals("0")){
					ggraficoAnterior.drawString(coord, centroX.intValue()+8, coordenadaPant+altotexto/2);
				}
			}
			
			//Se grafica un máximo de diez funciones
			exacto=Opciones.getPresicion();
			Opciones.setPresicion(false);
			presicion2=Opciones.getNumDecimales();
			Opciones.setNumDecimales(presicion);
			BigDecimal xviejo, yviejodecimal, xnuevo, ynuevodecimal;
			BigInteger yviejo, ynuevo;
			
			for(int j=0; j<=9; j++){
				if(Expresiones[j]!=null){
					if(alternar){
						ggraficoAnterior.setColor(colores[j]);
					}else{
						ggraficoAnterior.setColor(colorGrafica);
					}
					
					//Procedimiento que grafica la función
					xviejo=convPantCoordX(0);
					yviejodecimal=Evaluar(xviejo, j);
					if(yviejodecimal==null){
						yviejo=null;
					}else{
						yviejo=convCoordPantY(yviejodecimal);
					}
					for(int i=1; i<=ancho+numPuntos; i+=numPuntos){
						xnuevo=convPantCoordX(i);
						ynuevodecimal=Evaluar(xnuevo, j);
						if(ynuevodecimal==null){
							ynuevo=null;
						}else{
							ynuevo=convCoordPantY(ynuevodecimal);
						}
						
						if(yviejo!=null && ynuevo!=null){
							if((yviejo.compareTo(BigInteger.ZERO)>=0 && yviejo.compareTo(new BigInteger((new Integer(alto)).toString()))<=0) || (ynuevo.compareTo(BigInteger.ZERO)>=0 && ynuevo.compareTo(new BigInteger((new Integer(alto)).toString()))<=0)){ //Si no sale en la pantalla
								ggraficoAnterior.drawLine(i-numPuntos, yviejo.intValue(), i, ynuevo.intValue());
							}
						}
						
						xviejo=xnuevo;
						yviejo=ynuevo;
					}
				}
			}
			Opciones.setPresicion(exacto);
			Opciones.setNumDecimales(presicion2);
			
			Graphics2D g=(Graphics2D)ge;
			g.drawImage(graficoAnterior, null, 0, 0);
			
			repintar=false;
		}else{
			Graphics2D g=(Graphics2D)ge;
			g.drawImage(graficoAnterior, null, 0, 0);
			
			//Si se está definiendo un cuadro de zoom
			if(cuadro){
				int inicioCuadroX, inicioCuadroY;
				
				g.setColor(Color.GRAY);
				inicioCuadroX= xmouse<xmouse2 ? xmouse : xmouse2;
				inicioCuadroY= ymouse<ymouse2 ? ymouse : ymouse2;
				g.drawRect(inicioCuadroX, inicioCuadroY, Math.abs(xmouse2-xmouse), Math.abs(ymouse2-ymouse));
			}
		}
	}
	
	/*
	 *Función que evalúa la expresión i-ésima en el valor a.
	 */
	private BigDecimal Evaluar(BigDecimal a, int i){// throws ArithmeticException{
		Nodo exp, comodin;
		String tipo;
		
		exp=new Nodo("eval", 3);
		exp.agregaOperando(Expresiones[i].clonarNodo());
		exp.agregaOperando(Variables[i]);
		
		comodin=new Nodo("DecimalLargo",1);
		comodin.agregaOperando(new Nodo(a.toString()));
		
		exp.agregaOperando(comodin);
		
		exp=simplificar.simplifica(exp);
		tipo=exp.getTipo();
		if(tipo.equals("Indefinido") || tipo.length()<6 || "DecimalLargo".indexOf(tipo)==-1){
			return null;
		}
		
		return new BigDecimal(exp.getOperando(1).getTipo());
	}
	
	//FUNCIONES PUBLICAS
	
	/**
	 *Función que agrega la expresión dada.
	 *@param e La expresión para graficar.
	 */
	public void agregaExpresion(Nodo e){
		if(e.getTipo().equals(":=") || (e.getTipo().equals("=") && e.getOperando(1).getTipo().equals("Variable"))){
			e=e.getOperando(2);
		}
		
		if(Expresiones[expresionUltima]!=null && e.esIgual(Expresiones[expresionUltima])){
			numColores=(++numColores)%numTotalColores;
			colores[expresionUltima]=todosColores[numColores];
			repintar=true;
			this.repaint();
		}else if(verificarExpresion(e)){
			expresionUltima=(++expresionUltima)%10;
			Expresiones[expresionUltima]=e;
			numColores=(++numColores)%numTotalColores;
			colores[expresionUltima]=todosColores[numColores];
			repintar=true;
			this.repaint();
		}else{
			JOptionPane.showMessageDialog(this,"Hay un error en la expresión al graficar.","Error",JOptionPane.ERROR_MESSAGE);
		}
	}
	
	/**
	 *Función que borra la gráfica actual.
	 */
	public void borraExpresion(){
		if(Expresiones[expresionUltima]!=null){
			Expresiones[expresionUltima]=null;
			expresionUltima=(--expresionUltima)%10;
			repintar=true;
			this.repaint();
		}
	}
	
	/**
	 *Función que borra la gráfica indicada.
	 *@param i El número de gráfica a borrar.
	 */
	public void borraExpresion(int i){
		Expresiones[i]=null;
		if(i==expresionUltima){
			expresionUltima=(--expresionUltima)%10;
		}
		repintar=true;
		this.repaint();
	}
	
	/**
	 *Función que borra todas las gráficas.
	 */
	public void borraTodas(){
		Expresiones=new Nodo[10];
		expresionUltima=0;
		repintar=true;
		this.repaint();
	}
	
	/**
	 *Función que devuelve el valor mínimo en x
	 *representado en este objeto.
	 *@return El mínimo valor en x.
	 */
	public BigDecimal getMinX(){
		return xmin;
	}
	
	/**
	 *Función que define el valor mínimo en x
	 *representado en este objeto.
	 *@param nuevoMinX El nuevo valor mínimo para x.
	 */
	public void setMinX(int nuevoMinX){
		xmin=new BigDecimal(nuevoMinX);
		repintar=true;
		repaint();
	}
	
	/**
	 *Función que define el valor mínimo en x
	 *representado en este objeto.
	 *@param nuevoMinX El nuevo valor mínimo para x.
	 */
	public void setMinX(double nuevoMinX){
		xmin=new BigDecimal(nuevoMinX);
		repintar=true;
		repaint();
	}
	
	/**
	 *Función que define el valor mínimo en x
	 *representado en este objeto.
	 *@param nuevoMinX El nuevo valor mínimo para x.
	 */
	public void setMinX(BigDecimal nuevoMinX){
		xmin=nuevoMinX;
		repintar=true;
		repaint();
	}
	
	/**
	 *Función que devuelve el valor máximo en x
	 *representado en este objeto.
	 *@return El máximo valor en x.
	 */
	public BigDecimal getMaxX(){
		return xmax;
	}
	
	/**
	 *Función que define el valor máximo en x
	 *representado en este objeto.
	 *@param nuevoMaxX El nuevo valor máximo para x.
	 */
	public void setMaxX(int nuevoMaxX){
		xmax=new BigDecimal(nuevoMaxX);
		repintar=true;
		repaint();
	}
	
	/**
	 *Función que define el valor máximo en x
	 *representado en este objeto.
	 *@param nuevoMaxX El nuevo valor máximo para x.
	 */
	public void setMaxX(double nuevoMaxX){
		xmax=new BigDecimal(nuevoMaxX);
		repintar=true;
		repaint();
	}
	
	/**
	 *Función que define el valor máximo en x
	 *representado en este objeto.
	 *@param nuevoMaxX El nuevo valor máximo para x.
	 */
	public void setMaxX(BigDecimal nuevoMaxX){
		xmax=nuevoMaxX;
		repintar=true;
		repaint();
	}
	
	/**
	 *Función que devuelve el valor mínimo en y
	 *representado en este objeto.
	 *@return El mínimo valor en y.
	 */
	public BigDecimal getMinY(){
		return ymin;
	}
	
	/**
	 *Función que define el valor mínimo en y
	 *representado en este objeto.
	 *@param nuevoMinY El nuevo valor mínimo para y.
	 */
	public void setMinY(int nuevoMinY){
		ymin=new BigDecimal(nuevoMinY);
		repintar=true;
		repaint();
	}
	
	/**
	 *Función que define el valor mínimo en y
	 *representado en este objeto.
	 *@param nuevoMinY El nuevo valor mínimo para y.
	 */
	public void setMinY(double nuevoMinY){
		ymin=new BigDecimal(nuevoMinY);
		repintar=true;
		repaint();
	}
	
	/**
	 *Función que define el valor mínimo en y
	 *representado en este objeto.
	 *@param nuevoMinY El nuevo valor mínimo para y.
	 */
	public void setMinY(BigDecimal nuevoMinY){
		ymin=nuevoMinY;
		repintar=true;
		repaint();
	}
	
	/**
	 *Función que devuelve el valor máximo en y
	 *representado en este objeto.
	 *@return El máximo valor en y.
	 */
	public BigDecimal getMaxY(){
		return ymax;
	}
	
	/**
	 *Función que define el valor máximo en y
	 *representado en este objeto.
	 *@param nuevoMaxY El nuevo valor máximo para y.
	 */
	public void setMaxY(int nuevoMaxY){
		ymax=new BigDecimal(nuevoMaxY);
		repintar=true;
		repaint();
	}
	
	/**
	 *Función que define el valor máximo en y
	 *representado en este objeto.
	 *@param nuevoMaxY El nuevo valor máximo para y.
	 */
	public void setMaxY(double nuevoMaxY){
		ymax=new BigDecimal(nuevoMaxY);
		repintar=true;
		repaint();
	}
	
	/**
	 *Función que define el valor máximo en y
	 *representado en este objeto.
	 *@param nuevoMaxY El nuevo valor máximo para y.
	 */
	public void setMaxY(BigDecimal nuevoMaxY){
		ymax=nuevoMaxY;
		repintar=true;
		repaint();
	}
	
	/**
	 *Función que cambia el color de gráfica cuando es fijo.
	 *@param color El nuevo color.
	 */
	public void setColorGrafica(Color color){
		colorGrafica=color;
		repintar=true;
		this.repaint();
	}
	
	/**
	 *Función que cambia el color del fondo para graficar.
	 *@param color El nuevo color.
	 */
	public void setColorFondo(Color color){
		colorFondo=color;
		repintar=true;
		this.repaint();
	}
	
	/**
	 *Función que cambia el color de los ejes atenuados de la grafica.
	 *@param color El nuevo color.
	 */
	public void setColorEjesAtenuados(Color color){
		colorEjesAtenuados=color;
		repintar=true;
		this.repaint();
	}
	
	/**
	 *Función que cambia el color de los ejes de la grafica.
	 *@param color El nuevo color.
	 */
	public void setColorEjes(Color color){
		colorEjes=color;
		repintar=true;
		this.repaint();
	}
	
	/**
	 *Función que cambia la opción de alternar colores o usar uno fijo para graficar.
	 *@param alterna True si alterna o false en otro caso.
	 */
	public void setAlternarColor(boolean alterna){
		alternar=alterna;
		repintar=true;
		this.repaint();
	}
	
	/**
	 *Función que cambia el tipo de letra con que se pondrán los números en los ejes.
	 *@param letra La nueva letra.
	 */
	public void setTipoLetra(Font letra){
		if(tipoLetra!=letra){
			tipoLetra=letra;
			repintar=true;
			this.repaint();
		}
	}
	
	/**
	 *Función que vuelve a tomar los valores seleccionados en las opciones.
	 */
	public void retomarValores(){
		colorFondo=Opciones.getColorFondoGrafica();
		colorEjesAtenuados=Opciones.getColorEjesAtenuadosGrafica();
		colorEjes=Opciones.getColorEjesGrafica();
		colorGrafica=Opciones.getColorGrafica();
		alternar=Opciones.getAlternarColorGrafica();
		tipoLetra=Opciones.getTipoLetraEjesGrafica();
		numPuntos=Opciones.getNumPuntosGrafica();
		pixelesMarcasX=Opciones.getPixelesMarcasX();
		pixelesMarcasY=Opciones.getPixelesMarcasY();
		repintar=true;
		this.repaint();
	}
	
	/**
	 *Función que realiza un zoom hacia adentro en la gráfica.
	 */
	public void zoomIn(){
		zoom(1, 0);
	}
	
	/**
	 *Función que realiza un zoom vertical hacia adentro en la gráfica.
	 */
	public void zoomInVertical(){
		zoom(1, 2);
	}
	
	/**
	 *Función que realiza un zoom horizontal hacia adentro en la gráfica.
	 */
	public void zoomInHorizontal(){
		zoom(1, 1);
	}
	
	/**
	 *Función que realiza un zoom hacia afuera en la gráfica.
	 */
	public void zoomOut(){
		zoom(-1, 0);
	}
	
	/**
	 *Función que realiza un zoom vertical hacia afuera en la gráfica.
	 */
	public void zoomOutVertical(){
		zoom(-1,2);
	}
	
	/**
	 *Función que realiza un zoom horizontal hacia afuera en la gráfica.
	 */
	public void zoomOutHorizontal(){
		zoom(-1,1);
	}
	
	//FUNCIONES PRIVADAS
	/*
	 * Función que maneja el zoom
	 */
	private void zoom(int num, int forma){
		BigDecimal movimiento, comodin;
		
		if(forma==0 || forma==1){
			movimiento=xmax.subtract(xmin).multiply(new BigDecimal(num)).divide(new BigDecimal(10)); //convPixCoordX(-50);
			xmin=xmin.add(movimiento);
			xmax=xmax.subtract(movimiento);
			
			if(xmin.compareTo(xmax)==1){
				comodin=xmin;
				xmin=xmax;
				xmax=comodin;
			}
		}
		
		if(forma==0 || forma==2){
			movimiento=ymax.subtract(ymin).multiply(new BigDecimal(num)).divide(new BigDecimal(10)); //convPixCoordY(-50);
			ymin=ymin.add(movimiento);
			ymax=ymax.subtract(movimiento);
			
			if(ymin.compareTo(ymax)==1){
				comodin=ymin;
				ymin=ymax;
				ymax=comodin;
			}
		}
		
		repintar=true;
		this.repaint();
	}
	
	/*
	 *Convierte de coordenadas de la pantalla al
	 *sistema de coordenadas representado,
	 *todo esto en el eje x
	 */
	private BigDecimal convPantCoordX(int valor){
		return xmin.add(new BigDecimal(valor).multiply(anchoEjeX).divide(new BigDecimal(ancho), presicion, BigDecimal.ROUND_HALF_EVEN));
	}
	
	/*
	 *Convierte del sistema de coordenadas representado al
	 *sistema de coordenadas de pantalla, todo esto
	 *en el eje x.
	 */
	private BigInteger convCoordPantX(BigDecimal valor){
		return (valor).subtract(xmin).multiply(new BigDecimal(ancho)).divide(anchoEjeX, presicion, BigDecimal.ROUND_HALF_EVEN).round(new MathContext(presicion, RoundingMode.HALF_EVEN)).toBigInteger();
	}
	
	/*
	 *Convierte de coordenadas de la pantalla al
	 *sistema de coordenadas representado,
	 *todo esto en el eje y
	 */
	private BigDecimal convPantCoordY(int valor){
		return ymax.subtract(new BigDecimal(valor).multiply(anchoEjeY).divide(new BigDecimal(alto), presicion, BigDecimal.ROUND_HALF_EVEN));
	}
	
	/*
	 *Convierte del sistema de coordenadas representado al
	 *sistema de coordenadas de pantalla, todo esto
	 *en el eje y.
	 */
	private BigInteger convCoordPantY(BigDecimal valor){
		return ymax.subtract(valor).multiply(new BigDecimal(alto)).divide(anchoEjeY, presicion, BigDecimal.ROUND_HALF_EVEN).round(new MathContext(presicion, RoundingMode.HALF_EVEN)).toBigInteger();
	}
	
	/*
	 *Convierte una cantidad de pixeles dada
	 *a su correspondiente cantidad en las
	 *coordenadas representadas en el eje X.
	 */
	private BigDecimal convPixCoordX(int pixeles){
		return new BigDecimal(pixeles).multiply(anchoEjeX).divide(new BigDecimal(ancho), presicion, BigDecimal.ROUND_HALF_EVEN);
	}
	
	/*
	 *Convierte una cantidad de unidades dada
	 *en el sistema de coordenadas representado
	 *a su correspondiente cantidad en pixeles
	 *en el eje X.
	 */
	/*private BigInteger convCoordPixX(BigDecimal unidades){
		return unidades.multiply(new BigDecimal(ancho)).divide(anchoEjeX, presicion, BigDecimal.ROUND_HALF_EVEN).round(new MathContext(presicion, RoundingMode.HALF_EVEN)).toBigInteger();
	}*/
	
	/*
	 *Convierte una cantidad de pixeles dada
	 *a su correspondiente cantidad en las
	 *coordenadas representadas en el eje X.
	 */
	private BigDecimal convPixCoordY(int pixeles){
		return new BigDecimal(pixeles).multiply(anchoEjeY).divide(new BigDecimal(alto), presicion, BigDecimal.ROUND_HALF_EVEN);
	}
	
	/*
	 *Convierte una cantidad de unidades dada
	 *en el sistema de coordenadas representado
	 *a su correspondiente cantidad en pixeles
	 *en el eje Y.
	 */
	/*private BigInteger convCoordPixY(BigDecimal unidades){
		return unidades.multiply(new BigDecimal(ancho)).divide(anchoEjeY, presicion, BigDecimal.ROUND_HALF_EVEN).round(new MathContext(presicion, RoundingMode.HALF_EVEN)).toBigInteger();
	}*/
	
	/*
	 *Función que verifica si la expresión a graficar tiene un formato válido,
	 *es decir:
	 *con una variable
	 *y=f(x)
	 *f(x):=g(x)
	 */
	private boolean verificarExpresion(Nodo e){
		variables=new Vector<Nodo>();
		
		buscaVariables(e);
		
		if(variables.size()==1){
			Variables[expresionUltima+1]=(Nodo)variables.elementAt(0);
			return true;
		}else if(variables.size()==0){
			Nodo comodin=new Nodo("Variable");
			comodin.agregaOperando(new Nodo("x"));
			Variables[expresionUltima+1]=comodin;
			return true;
		}else{
			return false;
		}
	}
	
	/*
	 *Función que busca las variables de la expresión e
	 */
	private void buscaVariables(Nodo e){
		if(e.getTipo().equals("Variable")){
			boolean encontrado=false;
			int i=0;
			while(!encontrado && i<variables.size()){
				if(((Nodo)variables.elementAt(i)).esIgual(e)){
					encontrado=true;
				}
				i++;
			}
			if(!encontrado){
				variables.add(e);
				return;
			}
		}
		
		for(int i=1; i<=e.getNumeroOperandos();i++){
			buscaVariables(e.getOperando(i));
		}
	}
	
	//EVENTOS DEL MOUSE
	
	public void mouseExited(MouseEvent e){
	}
	
	public void mouseEntered(MouseEvent e){
	}
	
	public void mouseReleased(MouseEvent e){
		xmouse2=e.getX();
		ymouse2=e.getY();
		
		if(boton==MouseEvent.BUTTON2){
			cuadro=false;
			
			if(xmouse==xmouse2){
				xmouse2++;
			}
			
			if(ymouse==ymouse2){
				ymouse2++;
			}
			
			if(xmouse<xmouse2){
				xmax=convPantCoordX(xmouse2);
				xmin=convPantCoordX(xmouse);
			}else{
				xmax=convPantCoordX(xmouse);
				xmin=convPantCoordX(xmouse2);
			}
			
			if(ymouse<ymouse2){
				ymin=convPantCoordY(ymouse2);
				ymax=convPantCoordY(ymouse);
			}else{
				ymin=convPantCoordY(ymouse);
				ymax=convPantCoordY(ymouse2);
			}
			
			repintar=true;
			this.repaint();
		}
	}
	
	public void mousePressed(MouseEvent e){
		xmouse=e.getX();
		ymouse=e.getY();
		boton=e.getButton();
	}
	
	public void mouseClicked(MouseEvent e){
		if(e.getClickCount()>=2){
			if(e.getButton()==MouseEvent.BUTTON1){
				xmin=new BigDecimal(-5);
				xmax=new BigDecimal(5);
				ymin=new BigDecimal(-3);
				ymax=new BigDecimal(3);
			}else if(e.getButton()==MouseEvent.BUTTON3){
				BigDecimal movimiento;
				int x=e.getX();
				int y=e.getY();
				movimiento=convPixCoordX(ancho/2-x);
				xmin=xmin.subtract(movimiento);
				xmax=xmax.subtract(movimiento);
				
				movimiento=convPixCoordY(alto/2-y);
				ymin=ymin.add(movimiento);
				ymax=ymax.add(movimiento);
			}
			repintar=true;
			this.repaint();
		}
	}
	
	public void mouseMoved(MouseEvent e){
	}
	
	public void mouseDragged(MouseEvent e){
		BigDecimal movimiento, comodin;
		
		xmouse2=e.getX();
		ymouse2=e.getY();
		
		if(boton==MouseEvent.BUTTON1){
			movimiento=convPixCoordX(xmouse2-xmouse);
			xmin=xmin.subtract(movimiento);
			xmax=xmax.subtract(movimiento);
			
			movimiento=convPixCoordY(ymouse2-ymouse);
			ymin=ymin.add(movimiento);
			ymax=ymax.add(movimiento);
			
			xmouse=xmouse2;
			ymouse=ymouse2;
			repintar=true;
			this.repaint();
		}
		
		if(boton==MouseEvent.BUTTON2){
			cuadro=true;
			//repintar=true;
			this.repaint();
		}
		
		if(boton==MouseEvent.BUTTON3){
			movimiento=convPixCoordX(xmouse2-xmouse);
			xmin=xmin.subtract(movimiento);
			xmax=xmax.add(movimiento);
			
			if(xmin.compareTo(xmax)==1){
				comodin=xmin;
				xmin=xmax;
				xmax=comodin;
			}
			
			movimiento=convPixCoordY(ymouse2-ymouse);
			ymin=ymin.add(movimiento);
			ymax=ymax.subtract(movimiento);
			
			if(ymin.compareTo(ymax)==1){
				comodin=ymin;
				ymin=ymax;
				ymax=comodin;
			}
			
			xmouse=xmouse2;
			ymouse=ymouse2;
			
			repintar=true;
			this.repaint();
		}
	}
	
	public void mouseWheelMoved(MouseWheelEvent e){
		int num=e.getWheelRotation();
		zoom(num, 0);
	}
}