package itcr.mathexplorer.paneles;
 
import java.awt.*;
import javax.swing.*;
import java.math.*;
import java.awt.event.*;
import java.awt.image.*;
import java.util.*;
import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.herramientas.*;
import itcr.mathexplorer.tablas.*;
import itcr.mathexplorer.funciones.Simplificar;

/**
 * @(#)PanelGraficos3D.java
 *
 * Panel para la graficación de funciones en tres dimensiones
 *
 * @author Alexander Borbón Alpízar
 * @version 1.00 2007/11/1
 */
public class PanelGraficos3D extends JPanel implements MouseListener, MouseMotionListener, MouseWheelListener{
	static final long serialVersionUID=10014;
	
	//VARIABLES PRIVADAS
	//Este es el total de gráficas que se pueden realizar 
	private final int totalGraficas=10;
	//Este es el número de colores para graficar
	private final int numTotalColores=10;
	
	//Esta es la tabla que contiene a todas las funciones permitidas
	TablaFunciones funciones;
	
	//Esta es la tabla que contiene a todas las constantes permitidas
	TablaConstantes constantes;
	
	//Esta es la tabla que contiene a todos los operadores permitidos
	TablaOperadores operadores;
	
	//Esta es la tabla que contiene a todas las funciones permitidas
	TablaFuncionesUsuario funcionesUsuario;
	
	//El tamaño a graficar en coordenadas reales, se graficará de (xmin, ymin) a (xmax, ymax)
	private BigDecimal xmin, xmax, ymin, ymax, zmin, zmax; 
	private int ancho, alto, anchoAnterior, altoAnterior; //tamaño de la pantalla en pixeles
	private BigDecimal anchoEjeX, anchoEjeY, anchoEjeZ;
	private int presicion=40, pixelesMarcasX=40, pixelesMarcasY=40, pixelesMarcasZ=40;
	//Variables para los movimientos del mouse
	private int xmouse, ymouse, xmouse2, ymouse2, boton;
	
	private double anguloX, anguloY, anguloZ, anguloy, anguloz;// Variable que maneja el ángulo para la rotación.
	private Punto3D primerVector, segundoVector; //Vectores para sacar el vector de rotación.
	private boolean primeraVez;
	
	//Arreglo donde se guardar todas las expresiones a graficar
	private Nodo[] Expresiones;
	private Nodo[][] Variables;
	private int expresionUltima;//, total;
	private Vector<Nodo> variables; //variable donde se guarda la variable de la expresión ingresada.
	
	//Arreglo que contiene los colores para realizar las gráficas
	private Color[] colores, todosColores;
	private int numColores;
	
	private boolean exacto, alternar, repintar=true, evaluar;//, cuadro;
	
	//Función para simplificar las expresiones
	Simplificar simplificar;
	
	//Opciones
	Color colorFondo, colorEjesAtenuados, colorEjes, colorGrafica;
	Font tipoLetra;
	int numPuntos;
	
	//Se mantiene una copia anterior del gráfico para aumentar la velocidad del repintado
	BufferedImage graficoAnterior;
	
	//Arreglo que contiene la maya de puntos a graficar
	Punto3D[][][] mayaPuntos;
	
	//CONSTRUCTORES
	
	/**
	 *Constructor que define el área a grafica.
	 *Por defecto se grafica desde -5 a 5 en el eje
	 *X, de -5 a 5 en el eje Y y de -5 a 5 en el eje Z.
	 *@param tablaOperadores Tabla con todos los operadores del programa.
	 *@param tablaFunciones Tabla con todas las funciones del programa.
	 *@param tablaConstantes Tabla con todas las constantes del programa.
	 *@param tablaFuncionesUsuario Tabla con todas las funciones definidas por el usuario en el programa.
	 */
	public PanelGraficos3D(TablaOperadores tablaOperadores, TablaFunciones tablaFunciones, TablaConstantes tablaConstantes, TablaFuncionesUsuario tablaFuncionesUsuario){
		constructor(new BigDecimal(-5), new BigDecimal(5), new BigDecimal(-5), new BigDecimal(5), new BigDecimal(-5), new BigDecimal(5), tablaOperadores, tablaFunciones, tablaConstantes, tablaFuncionesUsuario);
	}
	
	/**
	 *Constructor vacío que define el área a grafica.
	 *Por defecto se grafica desde -5 a 5 en el eje
	 *X, de -5 a 5 en el eje Y y de -5 a 5 en el eje Z.
	 *@param e La expresión inicial a graficar
	 *@param tablaOperadores Tabla con todos los operadores del programa.
	 *@param tablaFunciones Tabla con todas las funciones del programa.
	 *@param tablaConstantes Tabla con todas las constantes del programa.
	 *@param tablaFuncionesUsuario Tabla con todas las funciones definidas por el usuario en el programa.
	 */
	public PanelGraficos3D(Nodo e, TablaOperadores tablaOperadores, TablaFunciones tablaFunciones, TablaConstantes tablaConstantes, TablaFuncionesUsuario tablaFuncionesUsuario){
		constructor(new BigDecimal(-5), new BigDecimal(5), new BigDecimal(-5), new BigDecimal(5), new BigDecimal(-5), new BigDecimal(5), tablaOperadores, tablaFunciones, tablaConstantes, tablaFuncionesUsuario);
		agregaExpresion(e);
	}
	
	/**
	 *Constructor que define el área a grafica.
	 *En este caso se grafica desde minX a maxX en el eje
	 *X, de minY a maxY en el eje Y y de minZ a maxZ en el eje Z.
	 *Si los valores mínimos son mayores a los máximos
	 *entonces se define de -3 a 3 en el eje correspondiente.
	 *@param minX El valor mínimo en el eje x a graficar.
	 *@param maxX El valor máximo en el eje x a graficar.
	 *@param minY El valor mínimo en el eje y a graficar.
	 *@param maxY El valor máximo en el eje y a graficar.
	 *@param minZ El valor mínimo en el eje z a graficar.
	 *@param maxZ El valor máximo en el eje z a graficar.
	 *@param tablaOperadores Tabla con todos los operadores del programa.
	 *@param tablaFunciones Tabla con todas las funciones del programa.
	 *@param tablaConstantes Tabla con todas las constantes del programa.
	 *@param tablaFuncionesUsuario Tabla con todas las funciones definidas por el usuario en el programa.
	 */
	public PanelGraficos3D(double minX, double maxX, double minY, double maxY, double minZ, double maxZ, TablaOperadores tablaOperadores, TablaFunciones tablaFunciones, TablaConstantes tablaConstantes, TablaFuncionesUsuario tablaFuncionesUsuario){
		constructor(new BigDecimal(minX), new BigDecimal(maxX), new BigDecimal(minY), new BigDecimal(maxY), new BigDecimal(minZ), new BigDecimal(maxZ), tablaOperadores, tablaFunciones, tablaConstantes, tablaFuncionesUsuario);
	}
	
	/**
	 *Constructor que define el área a grafica.
	 *En este caso se grafica desde minX a maxX en el eje
	 *X y de minY a maxY en el eje Y.
	 *Si los valores mínimos son mayores a los máximos
	 *entonces se define de -3 a 3 en el eje correspondiente.
	 *@param e La expresión inicial a graficar
	 *@param minX El valor mínimo en el eje x a graficar.
	 *@param maxX El valor máximo en el eje x a graficar.
	 *@param minY El valor mínimo en el eje y a graficar.
	 *@param maxY El valor máximo en el eje y a graficar.
	 *@param minZ El valor mínimo en el eje z a graficar.
	 *@param maxZ El valor máximo en el eje z a graficar.
	 *@param tablaOperadores Tabla con todos los operadores del programa.
	 *@param tablaFunciones Tabla con todas las funciones del programa.
	 *@param tablaConstantes Tabla con todas las constantes del programa.
	 *@param tablaFuncionesUsuario Tabla con todas las funciones definidas por el usuario en el programa.
	 */
	public PanelGraficos3D(Nodo e, double minX, double maxX, double minY, double maxY, double minZ, double maxZ, TablaOperadores tablaOperadores, TablaFunciones tablaFunciones, TablaConstantes tablaConstantes, TablaFuncionesUsuario tablaFuncionesUsuario){
		constructor(new BigDecimal(minX), new BigDecimal(maxX), new BigDecimal(minY), new BigDecimal(maxY), new BigDecimal(minZ), new BigDecimal(maxZ), tablaOperadores, tablaFunciones, tablaConstantes, tablaFuncionesUsuario);
		agregaExpresion(e);
	}
	
	/**
	 *Constructor que define el área a grafica.
	 *En este caso se grafica desde minX a maxX en el eje
	 *X y de minY a maxY en el eje Y.
	 *Si los valores mínimos son mayores a los máximos
	 *entonces se define de -3 a 3 en el eje correspondiente.
	 *@param minX El valor mínimo en el eje x a graficar.
	 *@param maxX El valor máximo en el eje x a graficar.
	 *@param minY El valor mínimo en el eje y a graficar.
	 *@param maxY El valor máximo en el eje y a graficar.
	 *@param minZ El valor mínimo en el eje z a graficar.
	 *@param maxZ El valor máximo en el eje z a graficar.
	 *@param tablaOperadores Tabla con todos los operadores del programa.
	 *@param tablaFunciones Tabla con todas las funciones del programa.
	 *@param tablaConstantes Tabla con todas las constantes del programa.
	 *@param tablaFuncionesUsuario Tabla con todas las funciones definidas por el usuario en el programa.
	 */
	public PanelGraficos3D(BigDecimal minX, BigDecimal maxX, BigDecimal minY, BigDecimal maxY, BigDecimal minZ, BigDecimal maxZ, TablaOperadores tablaOperadores, TablaFunciones tablaFunciones, TablaConstantes tablaConstantes, TablaFuncionesUsuario tablaFuncionesUsuario){
		constructor(minX, maxX, minY, maxZ, minY, maxZ, tablaOperadores, tablaFunciones, tablaConstantes, tablaFuncionesUsuario);
	}
	
	/**
	 *Constructor que define el área a grafica.
	 *En este caso se grafica desde minX a maxX en el eje
	 *X y de minY a maxY en el eje Y.
	 *Si los valores mínimos son mayores a los máximos
	 *entonces se define de -3 a 3 en el eje correspondiente.
	 *@param e La expresión inicial a graficar
	 *@param minX El valor mínimo en el eje x a graficar.
	 *@param maxX El valor máximo en el eje x a graficar.
	 *@param minY El valor mínimo en el eje y a graficar.
	 *@param maxY El valor máximo en el eje y a graficar.
	 *@param minZ El valor mínimo en el eje z a graficar.
	 *@param maxZ El valor máximo en el eje z a graficar.
	 *@param tablaOperadores Tabla con todos los operadores del programa.
	 *@param tablaFunciones Tabla con todas las funciones del programa.
	 *@param tablaConstantes Tabla con todas las constantes del programa.
	 *@param tablaFuncionesUsuario Tabla con todas las funciones definidas por el usuario en el programa.
	 */
	public PanelGraficos3D(Nodo e, BigDecimal minX, BigDecimal maxX, BigDecimal minY, BigDecimal maxY, BigDecimal minZ, BigDecimal maxZ, TablaOperadores tablaOperadores, TablaFunciones tablaFunciones, TablaConstantes tablaConstantes, TablaFuncionesUsuario tablaFuncionesUsuario){
		constructor(minX, maxX, minY, maxY, minZ, maxZ, tablaOperadores, tablaFunciones, tablaConstantes, tablaFuncionesUsuario);
		agregaExpresion(e);
	}
	
	private void constructor(BigDecimal minX, BigDecimal maxX, BigDecimal minY, BigDecimal maxY, BigDecimal minZ, BigDecimal maxZ, TablaOperadores tablaOperadores, TablaFunciones tablaFunciones, TablaConstantes tablaConstantes, TablaFuncionesUsuario tablaFuncionesUsuario){
		colorFondo=Opciones.getColorFondoGrafica3D();
		colorEjesAtenuados=Opciones.getColorEjesAtenuadosGrafica3D();
		colorEjes=Opciones.getColorEjesGrafica3D();
		colorGrafica=Opciones.getColorGrafica3D();
		alternar=Opciones.getAlternarColorGrafica3D();
		tipoLetra=Opciones.getTipoLetraEjesGrafica3D();
		numPuntos=Opciones.getNumPuntosGrafica3D();
		pixelesMarcasX=Opciones.getPixelesMarcasX3D();
		pixelesMarcasY=Opciones.getPixelesMarcasY3D();
		pixelesMarcasZ=Opciones.getPixelesMarcasZ3D();
		
		evaluar=true;
		primeraVez=true;
		anguloX=0;
		anguloY=0;
		anguloZ=0;
		primerVector=Punto3D.ZERO;
		segundoVector=Punto3D.ZERO;
		
		this.setBackground(colorFondo);
		
		if(minX.compareTo(maxX)==-1){
			xmin=minX;
			xmax=maxX;
		}else{
			xmin=new BigDecimal(-5);
			xmax=new BigDecimal(5);	
		}
		
		if(minY.compareTo(maxY)==-1){
			ymin=minY;
			ymax=maxY;
		}else{
			ymin=new BigDecimal(-5);
			ymax=new BigDecimal(5);
		}
		
		if(minZ.compareTo(maxZ)==-1){
			zmin=minZ;
			zmax=maxZ;
		}else{
			zmin=new BigDecimal(-5);
			zmax=new BigDecimal(5);
		}
		
		Expresiones=new Nodo[totalGraficas];
		Variables=new Nodo[totalGraficas][2];
		
		todosColores=new Color[numTotalColores];
		todosColores[0]=Color.BLACK;
		todosColores[1]=Color.BLUE;
		todosColores[2]=Color.GREEN;
		todosColores[3]=Color.RED;
		todosColores[4]=Color.YELLOW;
		todosColores[5]=Color.ORANGE;
		todosColores[6]=Color.CYAN;
		todosColores[7]=Color.MAGENTA;
		todosColores[8]=Color.PINK;
		todosColores[9]=Color.GRAY;
		numColores=0;
		
		colores=new Color[totalGraficas];
		
		expresionUltima=0;
		
		mayaPuntos=new Punto3D[totalGraficas][numPuntos+1][numPuntos+1];
		
		operadores=tablaOperadores;
		funciones=tablaFunciones;
		constantes=tablaConstantes;
		funcionesUsuario=tablaFuncionesUsuario;
		
		simplificar=new Simplificar(operadores, funciones, constantes, funcionesUsuario);
		
		this.addMouseListener(this);
		this.addMouseMotionListener(this);
		this.addMouseWheelListener(this);
	}
	
	//Método que repinta

	public void paintComponent(Graphics ge) {
		this.setBackground(colorFondo);
		super.paintComponent(ge);
		
		anchoAnterior=ancho;
		altoAnterior=alto;
		ancho=this.getWidth();
		alto=this.getHeight();
		
		if(anchoAnterior!=ancho || altoAnterior!=alto){
			repintar=true;
		}
		
		if(repintar){
			
			graficoAnterior= (BufferedImage)(this.createImage(ancho,alto));
            Graphics2D ggraficoAnterior = graficoAnterior.createGraphics();
            ggraficoAnterior.setBackground(colorFondo);
            ggraficoAnterior.clearRect(0,0,ancho,alto);
			ggraficoAnterior.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
			
			BigInteger centroX, centroY;
			int numMarcas, cont;
			int cifra, potencia, presicion2, presicion3;
			BigDecimal anchoMarcas, inicio;
			BigDecimal coordenada;
			int altotexto, anchotexto;
			String coord, coordcomodin, stringAnchoMarcas;
			FontMetrics tamanoletra;
			boolean negativo, exponente;
			Punto2D punto, punto2, punto3;
			
			anchoEjeX=xmax.subtract(xmin).stripTrailingZeros();
			anchoEjeY=ymax.subtract(ymin).stripTrailingZeros();
			anchoEjeZ=zmax.subtract(zmin).stripTrailingZeros();
			
			//Este procedimiento calcula la presicion necesaria para hacer un buen gráfico
			
			stringAnchoMarcas=anchoEjeX.toPlainString();
			if(stringAnchoMarcas.indexOf(".")!=-1){
				stringAnchoMarcas=stringAnchoMarcas.substring(stringAnchoMarcas.indexOf(".") + 1);
				presicion=1;
				while(stringAnchoMarcas.substring(0,1).equals("0")){
					stringAnchoMarcas=stringAnchoMarcas.substring(1);
					presicion++;
				}
			}else{
				presicion=1;
			}
			
			stringAnchoMarcas=anchoEjeY.toPlainString();
			if(stringAnchoMarcas.indexOf(".")!=-1){
				stringAnchoMarcas=stringAnchoMarcas.substring(stringAnchoMarcas.indexOf(".") + 1);
				presicion2=1;
				while(stringAnchoMarcas.substring(0,1).equals("0")){
					stringAnchoMarcas=stringAnchoMarcas.substring(1);
					presicion2++;
				}
			}else{
				presicion2=1;
			}
			
			stringAnchoMarcas=anchoEjeZ.toPlainString();
			if(stringAnchoMarcas.indexOf(".")!=-1){
				stringAnchoMarcas=stringAnchoMarcas.substring(stringAnchoMarcas.indexOf(".") + 1);
				presicion3=1;
				while(stringAnchoMarcas.substring(0,1).equals("0")){
					stringAnchoMarcas=stringAnchoMarcas.substring(1);
					presicion3++;
				}
			}else{
				presicion3=1;
			}
			
			presicion = Math.max(Math.max(presicion,presicion2),presicion3)+5;
			
			//EJES COORDENADOS
			
			//Eje X
			
			Punto2D puntoEjeInicio=Cambia3D2D(Rota3D(new Punto3D(xmin, new BigDecimal(0), new BigDecimal(0))));
			Punto2D puntoEjeFinal=Cambia3D2D(Rota3D(new Punto3D(xmax, new BigDecimal(0), new BigDecimal(0))));
			BigInteger ejeInicioX=convCoordPantY(puntoEjeInicio.getX());
			BigInteger ejeInicioY=convCoordPantZ(puntoEjeInicio.getY());
			BigInteger ejeFinalX=convCoordPantY(puntoEjeFinal.getX());
			BigInteger ejeFinalY=convCoordPantZ(puntoEjeFinal.getY());
			centroX=ejeInicioX.add(ejeFinalX).divide(new BigInteger("2"));
			centroY=ejeInicioY.add(ejeFinalY).divide(new BigInteger("2"));
			BigInteger movimientoX=BigInteger.ZERO, movimientoY=BigInteger.ZERO;
			
			ggraficoAnterior.setColor(colorEjes);
			
			if(centroX.compareTo(new BigInteger("30"))<0 || centroX.compareTo(new BigInteger((new Integer(ancho-30)).toString()))>0){
				movimientoX=(new BigInteger((new Long(ancho/2)).toString())).subtract(centroX);
				ggraficoAnterior.setColor(colorEjesAtenuados);
			}else if(centroY.compareTo(new BigInteger("30"))<0 || centroY.compareTo(new BigInteger((new Integer(alto-30)).toString()))>0){ //Si se sale de la pantalla
				movimientoY=(new BigInteger((new Long(alto/2)).toString())).subtract(centroY);
				ggraficoAnterior.setColor(colorEjesAtenuados);
			}
			
			ggraficoAnterior.setFont(tipoLetra); //Opciones de fuente para las marcas
			tamanoletra=ggraficoAnterior.getFontMetrics();
			altotexto=tamanoletra.getAscent();
			ggraficoAnterior.drawLine(ejeInicioX.add(movimientoX).intValue(), ejeInicioY.add(movimientoY).intValue(), ejeFinalX.add(movimientoX).intValue(), ejeFinalY.add(movimientoY).intValue());
			
			//Lo que sigue reliza las marcas en los ejes
			numMarcas=(new Double(ancho/pixelesMarcasX)).intValue(); //Cantidad de marcas que se realizarán
			numMarcas= numMarcas==0 ? 1 : numMarcas;
			anchoMarcas=anchoEjeX.divide(new BigDecimal(numMarcas), presicion, BigDecimal.ROUND_HALF_EVEN); //El ancho que tendrá cada marca
			if(anchoMarcas.compareTo(BigDecimal.ONE)>=0){
				stringAnchoMarcas=anchoMarcas.toPlainString();
				cifra=new Integer(stringAnchoMarcas.substring(0,1)); //El ancho de cada marca
				potencia=stringAnchoMarcas.indexOf(".") - 1;
			}else if(anchoMarcas.compareTo(new BigDecimal(0.7))>=0){
				cifra=1;
				potencia=0;
			}else if(anchoMarcas.compareTo(new BigDecimal(0.3))>=0){
				cifra=5;
				potencia=-1;
			}else if(anchoMarcas.compareTo(new BigDecimal(0.15))>=0){
				cifra=2;
				potencia=-1;
			}else if(anchoMarcas.compareTo(new BigDecimal(0.09))>=0){
				cifra=1;
				potencia=-1;
			}else{
				stringAnchoMarcas=anchoMarcas.toPlainString();
				
				if(stringAnchoMarcas.indexOf("E")!=-1){
					potencia=new Integer(stringAnchoMarcas.substring(stringAnchoMarcas.indexOf("E") + 1));
				}else{
					stringAnchoMarcas=stringAnchoMarcas.substring(stringAnchoMarcas.indexOf(".") + 1);
					potencia=-1;
					while(stringAnchoMarcas.substring(0,1).equals("0")){
						stringAnchoMarcas=stringAnchoMarcas.substring(1);
						potencia--;
					}
				}
				cifra=new Integer(stringAnchoMarcas.substring(0,1)); //El ancho de cada marca
			}
			
			if(potencia>=0){
				anchoMarcas=(new BigDecimal(cifra)).multiply((new BigDecimal(10)).pow(potencia)).stripTrailingZeros();
			}else{
				anchoMarcas=(new BigDecimal(cifra)).divide((new BigDecimal(10)).pow(-1*potencia), presicion, BigDecimal.ROUND_HALF_EVEN).stripTrailingZeros();
			}
			//inicio=xmin.divide(anchoMarcas, xmin.toString().indexOf(".")-Math.abs(potencia), BigDecimal.ROUND_CEILING).multiply(anchoMarcas).round(new MathContext(Math.abs(potencia), RoundingMode.HALF_EVEN)); //El valor donde se pone la primera marca
			inicio=xmin.divideToIntegralValue(anchoMarcas).multiply(anchoMarcas);//.round(new MathContext(Math.abs(potencia), RoundingMode.HALF_EVEN)); //El valor donde se pone la primera marca
			numMarcas=anchoEjeX.divide(anchoMarcas, presicion, BigDecimal.ROUND_HALF_EVEN).round(new MathContext(presicion, RoundingMode.HALF_EVEN)).intValue();
			
			exponente=false;
			for(int i=0; i<=numMarcas; i++){ //Dibujo de las marcas
				coordenada=inicio.add(new BigDecimal(i).multiply(anchoMarcas));
				
				Punto2D puntoCoordenada=Cambia3D2D(Rota3D(new Punto3D(coordenada, new BigDecimal(0), new BigDecimal(0))));
				int puntoCoordenadaX=convCoordPantY(puntoCoordenada.getX()).add(movimientoX).intValue();
				int puntoCoordenadaY=convCoordPantZ(puntoCoordenada.getY()).add(movimientoY).intValue();
				
				ggraficoAnterior.drawLine(puntoCoordenadaX, puntoCoordenadaY, puntoCoordenadaX, puntoCoordenadaY+5);
	
				coord=coordenada.stripTrailingZeros().toPlainString(); //new Double(coordenada - (coordenada%anchoMarcas)).toString(); //(new Long(Math.round(coordenada))).toString();
				
				if(coord.indexOf(".")!=-1){
					while(coord.substring(coord.length()-1).equals("0")){
						coord=coord.substring(0,coord.length()-1);
					}
					if(coord.substring(coord.length()-1).equals(".")){
						coord=coord.substring(0,coord.length()-1);
					}
					
					if(coord.substring(0,1).equals("-")){
						negativo=true;
						coordcomodin=coord.substring(1);
					}else{
						negativo=false;
						coordcomodin=coord;
					}
					
					cont=-1;
					while(!coordcomodin.equals("") && (coordcomodin.substring(0,1).equals("0") || coordcomodin.substring(0,1).equals("."))){
						coordcomodin=coordcomodin.substring(1);
						cont++;
					}
					if(coordcomodin.equals("")){
						coord="0";
					}
					
					if(coordcomodin.indexOf(".")==-1){
						cont+=coordcomodin.length()-1;
					}
					
					if(cont>=3 && cont+coordcomodin.length()>=5){
						if(negativo){
							coord="-".concat(coordcomodin).concat("·10^").concat((new Integer(-1*cont)).toString());
						}else{
							coord=coordcomodin.concat("·10^").concat((new Integer(-1*cont)).toString());
						}
					}
	
				}else if(exponente || coord.length()>=7){ //Si se debe poner en forma exponencial
					exponente=true;
					int j=0;
					while(coord.substring(coord.length()-1).equals("0")){
						coord=coord.substring(0,coord.length()-1);
						j++;
					}
					
					coord=coord.concat("·10^").concat((new Integer(j)).toString());
				}
				
				anchotexto=tamanoletra.stringWidth(coord);
				if(!coord.equals("0")){
					ggraficoAnterior.drawString(coord, puntoCoordenadaX-anchotexto/2, puntoCoordenadaY+8+altotexto);
				}
			}
			
			//Eje Y
			
			puntoEjeInicio=Cambia3D2D(Rota3D(new Punto3D(new BigDecimal(0), ymin, new BigDecimal(0))));
			puntoEjeFinal=Cambia3D2D(Rota3D(new Punto3D(new BigDecimal(0), ymax, new BigDecimal(0))));
			ejeInicioX=convCoordPantY(puntoEjeInicio.getX());
			ejeInicioY=convCoordPantZ(puntoEjeInicio.getY());
			ejeFinalX=convCoordPantY(puntoEjeFinal.getX());
			ejeFinalY=convCoordPantZ(puntoEjeFinal.getY());
			centroX=ejeInicioX.add(ejeFinalX).divide(new BigInteger("2"));
			centroY=ejeInicioY.add(ejeFinalY).divide(new BigInteger("2"));
			movimientoX=BigInteger.ZERO;
			movimientoY=BigInteger.ZERO;
			
			ggraficoAnterior.setColor(colorEjes);
			
			if(centroX.compareTo(new BigInteger("30"))<0 || centroX.compareTo(new BigInteger((new Integer(ancho-30)).toString()))>0){
				movimientoX=(new BigInteger((new Long(ancho/2)).toString())).subtract(centroX);
				ggraficoAnterior.setColor(colorEjesAtenuados);
			}else if(centroY.compareTo(new BigInteger("30"))<0 || centroY.compareTo(new BigInteger((new Integer(alto-30)).toString()))>0){ //Si se sale de la pantalla
				movimientoY=(new BigInteger((new Long(alto/2)).toString())).subtract(centroY);
				ggraficoAnterior.setColor(colorEjesAtenuados);
			}
			
			ggraficoAnterior.setFont(tipoLetra); //Opciones de fuente para las marcas
			tamanoletra=ggraficoAnterior.getFontMetrics();
			altotexto=tamanoletra.getAscent();
			ggraficoAnterior.drawLine(ejeInicioX.add(movimientoX).intValue(), ejeInicioY.add(movimientoY).intValue(), ejeFinalX.add(movimientoX).intValue(), ejeFinalY.add(movimientoY).intValue());
			
			//Lo que sigue reliza las marcas en los ejes
			numMarcas=(new Double(ancho/pixelesMarcasY)).intValue(); //Cantidad de marcas que se realizarán
			numMarcas= numMarcas==0 ? 1 : numMarcas;
			anchoMarcas=anchoEjeY.divide(new BigDecimal(numMarcas), presicion, BigDecimal.ROUND_HALF_EVEN); //El ancho que tendrá cada marca
			if(anchoMarcas.compareTo(BigDecimal.ONE)>=0){
				stringAnchoMarcas=anchoMarcas.toPlainString();
				cifra=new Integer(stringAnchoMarcas.substring(0,1)); //El ancho de cada marca
				potencia=stringAnchoMarcas.indexOf(".") - 1;
			}else if(anchoMarcas.compareTo(new BigDecimal(0.7))>=0){
				cifra=1;
				potencia=0;
			}else if(anchoMarcas.compareTo(new BigDecimal(0.3))>=0){
				cifra=5;
				potencia=-1;
			}else if(anchoMarcas.compareTo(new BigDecimal(0.15))>=0){
				cifra=2;
				potencia=-1;
			}else if(anchoMarcas.compareTo(new BigDecimal(0.09))>=0){
				cifra=1;
				potencia=-1;
			}else{
				stringAnchoMarcas=anchoMarcas.toPlainString();
				
				if(stringAnchoMarcas.indexOf("E")!=-1){
					potencia=new Integer(stringAnchoMarcas.substring(stringAnchoMarcas.indexOf("E") + 1));
				}else{
					stringAnchoMarcas=stringAnchoMarcas.substring(stringAnchoMarcas.indexOf(".") + 1);
					potencia=-1;
					while(stringAnchoMarcas.substring(0,1).equals("0")){
						stringAnchoMarcas=stringAnchoMarcas.substring(1);
						potencia--;
					}
				}
				cifra=new Integer(stringAnchoMarcas.substring(0,1)); //El ancho de cada marca
			}
			
			if(potencia>=0){
				anchoMarcas=(new BigDecimal(cifra)).multiply((new BigDecimal(10)).pow(potencia)).stripTrailingZeros();
			}else{
				anchoMarcas=(new BigDecimal(cifra)).divide((new BigDecimal(10)).pow(-1*potencia), presicion, BigDecimal.ROUND_HALF_EVEN).stripTrailingZeros();
			}
			//inicio=xmin.divide(anchoMarcas, xmin.toString().indexOf(".")-Math.abs(potencia), BigDecimal.ROUND_CEILING).multiply(anchoMarcas).round(new MathContext(Math.abs(potencia), RoundingMode.HALF_EVEN)); //El valor donde se pone la primera marca
			inicio=ymin.divideToIntegralValue(anchoMarcas).multiply(anchoMarcas);//.round(new MathContext(Math.abs(potencia), RoundingMode.HALF_EVEN)); //El valor donde se pone la primera marca
			numMarcas=anchoEjeY.divide(anchoMarcas, presicion, BigDecimal.ROUND_HALF_EVEN).round(new MathContext(presicion, RoundingMode.HALF_EVEN)).intValue();
			
			exponente=false;
			for(int i=0; i<=numMarcas; i++){ //Dibujo de las marcas
				coordenada=inicio.add(new BigDecimal(i).multiply(anchoMarcas));
				
				Punto2D puntoCoordenada=Cambia3D2D(Rota3D(new Punto3D(new BigDecimal(0), coordenada, new BigDecimal(0))));
				int puntoCoordenadaX=convCoordPantY(puntoCoordenada.getX()).add(movimientoX).intValue();
				int puntoCoordenadaY=convCoordPantZ(puntoCoordenada.getY()).add(movimientoY).intValue();
				
				ggraficoAnterior.drawLine(puntoCoordenadaX, puntoCoordenadaY, puntoCoordenadaX, puntoCoordenadaY+5);
	
				coord=coordenada.stripTrailingZeros().toPlainString(); //new Double(coordenada - (coordenada%anchoMarcas)).toString(); //(new Long(Math.round(coordenada))).toString();
				
				if(coord.indexOf(".")!=-1){
					while(coord.substring(coord.length()-1).equals("0")){
						coord=coord.substring(0,coord.length()-1);
					}
					if(coord.substring(coord.length()-1).equals(".")){
						coord=coord.substring(0,coord.length()-1);
					}
					
					if(coord.substring(0,1).equals("-")){
						negativo=true;
						coordcomodin=coord.substring(1);
					}else{
						negativo=false;
						coordcomodin=coord;
					}
					
					cont=-1;
					while(!coordcomodin.equals("") && (coordcomodin.substring(0,1).equals("0") || coordcomodin.substring(0,1).equals("."))){
						coordcomodin=coordcomodin.substring(1);
						cont++;
					}
					if(coordcomodin.equals("")){
						coord="0";
					}
					
					if(coordcomodin.indexOf(".")==-1){
						cont+=coordcomodin.length()-1;
					}
					
					if(cont>=3 && cont+coordcomodin.length()>=5){
						if(negativo){
							coord="-".concat(coordcomodin).concat("·10^").concat((new Integer(-1*cont)).toString());
						}else{
							coord=coordcomodin.concat("·10^").concat((new Integer(-1*cont)).toString());
						}
					}
	
				}else if(exponente || coord.length()>=7){ //Si se debe poner en forma exponencial
					exponente=true;
					int j=0;
					while(coord.substring(coord.length()-1).equals("0")){
						coord=coord.substring(0,coord.length()-1);
						j++;
					}
					
					coord=coord.concat("·10^").concat((new Integer(j)).toString());
				}
				
				anchotexto=tamanoletra.stringWidth(coord);
				if(!coord.equals("0")){
					ggraficoAnterior.drawString(coord, puntoCoordenadaX-anchotexto/2, puntoCoordenadaY+8+altotexto);
				}
			}
			
			//Eje Z
			
			puntoEjeInicio=Cambia3D2D(Rota3D(new Punto3D(new BigDecimal(0), new BigDecimal(0), zmin)));
			puntoEjeFinal=Cambia3D2D(Rota3D(new Punto3D(new BigDecimal(0), new BigDecimal(0), zmax)));
			ejeInicioX=convCoordPantY(puntoEjeInicio.getX());
			ejeInicioY=convCoordPantZ(puntoEjeInicio.getY());
			ejeFinalX=convCoordPantY(puntoEjeFinal.getX());
			ejeFinalY=convCoordPantZ(puntoEjeFinal.getY());
			centroX=ejeInicioX.add(ejeFinalX).divide(new BigInteger("2"));
			centroY=ejeInicioY.add(ejeFinalY).divide(new BigInteger("2"));
			movimientoX=BigInteger.ZERO;
			movimientoY=BigInteger.ZERO;
			
			ggraficoAnterior.setColor(colorEjes);
			
			if(centroX.compareTo(new BigInteger("30"))<0 || centroX.compareTo(new BigInteger((new Integer(ancho-30)).toString()))>0){
				movimientoX=(new BigInteger((new Long(ancho/2)).toString())).subtract(centroX);
				ggraficoAnterior.setColor(colorEjesAtenuados);
			}else if(centroY.compareTo(new BigInteger("30"))<0 || centroY.compareTo(new BigInteger((new Integer(alto-30)).toString()))>0){ //Si se sale de la pantalla
				movimientoY=(new BigInteger((new Long(alto/2)).toString())).subtract(centroY);
				ggraficoAnterior.setColor(colorEjesAtenuados);
			}
			
			ggraficoAnterior.setFont(tipoLetra); //Opciones de fuente para las marcas
			tamanoletra=ggraficoAnterior.getFontMetrics();
			altotexto=tamanoletra.getAscent();
			ggraficoAnterior.drawLine(ejeInicioX.add(movimientoX).intValue(), ejeInicioY.add(movimientoY).intValue(), ejeFinalX.add(movimientoX).intValue(), ejeFinalY.add(movimientoY).intValue());
			
			//Lo que sigue reliza las marcas en los ejes
			numMarcas=(new Double(ancho/pixelesMarcasZ)).intValue(); //Cantidad de marcas que se realizarán
			numMarcas= numMarcas==0 ? 1 : numMarcas;
			anchoMarcas=anchoEjeZ.divide(new BigDecimal(numMarcas), presicion, BigDecimal.ROUND_HALF_EVEN); //El ancho que tendrá cada marca
			if(anchoMarcas.compareTo(BigDecimal.ONE)>=0){
				stringAnchoMarcas=anchoMarcas.toPlainString();
				cifra=new Integer(stringAnchoMarcas.substring(0,1)); //El ancho de cada marca
				potencia=stringAnchoMarcas.indexOf(".") - 1;
			}else if(anchoMarcas.compareTo(new BigDecimal(0.7))>=0){
				cifra=1;
				potencia=0;
			}else if(anchoMarcas.compareTo(new BigDecimal(0.3))>=0){
				cifra=5;
				potencia=-1;
			}else if(anchoMarcas.compareTo(new BigDecimal(0.15))>=0){
				cifra=2;
				potencia=-1;
			}else if(anchoMarcas.compareTo(new BigDecimal(0.09))>=0){
				cifra=1;
				potencia=-1;
			}else{
				stringAnchoMarcas=anchoMarcas.toPlainString();
				
				if(stringAnchoMarcas.indexOf("E")!=-1){
					potencia=new Integer(stringAnchoMarcas.substring(stringAnchoMarcas.indexOf("E") + 1));
				}else{
					stringAnchoMarcas=stringAnchoMarcas.substring(stringAnchoMarcas.indexOf(".") + 1);
					potencia=-1;
					while(stringAnchoMarcas.substring(0,1).equals("0")){
						stringAnchoMarcas=stringAnchoMarcas.substring(1);
						potencia--;
					}
				}
				cifra=new Integer(stringAnchoMarcas.substring(0,1)); //El ancho de cada marca
			}
			
			if(potencia>=0){
				anchoMarcas=(new BigDecimal(cifra)).multiply((new BigDecimal(10)).pow(potencia)).stripTrailingZeros();
			}else{
				anchoMarcas=(new BigDecimal(cifra)).divide((new BigDecimal(10)).pow(-1*potencia), presicion, BigDecimal.ROUND_HALF_EVEN).stripTrailingZeros();
			}
			//inicio=xmin.divide(anchoMarcas, xmin.toString().indexOf(".")-Math.abs(potencia), BigDecimal.ROUND_CEILING).multiply(anchoMarcas).round(new MathContext(Math.abs(potencia), RoundingMode.HALF_EVEN)); //El valor donde se pone la primera marca
			inicio=zmin.divideToIntegralValue(anchoMarcas).multiply(anchoMarcas);//.round(new MathContext(Math.abs(potencia), RoundingMode.HALF_EVEN)); //El valor donde se pone la primera marca
			numMarcas=anchoEjeZ.divide(anchoMarcas, presicion, BigDecimal.ROUND_HALF_EVEN).round(new MathContext(presicion, RoundingMode.HALF_EVEN)).intValue();
			
			exponente=false;
			for(int i=0; i<=numMarcas; i++){ //Dibujo de las marcas
				coordenada=inicio.add(new BigDecimal(i).multiply(anchoMarcas));
				
				Punto2D puntoCoordenada=Cambia3D2D(Rota3D(new Punto3D(new BigDecimal(0), new BigDecimal(0), coordenada)));
				int puntoCoordenadaX=convCoordPantY(puntoCoordenada.getX()).add(movimientoX).intValue();
				int puntoCoordenadaY=convCoordPantZ(puntoCoordenada.getY()).add(movimientoY).intValue();
				
				ggraficoAnterior.drawLine(puntoCoordenadaX, puntoCoordenadaY, puntoCoordenadaX+5, puntoCoordenadaY);
	
				coord=coordenada.stripTrailingZeros().toPlainString(); //new Double(coordenada - (coordenada%anchoMarcas)).toString(); //(new Long(Math.round(coordenada))).toString();
				
				if(coord.indexOf(".")!=-1){
					while(coord.substring(coord.length()-1).equals("0")){
						coord=coord.substring(0,coord.length()-1);
					}
					if(coord.substring(coord.length()-1).equals(".")){
						coord=coord.substring(0,coord.length()-1);
					}
					
					if(coord.substring(0,1).equals("-")){
						negativo=true;
						coordcomodin=coord.substring(1);
					}else{
						negativo=false;
						coordcomodin=coord;
					}
					
					cont=-1;
					while(!coordcomodin.equals("") && (coordcomodin.substring(0,1).equals("0") || coordcomodin.substring(0,1).equals("."))){
						coordcomodin=coordcomodin.substring(1);
						cont++;
					}
					if(coordcomodin.equals("")){
						coord="0";
					}
					
					if(coordcomodin.indexOf(".")==-1){
						cont+=coordcomodin.length()-1;
					}
					
					if(cont>=3 && cont+coordcomodin.length()>=5){
						if(negativo){
							coord="-".concat(coordcomodin).concat("·10^").concat((new Integer(-1*cont)).toString());
						}else{
							coord=coordcomodin.concat("·10^").concat((new Integer(-1*cont)).toString());
						}
					}
	
				}else if(exponente || coord.length()>=7){ //Si se debe poner en forma exponencial
					exponente=true;
					int j=0;
					while(coord.substring(coord.length()-1).equals("0")){
						coord=coord.substring(0,coord.length()-1);
						j++;
					}
					
					coord=coord.concat("·10^").concat((new Integer(j)).toString());
				}
				
				anchotexto=tamanoletra.stringWidth(coord);
				if(!coord.equals("0")){
					ggraficoAnterior.drawString(coord, puntoCoordenadaX+8, puntoCoordenadaY+altotexto/2); //puntoCoordenadaX-anchotexto/2, puntoCoordenadaY+8+altotexto);
				}
			}
			
			//PROCEDIMIENTO QUE GRAFICA LAS FUNCIONES
			
			//Se grafica un máximo de diez funciones
			exacto=Opciones.getPresicion();
			Opciones.setPresicion(false);
			presicion2=Opciones.getNumDecimales();
			Opciones.setNumDecimales(presicion);
			BigDecimal pasoX, pasoY, posX=xmin, posY=ymin, posZ;
			Punto2D[][] mayaPuntos2D;
			BigInteger X, Y, X2, Y2;
			
			for(int k=0; k<totalGraficas; k++){
				//Se realiza la maya para la función
				if(Expresiones[k]!=null){
					if(alternar){
						ggraficoAnterior.setColor(colores[k]);
					}else{
						ggraficoAnterior.setColor(colorGrafica);
					}
					
					//Procedimiento para realizar la maya
					if(evaluar){
						mayaPuntos=new Punto3D[totalGraficas][numPuntos+1][numPuntos+1];
						pasoX=xmax.subtract(xmin).divide(new BigDecimal(numPuntos), presicion, BigDecimal.ROUND_HALF_EVEN);
						pasoY=ymax.subtract(ymin).divide(new BigDecimal(numPuntos), presicion, BigDecimal.ROUND_HALF_EVEN);
						
						posX=xmin;
						for(int i=0; i<=numPuntos; i++){
							posY=ymin;
							for(int j=0; j<=numPuntos; j++){
								posZ=Evaluar(posX, posY, k);
								if(posZ!=null){
									mayaPuntos[k][i][j]=new Punto3D(posX, posY, posZ);
								}else{
									mayaPuntos[k][i][j]=null;
								}
								posY=posY.add(pasoY);
							}
							posX=posX.add(pasoX);
						}
						evaluar=false;
					}
					
					//Se convierten los puntos a 2D
					mayaPuntos2D=new Punto2D[numPuntos+1][numPuntos+1];
					for(int i=0; i<=numPuntos; i++){
						for(int j=0; j<=numPuntos; j++){
							//mayaPuntos[k][i][j]=Rota3D(mayaPuntos[k][i][j]).redondear(5);
							mayaPuntos2D[i][j]=Cambia3D2D(Rota3D(mayaPuntos[k][i][j]));
						}
					}
					
					//Se grafica la maya
					for(int i=0; i<=numPuntos; i++){
						for(int j=0; j<=numPuntos; j++){
							punto=mayaPuntos2D[i][j];
							if(punto!=null){
								X=convCoordPantY(punto.getX());
								Y=convCoordPantZ(punto.getY());
								if(X.compareTo(new BigInteger("-100"))>0 && X.compareTo(new BigInteger((new Integer(ancho+100)).toString()))<0 && Y.compareTo(new BigInteger("-100"))>0 && Y.compareTo(new BigInteger((new Integer(alto+100)).toString()))<0){
									ggraficoAnterior.fillOval(X.intValue(), Y.intValue(), 2, 2);
									if(i!=numPuntos && j!=numPuntos){
										punto2=mayaPuntos2D[i+1][j];
										if(punto2!=null){
											X2=convCoordPantY(punto2.getX());
											Y2=convCoordPantZ(punto2.getY());
											if(X2.compareTo(new BigInteger("-100"))>0 && X2.compareTo(new BigInteger((new Integer(ancho+100)).toString()))<0 && Y2.compareTo(new BigInteger("-100"))>0 && Y2.compareTo(new BigInteger((new Integer(alto+100)).toString()))<0){
												ggraficoAnterior.drawLine(X.intValue(), Y.intValue(), X2.intValue(), Y2.intValue());
											}
										}
										punto3=mayaPuntos2D[i][j+1];
										if(punto3!=null){
											X2=convCoordPantY(punto3.getX());
											Y2=convCoordPantZ(punto3.getY());
											if(X2.compareTo(new BigInteger("-100"))>0 && X2.compareTo(new BigInteger((new Integer(ancho+100)).toString()))<0 && Y2.compareTo(new BigInteger("-100"))>0 && Y2.compareTo(new BigInteger((new Integer(alto+100)).toString()))<0){
												ggraficoAnterior.drawLine(X.intValue(), Y.intValue(), X2.intValue(), Y2.intValue());
											}
										}
									}else if(i!=numPuntos && j==numPuntos){
										punto2=mayaPuntos2D[i+1][j];
										if(punto2!=null){
											X2=convCoordPantY(punto2.getX());
											Y2=convCoordPantZ(punto2.getY());
											if(X2.compareTo(new BigInteger("-100"))>0 && X2.compareTo(new BigInteger((new Integer(ancho+100)).toString()))<0 && Y2.compareTo(new BigInteger("-100"))>0 && Y2.compareTo(new BigInteger((new Integer(alto+100)).toString()))<0){
												ggraficoAnterior.drawLine(X.intValue(), Y.intValue(), X2.intValue(), Y2.intValue());
											}
										}
									}else if (j!=numPuntos && i==numPuntos){
										punto3=mayaPuntos2D[i][j+1];
										if(punto3!=null){
											X2=convCoordPantY(punto3.getX());
											Y2=convCoordPantZ(punto3.getY());
											if(X2.compareTo(new BigInteger("-100"))>0 && X2.compareTo(new BigInteger((new Integer(ancho+100)).toString()))<0 && Y2.compareTo(new BigInteger("-100"))>0 && Y2.compareTo(new BigInteger((new Integer(alto+100)).toString()))<0){
												ggraficoAnterior.drawLine(X.intValue(), Y.intValue(), X2.intValue(), Y2.intValue());
											}
										}
									}
								}
							}
						}
					}
				}
			}
			Opciones.setPresicion(exacto);
			Opciones.setNumDecimales(presicion2);
			
			//ggraficoAnterior.drawOval(ancho/2-Math.min(ancho, alto)/2, alto/2-Math.min(ancho, alto)/2, Math.min(ancho, alto), Math.min(ancho, alto));
			
			Graphics2D g=(Graphics2D)ge;
			g.drawImage(graficoAnterior, null, 0, 0);
			
			repintar=false;
			
		}else{
			Graphics2D g=(Graphics2D)ge;
			g.drawImage(graficoAnterior, null, 0, 0);
			
			//Si se está definiendo un cuadro de zoom
			/*if(cuadro){
				int inicioCuadroX, inicioCuadroY;
				
				g.setColor(Color.GRAY);
				inicioCuadroX= xmouse<xmouse2 ? xmouse : xmouse2;
				inicioCuadroY= ymouse<ymouse2 ? ymouse : ymouse2;
				g.drawRect(inicioCuadroX, inicioCuadroY, Math.abs(xmouse2-xmouse), Math.abs(ymouse2-ymouse));
			}*/
		}
	}
	
	/*
	 * Función que rota un punto en 3D
	 */
	private Punto3D Rota3D(Punto3D punto){
		if(punto!=null){
			return Rota3D(Rota3D(Rota3D(punto, Punto3D.X , anguloX), Punto3D.Y , anguloY), Punto3D.Z , anguloZ);
		}else{
			return null;
		}
	}
	
	/*
	 * Función que rota un punto en 3D
	 */
	private Punto3D Rota3D(Punto3D punto, Punto3D vectorRotacion, double angulo){
		if(punto!=null){
			Punto3D division=vectorRotacion.divide(MetodosNumericos.calculeRaiz(vectorRotacion.productoPunto(vectorRotacion),presicion));
			Punto3D producto=division.productoCruz(punto);
			
			return punto.suma(producto.multiplica(new BigDecimal(Math.sin(angulo)))).suma(division.productoCruz(producto).multiplica(new BigDecimal("2").multiply(new BigDecimal(Math.pow(Math.sin(0.5 * angulo),2))))).redondear(presicion);
		}else{
			return null;
		}
	}
	
	/*
	 * Función que transforma un punto en 3D a un punto en 2D
	 */
	private Punto2D Cambia3D2D(Punto3D punto){
		if(punto!=null){
			//return new Punto2D(punto.getY().subtract(CambiaX(punto.getX())), punto.getZ().subtract(CambiaX(punto.getX()).divide(new BigDecimal("2"), presicion, BigDecimal.ROUND_HALF_EVEN)));
			return new Punto2D(punto.getY().add(CambiaX(punto.getX())), punto.getZ().add(CambiaX(punto.getX()).divide(new BigDecimal("2"), presicion, BigDecimal.ROUND_HALF_EVEN)));
		}else{
			return null;
		}
	}
	
	/*
	 * Función que transforma la coordenada X
	 */
	private BigDecimal CambiaX(BigDecimal x){
		//return (x.subtract(xmin)).multiply(ymin.subtract(ymax)).divide(xmin.subtract(xmax), presicion, BigDecimal.ROUND_HALF_EVEN).add(ymin);
		return (x.subtract(xmin)).multiply(ymax.subtract(ymin)).divide(xmin.subtract(xmax), presicion, BigDecimal.ROUND_HALF_EVEN).add(ymax);
	}
	
	/*
	 *Función que evalúa la expresión i-ésima en el valor (a,b).
	 */
	private BigDecimal Evaluar(BigDecimal a, BigDecimal b, int i){// throws ArithmeticException{
		Nodo exp, exp2, comodin;
		String tipo;
		
		exp=new Nodo("eval", 3);
		exp.agregaOperando(Expresiones[i].clonarNodo());
		exp.agregaOperando(Variables[i][0]);
		
		comodin=new Nodo("DecimalLargo",1);
		comodin.agregaOperando(new Nodo(a.toString()));
		
		exp.agregaOperando(comodin);
		
		exp=simplificar.simplifica(exp);
		
		exp2=new Nodo("eval", 3);
		exp2.agregaOperando(exp);
		exp2.agregaOperando(Variables[i][1]);
		
		comodin=new Nodo("DecimalLargo",1);
		comodin.agregaOperando(new Nodo(b.toString()));
		
		exp2.agregaOperando(comodin);
		
		exp2=simplificar.simplifica(exp2);
		
		tipo=exp2.getTipo();
		if(tipo.equals("Indefinido") || tipo.length()<5 || "DecimalLargoEnteroLargo".indexOf(tipo)==-1){
			return null;
		}
		
		return new BigDecimal(exp2.getOperando(1).getTipo());
	}
	
	//FUNCIONES PUBLICAS
	
	/**
	 *Función que agrega la expresión dada.
	 *@param e La expresión para graficar.
	 */
	public void agregaExpresion(Nodo e){
		if(e.getTipo().equals(":=") || (e.getTipo().equals("=") && e.getOperando(1).getTipo().equals("Variable"))){
			e=e.getOperando(2);
		}
		
		if(Expresiones[expresionUltima]!=null && e.esIgual(Expresiones[expresionUltima])){
			numColores=(++numColores)%numTotalColores;
			colores[expresionUltima]=todosColores[numColores];
			repintar=true;
			this.repaint();
		}else if(verificarExpresion(e)){
			expresionUltima=(++expresionUltima)%totalGraficas;
			Expresiones[expresionUltima]=e;
			numColores=(++numColores)%numTotalColores;
			colores[expresionUltima]=todosColores[numColores];
			evaluar=true;
			repintar=true;
			this.repaint();
		}else{
			JOptionPane.showMessageDialog(this,"Hay un error en la expresión al graficar.","Error",JOptionPane.ERROR_MESSAGE);
		}
	}
	
	/**
	 *Función que borra la gráfica actual.
	 */
	public void borraExpresion(){
		if(Expresiones[expresionUltima]!=null){
			Expresiones[expresionUltima]=null;
			expresionUltima=(--expresionUltima)%totalGraficas;
			repintar=true;
			this.repaint();
		}
	}
	
	/**
	 *Función que borra la gráfica indicada.
	 *@param i El número de gráfica a borrar.
	 */
	public void borraExpresion(int i){
		Expresiones[i]=null;
		if(i==expresionUltima){
			expresionUltima=(--expresionUltima)%totalGraficas;
		}
		repintar=true;
		this.repaint();
	}
	
	/**
	 *Función que borra todas las gráficas.
	 */
	public void borraTodas(){
		Expresiones=new Nodo[totalGraficas];
		expresionUltima=0;
		repintar=true;
		this.repaint();
	}
	
	/**
	 *Función que devuelve el valor mínimo en x
	 *representado en este objeto.
	 *@return El mínimo valor en x.
	 */
	public BigDecimal getMinX(){
		return xmin;
	}
	
	/**
	 *Función que define el valor mínimo en x
	 *representado en este objeto.
	 *@param nuevoMinX El nuevo valor mínimo para x.
	 */
	public void setMinX(int nuevoMinX){
		xmin=new BigDecimal(nuevoMinX);
		repintar=true;
		repaint();
	}
	
	/**
	 *Función que define el valor mínimo en x
	 *representado en este objeto.
	 *@param nuevoMinX El nuevo valor mínimo para x.
	 */
	public void setMinX(double nuevoMinX){
		xmin=new BigDecimal(nuevoMinX);
		repintar=true;
		repaint();
	}
	
	/**
	 *Función que define el valor mínimo en x
	 *representado en este objeto.
	 *@param nuevoMinX El nuevo valor mínimo para x.
	 */
	public void setMinX(BigDecimal nuevoMinX){
		xmin=nuevoMinX;
		repintar=true;
		repaint();
	}
	
	/**
	 *Función que devuelve el valor máximo en x
	 *representado en este objeto.
	 *@return El máximo valor en x.
	 */
	public BigDecimal getMaxX(){
		return xmax;
	}
	
	/**
	 *Función que define el valor máximo en x
	 *representado en este objeto.
	 *@param nuevoMaxX El nuevo valor máximo para x.
	 */
	public void setMaxX(int nuevoMaxX){
		xmax=new BigDecimal(nuevoMaxX);
		repintar=true;
		repaint();
	}
	
	/**
	 *Función que define el valor máximo en x
	 *representado en este objeto.
	 *@param nuevoMaxX El nuevo valor máximo para x.
	 */
	public void setMaxX(double nuevoMaxX){
		xmax=new BigDecimal(nuevoMaxX);
		repintar=true;
		repaint();
	}
	
	/**
	 *Función que define el valor máximo en x
	 *representado en este objeto.
	 *@param nuevoMaxX El nuevo valor máximo para x.
	 */
	public void setMaxX(BigDecimal nuevoMaxX){
		xmax=nuevoMaxX;
		repintar=true;
		repaint();
	}
	
	/**
	 *Función que devuelve el valor mínimo en y
	 *representado en este objeto.
	 *@return El mínimo valor en y.
	 */
	public BigDecimal getMinY(){
		return ymin;
	}
	
	/**
	 *Función que define el valor mínimo en y
	 *representado en este objeto.
	 *@param nuevoMinY El nuevo valor mínimo para y.
	 */
	public void setMinY(int nuevoMinY){
		ymin=new BigDecimal(nuevoMinY);
		repintar=true;
		repaint();
	}
	
	/**
	 *Función que define el valor mínimo en y
	 *representado en este objeto.
	 *@param nuevoMinY El nuevo valor mínimo para y.
	 */
	public void setMinY(double nuevoMinY){
		ymin=new BigDecimal(nuevoMinY);
		repintar=true;
		repaint();
	}
	
	/**
	 *Función que define el valor mínimo en y
	 *representado en este objeto.
	 *@param nuevoMinY El nuevo valor mínimo para y.
	 */
	public void setMinY(BigDecimal nuevoMinY){
		ymin=nuevoMinY;
		repintar=true;
		repaint();
	}
	
	/**
	 *Función que devuelve el valor máximo en y
	 *representado en este objeto.
	 *@return El máximo valor en y.
	 */
	public BigDecimal getMaxY(){
		return ymax;
	}
	
	/**
	 *Función que define el valor máximo en y
	 *representado en este objeto.
	 *@param nuevoMaxY El nuevo valor máximo para y.
	 */
	public void setMaxY(int nuevoMaxY){
		ymax=new BigDecimal(nuevoMaxY);
		repintar=true;
		repaint();
	}
	
	/**
	 *Función que define el valor máximo en y
	 *representado en este objeto.
	 *@param nuevoMaxY El nuevo valor máximo para y.
	 */
	public void setMaxY(double nuevoMaxY){
		ymax=new BigDecimal(nuevoMaxY);
		repintar=true;
		repaint();
	}
	
	/**
	 *Función que define el valor máximo en y
	 *representado en este objeto.
	 *@param nuevoMaxY El nuevo valor máximo para y.
	 */
	public void setMaxY(BigDecimal nuevoMaxY){
		ymax=nuevoMaxY;
		repintar=true;
		repaint();
	}
	
	/**
	 *Función que devuelve el valor mínimo en z
	 *representado en este objeto.
	 *@return El mínimo valor en z.
	 */
	public BigDecimal getMinZ(){
		return zmin;
	}
	
	/**
	 *Función que define el valor mínimo en z
	 *representado en este objeto.
	 *@param nuevoMinZ El nuevo valor mínimo para z.
	 */
	public void setMinZ(int nuevoMinZ){
		zmin=new BigDecimal(nuevoMinZ);
		repintar=true;
		repaint();
	}
	
	/**
	 *Función que define el valor mínimo en z
	 *representado en este objeto.
	 *@param nuevoMinZ El nuevo valor mínimo para z.
	 */
	public void setMinZ(double nuevoMinZ){
		zmin=new BigDecimal(nuevoMinZ);
		repintar=true;
		repaint();
	}
	
	/**
	 *Función que define el valor mínimo en z
	 *representado en este objeto.
	 *@param nuevoMinZ El nuevo valor mínimo para z.
	 */
	public void setMinZ(BigDecimal nuevoMinZ){
		zmin=nuevoMinZ;
		repintar=true;
		repaint();
	}
	
	/**
	 *Función que devuelve el valor máximo en z
	 *representado en este objeto.
	 *@return El máximo valor en z.
	 */
	public BigDecimal getMaxZ(){
		return zmax;
	}
	
	/**
	 *Función que define el valor máximo en z
	 *representado en este objeto.
	 *@param nuevoMaxZ El nuevo valor máximo para z.
	 */
	public void setMaxZ(int nuevoMaxZ){
		zmax=new BigDecimal(nuevoMaxZ);
		repintar=true;
		repaint();
	}
	
	/**
	 *Función que define el valor máximo en z
	 *representado en este objeto.
	 *@param nuevoMaxZ El nuevo valor máximo para z.
	 */
	public void setMaxZ(double nuevoMaxZ){
		zmax=new BigDecimal(nuevoMaxZ);
		repintar=true;
		repaint();
	}
	
	/**
	 *Función que define el valor máximo en z
	 *representado en este objeto.
	 *@param nuevoMaxZ El nuevo valor máximo para z.
	 */
	public void setMaxZ(BigDecimal nuevoMaxZ){
		zmax=nuevoMaxZ;
		repintar=true;
		repaint();
	}
	
	/**
	 *Función que cambia el color de gráfica cuando es fijo.
	 *@param color El nuevo color.
	 */
	public void setColorGrafica(Color color){
		colorGrafica=color;
		repintar=true;
		this.repaint();
	}
	
	/**
	 *Función que cambia el color del fondo para graficar.
	 *@param color El nuevo color.
	 */
	public void setColorFondo(Color color){
		colorFondo=color;
		repintar=true;
		this.repaint();
	}
	
	/**
	 *Función que cambia el color de los ejes atenuados de la grafica.
	 *@param color El nuevo color.
	 */
	public void setColorEjesAtenuados(Color color){
		colorEjesAtenuados=color;
		repintar=true;
		this.repaint();
	}
	
	/**
	 *Función que cambia el color de los ejes de la grafica.
	 *@param color El nuevo color.
	 */
	public void setColorEjes(Color color){
		colorEjes=color;
		repintar=true;
		this.repaint();
	}
	
	/**
	 *Función que cambia la opción de alternar colores o usar uno fijo para graficar.
	 *@param alterna True si alterna o false en otro caso.
	 */
	public void setAlternarColor(boolean alterna){
		alternar=alterna;
		repintar=true;
		this.repaint();
	}
	
	/**
	 *Función que cambia el tipo de letra con que se pondrán los números en los ejes.
	 *@param letra La nueva letra.
	 */
	public void setTipoLetra(Font letra){
		if(tipoLetra!=letra){
			tipoLetra=letra;
			repintar=true;
			this.repaint();
		}
	}
	
	/**
	 *Función que vuelve a tomar los valores seleccionados en las opciones.
	 */
	public void retomarValores(){
		colorFondo=Opciones.getColorFondoGrafica3D();
		colorEjesAtenuados=Opciones.getColorEjesAtenuadosGrafica3D();
		colorEjes=Opciones.getColorEjesGrafica3D();
		colorGrafica=Opciones.getColorGrafica3D();
		alternar=Opciones.getAlternarColorGrafica3D();
		tipoLetra=Opciones.getTipoLetraEjesGrafica3D();
		numPuntos=Opciones.getNumPuntosGrafica3D();
		pixelesMarcasX=Opciones.getPixelesMarcasX3D();
		pixelesMarcasY=Opciones.getPixelesMarcasY3D();
		pixelesMarcasZ=Opciones.getPixelesMarcasZ3D();
		repintar=true;
		this.repaint();
	}
	
	/**
	 *Función que realiza un zoom hacia adentro en la gráfica.
	 */
	public void zoomIn(){
		zoom(1,0);
	}
	
	/**
	 *Función que realiza un zoom hacia adentro en la gráfica en el eje X.
	 */
	public void zoomInX(){
		zoom(1, 1);
	}
	
	/**
	 *Función que realiza un zoom hacia adentro en la gráfica en el eje Y.
	 */
	public void zoomInY(){
		zoom(1, 2);
	}
	
	/**
	 *Función que realiza un zoom hacia adentro en la gráfica en el eje Z.
	 */
	public void zoomInZ(){
		zoom(1, 3);
	}
	
	/**
	 *Función que realiza un zoom hacia afuera en la gráfica.
	 */
	public void zoomOut(){
		zoom(-1, 0);
	}
	
	/**
	 *Función que realiza un zoom hacia afuera en la gráfica en el eje X.
	 */
	public void zoomOutX(){
		zoom(-1, 1);
	}
	
	/**
	 *Función que realiza un zoom hacia afuera en la gráfica en el eje Y.
	 */
	public void zoomOutY(){
		zoom(-1, 2);
	}
	
	/**
	 *Función que realiza un zoom hacia afuera en la gráfica en el eje Z.
	 */
	public void zoomOutZ(){
		zoom(-1, 3);
	}
	
	//FUNCIONES PRIVADAS
	/*
	 * Función que maneja el zoom
	 */
	private void zoom(int num, int forma){
		BigDecimal movimiento, comodin;
		
		if(forma==0 || forma==1){
			movimiento=xmax.subtract(xmin).multiply(new BigDecimal(num)).divide(new BigDecimal(10)); //convPixCoordX(-50);
			xmin=xmin.add(movimiento);
			xmax=xmax.subtract(movimiento);
			
			if(xmin.compareTo(xmax)==1){
				comodin=xmin;
				xmin=xmax;
				xmax=comodin;
			}
		}
		
		if(forma==0 || forma==2){
			movimiento=ymax.subtract(ymin).multiply(new BigDecimal(num)).divide(new BigDecimal(10)); //convPixCoordY(-50);
			ymin=ymin.add(movimiento);
			ymax=ymax.subtract(movimiento);
			
			if(ymin.compareTo(ymax)==1){
				comodin=ymin;
				ymin=ymax;
				ymax=comodin;
			}
		}
		
		if(forma==0 || forma==3){
			movimiento=zmax.subtract(zmin).multiply(new BigDecimal(num)).divide(new BigDecimal(10)); //convPixCoordZ(50);
			zmin=zmin.add(movimiento);
			zmax=zmax.subtract(movimiento);
			
			if(zmin.compareTo(zmax)==1){
				comodin=zmin;
				zmin=zmax;
				zmax=comodin;
			}
		}
		
		evaluar=true;
		repintar=true;
		this.repaint();
	}
	
	/*
	 *Convierte de coordenadas de la pantalla al
	 *sistema de coordenadas representado,
	 *todo esto en el eje x
	 */
	/*private BigDecimal convPantCoordX(int valor){
		return xmin.add(new BigDecimal(valor).multiply(anchoEjeX).divide(new BigDecimal(ancho), presicion, BigDecimal.ROUND_HALF_EVEN));
	}*/
	
	/*
	 *Convierte del sistema de coordenadas representado al
	 *sistema de coordenadas de pantalla, todo esto
	 *en el eje x.
	 */
	/*private BigInteger convCoordPantX(BigDecimal valor){
		return (valor).subtract(xmin).multiply(new BigDecimal(ancho)).divide(anchoEjeX, presicion, BigDecimal.ROUND_HALF_EVEN).round(new MathContext(presicion, RoundingMode.HALF_EVEN)).toBigInteger();
	}*/
	
	/*
	 *Convierte de coordenadas de la pantalla al
	 *sistema de coordenadas representado,
	 *todo esto en el eje y
	 */
	/*private BigDecimal convPantCoordY(int valor){
		return ymin.add(new BigDecimal(valor).multiply(anchoEjeY).divide(new BigDecimal(ancho), presicion, BigDecimal.ROUND_HALF_EVEN));
	}*/
	
	/*
	 *Convierte del sistema de coordenadas representado al
	 *sistema de coordenadas de pantalla, todo esto
	 *en el eje y.
	 */
	private BigInteger convCoordPantY(BigDecimal valor){
		return (valor).subtract(ymin).multiply(new BigDecimal(ancho)).divide(anchoEjeY, presicion, BigDecimal.ROUND_HALF_EVEN).round(new MathContext(presicion, RoundingMode.HALF_EVEN)).toBigInteger();
	}
	
	/*
	 *Convierte de coordenadas de la pantalla al
	 *sistema de coordenadas representado,
	 *todo esto en el eje z
	 */
	/*private BigDecimal convPantCoordZ(int valor){
		return zmax.subtract(new BigDecimal(valor).multiply(anchoEjeZ).divide(new BigDecimal(alto), presicion, BigDecimal.ROUND_HALF_EVEN));
	}*/
	
	/*
	 *Convierte del sistema de coordenadas representado al
	 *sistema de coordenadas de pantalla, todo esto
	 *en el eje z.
	 */
	private BigInteger convCoordPantZ(BigDecimal valor){
		return zmax.subtract(valor).multiply(new BigDecimal(alto)).divide(anchoEjeZ, presicion, BigDecimal.ROUND_HALF_EVEN).round(new MathContext(presicion, RoundingMode.HALF_EVEN)).toBigInteger();
	}
	
	/*
	 *Convierte una cantidad de pixeles dada
	 *a su correspondiente cantidad en las
	 *coordenadas representadas en el eje X.
	 */
	private BigDecimal convPixCoordX(int pixeles){
		return new BigDecimal(pixeles).multiply(anchoEjeX).divide(new BigDecimal(ancho), presicion, BigDecimal.ROUND_HALF_EVEN);
	}
	
	/*
	 *Convierte una cantidad de pixeles dada
	 *a su correspondiente cantidad en las
	 *coordenadas representadas en el eje Y.
	 */
	private BigDecimal convPixCoordY(int pixeles){
		return new BigDecimal(pixeles).multiply(anchoEjeY).divide(new BigDecimal(ancho), presicion, BigDecimal.ROUND_HALF_EVEN);
	}
	
	/*
	 *Convierte una cantidad de unidades dada
	 *en el sistema de coordenadas representado
	 *a su correspondiente cantidad en pixeles
	 *en el eje Y.
	 */
	/*private BigInteger convCoordPixY(BigDecimal unidades){
		return unidades.multiply(new BigDecimal(ancho)).divide(anchoEjeY, presicion, BigDecimal.ROUND_HALF_EVEN).round(new MathContext(presicion, RoundingMode.HALF_EVEN)).toBigInteger();
	}*/
	
	/*
	 *Convierte una cantidad de pixeles dada
	 *a su correspondiente cantidad en las
	 *coordenadas representadas en el eje Z.
	 */
	private BigDecimal convPixCoordZ(int pixeles){
		return new BigDecimal(pixeles).multiply(anchoEjeZ).divide(new BigDecimal(alto), presicion, BigDecimal.ROUND_HALF_EVEN);
	}
	
	/*
	 *Convierte una cantidad de unidades dada
	 *en el sistema de coordenadas representado
	 *a su correspondiente cantidad en pixeles
	 *en el eje Z.
	 */
	/*private BigInteger convCoordPixZ(BigDecimal unidades){
		return unidades.multiply(new BigDecimal(ancho)).divide(anchoEjeZ, presicion, BigDecimal.ROUND_HALF_EVEN).round(new MathContext(presicion, RoundingMode.HALF_EVEN)).toBigInteger();
	}*/
	
	/*
	 *Función que verifica si la expresión a graficar tiene un formato válido,
	 *es decir:
	 *con una variable
	 *y=f(x)
	 *f(x):=g(x)
	 */
	private boolean verificarExpresion(Nodo e){
		variables=new Vector<Nodo>();
		
		buscaVariables(e);
		
		if(variables.size()==2){
			if(((Nodo)variables.elementAt(1)).getOperando(1).getTipo().equals("x") || ((Nodo)variables.elementAt(0)).getOperando(1).getTipo().equals("y")){
				Variables[expresionUltima+1][0]=(Nodo)variables.elementAt(1);
				Variables[expresionUltima+1][1]=(Nodo)variables.elementAt(0);
			}else{
				Variables[expresionUltima+1][0]=(Nodo)variables.elementAt(1);
				Variables[expresionUltima+1][1]=(Nodo)variables.elementAt(0);
			}
			return true;
		}else if(variables.size()==1){
			Nodo comodin=new Nodo("Variable");
			
			if(((Nodo)variables.elementAt(0)).getOperando(1).getTipo().equals("y")){
				Variables[expresionUltima+1][1]=(Nodo)variables.elementAt(0);
				comodin.agregaOperando(new Nodo("x"));
				Variables[expresionUltima+1][0]=comodin;
			}else{
				Variables[expresionUltima+1][0]=(Nodo)variables.elementAt(0);
				comodin.agregaOperando(new Nodo("y"));
				Variables[expresionUltima+1][1]=comodin;
			}
			
			return true;
		}else if(variables.size()==0){
			Nodo comodin=new Nodo("Variable");
			comodin.agregaOperando(new Nodo("x"));
			Variables[expresionUltima+1][0]=comodin;
			
			comodin=new Nodo("Variable");
			comodin.agregaOperando(new Nodo("y"));
			Variables[expresionUltima+1][1]=comodin;
			
			return true;
		}else{
			return false;
		}
	}
	
	/*
	 *Función que busca las variables de la expresión e
	 */
	private void buscaVariables(Nodo e){
		if(e.getTipo().equals("Variable")){
			boolean encontrado=false;
			int i=0;
			while(!encontrado && i<variables.size()){
				if(((Nodo)variables.elementAt(i)).esIgual(e)){
					encontrado=true;
				}
				i++;
			}
			if(!encontrado){
				variables.add(e);
				return;
			}
		}
		
		for(int i=1; i<=e.getNumeroOperandos();i++){
			buscaVariables(e.getOperando(i));
		}
	}
	
	private double calculaAngulo(Punto3D vector1, Punto3D vector2){
		double angulo;
		BigDecimal divisor=vector1.norma(presicion).multiply(vector2.norma(presicion));
		if(divisor.compareTo(BigDecimal.ZERO)==0){
			angulo=0;
		}else{
			angulo=vector1.productoPunto(vector2).divide(divisor,presicion, BigDecimal.ROUND_HALF_DOWN).doubleValue();
			if(angulo<=1){
				angulo=Math.acos(angulo);
			}else{
				angulo=0;
			}
		}
		//System.out.println(vector1.productoPunto(vector2).toPlainString() + " " +divisor.toPlainString() + " "+ vector1.productoPunto(vector2).divide(divisor,presicion, BigDecimal.ROUND_HALF_DOWN).doubleValue());
		return angulo;
	}
	
	//EVENTOS DEL MOUSE
	
	public void mouseExited(MouseEvent e){
	}
	
	public void mouseEntered(MouseEvent e){
	}
	
	public void mouseReleased(MouseEvent e){
		xmouse2=e.getX();
		ymouse2=e.getY();
		
		if(e.getButton()==MouseEvent.BUTTON1){
			primeraVez=false;
		}
		
		/*if(boton==MouseEvent.BUTTON2){
			cuadro=false;
			
			if(xmouse==xmouse2){
				xmouse2++;
			}
			
			if(ymouse==ymouse2){
				ymouse2++;
			}
			
			if(xmouse<xmouse2){
				xmax=convPantCoordX(xmouse2);
				xmin=convPantCoordX(xmouse);
			}else{
				xmax=convPantCoordX(xmouse);
				xmin=convPantCoordX(xmouse2);
			}
			
			if(ymouse<ymouse2){
				ymin=convPantCoordY(ymouse2);
				ymax=convPantCoordY(ymouse);
			}else{
				ymin=convPantCoordY(ymouse);
				ymax=convPantCoordY(ymouse2);
			}
			
			repintar=true;
			this.repaint();
		}*/
	}
	
	public void mousePressed(MouseEvent e){
		xmouse=e.getX();
		ymouse=e.getY();
		boton=e.getButton();
	}
	
	public void mouseClicked(MouseEvent e){
		if(e.getClickCount()>=2){
			if(e.getButton()==MouseEvent.BUTTON1){
				xmin=new BigDecimal(-5);
				xmax=new BigDecimal(5);
				ymin=new BigDecimal(-5);
				ymax=new BigDecimal(5);
				zmin=new BigDecimal(-5);
				zmax=new BigDecimal(5);
				primeraVez=true;
				anguloX=0;
				anguloY=0;
				anguloZ=0;
				primerVector=Punto3D.ZERO;
				segundoVector=Punto3D.ZERO;
			}else if(e.getButton()==MouseEvent.BUTTON3){
				BigDecimal movimiento;
				int x=e.getX();
				int y=e.getY();
				
				movimiento=convPixCoordX(ancho/2-x);
				xmin=xmin.subtract(movimiento);
				xmax=xmax.subtract(movimiento);
				
				movimiento=convPixCoordY(ancho/2-x);
				ymin=ymin.subtract(movimiento);
				ymax=ymax.subtract(movimiento);
				
				movimiento=convPixCoordZ(alto/2-y);
				zmin=zmin.add(movimiento);
				zmax=zmax.add(movimiento);
			}
			evaluar=true;
			repintar=true;
			this.repaint();
		}
	}
	
	public void mouseMoved(MouseEvent e){
	}
	
	public void mouseDragged(MouseEvent e){
		BigDecimal movimiento, comodin;
		
		xmouse2=e.getX();
		ymouse2=e.getY();
		
		/*if(boton==MouseEvent.BUTTON2){
			cuadro=true;
			//repintar=true;
			this.repaint();
		}*/
		
		if(boton==MouseEvent.BUTTON1){
			double x1, y1, z1;
			double x2, y2, z2;
			Punto3D primerVectorX, primerVectorY, primerVectorZ, segundoVectorX, segundoVectorY, segundoVectorZ;
			BigDecimal comparar;
			
			y1=xmouse-ancho/2;
			z1=-ymouse+alto/2;
			x1=Math.sqrt(Math.pow(Math.min(ancho, alto)/2,2)-Math.pow(y1, 2)-Math.pow(z1, 2));
			
			y2=xmouse2-ancho/2;
			z2=-ymouse2+alto/2;
			x2=Math.sqrt(Math.pow(Math.min(ancho, alto)/2,2)-Math.pow(y2, 2)-Math.pow(z2, 2));
			
			//System.out.println(Math.min(ancho, alto)/2);
			System.out.println(x1 + " " + y1 + " " + z1);
			System.out.println(x2 + " " + y2 + " " + z2);
			
			primerVector=new Punto3D(new BigDecimal(x1),new BigDecimal(y1), new BigDecimal(z1));
			segundoVector=new Punto3D(new BigDecimal(x2),new BigDecimal(y2), new BigDecimal(z2));
			//primerVector=Rota3D(Rota3D(primerVector, Punto3D.Z, 0.5), Punto3D.Y, 0.2).redondear(7);
			//segundoVector=Rota3D(Rota3D(segundoVector, Punto3D.Z, 0.5), Punto3D.Y, 0.2).redondear(7);
			
			primerVectorX=new Punto3D(BigDecimal.ZERO,primerVector.getY(), primerVector.getZ());
			primerVectorY=new Punto3D(primerVector.getX(),BigDecimal.ZERO, primerVector.getZ());
			primerVectorZ=new Punto3D(primerVector.getX(),primerVector.getY(), BigDecimal.ZERO);
			segundoVectorX=new Punto3D(BigDecimal.ZERO,segundoVector.getY(), segundoVector.getZ());
			segundoVectorY=new Punto3D(segundoVector.getX(),BigDecimal.ZERO, segundoVector.getZ());
			segundoVectorZ=new Punto3D(segundoVector.getX(),segundoVector.getY(), BigDecimal.ZERO);
			
			if(primeraVez){ //primerVector.equals(Punto3D.ZERO) || (primerVector.getX().compareTo(new BigDecimal(x1))==0 && primerVector.getY().compareTo(new BigDecimal(y1))==0 && primerVector.getZ().compareTo(new BigDecimal(z1))==0)){
				//comparar=primerVectorX.productoCruz(segundoVectorX).getX();
				/*if(comparar.compareTo(BigDecimal.ZERO)>0){
					anguloX=calculaAngulo(primerVectorX, segundoVectorX);
				}else if(comparar.compareTo(BigDecimal.ZERO)<0){
					anguloX=-1*calculaAngulo(primerVectorX, segundoVectorX);
				}else{
					anguloX=0;
				}*/
				comparar=primerVectorY.productoCruz(segundoVectorY).getY();
				if(comparar.compareTo(BigDecimal.ZERO)>0){
					anguloY=calculaAngulo(primerVectorY, segundoVectorY);
				}else if(comparar.compareTo(BigDecimal.ZERO)<0){
					anguloY=-1*calculaAngulo(primerVectorY, segundoVectorY);
				}else{
					anguloY=0;
				}
				comparar=primerVectorZ.productoCruz(segundoVectorZ).getZ();
				if(comparar.compareTo(BigDecimal.ZERO)>0){
					anguloZ=calculaAngulo(primerVectorZ, segundoVectorZ);
				}else if(comparar.compareTo(BigDecimal.ZERO)<0){
					anguloZ=-1*calculaAngulo(primerVectorZ, segundoVectorZ);
				}else{
					anguloZ=0;
				}
			}else{
				anguloY-=anguloy;
				anguloZ-=anguloz;
				comparar=primerVectorY.productoCruz(segundoVectorY).getY();
				if(comparar.compareTo(BigDecimal.ZERO)>0){
					anguloy=calculaAngulo(primerVectorY, segundoVectorY);
				}else if(comparar.compareTo(BigDecimal.ZERO)<0){
					anguloy=-1*calculaAngulo(primerVectorY, segundoVectorY);
				}else{
					anguloy=0;
				}
				comparar=primerVectorZ.productoCruz(segundoVectorZ).getZ();
				if(comparar.compareTo(BigDecimal.ZERO)>0){
					anguloz=calculaAngulo(primerVectorZ, segundoVectorZ);
				}else if(comparar.compareTo(BigDecimal.ZERO)<0){
					anguloz=-1*calculaAngulo(primerVectorZ, segundoVectorZ);
				}else{
					anguloz=0;
				}
				anguloY+=anguloy;
				anguloZ+=anguloz;
			}
			/*}else if(primerVector.getX().compareTo(new BigDecimal(x1))==0 && primerVector.getY().compareTo(new BigDecimal(y1))==0 && primerVector.getZ().compareTo(new BigDecimal(z1))==0){
				segundoVector=new Punto3D(new BigDecimal(x2),new BigDecimal(y2), new BigDecimal(z2));
				if(primerVectorX.productoCruz(segundoVectorX).getX().compareTo(BigDecimal.ZERO)>=0){
					anguloX=calculaAngulo(primerVectorX, segundoVectorX);
				}else{
					anguloX=-1*calculaAngulo(primerVectorX, segundoVectorX);
				}
				if(primerVectorY.productoCruz(segundoVectorY).getY().compareTo(BigDecimal.ZERO)>=0){
					anguloY=calculaAngulo(primerVectorY, segundoVectorY);
				}else{
					anguloY=-1*calculaAngulo(primerVectorY, segundoVectorY);
				}
				if(primerVectorZ.productoCruz(segundoVectorZ).getZ().compareTo(BigDecimal.ZERO)>=0){
					anguloZ=calculaAngulo(primerVectorZ, segundoVectorZ);
				}else{
					anguloZ=-1*calculaAngulo(primerVectorZ, segundoVectorZ);
				}*/
			//}else{
				/*Punto3D segundovector=new Punto3D(new BigDecimal(x2),new BigDecimal(y2), new BigDecimal(z2));
				
				xmouse=xmouse2;
				ymouse=ymouse2;
				
				vectorRotacion=primervector.productoCruz(segundovector);
				vectorRotacion=Rota3D(vectorRotacion, Punto3D.Z, 0.5);
				vectorRotacion=Rota3D(vectorRotacion, Punto3D.Y, 0.2);*/
				//vectorRotacion=Rota3D(vectorRotacion, new Punto3D(BigDecimal.ZERO, BigDecimal.ONE, BigDecimal.ONE), 0.3);
				/*divisor=primervector.norma().multiply(segundovector.norma());
				if(divisor.compareTo(BigDecimal.ZERO)==0){
					angulo=0;
				}else{
					angulo=Math.acos(primervector.productoPunto(segundovector).divide(primervector.norma().multiply(segundovector.norma()),2).doubleValue());//MetodosNumericos.calculeArcoseno(primerVector.productoPunto(segundoVector).divide(primerVector.norma().multiply(segundoVector.norma()),2).subtract(new BigDecimal(Math.PI/2)), 2).doubleValue();
				}*/
				//angulo=calculaAngulo(primervector, segundovector);
				//primerVector=Rota3D(primerVector, vectorRotacion, angulo);
				//segundoVector=Rota3D(segundoVector, vectorRotacion, angulo);
			//}
			//vectorRotacion=primerVector.productoCruz(segundoVector);
			//vectorRotacion=Rota3D(vectorRotacion, Punto3D.Z, 0.5);
			//vectorRotacion=Rota3D(vectorRotacion, Punto3D.Y, 0.2).redondear(7);
			//vectorRotacion=Rota3D(vectorRotacion, new Punto3D(BigDecimal.ZERO, BigDecimal.ONE, BigDecimal.ONE), 0.3);
			/*divisor=primerVector.norma().multiply(segundoVector.norma());
			if(divisor.compareTo(BigDecimal.ZERO)==0){
				angulo=0;
			}else{
				angulo=Math.acos(primerVector.productoPunto(segundoVector).divide(primerVector.norma().multiply(segundoVector.norma()),2).doubleValue());//MetodosNumericos.calculeArcoseno(primerVector.productoPunto(segundoVector).divide(primerVector.norma().multiply(segundoVector.norma()),2).subtract(new BigDecimal(Math.PI/2)), 2).doubleValue();
			}*/
			//angulo=calculaAngulo(primerVector, segundoVector);
			//System.out.println("Vector de rotación: (" + vectorRotacion.getX() + ", " + vectorRotacion.getY() + ", " + vectorRotacion.getZ() + ")");
			
			//xmouse=xmouse2;
			//ymouse=ymouse2;
			
			System.out.println(anguloX + " " + anguloY + " " + anguloZ);
			
			evaluar=false;
			repintar=true;
			this.repaint();
		}
		
		if(boton==MouseEvent.BUTTON2){
			movimiento=convPixCoordX(xmouse2-xmouse);
			xmin=xmin.subtract(movimiento);
			xmax=xmax.add(movimiento);
			
			if(xmin.compareTo(xmax)==1){
				comodin=xmin;
				xmin=xmax;
				xmax=comodin;
			}
			
			movimiento=convPixCoordY(xmouse2-xmouse);
			ymin=ymin.subtract(movimiento);
			ymax=ymax.add(movimiento);
			
			if(ymin.compareTo(ymax)==1){
				comodin=ymin;
				ymin=ymax;
				ymax=comodin;
			}
			
			movimiento=convPixCoordZ(ymouse2-ymouse);
			zmin=zmin.add(movimiento);
			zmax=zmax.subtract(movimiento);
			
			if(zmin.compareTo(zmax)==1){
				comodin=zmin;
				zmin=zmax;
				zmax=comodin;
			}
			
			xmouse=xmouse2;
			ymouse=ymouse2;
			
			evaluar=true;
			repintar=true;
			this.repaint();
		}
		
		if(boton==MouseEvent.BUTTON3){
			movimiento=convPixCoordX(xmouse2-xmouse);
			xmin=xmin.add(movimiento);
			xmax=xmax.add(movimiento);
			
			movimiento=convPixCoordY(xmouse2-xmouse);
			ymin=ymin.subtract(movimiento);
			ymax=ymax.subtract(movimiento);
			
			movimiento=convPixCoordZ(ymouse2-ymouse);
			zmin=zmin.add(movimiento);
			zmax=zmax.add(movimiento);
			
			xmouse=xmouse2;
			ymouse=ymouse2;
			evaluar=true;
			repintar=true;
			this.repaint();
		}
	}
	
	public void mouseWheelMoved(MouseWheelEvent e){
		int num=e.getWheelRotation();
		zoom(num, 0);
	}
}