package itcr.mathexplorer.paneles;

import javax.swing.*;
import itcr.mathexplorer.tablas.*;
import itcr.mathexplorer.expresion.*;

/**
 *@(#)VentanaGraficos2D.java
 *
 *Este es el panel donde se realizan las gráficas en dos dimensiones.
 *
 *@author Alexander Borbón Alpízar
 */
public class VentanaGraficos2D extends JInternalFrame{
	static final long serialVersionUID=10014;
	
	//Variables privadas
	
	private PanelGraficos2D panelGraficos2D;
	
	//Constructores
	/**
	 *Constructor de la ventana que contiene los gráficos en dos dimensiones.
	 *@param tablaOperadores Tabla con todos los operadores del programa.
	 *@param tablaFunciones Tabla con todas las funciones del programa.
	 *@param tablaConstantes Tabla con todas las constantes del programa.
	 *@param tablaFuncionesUsuario Tabla con todas las funciones definidas por el usuario en el programa.
	 */
	public VentanaGraficos2D(TablaOperadores tablaOperadores, TablaFunciones tablaFunciones, TablaConstantes tablaConstantes, TablaFuncionesUsuario tablaFuncionesUsuario){
		super("Graficos 2D",true,true,true,true);
		panelGraficos2D=new PanelGraficos2D(tablaOperadores, tablaFunciones, tablaConstantes, tablaFuncionesUsuario);
		this.add(panelGraficos2D);
	}
	
	/**
	 *Función que agrega una expresión para graficar.
	 *@param e La expresión para graficar
	 */
	public void agregaExpresion(Nodo e){
		panelGraficos2D.agregaExpresion(e);
	}
	
	/**
	 *Función que borra una expresión para graficar.
	 */
	public void borraExpresion(){
		panelGraficos2D.borraExpresion();
	}
	
	/**
	 *Función que borra todas las expresiones para graficar.
	 */
	public void borraTodas(){
		panelGraficos2D.borraTodas();
	}
	
	/**
	 *Función que retoma los valores de las opciones para graficar.
	 */
	public void retomarValores(){
		panelGraficos2D.retomarValores();
	}
	
	/**
	 *Función que realiza un zoom hacia adentro en la gráfica.
	 */
	public void zoomIn(){
		panelGraficos2D.zoomIn();
	}
	
	/**
	 *Función que realiza un zoom vertical hacia adentro en la gráfica.
	 */
	public void zoomInVertical(){
		panelGraficos2D.zoomInVertical();
	}
	
	/**
	 *Función que realiza un zoom horizontal hacia adentro en la gráfica.
	 */
	public void zoomInHorizontal(){
		panelGraficos2D.zoomInHorizontal();
	}
	
	/**
	 *Función que realiza un zoom hacia afuera en la gráfica.
	 */
	public void zoomOut(){
		panelGraficos2D.zoomOut();
	}
	
	/**
	 *Función que realiza un zoom vertical hacia afuera en la gráfica.
	 */
	public void zoomOutVertical(){
		panelGraficos2D.zoomOutVertical();
	}
	
	/**
	 *Función que realiza un zoom horizontal hacia afuera en la gráfica.
	 */
	public void zoomOutHorizontal(){
		panelGraficos2D.zoomOutHorizontal();
	}
}