package itcr.mathexplorer.paneles;

import javax.swing.*;
import itcr.mathexplorer.tablas.*;
import itcr.mathexplorer.expresion.*;

/**
 *@(#)VentanaGraficos2D.java
 *
 *Este es el panel donde se realizan las gráficas en tres dimensiones.
 *
 *@author Alexander Borbón Alpízar
 */
public class VentanaGraficos3D extends JInternalFrame{
	static final long serialVersionUID=10014;
	
	//Variables privadas
	
	private PanelGraficos3D panelGraficos3D;
	
	//Constructores
	/**
	 *Constructor de la ventana que contiene los gráficos en dos dimensiones.
	 *@param tablaOperadores Tabla con todos los operadores del programa.
	 *@param tablaFunciones Tabla con todas las funciones del programa.
	 *@param tablaConstantes Tabla con todas las constantes del programa.
	 *@param tablaFuncionesUsuario Tabla con todas las funciones definidas por el usuario en el programa.
	 */
	public VentanaGraficos3D(TablaOperadores tablaOperadores, TablaFunciones tablaFunciones, TablaConstantes tablaConstantes, TablaFuncionesUsuario tablaFuncionesUsuario){
		super("Graficos 3D",true,true,true,true);
		panelGraficos3D=new PanelGraficos3D(tablaOperadores, tablaFunciones, tablaConstantes, tablaFuncionesUsuario);
		this.add(panelGraficos3D);
	}
	
	/**
	 *Función que agrega una expresión para graficar.
	 *@param e La expresión para graficar
	 */
	public void agregaExpresion(Nodo e){
		panelGraficos3D.agregaExpresion(e);
	}
	
	/**
	 *Función que borra una expresión para graficar.
	 */
	public void borraExpresion(){
		panelGraficos3D.borraExpresion();
	}
	
	/**
	 *Función que borra todas las expresiones para graficar.
	 */
	public void borraTodas(){
		panelGraficos3D.borraTodas();
	}
	
	/**
	 *Función que retoma los valores de las opciones para graficar.
	 */
	public void retomarValores(){
		panelGraficos3D.retomarValores();
	}
	
	/**
	 *Función que realiza un zoom hacia adentro en la gráfica.
	 */
	public void zoomIn(){
		panelGraficos3D.zoomIn();
	}
	
	/**
	 *Función que realiza un zoom hacia adentro en la gráfica en el eje X.
	 */
	public void zoomInX(){
		panelGraficos3D.zoomInX();
	}
	
	/**
	 *Función que realiza un zoom hacia adentro en la gráfica en el eje Y.
	 */
	public void zoomInY(){
		panelGraficos3D.zoomInY();
	}
	
	/**
	 *Función que realiza un zoom hacia adentro en la gráfica en el eje Z.
	 */
	public void zoomInZ(){
		panelGraficos3D.zoomInZ();
	}
	
	/**
	 *Función que realiza un zoom hacia afuera en la gráfica.
	 */
	public void zoomOut(){
		panelGraficos3D.zoomOut();
	}
	
	/**
	 *Función que realiza un zoom hacia afuera en la gráfica en el eje X.
	 */
	public void zoomOutX(){
		panelGraficos3D.zoomOutX();
	}
	
	/**
	 *Función que realiza un zoom hacia afuera en la gráfica en el eje Y.
	 */
	public void zoomOutY(){
		panelGraficos3D.zoomOutY();
	}
	
	/**
	 *Función que realiza un zoom hacia afuera en la gráfica en el eje Z.
	 */
	public void zoomOutZ(){
		panelGraficos3D.zoomOutZ();
	}
}