package itcr.mathexplorer.paneles;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;
import java.io.*;
import javax.swing.border.*;
import itcr.mathexplorer.traductores.*;
import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.tipos.*;
import itcr.mathexplorer.tablas.*;
import itcr.mathexplorer.exceptions.*;
import itcr.mathexplorer.funciones.Simplificar;

/**
 *@(#)VentanaInternaExpresiones.java
 *
 *Este es el panel donde se colocan todas las expresiones.
 *
 *@author Alexander Borbón Alpízar
 */
public class VentanaInternaExpresiones extends JInternalFrame implements MouseListener, KeyListener{
	static final long serialVersionUID=10015;
	
	//VARIABLES GLOBALES
	
	//Esta es la tabla que contiene a todas las funciones permitidas
	TablaFunciones funciones;
	
	//Esta es la tabla que contiene a todas las constantes permitidas
	TablaConstantes constantes;
	
	//Esta es la tabla que contiene a todos los operadores permitidos
	TablaOperadores operadores;
	
	//Esta es la tabla que contiene a todas las funciones permitidas
	TablaFuncionesUsuario funcionesUsuario;
	
	//Campo de trabajo con las expresiones
	private PanelExpresiones panelExpresiones;
	private ScrollPanelExpresiones scrollPaneExpresiones;
	
	//Aquí se guardan todas las expresiones de esta ventana
	private Vector<Expresion> expresiones;
	
	//Variable donde se guarda la expresion que está elegida actualmente en esta ventana
	private Expresion expresionActual;
	
	//Esta variable marca si se presentaron cambios en la ventana
	private boolean cambios;
	
	//private int numeroVentana;
	/**
	 * Esta variable cuenta el número de ventanas que se utilizan.
	 */
	static public int contadorFrames=1;
	
	/**
	 * La distancia en x que se deja al graficar una nueva ventana.
	 */
	static public final int xOffset = 30;
	
	/**
	 * La distancia en y que se deja al graficar una nueva ventana.
	 */
	static public final int yOffset = 30;
	
	/**
	 * El ancho por defecto de una nueva ventana.
	 */
	static public final int ancho = 400;
	
	/**
	 * El alto por defecto de una nueva ventana.
	 */
	static public final int alto = 230;
	
	File archivoActual;
	
	JTextField campoTexto;
	
	int imagenElegida;
	
	private Color verde, azul;
	
	//CONSTRUCTORES
	/**
	 *Constructor de la ventana que contiene las expresiones.
	 *@param titulo El título que tendrá la ventana creada.
	 *@param campoTextoExpresion El campo de texto del programa donde se digitan las expresiones.
	 *@param tablaOperadores Tabla con todos los operadores del programa.
	 *@param tablaFunciones Tabla con todas las funciones del programa.
	 *@param tablaConstantes Tabla con todas las constantes del programa.
	 *@param tablaFuncionesUsuario Tabla con todas las funciones definidas por el usuario en el programa.
	 */
	public VentanaInternaExpresiones(String titulo, JTextField campoTextoExpresion,TablaOperadores tablaOperadores, TablaFunciones tablaFunciones, TablaConstantes tablaConstantes, TablaFuncionesUsuario tablaFuncionesUsuario){
		
		super(titulo + " " + contadorFrames,true,true,true,true);

		operadores=tablaOperadores;
		funciones=tablaFunciones;
		constantes=tablaConstantes;
		funcionesUsuario=tablaFuncionesUsuario;
		
		//Estos son los paneles donde se muestran las expresiones
		
		panelExpresiones=new PanelExpresiones();
		
		scrollPaneExpresiones=new ScrollPanelExpresiones();
		scrollPaneExpresiones.getViewport().add(panelExpresiones,BorderLayout.CENTER);
		
		this.add(scrollPaneExpresiones, BorderLayout.CENTER);
		this.setVisible(true);
		
		this.reshape(xOffset*((contadorFrames-1)%6 + (contadorFrames-1)/6), yOffset*((contadorFrames-1)%6),ancho,alto);
		this.setDefaultCloseOperation(JInternalFrame.DO_NOTHING_ON_CLOSE);
		
		cambios=false;
		expresiones=new Vector<Expresion>();
		campoTexto=campoTextoExpresion;
		
		verde=new Color(50, 200, 50);
		azul=new Color(100, 100, 255);
		
		campoTexto.addKeyListener(this);
		
		//numeroVentana=contadorFrames;
		
		contadorFrames++;
	}
	
	//FUNCIONES PUBLICAS
	
	/**
	 *Esta función agrega la expresión en esta ventana.
	 *@param expresion La expresión en un string para trabajar con ella.
	 *@exception SintaxException Error en la expresión.
	 */
	public void agregaExpresion(String expresion) throws SintaxException{
		try{
			Simplificar simplificar=new Simplificar(operadores, funciones, constantes, funcionesUsuario);
			
			Expresion nueva=new Expresion(operadores, funciones, constantes, funcionesUsuario);
			nueva.cambiaExpresionInfijo(expresion, false);
			panelExpresiones.agregaExpresion(nueva.obtenExpresionImagen(), " Exp: ");
			nueva.obtenExpresionImagen().addMouseListener(this);
			
			expresiones.add(nueva);
			nueva.obtenExpresionImagen().setNumeroImagen(expresiones.size()-1);
			
			Expresion simplificada=new Expresion(operadores, funciones, constantes, funcionesUsuario);
			simplificada.cambiaExpresionPrefijo(simplificar.simplifica(nueva.obtenExpresionPrefijo().clonarNodo()), true);
			panelExpresiones.agregaExpresion(simplificada.obtenExpresionImagen(), "    Sol: ");
			simplificada.obtenExpresionImagen().addMouseListener(this);
			
			((Expresion)expresiones.elementAt(imagenElegida)).obtenExpresionImagen().setBorder(null);
			
			expresiones.add(simplificada);
			imagenElegida=expresiones.size()-1;
			simplificada.obtenExpresionImagen().setNumeroImagen(imagenElegida);
			simplificada.obtenExpresionImagen().setBorder(new LineBorder(verde,1));
			
			expresionActual=simplificada;
			
			panelExpresiones.setSize(new Dimension(1,1));
			panelExpresiones.revalidate();
			cambios=true;
		}catch(SintaxException e){
			e.printStackTrace();
			throw e;
		}
	}
	
	/**
	 *Esta función indica si la ventana ha sufrido cambios sin guardar.
	 *@return True si han habido cambios, false en caso contrario.
	 */
	public boolean Cambios(){
		return cambios;
	}
	
	/**
	 *Esta función devuelve el archivo actual de la ventana.
	 *@return El archivo relacionado con la ventana.
	 */
	public File getArchivoActual(){
		return archivoActual;
	}
	
	/**
	 *Esta función devuelve el directorio actual donde se guarda el archivo
	 *asociado con esta ventana.
	 *@return El directorio de trabajo.
	 */
	public String getDirectorioActual(){
		return archivoActual.getParent();
	}
	
	/**
	 *Método para guardar el archivo
	 *@param archivo El archivo donde se guardará el documento.
	 *@exception IOException Error al guardar el archivo.
	 */
	public void guardarArchivo(File archivo) throws IOException{
		try{
			DataOutputStream archivoSalida=new DataOutputStream(new FileOutputStream(archivo));
			
			for(int i=0; i<expresiones.size(); i++){
				archivoSalida.writeUTF(((Expresion)expresiones.elementAt(i)).obtenExpresionInfijo());
			}
			archivoSalida.flush();
			archivoSalida.close();
			cambios=false;
			archivoActual=archivo;
		}catch(IOException e){
			e.printStackTrace();
			throw new IOException("Error al guardar el archivo: "+e.getMessage()+archivo.getName());
		}
	}
	
	/**
	 *Método para guardar el archivo
	 *@return True si el usuario escoge guardar el archivo o false si escoge cancelar.
	 *@exception IOException Error al guardar el archivo.
	 */
	public boolean guardarArchivo() throws IOException{
		JFileChooser chooser;
		int respuesta1, respuesta2;
		File archivo;
		
		if(archivoActual!=null){
			chooser = new JFileChooser(getDirectorioActual());
		}else{
			chooser = new JFileChooser();
		}

		MiFileFilter filter = new MiFileFilter();
		chooser.setFileFilter(filter);
		respuesta1 = chooser.showSaveDialog(this);
		if(respuesta1 == JFileChooser.APPROVE_OPTION) {
			archivo=chooser.getSelectedFile();
			if(!archivo.getParentFile().exists()){
				archivo.getParentFile().mkdirs();
				if(archivo.toString().indexOf(".mae")==-1){
					archivo=new File(archivo.toString().concat(".mae"));
				}
				guardarArchivo(archivo);
			}else if(archivo.exists()){
				respuesta2 = JOptionPane.showConfirmDialog(this,"¿Deseas sobreescribir el archivo?","El archivo ya existe",JOptionPane.YES_NO_OPTION);
				if(respuesta2 == JOptionPane.YES_OPTION){
					if(archivo.toString().indexOf(".mae")==-1){
						archivo=new File(archivo.toString().concat(".mae"));
					}
					guardarArchivo(archivo);
				}
			}else{
				if(archivo.toString().indexOf(".mae")==-1){
					archivo=new File(archivo.toString().concat(".mae"));
				}
				guardarArchivo(archivo);
			}
			return true;
		}else{
			return false;
		}
	}
	
	/**
	 *Método para abrir un archivo.
	 *@exception IOException Error al abrir el archivo.
	 */
	public void abrirArchivo() throws IOException{
		JFileChooser chooser;
		String dato;
		int respuesta1;
		File archivo;
		
		if(archivoActual!=null){
			chooser = new JFileChooser(getDirectorioActual());
		}else{
			chooser = new JFileChooser();
		}

		MiFileFilter filter = new MiFileFilter();
		chooser.setFileFilter(filter);
		respuesta1 = chooser.showOpenDialog(this);
		if(respuesta1 == JFileChooser.APPROVE_OPTION) {
			if(chooser.getSelectedFile().exists()){
				archivo=chooser.getSelectedFile();
				try{
					Simplificar simplificar=new Simplificar(operadores, funciones, constantes, funcionesUsuario);
					expresiones=new Vector<Expresion>();
					
					DataInputStream archivoEntrada = new DataInputStream(new FileInputStream(archivo));
					
					try{
						while(true){
							dato=archivoEntrada.readUTF();
							Expresion nueva=new Expresion(operadores, funciones, constantes, funcionesUsuario);
							expresiones.add(nueva.cambiaExpresionInfijo(dato, false));
							dato=archivoEntrada.readUTF();
							Expresion simplificada=new Expresion(operadores, funciones, constantes, funcionesUsuario);
							expresiones.add(simplificada.cambiaExpresionPrefijo(simplificar.simplifica(simplificada.cambiaExpresionInfijo(dato, false).obtenExpresionPrefijo().clonarNodo()), true));
						}
					}catch(EOFException e){} //Saca hasta que se acaba el archivo
					
					archivoEntrada.close();
					
					panelExpresiones=new PanelExpresiones();
					scrollPaneExpresiones.getViewport().add(panelExpresiones,BorderLayout.CENTER);
					
					for(int i=0; i<expresiones.size()-1;i=i+2){
						panelExpresiones.agregaExpresion(((Expresion)expresiones.elementAt(i)).obtenExpresionImagen(), " Exp: ");
						panelExpresiones.agregaExpresion(((Expresion)expresiones.elementAt(i+1)).obtenExpresionImagen(), "    Sol: ");
					}
					
					
					cambios=false;
					archivoActual=archivo;
				}catch(IOException e){
					e.printStackTrace();
					throw new IOException("Error al abrir el archivo: "+e.getMessage() + archivo.getName());
				}catch(Exception e){
					e.printStackTrace();
					throw new IOException("Error al abrir el archivo: "+e.getMessage() + archivo.getName());
				}
			}else{
				JOptionPane.showMessageDialog(this,"¿El archivo que desea abrir no existe!","El archivo no existe",JOptionPane.ERROR_MESSAGE);
			}
		}else{
			this.dispose();
		}
	}
	
	/**
	 *Método para guardar el archivo
	 *@param direccion La dirección donde se guardará el archivo.
	 *@exception IOException Error al guardar el archivo.
	 */
	/*public void exportarTexto(String direccion) throws IOException{
		try{
			PrintWriter archivoSalida = new PrintWriter(new FileWriter(direccion,false), true);
			for(int i=0;i<expresiones.size(); i++){
				archivoSalida.println(((Expresion)expresiones.elementAt(i)).obtenExpresionInfijo());
			}
			archivoSalida.close();
			cambios=false;
			archivoActual=direccion;
		}catch(IOException e){
			throw new IOException("Error al guardar el archivo: "+direccion);
		}
	}*/
	
	/**
	 *Método para abrir un archivo.
	 *@exception IOException Error al abrir el archivo.
	 */
	/*public void importarTexto() throws IOException{
		JFileChooser chooser;
		String archivo, directorio, datos;
		int respuesta1;
		Expresion expresionArchivo=new Expresion(operadores, funciones, constantes);
		
		expresiones =new Vector();
		
		if(archivoActual!=null){
			chooser = new JFileChooser(getDirectorioActual());
		}else{
			chooser = new JFileChooser();
		}

		MiFileFilter filter = new MiFileFilter();
		chooser.setFileFilter(filter);
		respuesta1 = chooser.showOpenDialog(this);
		if(respuesta1 == JFileChooser.APPROVE_OPTION) {
			if(chooser.getSelectedFile().exists()){
				archivo=chooser.getSelectedFile().toString();
				try{
					panelExpresiones=new PanelExpresiones();
					scrollPaneExpresiones.getViewport().add(panelExpresiones,BorderLayout.CENTER);
					
					BufferedReader archivoEntrada = new BufferedReader(new FileReader(archivo));
					int i=0;
					boolean tipo;
					while((datos=archivoEntrada.readLine())!=null){
						if(i%2==0){
							tipo=false;
						}else{
							tipo=true;
						}
						expresionArchivo=new Expresion(operadores, funciones, constantes);
						expresionArchivo.cambiaExpresionInfijo(datos,tipo);
						expresiones.add(expresionArchivo);
						if(!tipo){
							panelExpresiones.agregaExpresion(expresionArchivo.ObtenExpresionImagen(), " Exp: ");
						}else{
							panelExpresiones.agregaExpresion(expresionArchivo.ObtenExpresionImagen(), "    Sol: ");
						}
						
						i++;
					}
					archivoEntrada.close();
					cambios=false;
					archivoActual=archivo;
				}catch(IOException e){
					e.printStackTrace();
					throw new IOException("Error al guardar el archivo: "+archivo);
				}
			}else{
				JOptionPane.showMessageDialog(this,"¿El archivo que desea abrir no existe!","El archivo no existe",JOptionPane.ERROR_MESSAGE);
			}
		}
	}*/
	
	/**
	 *Retorna le número de ventana de esta ventana.
	 *@return El número de ventana.
	 */
	//public int getNumeroVentana(){
	//	return numeroVentana;
	//}
	
	/**
	 *Define el color de fondo de esta ventana.
	 *@param bg El nuevo color del fondo.
	 */
	public void setColorFondo(Color bg){
		panelExpresiones.setColorFondo(bg);
	}
	
	/**
	 *Define el color de la letra de esta ventana.
	 *@param bg El nuevo color de la letra.
	 */
	public void setColorLetra(Color bg){
		panelExpresiones.setColorLetra(bg);
	}
	
	/**
	 *Define el tipo de la letra de esta ventana.
	 *@param tipoLetra El nuevo tipo de letra.
	 */
	public void setTipoLetra(Font tipoLetra){
		panelExpresiones.setTipoLetra(tipoLetra);
	}
	
	/**
	 *Devuelve el número de expresiones de la ventana.
	 *@return El número de expresiones que hay actualmente en esta ventana.
	 */
	public int getNumeroExpresiones(){
		return expresiones.size();
	}
	
	/**
	 *Devuelve la expresión que actualmente está seleccionada en esta ventana.
	 *@return La expresión seleccionada.
	 */
	public Expresion getExpresionActual(){
		return expresionActual;
	}
	
	//Auditor de eventos para el mouse
	public void mouseExited(MouseEvent e){
		if(((TraductorPrefijoImagen)e.getSource()).getNumeroImagen()==imagenElegida){
			((TraductorPrefijoImagen)e.getSource()).setBorder(new LineBorder(verde,1));
		}else{
			((TraductorPrefijoImagen)e.getSource()).setBorder(null);
		}
	}
	
	public void mouseEntered(MouseEvent e){
		((TraductorPrefijoImagen)e.getSource()).setBorder(new LineBorder(azul,1));
	}
	
	public void mouseReleased(MouseEvent e){ }
	
	public void mousePressed(MouseEvent e){	}
	
	public void mouseClicked(MouseEvent e){
		if(e.getClickCount()==2){
			if(e.getButton()==MouseEvent.BUTTON1){
				campoTexto.setText(campoTexto.getText().substring(0,campoTexto.getSelectionStart()).concat(((Expresion)expresiones.elementAt(((TraductorPrefijoImagen)e.getSource()).getNumeroImagen())).obtenExpresionInfijo()).concat(campoTexto.getText().substring(campoTexto.getSelectionEnd())));
			}else if(e.getButton()==MouseEvent.BUTTON3){
				campoTexto.setText(campoTexto.getText().substring(0,campoTexto.getSelectionStart()).concat("(".concat(((Expresion)expresiones.elementAt(((TraductorPrefijoImagen)e.getSource()).getNumeroImagen())).obtenExpresionInfijo()).concat(")".concat(campoTexto.getText().substring(campoTexto.getSelectionEnd())))));
			}
		}
		
		((Expresion)expresiones.elementAt(imagenElegida)).obtenExpresionImagen().setBorder(null);
		imagenElegida=((TraductorPrefijoImagen)e.getSource()).getNumeroImagen();
		((TraductorPrefijoImagen)e.getSource()).setBorder(new LineBorder(verde,1));
		
		expresionActual=(Expresion)expresiones.elementAt(((TraductorPrefijoImagen)e.getSource()).getNumeroImagen());
		
		campoTexto.requestFocus();
	}
	
	//public void mouseMoved(MouseEvent e){ }
	
	//public void mouseDragged(MouseEvent e){	}
	
	//Auditor para los eventos de teclado
	public void keyPressed(KeyEvent e) { }
	
	public void keyReleased(KeyEvent e) {
		if(e.getSource()==campoTexto){
			if(e.getKeyCode()==KeyEvent.VK_F2){
				campoTexto.setText(campoTexto.getText().substring(0,campoTexto.getSelectionStart()).concat(((Expresion)expresiones.elementAt(imagenElegida)).obtenExpresionInfijo()).concat(campoTexto.getText().substring(campoTexto.getSelectionEnd())));
			}else if(e.getKeyCode()==KeyEvent.VK_F3){
				campoTexto.setText(campoTexto.getText().substring(0,campoTexto.getSelectionStart()).concat("(".concat(((Expresion)expresiones.elementAt(imagenElegida)).obtenExpresionInfijo()).concat(")".concat(campoTexto.getText().substring(campoTexto.getSelectionEnd())))));
			}
		}
	}
	
	public void keyTyped(KeyEvent e) { }
	
}//Fin de VentanaInternaExpresiones