package itcr.mathexplorer.tablas;

import java.util.Hashtable;
import itcr.mathexplorer.tipos.FuncionUsuario;

/**
 *@(#)TablaFuncionesUsuario.java
 *
 *Clase que construye una tabla de Hash para que se escriban en ella
 *todas las constantes disponibles en el programa.
 *
 *@author Alexander Borbón Alpízar
 */
public class TablaFuncionesUsuario extends Hashtable<String, FuncionUsuario>{
	static final long serialVersionUID=10010;
	
	//CONSTRUCTORES
	/**
	 * Constructor vacío de la Tabla donde se guardan las funciones del usuario del programa.
	 */
	public TablaFuncionesUsuario(){	}


	//FUNCIONES PUBLICAS
	
	
	/** 
	 *Agrega a la tabla de funciones una nueva nueva función definida por el usuario.
	 *@param llave La llave asociada a la función introducida
	 *@param nuevaFuncion La nueva función introducida
	 *@return El valor previo que había en la llave seleccionada o null si no había nada
	 */
	public Object agregaFuncion(String llave, FuncionUsuario nuevaFuncion)
	{
		return super.put(llave, nuevaFuncion);
	}
	
	/**
	 *Devuelve la constante asociada a la llave
	 *@param llave La llave asociada a la función buscada
	 *@return La expresión asociada con la llave dada
	 */
	public FuncionUsuario devuelveFuncion(String llave)
	{
		return (FuncionUsuario)super.get(llave);
	}
	
	/**
	 *Esta función busca si en esta tabla se encuentra la llave buscada
	 *@param llave La llave que se busca en la tabla
	 *@return Devuelve true si la llave aparece o false si no aparece.
	 */
	public boolean apareceLlave(String llave){
		return super.containsKey(llave);
	}
	
	/**
	 *Esta función devuelve el número de funciones que están actualmente en la tabla.
	 *@return El número de funciones que están en la tabla.
	 */
	public int numeroFunciones(){
		return super.size();
	}

}//fin de TablaFunciones