package itcr.mathexplorer.tipos;

import java.math.BigInteger;
import java.math.BigDecimal;
import java.math.MathContext;

/**
 *@(#)BigRational.java
 *
 * Clase para el manejo de números racionales con cualquier presición
 * de dígitos, para esta clase se utiliza la clase de java BigInteger
 *
 * @author Alexander Borbón Alpízar
 */
public class BigRational {
	
	//VARIABLES
	
	/*
	 *Variables privadas donde se guardará el numerador y el denominador del racional.
	 *El acceso a estas variables sólo es posible por medio de las funciones públicas
	 */
	private BigInteger numerador, denominador;



	//CONSTRUCTORES
	
	/**
	 *Constructor por defecto que crea el racional 0/1
	 */
    public BigRational() {
    	numerador=new BigInteger("0");
    	denominador=new BigInteger("1");
    }
    
	/**
	 *Constructor de un número BigRational dados dos enteros largos representando su numerador y su denominador.
	 *@param Numerador El numerador del nuevo número racional.
	 *@param Denominador El denominador del nuevo número racional.
	 *@exception NumberFormatException Si hay algún problema con el formato del número a construir.
	 *@exception ArithmeticException Si hay división por cero.
	 */
    public BigRational(BigInteger Numerador, BigInteger Denominador) throws NumberFormatException, ArithmeticException{
    	try{
    		construirRational(Numerador.toString(), Denominador.toString());
    	}catch(NumberFormatException e){
    		e.printStackTrace();
    		throw e;
    	}catch(ArithmeticException e){
    		e.printStackTrace();
    		throw e;
    	}
    }
    
	/**
	 *Constructor que traduce un BigInteger a un número BigRational.
	 *@param val El número a ser traducido a BigRational.
	 *@exception NumberFormatException Si hay algún problema con el formato del número a construir.
	 *@exception ArithmeticException Si hay división por cero.
	 */
    public BigRational(BigInteger val) throws NumberFormatException, ArithmeticException{
    	try{
    		construirRational(val.toString(), "1");
    	}catch(NumberFormatException e){
    		e.printStackTrace();
    		throw e;
    	}catch(ArithmeticException e){
    		e.printStackTrace();
    		throw e;
    	}
    }
    
	/**
	 *Constructor de un número BigRational dados dos Strings representando su numerador y su denominador.
	 *@param Numerador El numerador del nuevo número racional.
	 *@param Denominador El denominador del nuevo número racional.
	 *@exception NumberFormatException Si hay algún problema con el formato del número a construir.
	 *@exception ArithmeticException Si hay división por cero.
	 */
    public BigRational(String Numerador, String Denominador) throws NumberFormatException, ArithmeticException{
    	try{
    		construirRational(Numerador, Denominador);
    	}catch(NumberFormatException e){
    		e.printStackTrace();
    		throw e;
    	}catch(ArithmeticException e){
    		e.printStackTrace();
    		throw e;
    	}
    }
    
    //FUNCIONES PÚBLICAS
    
    /**
     *Calcula el valor absoluto del número racional.
     *@return El valor absoluto del número racional.
     */
    public BigRational abs(){
    	if (numerador.signum()==-1){
    		return this.negate();
    	}else{
    		return new BigRational(numerador,denominador);
    	}
    }
    
    /**
     *Determina el signo del número racional.
     *@return -1, 0, 1 dependiendo si es negativo, cero o positivo.
     */
    public int signum(){	
    	return numerador.signum();
    }
    
    /**
     *Calcula el opuesto del número racional.
     *@return El número opuesto del número racional.
     */
    public BigRational negate(){	
    	return new BigRational(numerador.negate(),denominador);
    }
    
    /**
     *Devuelve el numerador del número racional.
     *@return El numerador del número racional.
     */
    public BigInteger getNumerator(){
    	return new BigInteger(numerador.toString());
    }
    
    /**
     *Devuelve el denominador del número racional.
     *@return El denominador del número racional.
     */
    public BigInteger getDenominator(){
    	return new BigInteger(denominador.toString());
    }
    
    /**
     *Calcula la suma de this+val, ambos de tipo BigRational
     *@param val El número racional a ser sumado con this
     *@return La suma de los números this+val
     */
    public BigRational add(BigRational val){
    	BigInteger maximocomundivisor=denominador.gcd(val.getDenominator());
    	return new BigRational((numerador.multiply(val.getDenominator().divide(maximocomundivisor))).add(denominador.divide(maximocomundivisor).multiply(val.getNumerator())),denominador.multiply(val.getDenominator()).divide(maximocomundivisor));
    }
    
    /**
     *Calcula la resta de this-val, ambos de tipo BigRational
     *@param val El número racional a ser restado con this
     *@return La resta de los números this-val
     */
    public BigRational subtract(BigRational val){
    	return this.add(val.negate());
    }
    
    /**
     *Calcula la multiplicación de this*val, ambos de tipo BigRational
     *@param val El número racional a ser multiplicado con this
     *@return La multiplicación de los números this*val
     */
    public BigRational multiply(BigRational val){
    	BigInteger maximocomundivisor1=numerador.gcd(val.getDenominator());
    	BigInteger maximocomundivisor2=denominador.gcd(val.getNumerator());
    	return new BigRational(numerador.divide(maximocomundivisor1).multiply(val.getNumerator().divide(maximocomundivisor2)),denominador.divide(maximocomundivisor2).multiply(val.getDenominator().divide(maximocomundivisor1)));
    }
    
    /**
     *Calcula la división de this/val, ambos de tipo BigRational
     *@param val El número racional a ser dividido con this
     *@return La división de los números this/val
     *@exception ArithmeticException División por cero
     */
    public BigRational divide(BigRational val) throws ArithmeticException{
    	BigInteger maximocomundivisor1=numerador.gcd(val.getNumerator());
    	BigInteger maximocomundivisor2=denominador.gcd(val.getDenominator());
    	
    	BigRational Resultado;
    	try{
    		Resultado= new BigRational(numerador.divide(maximocomundivisor1).multiply(val.getDenominator().divide(maximocomundivisor2)),denominador.divide(maximocomundivisor2).multiply(val.getNumerator().divide(maximocomundivisor1)));
    	}catch(ArithmeticException e){
    		e.printStackTrace();
    		throw e;
    	}
    	return Resultado;
    }
    
    /**
     *Compara el BigRational con val para ver si es menor, igual o mayor.
     *@param val El BigRational con el que se hará la comparación.
     *@return -1, 0 ó 1 si el BigRational es menor, igual o mayor, respectivamente, que val.
     */
    public int compareTo(BigRational val){
    	return (numerador.multiply(val.getDenominator())).compareTo(denominador.multiply(val.getNumerator()));
    }
    
    /**
     *Compara el BigRational con val para ver si son iguales.
     *@param val El BigRational con el que se hará la comparación.
     *@return true si el BigRational es igual a val, false en caso contrario.
     */
    public boolean equals(BigRational val){
    	return (this.compareTo(val)==0);
    }
    
    /**
     *Devuelve el BigRational menor entre this y val.
     *@param val El BigRational con el que se hará la comparación.
     *@return El BigRational menor entre this y val.
     */
    public BigRational min(BigRational val){
    	if (this.compareTo(val)==-1){
    		return new BigRational(numerador, denominador);
    	}else{
    		return new BigRational(val.getNumerator(), val.getDenominator());
    	}
    }
    
    /**
     *Devuelve el BigRational mayor entre this y val.
     *@param val El BigRational con el que se hará la comparación.
     *@return El BigRational mayor entre this y val.
     */
    public BigRational max(BigRational val){
    	if (this.compareTo(val)==1){
    		return new BigRational(numerador, denominador);
    	}else{
    		return new BigRational(val.getNumerator(), val.getDenominator());
    	}
    }
    
    /**
     *Calcula el BigRational elevado a exponent.
     *@param exponent El exponente de la potencia.
     *@return El BigRational resultado de hacer la potencia this^exponent.
     */
    public BigRational pow(int exponent){
    	if (exponent<0){
    		construirRational(denominador.toString(), numerador.toString());
    		exponent*=-1;
    	}
    	return new BigRational(numerador.pow(exponent),denominador.pow(exponent));
    }
    
    
    /**
     *Convierte este BigRational a BigInteger, no indica si hubo pérdida de decimales.
     *@return La parte entera del número BigRational perdiendo sus decimales.
     */
    public BigInteger toBigInteger(){
    	return numerador.divide(denominador);
    }
    
    /**
     *Convierte este BigRational a BigInteger, lanza una exception si hubo pérdida de decimales.
     *@return El número BigRational como BigInteger.
     *@exception ArithmeticException Se lanza cuando la división no es exacta (hubo pérdida de decimales).
     */
    public BigInteger toBigIntegerExact() throws ArithmeticException{
    	if (!denominador.equals(BigInteger.ONE)){
    		throw new ArithmeticException("La conversión no es exacta.");
    	}
    	return new BigInteger(numerador.toString());
    }
    
    /**
     *Convierte este BigRational a BigDecimal.
     *@return El número BigRational como BigDecimal.
     *@exception ArithmeticException Se lanza si la división tiene expansión decimal infinita.
     */
    public BigDecimal toBigDecimal() throws ArithmeticException{
    	try{
    		BigDecimal bigdecimalnumerador= new BigDecimal(numerador);
    		BigDecimal bigdecimaldenominador= new BigDecimal(denominador);
    		return bigdecimalnumerador.divide(bigdecimaldenominador);
    	}catch(ArithmeticException e){
    		e.printStackTrace();
    		throw e;
    	}
    }
    
    /**
     *Convierte este BigRational a BigDecimal con la precisión dada.
     *@param precision La presición con la que se quiere el resultado (el número de decimales).
     *@return El número BigRational como BigDecimal con la precisión dada.
     *@exception ArithmeticException Se lanza si hay problemas al realizar la división.
     */
    public BigDecimal toBigDecimal(int precision) throws ArithmeticException{
    	try{
    		BigDecimal bigdecimalnumerador= new BigDecimal(numerador);
    		BigDecimal bigdecimaldenominador= new BigDecimal(denominador);
    		return bigdecimalnumerador.divide(bigdecimaldenominador, new MathContext(precision));
    	}catch(ArithmeticException e){
    		e.printStackTrace();
    		throw e;
    	}
    }
    
    /**
     *Transforma el BigRational a String
     *@return El BigRational como String
     */
    public String toString(){
    	return new String(numerador.toString() + " / " + denominador.toString());
    }
    
    //FUNCIONES PRIVADAS
    
    /*
     *Función para construir el número racional, esta función se fija que el denominador
     *sea positivo y que no sea cero y lo arregla de ser necesario.
     *También simplifica la fracción en caso de ser necesario.
     */
    private void construirRational(String Numerador, String Denominador) throws NumberFormatException, ArithmeticException{
    	int signo;
    	BigInteger maximocomundivisor;
    	
    	try{
    		numerador=new BigInteger(Numerador.toString());
    	}catch(NumberFormatException e){
    		throw new NumberFormatException("El numerador del número racional tiene un formato erróneo.");
    	}
    	try{
    		denominador=new BigInteger(Denominador.toString());
    	}catch(NumberFormatException e){
    		e.printStackTrace();
    		throw new NumberFormatException("El denominador del número racional tiene un formato erróneo.");
    	}
    	
    	signo=denominador.signum();
    	if (signo==-1){
    		numerador=numerador.negate();
    		denominador=denominador.negate();
    	}else if (signo==0){
    		numerador=BigInteger.ZERO;
    		denominador=BigInteger.ONE;
    		throw new ArithmeticException("División por cero.");
    	}
    	
    	maximocomundivisor=numerador.gcd(denominador);
    	if(maximocomundivisor.equals(BigInteger.ONE)==false){
    		numerador=numerador.divide(maximocomundivisor);
    		denominador=denominador.divide(maximocomundivisor);
    	}
    }//construirRational
    
    
}//BigRational