package itcr.mathexplorer.tipos;

import itcr.mathexplorer.expresion.*;

/**
 *@(#)FuncionUsuario.java
 *
 * Clase para el manejo de las funciones definidas por el usuario.
 *
 * @author Alexander Borbón Alpízar
 */
public class FuncionUsuario {
	
	//VARIABLES PRIVADAS, solo se tiene acceso a travez de funciones
	
	private String nombre;
	private Nodo nombreVariables; //Esta variable contiene el nombre de la función y las variables
	private Nodo expresion; //Esta contiene la expresión correspondiente a la función
	
	
	//CONSTRUCTORES
	
	/**
	 *Constructor sin parámetros.
	 */
	public FuncionUsuario(){
		nombre=new String();
		nombreVariables=new Nodo();
		expresion=new Nodo();
	}
	
	/**
	 *Constructor que recibe la función definida por el usuario ya parseada y lista.
	 *@param nuevaFuncion La expresión con el nombre de la función y las variables.
	 *@param nuevaExpresion La expresión correspondiente a la función dada.
	 */
	public FuncionUsuario(Nodo nuevaFuncion, Nodo nuevaExpresion){
		nombre=nuevaFuncion.getTipo();
		nombreVariables=nuevaFuncion;
		expresion=nuevaExpresion;
	}
    
    //FUNCIONES PÚBLICAS
    
    /**
     *Define si la función está definida con el número de parámetros dado.
     *@param numeroParametros El número de parámetros que recibe la función, se debe comparar con el número de parámetros con los que está definida la función.
     *@return True si admite el número de parámetros dado, false en caso contario.
     */
    public boolean permiteNumeroParametros(int numeroParametros){
    	if (numeroParametros==nombreVariables.getNumeroOperandos()){
    		return true;
    	}else{
    		return false;
    	}
    }
    
    /**
     *Devuelve la expresión correspondiente a la función.
     *@return La expresión correspondiente con la función.
     */
    public Nodo obtenExpresion(){
    	return expresion.clonarNodo();
    }
    
    /**
     *Devuelve la función con sus variables.
     *@return Un Nodo con la función correspondiente y sus variables.
     */
    public Nodo obtenFuncion(){
    	return nombreVariables.clonarNodo();
    }
    
    /**
     *Cambia la función con sus variables.
     *@param nuevaFuncion La nueva función con sus variables.
     *@return La FuncionUsuario para recursividad.
     */
    public FuncionUsuario cambiaFuncion(Nodo nuevaFuncion){
    	nombreVariables=nuevaFuncion;
    	
    	return this;
    }
    
    /**
     *Cambia la expresión correspondiente a la función.
     *@param nuevaExpresion La nueva expresión ligada a la función.
     *@return La FuncionUsuario para recursividad.
     */
    public FuncionUsuario cambiaExpresion(Nodo nuevaExpresion){
    	expresion=nuevaExpresion;
    	
    	return this;
    }
    
    /**
     *Función que devuelve el nombre de la función
     * @return Un string con el nombre de la función
     */
    public String obtenNombre(){
    	return nombre;
    }
    
    /**
     * Cambia el nombre de la función
     * @param nom El nuevo nombre
     */
    public void cambiaNombre(String nom){
    	nombre=nom;
    }
    
}//FuncionUsuario