package itcr.mathexplorer.traductores;

import java.awt.*;
import java.awt.geom.*;
import javax.swing.*;
import itcr.mathexplorer.exceptions.*;
import itcr.mathexplorer.tablas.*;
import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.herramientas.Opciones;
import itcr.mathexplorer.funciones.SimplificarNumeros;
import java.util.Vector;

/**
 *@(#)TraductorPrefijoImagen.java
 *
 *Clase que recibe una expresión en notación prefijo
 *y la formatea en forma de gráfico.
 *
 *Para formatear de manera gráfica la expresión exprEnPrefijo se 
 *utiliza Obj.Graficar(exprEnPostfijo).
 *
 *Esta clase está pensada para utilizarse en combinación con
 *la clase TraductorPrefijo que traduce una expresión
 *matemática escrita en lenguaje normal a postfijo.
 *
 *Esta clase en realidad extiende un JPanel,
 *ella es un JPanel y lo que hace es graficar en el JPanel
 *la expresión formateada.
 *
 *@author Alexander Borbón Alpízar
 */
public class TraductorPrefijoImagen extends JPanel{
	static final long serialVersionUID=10013;
	
	//VARIABLES GLOBALES
	
	private Nodo expresion;
	
	private TablaOperadores operadores;
	private TablaFunciones funciones;
	private TablaConstantes constantes;
	private TablaFuncionesUsuario funcionesUsuario;
	
	//Esta variable indica si se debe hacer la imagen tal y como el usuario la digitó o arreglando la expresión simplificada
	//false significa que hay que hacerlo tal y como se escribió y true significa que se debe arreglar.
	private boolean tipografico;
	
	//Esta variable indica si hay que repintar toda la expresión o solo volver a pegar la imagen
	private boolean repintar;
	
	//Esta es la variable que guarda la imagen para el repintado
	private Image laImagen;
	
	//Este JPanel para el auditor
	//private JPanel esteJPanel=this;
	
	private int numeroImagen;


	//CONSTRUCTORES
	
	/**
	 *Constructor del traductor de la expresión en prefijo a imagen.
	 *@param tablaoperadores Tabla con todos los operadores del programa.
	 *@param tablafunciones Tabla con todas las funciones del programa.
	 *@param tablaconstantes Tabla con todas las constantes del programa.
	 *@param tablafuncionesusuario Tabla con todas las funciones definidas por el usuario en el programa.
	 */
	public TraductorPrefijoImagen(TablaOperadores tablaoperadores, TablaFunciones tablafunciones, TablaConstantes tablaconstantes, TablaFuncionesUsuario tablafuncionesusuario){
		super();
		this.setBackground(Opciones.getColorFondo());
		expresion=new Nodo();
		operadores=tablaoperadores;
		funciones=tablafunciones;
		constantes=tablaconstantes;
		funcionesUsuario=tablafuncionesusuario;
		tipografico=false;
		repintar=true;
		//this.addMouseListener(this);
	}
	
	/**
	 *Constructor del traductor de la expresión en prefijo a imagen.
	 *@param expresionPrefijo La expresión a traducir.
	 *@param tablaoperadores Tabla con todos los operadores del programa.
	 *@param tablafunciones Tabla con todas las funciones del programa.
	 *@param tablaconstantes Tabla con todas las constantes del programa.
	 *@param tablafuncionesusuario Tabla con todas las funciones definidas por el usuario en el programa.
	 */
	public TraductorPrefijoImagen(Nodo expresionPrefijo, TablaOperadores tablaoperadores, TablaFunciones tablafunciones, TablaConstantes tablaconstantes, TablaFuncionesUsuario tablafuncionesusuario){
		super();
		this.setBackground(Opciones.getColorFondo());
		expresion=expresionPrefijo;
		operadores=tablaoperadores;
		funciones=tablafunciones;
		constantes=tablaconstantes;
		funcionesUsuario=tablafuncionesusuario;
		tipografico=false;
		repintar=true;
		repaint();
	}
	
	//FUNCIONES PUBLICAS
	
	/**
	 *Esta función cambia la expresión representada en este JPanel
	 *@param expresionPrefijo El string con la expresión a representar, esta tiene que estar en postfijo.
	 *@param tipoGrafico Indica si hay que graficar tal y como la expresión está (false) o si hay que arreglar la imagen (true).
	 *@exception SintaxException Error en la expresión.
	 */
	public void cambiaExpresion(Nodo expresionPrefijo, boolean tipoGrafico) throws SintaxException{
		try{
			expresion=expresionPrefijo;
			tipografico=tipoGrafico;
			repintar=true;
			repaint();
		}catch(Exception e){
			e.printStackTrace();
			throw new SintaxException("Error al graficar la expresión");
		}
	}
	
	/**
	 *Esta función le indica al panel si debe volver a repintar toda la imagen
	 *ya sea porque cambio la expresión o cambio el color de fondo o letra o
	 *solo volver a pintar la imagen tal y como estaba.
	 *@param valor True si hay que volver a pintar o false en caso contrario
	 */
	public void pintarImagen(boolean valor){
		repintar=valor;
	}

	public void paintComponent(Graphics g){
		super.paintComponent(g);
		this.setBackground(Opciones.getColorFondo());
		
		try{
			if (!expresion.equals("") && repintar){
				Font fuente=Opciones.getTipoLetra();
				laImagen=graficaExpresion(expresion, (Graphics2D)g, fuente.getSize());
				this.setSize(laImagen.getWidth(this)+10,laImagen.getHeight(this)+10);
				this.setPreferredSize(new Dimension(laImagen.getWidth(this)+10,laImagen.getHeight(this)+10));
				g.drawImage(laImagen,5,5,this);
				repintar=false;
			}else{
				g.drawImage(laImagen,5,5,this);
			}
		}catch(Exception e){
			e.printStackTrace();
			throw new SintaxException("Error al graficar");
		}
		
	}
	

	//FUNCIONES PRIVADAS
	
	private Image graficaExpresion(Nodo expresionPrefijo, Graphics2D g, int tamanoLetra) throws SintaxException{
		Image a, b, c, d;
		int altoa, anchoa, altob, anchob, altoc, anchoc, altod, anchod;
		Graphics2D ga, gb, gc, gd;
		FontMetrics metricaletra;
		int altocompletotexto;
		int altotexto, anchotexto, anchotexto2, espacio;
		String tipo, tipo2;
		Nodo comodin;
		
		try{
			opcionesGraphics2D(g, this.getWidth(), this.getHeight(), tamanoLetra);
			metricaletra= g.getFontMetrics();
			altocompletotexto=metricaletra.getAscent()+metricaletra.getDescent();
			altotexto=metricaletra.getAscent();
			
			tipo=expresionPrefijo.getTipo();
			
			if (tipo.equals("Variable")){ //Si el tipo Variable
				tipo2=expresionPrefijo.getOperando(1).getTipo();
				anchotexto=metricaletra.stringWidth(tipo2);
				
				a=createImage(anchotexto+2,altocompletotexto);
				ga=(Graphics2D)a.getGraphics();
				opcionesGraphics2D(ga,anchotexto,altocompletotexto, tamanoLetra);
				
				/*Font fuente=Opciones.getTipoLetra();
				if(!fuente.isItalic()){
					fuente=new Font(fuente.getFontName(), Font.ITALIC, tamanoLetra>=8?tamanoLetra:8);
					ga.setFont(fuente);
				}
				
				anchotexto=ga.getFontMetrics().stringWidth(tipo2);
				
				a=createImage(anchotexto,altocompletotexto);
				ga=(Graphics2D)a.getGraphics();
				opcionesGraphics2D(ga,anchotexto,altocompletotexto, tamanoLetra);*/
				
				Font fuente=Opciones.getTipoLetra();
				if(!fuente.isItalic()){
					fuente=new Font(fuente.getFontName(), Font.ITALIC, tamanoLetra>=8?tamanoLetra:8);
					ga.setFont(fuente);
				}
				
				ga.drawString(tipo2,0,altotexto);
				
				return a;
				
			}else if ((tipo.length()>5 && ("EnteroLargo").indexOf(tipo)!=-1) || constantes.apareceLlave(tipo)){ //Si el tipo es Entero, EnteroLargo o una constante
				tipo2=expresionPrefijo.getOperando(1).getTipo();
				anchotexto=metricaletra.stringWidth(tipo2);
				
				a=createImage(anchotexto,altocompletotexto);
				ga=(Graphics2D)a.getGraphics();
				opcionesGraphics2D(ga,anchotexto,altocompletotexto, tamanoLetra);
				
				ga.drawString(tipo2,0,altotexto);
				
				return a;
				
			}else if (tipo.length()>5 && ("DecimalLargo").indexOf(tipo)!=-1){ //Si el tipo es Decimal o DecimalLargo
				tipo2=expresionPrefijo.getOperando(1).getTipo();
				
				int pos=tipo2.indexOf(".");
				int num=pos+Opciones.getNumDecimales()-9;
				if(pos!=-1 && num<=tipo2.length()){
					tipo2=tipo2.substring(0,num);
				}
				
				anchotexto=metricaletra.stringWidth(tipo2);
				
				a=createImage(anchotexto,altocompletotexto);
				ga=(Graphics2D)a.getGraphics();
				opcionesGraphics2D(ga,anchotexto,altocompletotexto, tamanoLetra);
				
				ga.drawString(tipo2,0,altotexto);
				
				return a;
			}else if (tipo.equals("Racional")){ //Si el tipo es Racional
				tipo=expresionPrefijo.getOperando(1).getTipo();
				anchoa=metricaletra.stringWidth(tipo);
				tipo2=expresionPrefijo.getOperando(2).getTipo();
				anchob=metricaletra.stringWidth(tipo2);
				
				altoc=2*altocompletotexto+9;
				anchoc=Math.max(anchoa+8,anchob+8);
				
				a=createImage(anchoc,altoc);
				ga=(Graphics2D)a.getGraphics();
				opcionesGraphics2D(ga,anchoc,altoc, tamanoLetra);
				
				ga.drawString(tipo,(anchoc-anchoa)/2,altotexto);
				ga.draw(new Line2D.Double(0,altocompletotexto+5,anchoc+8,altocompletotexto+5));
				ga.drawString(tipo2,(anchoc-anchob)/2,altocompletotexto + 9+altotexto);
				
				return a;
			}else if (tipo.equals("Constante")){
				
				tipo2=((Nodo)expresionPrefijo.getOperando(1)).getTipo();
				
				if(tipo2.equals("#pi")){
					tipo2=tipo2.substring(1);
					anchotexto=metricaletra.stringWidth(tipo2);
					
					a=createImage(anchotexto,altocompletotexto);
					ga=(Graphics2D)a.getGraphics();
					opcionesGraphics2D(ga,anchotexto,altocompletotexto, tamanoLetra);
					
					ga.draw(new Line2D.Double(anchotexto/3,3*altocompletotexto/8,anchotexto/3,3*altocompletotexto/4));
					ga.draw(new Line2D.Double(2*anchotexto/3,3*altocompletotexto/8,2*anchotexto/3,3*altocompletotexto/4));
					ga.draw(new Line2D.Double(anchotexto/5,3*altocompletotexto/8+1,anchotexto/3-2,3*altocompletotexto/8));
					ga.draw(new Line2D.Double(anchotexto/3-2,3*altocompletotexto/8,2*anchotexto/3+1,3*altocompletotexto/8));
					ga.draw(new Line2D.Double(2*anchotexto/3+1,3*altocompletotexto/8,4*anchotexto/5,3*altocompletotexto/8-1));
					
					return a;
					
				}else{
					tipo2=tipo2.substring(1);
					
					anchotexto=metricaletra.stringWidth(tipo2);
					
					a=createImage(anchotexto,altocompletotexto);
					ga=(Graphics2D)a.getGraphics();
					opcionesGraphics2D(ga,anchotexto,altocompletotexto, tamanoLetra);
					ga.setFont(ga.getFont().deriveFont(Font.ITALIC));
					
					ga.drawString(tipo2,0,altotexto);
					
					return a;
				}
			}else if (tipo.equals("Boolean")){
				tipo2=((Nodo)expresionPrefijo.getOperando(1)).getTipo();
				
				anchotexto=metricaletra.stringWidth(tipo2);
				
				a=createImage(anchotexto,altocompletotexto);
				ga=(Graphics2D)a.getGraphics();
				opcionesGraphics2D(ga,anchotexto,altocompletotexto, tamanoLetra);
				//ga.setFont(ga.getFont().deriveFont(Font.ITALIC));
				
				ga.drawString(tipo2,0,altotexto);
				
				return a;
			}else if(tipo.equals(":=")){
				anchotexto=metricaletra.stringWidth(tipo);
				
				a=graficaExpresion(expresionPrefijo.getOperando(1),g, tamanoLetra);
				altoa=a.getHeight(this);
				anchoa=a.getWidth(this);
				
				b=graficaExpresion(expresionPrefijo.getOperando(2),g, tamanoLetra);
				altob=b.getHeight(this);
				anchob=b.getWidth(this);
				
				altoc=Math.max(altoa, altob);
				anchoc=anchoa+anchotexto+anchob+4;
				
				c=createImage(anchoc,altoc);
				gc=(Graphics2D)c.getGraphics();
				opcionesGraphics2D(gc,anchoc,altoc, tamanoLetra);
				
				gc.drawImage(a,0,(altoc-altoa)/2,this);
				gc.drawString(tipo,anchoa+2,(altotexto+altoc)/2);
				gc.drawImage(b,anchoa+4+anchotexto,(altoc-altob)/2,this);
				
				return c;
			}else if(tipo.equals("Lista")){
				if(expresionPrefijo.getNumeroOperandos()!=0){
					a=graficaExpresion(expresionPrefijo.getOperando(1),g, tamanoLetra);
					altoa=a.getHeight(this);
					anchoa=a.getWidth(this);
					
					tipo2=" , ";
					for(int i=2;i<=expresionPrefijo.getNumeroOperandos();i++){
						
						b=graficaExpresion(expresionPrefijo.getOperando(i),g, tamanoLetra);
						altob=b.getHeight(this);
						anchob=b.getWidth(this);
						
						anchotexto2=metricaletra.stringWidth(tipo2);
						
						altoc=Math.max(Math.max(altoa,altob),altotexto);
						anchoc=anchoa+anchotexto2+anchob+2;
						
						c=createImage(anchoc,altoc);
						gc=(Graphics2D)c.getGraphics();
						opcionesGraphics2D(gc,anchoc,altoc, tamanoLetra);
						
						gc.drawImage(a,0,(altoc-altoa)/2,this);
						gc.drawString(tipo2,anchoa+2,altoc/2+altotexto/3);
						gc.drawImage(b,anchoa+2+anchotexto2,(altoc-altob)/2,this);
						
						a=c;
						altoa=a.getHeight(this);
						anchoa=a.getWidth(this);
					}
					
					espacio=Math.min(Math.max(4,altoa/6),16);
					
					altoc=Math.max(altoa,altotexto);
					anchoc=anchoa+2*espacio+4;
					
					c=createImage(anchoc,altoc);
					gc=(Graphics2D)c.getGraphics();
					opcionesGraphics2D(gc,anchoc,altoc, tamanoLetra);
					
					int x1=espacio/2;
					int x2=-x1;
					if(espacio%2==1){
						x2--;
					}
					
					//Primer llave
					gc.drawArc(x1, 0, espacio, altoc/3, 90, 90);
					gc.drawLine(x1, altoc/6, x1, altoc/3);
					gc.drawArc(x2, altoc/6, espacio, altoc/3, 0, -90);
					gc.drawArc(x2, altoc/2, espacio, altoc/3, 0, 90);
					gc.drawLine(x1, 2*altoc/3, x1, 5*altoc/6);
					gc.drawArc(x1, 2*altoc/3, espacio, altoc/3, -90, -90);
					//Lo de adentro
					gc.drawImage(a, espacio+2,(altoc-altoa)/2,this);
					//Segunda llave
					gc.drawArc(anchoa+espacio+3+x2, 0, espacio, altoc/3, 90, -90);
					gc.drawLine(anchoa+espacio+3+x1, altoc/6, anchoa+espacio+3+x1, altoc/3);
					gc.drawArc(anchoa+espacio+3+x1, altoc/6, espacio, altoc/3, -90, -90);
					gc.drawArc(anchoa+espacio+3+x1, altoc/2, espacio, altoc/3, 90, 90);
					gc.drawLine(anchoa+espacio+3+x1, 2*altoc/3, anchoa+espacio+3+x1, 5*altoc/6);
					gc.drawArc(anchoa+espacio+3+x2, 2*altoc/3, espacio, altoc/3, 0, -90);
				}else{
					espacio=4;
					
					altoc=altotexto;
					anchoc=2*espacio+4;
					
					c=createImage(anchoc,altoc);
					gc=(Graphics2D)c.getGraphics();
					opcionesGraphics2D(gc,anchoc,altoc, tamanoLetra);
					
					int x1=espacio/2;
					int x2=-x1;
					if(espacio%2==1){
						x2--;
					}
					
					//Primer llave
					gc.drawArc(x1, 0, espacio, altoc/3, 90, 90);
					gc.drawLine(x1, altoc/6, x1, altoc/3);
					gc.drawArc(x2, altoc/6, espacio, altoc/3, 0, -90);
					gc.drawArc(x2, altoc/2, espacio, altoc/3, 0, 90);
					gc.drawLine(x1, 2*altoc/3, x1, 5*altoc/6);
					gc.drawArc(x1, 2*altoc/3, espacio, altoc/3, -90, -90);
					//Segunda llave
					gc.drawArc(espacio+3+x2, 0, espacio, altoc/3, 90, -90);
					gc.drawLine(espacio+3+x1, altoc/6, espacio+3+x1, altoc/3);
					gc.drawArc(espacio+3+x1, altoc/6, espacio, altoc/3, -90, -90);
					gc.drawArc(espacio+3+x1, altoc/2, espacio, altoc/3, 90, 90);
					gc.drawLine(espacio+3+x1, 2*altoc/3, espacio+3+x1, 5*altoc/6);
					gc.drawArc(espacio+3+x2, 2*altoc/3, espacio, altoc/3, 0, -90);
				}
				
				return c;
			}else if(tipo.equals("Matriz")){
				if(expresionPrefijo.getNumeroOperandos()>1){
					Vector<Image> graficos=new Vector<Image>();
					
					int filas=expresionPrefijo.getNumeroOperandos();
					int columnas=expresionPrefijo.getOperando(1).getNumeroOperandos();
					
					altoa=0;
					anchoa=0;
					
					for(int i=1;i<=filas;i++){
						for(int j=1;j<=columnas;j++){
							a=graficaExpresion(expresionPrefijo.getOperando(i).getOperando(j),g, tamanoLetra);
							graficos.addElement(a);
							altoa=Math.max(altoa, a.getHeight(this));
							anchoa=Math.max(anchoa, a.getWidth(this));
						}
					}
					
					altoa+=10;
					anchoa+=10;
					
					espacio=Math.min(Math.max(4,altoa/6),16);
					
					altoc=filas*Math.max(altoa,altotexto);
					anchoc=columnas*anchoa+2*espacio+4;
					
					c=createImage(anchoc,altoc);
					gc=(Graphics2D)c.getGraphics();
					opcionesGraphics2D(gc,anchoc,altoc, tamanoLetra);
					
					//Primer llave
					gc.drawLine(0, 0, espacio, 0);
					gc.drawLine(0, 0, 0, altoc-1);
					gc.drawLine(0, altoc-1, espacio, altoc-1);
					
					//Lo de adentro
					int k=0;
					for(int i=1;i<=filas;i++){
						for(int j=1;j<=columnas;j++){
							b=(Image)graficos.elementAt(k);
							altob=b.getHeight(this);
							anchob=b.getWidth(this);
							
							gc.drawImage(b, (j-1)*anchoa+espacio+1+(anchoa-anchob)/2,(i-1)*altoa+(altoa-altob)/2,this);
							
							k++;
						}
					}
					
					//Segunda llave
					gc.drawLine(columnas*anchoa+espacio+3, 0, columnas*anchoa+2*espacio+3, 0);
					gc.drawLine(columnas*anchoa+2*espacio+3, 0, columnas*anchoa+2*espacio+3, altoc-1);
					gc.drawLine(columnas*anchoa+2*espacio+3, altoc-1, columnas*anchoa+espacio+3, altoc-1);
				}else if(expresionPrefijo.getNumeroOperandos()==1){
					a=graficaExpresion(expresionPrefijo.getOperando(1).getOperando(1),g, tamanoLetra);
					altoa=a.getHeight(this);
					anchoa=a.getWidth(this);
					
					tipo2=" , ";
					for(int i=2;i<=expresionPrefijo.getOperando(1).getNumeroOperandos();i++){
						
						b=graficaExpresion(expresionPrefijo.getOperando(1).getOperando(i),g, tamanoLetra);
						altob=b.getHeight(this);
						anchob=b.getWidth(this);
						
						anchotexto2=metricaletra.stringWidth(tipo2);
						
						altoc=Math.max(Math.max(altoa,altob),altotexto);
						anchoc=anchoa+anchotexto2+anchob+2;
						
						c=createImage(anchoc,altoc);
						gc=(Graphics2D)c.getGraphics();
						opcionesGraphics2D(gc,anchoc,altoc, tamanoLetra);
						
						gc.drawImage(a,0,(altoc-altoa)/2,this);
						gc.drawString(tipo2,anchoa+2,altoc/2+altotexto/3);
						gc.drawImage(b,anchoa+2+anchotexto2,(altoc-altob)/2,this);
						
						a=c;
						altoa=a.getHeight(this);
						anchoa=a.getWidth(this);
					}
					
					espacio=Math.min(Math.max(4,altoa/6),16);
					
					altoc=Math.max(altoa,altotexto);
					anchoc=anchoa+2*espacio+5;
					
					c=createImage(anchoc,altoc);
					gc=(Graphics2D)c.getGraphics();
					opcionesGraphics2D(gc,anchoc,altoc, tamanoLetra);
					
					//Primer llave
					gc.drawLine(0, 0, espacio, 0);
					gc.drawLine(0, 0, 0, altoc-1);
					gc.drawLine(0, altoc-1, espacio, altoc-1);
					//Lo de adentro
					gc.drawImage(a, espacio+2,(altoc-altoa)/2,this);
					//Segunda llave
					gc.drawLine(anchoa+espacio+4, 0, anchoa+2*espacio+4, 0);
					gc.drawLine(anchoa+2*espacio+4, 0, anchoa+2*espacio+4, altoc-1);
					gc.drawLine(anchoa+2*espacio+4, altoc-1, anchoa+espacio+4, altoc-1);
				}else{
					espacio=4;
					
					altoc=altotexto;
					anchoc=2*espacio+4;
					
					c=createImage(anchoc,altoc);
					gc=(Graphics2D)c.getGraphics();
					opcionesGraphics2D(gc,anchoc,altoc, tamanoLetra);
					
					int x1=espacio/2;
					int x2=-x1;
					if(espacio%2==1){
						x2--;
					}
					
					//Primer parentesis cuadrado
					gc.drawLine(0, 0, espacio, 0);
					gc.drawLine(0, 0, 0, altoc-1);
					gc.drawLine(0, altoc-1, espacio, altoc-1);
					//Segunda llave
					gc.drawLine(espacio+3, 0, 2*espacio+3, 0);
					gc.drawLine(2*espacio+3, 0, 2*espacio+3, altoc-1);
					gc.drawLine(2*espacio+3, altoc-1, espacio+3, altoc-1);
				}
				
				return c;
			}else if(operadores.apareceLlave(tipo)){ //Si es un operador o una asignación
				a=graficaExpresion(expresionPrefijo.getOperando(1),g, tamanoLetra);
				altoa=a.getHeight(this);
				anchoa=a.getWidth(this);
				tipo2=expresionPrefijo.getOperando(1).getTipo();
				
				//Si lo anterior era una suma y lo que sigue es una potencia hay que subirla un poquito
				if((tipo.equals("·") || tipo.equals("+")) && tipo2.equals("^")){
					altoa+=altotexto/2 + 3;
				}
				
				//Si hay que ponerle paréntesis al primer término
				if(operadores.apareceLlave(tipo2) && ((!tipo.equals("/") &&operadores.devuelveOperador(tipo).prioridad()>operadores.devuelveOperador(tipo2).prioridad()) || (tipo.equals("^") && tipo.equals("^")))){
					espacio=Math.min(Math.max(8,altoa/6),16);
					
					anchoc=anchoa+2*espacio+2;
					altoc=altoa;
					
					c=createImage(anchoc,altoc);
					gc=(Graphics2D)c.getGraphics();
					opcionesGraphics2D(gc,anchoc,altoc, tamanoLetra);
					
					gc.draw(new Arc2D.Double(0,0,2*espacio-1,altoc,100,160, Arc2D.OPEN));
					gc.drawImage(a,espacio,0,this);
					gc.draw(new Arc2D.Double(anchoa+1,0,2*espacio-1,altoc,280,160, Arc2D.OPEN));
					a=c;
					altoa=a.getHeight(this);
					anchoa=a.getWidth(this);
				}
				
				//Se inicia con los operadores que no son binarios
				if(tipo.equals("!") || tipo.equals("%")){
					anchotexto=metricaletra.stringWidth(tipo);
					
					altoc=Math.max(altotexto,altoa);
					anchoc=anchoa+anchotexto+2;
					
					c=createImage(anchoc,altoc);
					gc=(Graphics2D)c.getGraphics();
					opcionesGraphics2D(gc,anchoc,altoc, tamanoLetra);
					
					gc.drawImage(a,0,(altoc-altoa)/2,this);
					gc.drawString(tipo,anchoa+2,(altoc+altotexto)/2-1);
					
					return c;
				}else if(!operadores.devuelveOperador(tipo).esBinario()){
					espacio=Math.min(Math.max(8,altoa/6),16);
					
					anchoc=anchoa+2*espacio+2;
					altoc=altoa;
					
					c=createImage(anchoc,altoc);
					gc=(Graphics2D)c.getGraphics();
					opcionesGraphics2D(gc,anchoc,altoc, tamanoLetra);
					
					gc.draw(new Arc2D.Double(0,0,2*espacio-1,altoc,100,160, Arc2D.OPEN));
					gc.drawImage(a,espacio,0,this);
					gc.draw(new Arc2D.Double(anchoa+1,0,2*espacio-1,altoc,280,160, Arc2D.OPEN));
					a=c;
					altoa=a.getHeight(this);
					anchoa=a.getWidth(this);
					
					anchotexto=metricaletra.stringWidth(tipo);
					
					anchoc=anchoa+anchotexto+2;
					altoc=altoa;
					
					c=createImage(anchoc,altoc);
					gc=(Graphics2D)c.getGraphics();
					opcionesGraphics2D(gc,anchoc,altoc, tamanoLetra);
					
					gc.drawString(tipo,0,(altotexto+altoc)/2);
					gc.drawImage(a,anchotexto+2,0,this);
					
					return c;
				}else if(tipo.equals("/")){
					b=graficaExpresion(expresionPrefijo.getOperando(2),g, tamanoLetra);
					altob=b.getHeight(this);
					anchob=b.getWidth(this);
					
					altoc=altoa+9+altob;
					anchoc=Math.max(anchoa+8,anchob+8);
					
					c=createImage(anchoc,altoc);
					gc=(Graphics2D)c.getGraphics();
					opcionesGraphics2D(gc,anchoc,altoc, tamanoLetra);
					
					gc.drawImage(a,(anchoc-anchoa)/2,0,this);
					gc.draw(new Line2D.Double(0,altoa+5,anchoc+8,altoa+5));
					gc.drawImage(b,(anchoc-anchob)/2,altoa + 9,this);
					
					return c;
				}else if(tipo.equals("^")){
					//Si el exponente es negativo se maneja como división
					if(tipografico && expresionPrefijo.getOperando(2).getTipo().length()>5 &&("EnteroLargo DecimalLargo Racional").indexOf(expresionPrefijo.getOperando(2).getTipo())!=-1 && expresionPrefijo.getOperando(2).esIgual(SimplificarNumeros.MENOSUNO)){
						anchotexto=metricaletra.stringWidth("1");
						
						b=createImage(anchotexto,altocompletotexto);
						gb=(Graphics2D)b.getGraphics();
						opcionesGraphics2D(gb,anchotexto,altocompletotexto, tamanoLetra);
						
						gb.drawString("1",0,altotexto);
						
						altob=b.getHeight(this);
						anchob=b.getWidth(this);
						
						altoc=altocompletotexto+9+altoa;
						anchoc=Math.max(anchoa+8,anchob+8);
						
						c=createImage(anchoc,altoc);
						gc=(Graphics2D)c.getGraphics();
						opcionesGraphics2D(gc,anchoc,altoc, tamanoLetra);
						
						gc.drawImage(b,(anchoc-anchob)/2,0,this);
						gc.draw(new Line2D.Double(0,altob+5,anchoc+8,altob+5));
						gc.drawImage(a,(anchoc-anchoa)/2,altob + 9,this);
					}else if(tipografico && expresionPrefijo.getOperando(2).getTipo().length()>5 && ("EnteroLargo DecimalLargo Racional").indexOf(expresionPrefijo.getOperando(2).getTipo())!=-1 && (expresionPrefijo.getOperando(2).getOperando(1).getTipo()).indexOf("-")!=-1){
						comodin=expresionPrefijo.clonarNodo();
						String numero=comodin.getOperando(2).getOperando(1).getTipo();
						numero=numero.substring(1);
						comodin.getOperando(2).eliminaOperando(1);
						comodin.getOperando(2).agregaOperando(new Nodo(numero),1);
						
						b=graficaExpresion(comodin,g, tamanoLetra);
						altob=b.getHeight(this);
						anchob=b.getWidth(this);

						anchotexto=metricaletra.stringWidth("1");
						
						a=createImage(anchotexto,altocompletotexto);
						ga=(Graphics2D)a.getGraphics();
						opcionesGraphics2D(ga,anchotexto,altocompletotexto, tamanoLetra);
						
						ga.drawString("1",0,altotexto);
						
						altoa=a.getHeight(this);
						anchoa=a.getWidth(this);
						altoc=altoa+9+altob;
						anchoc=Math.max(anchoa+8,anchob+8);
						
						c=createImage(anchoc,altoc);
						gc=(Graphics2D)c.getGraphics();
						opcionesGraphics2D(gc,anchoc,altoc, tamanoLetra);
						
						gc.drawImage(a,(anchoc-anchoa)/2,0,this);
						gc.draw(new Line2D.Double(0,altoa+5,anchoc+8,altoa+5));
						gc.drawImage(b,(anchoc-anchob)/2,altoa + 9,this);
					}else{	//Si el exponente no es negativo
						b=graficaExpresion(expresionPrefijo.getOperando(2),g, tamanoLetra-1);
						altob=b.getHeight(this);
						anchob=b.getWidth(this);
						
						anchotexto=metricaletra.stringWidth(tipo);
						
						altoc=Math.max(altob+2*altoa/3,altoa);
						anchoc=anchoa+anchob+2;
						
						c=createImage(anchoc,altoc);
						gc=(Graphics2D)c.getGraphics();
						opcionesGraphics2D(gc,anchoc,altoc, tamanoLetra);
						
						gc.drawImage(a,0,altoc-altoa,this);
						gc.drawImage(b,anchoa+2,0,this);
					}
					
					return c;
				}else if(tipo.equals("·")){
					boolean multiplicamenosuno=false, multiplicauno=false, numerador=true;
					
					//c es la imagen con el numerador y d con el denominador
					c=null;
					d=null;
					
					//El primer término se maneja por separado
					if(expresionPrefijo.getOperando(1).esIgual(SimplificarNumeros.MENOSUNO)){
						comodin=new Nodo("Entero",1);
						comodin.agregaOperando(new Nodo("-"));
						a=graficaExpresion(comodin,g, tamanoLetra);
						multiplicamenosuno=true;
						altoa=a.getHeight(this);
						anchoa=a.getWidth(this);
					}else if(expresionPrefijo.getOperando(1).esIgual(SimplificarNumeros.UNO)){
						multiplicauno=true;
					}
					
					for(int i=1;i<=expresionPrefijo.getNumeroOperandos();i++){
						tipo2=expresionPrefijo.getOperando(i).getTipo();

						if(tipografico && tipo2.equals("^") && expresionPrefijo.getOperando(i).getOperando(2).esIgual(SimplificarNumeros.MENOSUNO)){ //expresionPrefijo.getOperando(i).getOperando(2).getTipo().length()>5 && ("EnteroLargo DecimalLargo Racional").indexOf(expresionPrefijo.getOperando(i).getOperando(2).getTipo())!=-1 && expresionPrefijo.getOperando(i).getOperando(2).esIgual(SimplificarNumeros.MENOSUNO)){
							b=graficaExpresion(expresionPrefijo.getOperando(i).getOperando(1),g, tamanoLetra);
							altob=b.getHeight(this);
							anchob=b.getWidth(this);
							numerador=false;
						}else if(tipografico && tipo2.equals("^") && expresionPrefijo.getOperando(i).getOperando(2).getTipo().length()>5 && ("EnteroLargo DecimalLargo Racional").indexOf(expresionPrefijo.getOperando(i).getOperando(2).getTipo())!=-1 && (expresionPrefijo.getOperando(i).getOperando(2).getOperando(1).getTipo()).indexOf("-")!=-1){
							comodin=expresionPrefijo.getOperando(i).clonarNodo();
							String numero=comodin.getOperando(2).getOperando(1).getTipo();
							numero=numero.substring(1);
							comodin.getOperando(2).eliminaOperando(1);
							comodin.getOperando(2).agregaOperando(new Nodo(numero),1);
							
							b=graficaExpresion(comodin,g, tamanoLetra);
							altob=b.getHeight(this)+altotexto/2 + 3;
							anchob=b.getWidth(this);
							
							//Si lo anterior era una suma y lo que sigue es una potencia hay que subirla un poquito
							/*if(tipo2.equals("^")){
								altob+=altotexto/2 + 3;
							}*/
							
							numerador=false;
						}else{
							b=graficaExpresion(expresionPrefijo.getOperando(i),g, tamanoLetra);
							altob=b.getHeight(this);
							anchob=b.getWidth(this);
							
							//Si lo que sigue es una potencia hay que subirla un poquito
							if(tipo2.equals("^")){
								altob+=altotexto/2 + 3;
							}
							
							//Si hay que ponerle paréntesis al primer término
							if(i==1 && tipo2.equals("+")){
								espacio=Math.min(Math.max(8,altob/6),16);
								
								anchoc=anchob+2*espacio+2;
								altoc=altob;
								
								c=createImage(anchoc,altoc);
								gc=(Graphics2D)c.getGraphics();
								opcionesGraphics2D(gc,anchoc,altoc, tamanoLetra);
								
								gc.draw(new Arc2D.Double(0,0,2*espacio-1,altoc,100,160, Arc2D.OPEN));
								gc.drawImage(b,espacio,0,this);
								gc.draw(new Arc2D.Double(anchob+1,0,2*espacio-1,altoc,280,160, Arc2D.OPEN));
								b=c;
								c=null;
								altob=b.getHeight(this);
								anchob=b.getWidth(this);
							}
							
							
							numerador=true;
						}
						
						anchotexto=metricaletra.stringWidth(tipo);
						
						if(numerador){ //Si es un término que va en el numerador
							if(c==null){
								if(multiplicamenosuno){
									c=a;
									//multiplicamenosuno=false;
								}else if(multiplicauno){
									multiplicauno=false;
								}else{
									c=createImage(anchob,altob);
									gc=(Graphics2D)c.getGraphics();
									opcionesGraphics2D(gc,anchob,altob, tamanoLetra);
									
									gc.drawImage(b,0,0,this);
									//c=b;
								}
							}else{
								altoc=c.getHeight(this);
								anchoc=c.getWidth(this);
								altoa=Math.max(Math.max(altoc,altob),altotexto);
								
								if(operadores.apareceLlave(tipo2) && !tipo2.equals("^") && !tipo2.equals("/") && operadores.devuelveOperador(tipo).prioridad()>operadores.devuelveOperador(tipo2).prioridad()){
									espacio=Math.min(Math.max(8,altob/6),16);
									
									if(multiplicamenosuno){
										anchoa=anchoc+anchob+2*espacio+2;
										
										a=createImage(anchoa,altoa);
										ga=(Graphics2D)a.getGraphics();
										opcionesGraphics2D(ga,anchoa,altoa, tamanoLetra);
										
										ga.drawImage(c,0,(altoa-altoc)/2,this);
										ga.draw(new Arc2D.Double(anchoc+4,0,2*espacio-1,altoa,100,160, Arc2D.OPEN));
										ga.drawImage(b,anchoc+2+espacio,(altoa-altob)/2,this);
										ga.draw(new Arc2D.Double(anchoc+2+anchob,0,2*espacio-1,altoa,280,160, Arc2D.OPEN));
										multiplicamenosuno=false;
									}else{
										anchoa=anchoc+anchotexto+anchob+2*espacio+2;
										
										a=createImage(anchoa,altoa);
										ga=(Graphics2D)a.getGraphics();
										opcionesGraphics2D(ga,anchoa,altoa, tamanoLetra);
										
										ga.drawImage(c,0,(altoa-altoc)/2,this);
										ga.drawString(tipo,anchoc+1,(altotexto+altoa)/2);
										ga.draw(new Arc2D.Double(anchoc+4+anchotexto,0,2*espacio-1,altoa,100,160, Arc2D.OPEN));
										ga.drawImage(b,anchoc+2+anchotexto+espacio,(altoa-altob)/2,this);
										ga.draw(new Arc2D.Double(anchoc+2+anchotexto+anchob,0,2*espacio-1,altoa,280,160, Arc2D.OPEN));
										multiplicamenosuno=false;
									}
									
									c=a;
								}else{
									if(multiplicamenosuno){
										anchoa=anchoc+anchob+4;
										
										a=createImage(anchoa,altoa);
										ga=(Graphics2D)a.getGraphics();
										opcionesGraphics2D(ga,anchoa,altoa, tamanoLetra);
										
										ga.drawImage(c,0,(altoa-altoc)/2,this);
										ga.drawImage(b,anchoc+4,(altoa-altob)/2,this);
										multiplicamenosuno=false;	
									}else{
										anchoa=anchoc+anchotexto+anchob+4;
										
										a=createImage(anchoa,altoa);
										ga=(Graphics2D)a.getGraphics();
										opcionesGraphics2D(ga,anchoa,altoa, tamanoLetra);
										
										ga.drawImage(c,0,(altoa-altoc)/2,this);
										ga.drawString(tipo,anchoc+2,(altotexto+altoa)/2);
										ga.drawImage(b,anchoc+4+anchotexto,(altoa-altob)/2,this);
										multiplicamenosuno=false;
									}
									
									c=a;
								}
							}
						}else{ //Es un término del denominador
							if(d==null){
								d=createImage(anchob,altob);
								gd=(Graphics2D)d.getGraphics();
								opcionesGraphics2D(gd,anchob,altob, tamanoLetra);
								
								gd.drawImage(b,0,0,this);
								//d=b;
							}else{
								altod=d.getHeight(this);
								anchod=d.getWidth(this);
								altoa=Math.max(Math.max(altod,altob),altotexto);
								
								if(operadores.apareceLlave(tipo2) && !tipo2.equals("^") && !tipo2.equals("/") && operadores.devuelveOperador(tipo).prioridad()>operadores.devuelveOperador(tipo2).prioridad()){
									espacio=Math.min(Math.max(8,altob/6),16);
									
									anchoa=anchod+anchotexto+anchob+2*espacio+2;
									
									a=createImage(anchoa,altoa);
									ga=(Graphics2D)a.getGraphics();
									opcionesGraphics2D(ga,anchoa,altoa, tamanoLetra);
									
									ga.drawImage(d,0,(altoa-altod)/2,this);
									ga.drawString(tipo,anchod+1,(altotexto+altoa)/2);
									ga.draw(new Arc2D.Double(anchod+4+anchotexto,0,2*espacio-1,altoa,100,160, Arc2D.OPEN));
									ga.drawImage(b,anchod+2+anchotexto+espacio,(altoa-altob)/2,this);
									ga.draw(new Arc2D.Double(anchod+2+anchotexto+anchob,0,2*espacio-1,altoa,280,160, Arc2D.OPEN));
									
									d=a;
								}else{
									anchoa=anchod+anchotexto+anchob+4;
									
									a=createImage(anchoa,altoa);
									ga=(Graphics2D)a.getGraphics();
									opcionesGraphics2D(ga,anchoa,altoa, tamanoLetra);
									
									ga.drawImage(d,0,(altoa-altod)/2,this);
									ga.drawString(tipo,anchod+2,(altotexto+altod)/2);
									ga.drawImage(b,anchod+4+anchotexto,(altoa-altob)/2,this);
									
									d=a;
								}
							}
						}
					}
					
					if(c==null){
						anchotexto=metricaletra.stringWidth("1");
						
						c=createImage(anchotexto,altocompletotexto);
						gc=(Graphics2D)c.getGraphics();
						opcionesGraphics2D(gc,anchotexto,altocompletotexto, tamanoLetra);
						
						gc.drawString("1",0,altotexto);
						
						altod=d.getHeight(this);
						anchod=d.getWidth(this);
						altoc=c.getHeight(this);
						anchoc=c.getWidth(this);
						
						altoa=altocompletotexto+9+altod;
						anchoa=Math.max(anchoc+8,anchod+8);
						
						a=createImage(anchoa,altoa);
						ga=(Graphics2D)a.getGraphics();
						opcionesGraphics2D(ga,anchoa,altoa, tamanoLetra);
						
						ga.drawImage(c,(anchoa-anchoc)/2,0,this);
						ga.draw(new Line2D.Double(0,altoc+5,anchoa+8,altoc+5));
						ga.drawImage(d,(anchoa-anchod)/2,altoc + 9,this);
					}else if(d==null){
						a=c;
					}else if(multiplicamenosuno){
						anchotexto=metricaletra.stringWidth("-1");
						
						c=createImage(anchotexto,altocompletotexto);
						gc=(Graphics2D)c.getGraphics();
						opcionesGraphics2D(gc,anchotexto,altocompletotexto, tamanoLetra);
						
						gc.drawString("-1",0,altotexto);
						
						altod=d.getHeight(this);
						anchod=d.getWidth(this);
						altoc=c.getHeight(this);
						anchoc=c.getWidth(this);
						
						altoa=altocompletotexto+9+altod;
						anchoa=Math.max(anchoc+8,anchod+8);
						
						a=createImage(anchoa,altoa);
						ga=(Graphics2D)a.getGraphics();
						opcionesGraphics2D(ga,anchoa,altoa, tamanoLetra);
						
						ga.drawImage(c,(anchoa-anchoc)/2,0,this);
						ga.draw(new Line2D.Double(0,altoc+5,anchoa+8,altoc+5));
						ga.drawImage(d,(anchoa-anchod)/2,altoc + 9,this);
					}else{
						altod=d.getHeight(this);
						anchod=d.getWidth(this);
						altoc=c.getHeight(this);
						anchoc=c.getWidth(this);
						
						altoa=altoc+9+altod;
						anchoa=Math.max(anchoc+8,anchod+8);
						
						a=createImage(anchoa,altoa);
						ga=(Graphics2D)a.getGraphics();
						opcionesGraphics2D(ga,anchoa,altoa, tamanoLetra);
						
						ga.drawImage(c,(anchoa-anchoc)/2,0,this);
						ga.draw(new Line2D.Double(0,altoc+5,anchoa+8,altoc+5));
						ga.drawImage(d,(anchoa-anchod)/2,altoc + 9,this);
					}
					
					return a;
				}else{ //Cualquier otro operador
					c=createImage(1,1);
					
					for(int i=2;i<=expresionPrefijo.getNumeroOperandos();i++){
						tipo2=expresionPrefijo.getOperando(i).getTipo();
						
						if(operadores.apareceLlave(tipo2) && !tipo2.equals("^") && !tipo2.equals("/") && operadores.devuelveOperador(tipo).prioridad()>operadores.devuelveOperador(tipo2).prioridad()){
							b=graficaExpresion(expresionPrefijo.getOperando(i),g, tamanoLetra);
							altob=b.getHeight(this);
							anchob=b.getWidth(this);
							
							anchotexto=metricaletra.stringWidth(tipo);
							
							altoc=Math.max(Math.max(altoa,altob),altotexto);
							
							espacio=Math.min(Math.max(8,altob/6),16);
							
							anchoc=anchoa+anchotexto+anchob+2*espacio+2;
							
							c=createImage(anchoc,altoc);
							gc=(Graphics2D)c.getGraphics();
							opcionesGraphics2D(gc,anchoc,altoc, tamanoLetra);
							
							gc.drawImage(a,0,(altoc-altoa)/2,this);
							if(tipo.equals("+")){ //Si es una suma hay que ponerla un poquito más arriba
								gc.drawString(tipo,anchoa+1,(altotexto+altoc)/2-2);
							}else{
								gc.drawString(tipo,anchoa+1,(altotexto+altoc)/2);
							}
							gc.draw(new Arc2D.Double(anchoa+4+anchotexto,0,2*espacio-1,altoc,100,160, Arc2D.OPEN));
							gc.drawImage(b,anchoa+2+anchotexto+espacio,(altoc-altob)/2,this);
							gc.draw(new Arc2D.Double(anchoa+2+anchotexto+anchob,0,2*espacio-1,altoc,280,160, Arc2D.OPEN));
							
						//Si es una suma y el siguiente término es una multiplicación de un número negativo por algo entonces le quita el menos (a+-3*b = a-3*b)
						}else if(tipo.equals("+") && (tipo2.equals("·") && expresionPrefijo.getOperando(i).getOperando(1).getTipo().length()>5 && ("EnteroLargo DecimalLargo Racional").indexOf(expresionPrefijo.getOperando(i).getOperando(1).getTipo())!=-1 && (expresionPrefijo.getOperando(i).getOperando(1).getOperando(1).getTipo()).indexOf("-")!=-1)){
							comodin=expresionPrefijo.getOperando(i).clonarNodo();
							
							tipo2=comodin.getOperando(1).getOperando(1).getTipo().substring(1);
							comodin.getOperando(1).eliminaOperando(1);
							comodin.getOperando(1).agregaOperando(new Nodo(tipo2),1);
							
							b=graficaExpresion(comodin,g, tamanoLetra);
							altob=b.getHeight(this);
							anchob=b.getWidth(this);
							
							//Si lo anterior era una suma y lo que sigue es una potencia hay que subirla un poquito
							/*if(tipo2.equals("^")){
								altob+=altotexto/2 + 3;
							}*/
							
							anchotexto=metricaletra.stringWidth("-");
							
							altoc=Math.max(Math.max(altoa,altob),altotexto);
							
							anchoc=anchoa+anchotexto+anchob+4;
							
							c=createImage(anchoc,altoc);
							gc=(Graphics2D)c.getGraphics();
							opcionesGraphics2D(gc,anchoc,altoc, tamanoLetra);
							
							gc.drawImage(a,0,(altoc-altoa)/2,this);
							gc.drawString("-",anchoa+2,(altotexto+altoc)/2-2);
							gc.drawImage(b,anchoa+4+anchotexto,(altoc-altob)/2,this);
							
						//En este caso es si es una suma y el segundo término es un número negativo (a+-2 = a-2)
						}else if(tipo.equals("+") && tipo2.length()>5 && ("EnteroLargo DecimalLargo Racional").indexOf(tipo2)!=-1 && expresionPrefijo.getOperando(i).getOperando(1).getTipo().indexOf("-")!=-1){
							comodin=expresionPrefijo.getOperando(i).clonarNodo();
							
							tipo2=comodin.getOperando(1).getTipo().substring(1);
							comodin.eliminaOperando(1);
							comodin.agregaOperando(new Nodo(tipo2),1);
							
							b=graficaExpresion(comodin,g, tamanoLetra);
							altob=b.getHeight(this);
							anchob=b.getWidth(this);
							
							anchotexto=metricaletra.stringWidth("-");
							
							altoc=Math.max(Math.max(altoa,altob),altotexto);
							
							anchoc=anchoa+anchotexto+anchob+4;
							
							c=createImage(anchoc,altoc);
							gc=(Graphics2D)c.getGraphics();
							opcionesGraphics2D(gc,anchoc,altoc, tamanoLetra);
							
							gc.drawImage(a,0,(altoc-altoa)/2,this);
							gc.drawString("-",anchoa+2,(altotexto+altoc)/2-2);
							gc.drawImage(b,anchoa+4+anchotexto,(altoc-altob)/2,this);
						}else{
							b=graficaExpresion(expresionPrefijo.getOperando(i),g, tamanoLetra);
							altob=b.getHeight(this);
							anchob=b.getWidth(this);
							
							//Si lo anterior era una suma y lo que sigue es una potencia hay que subirla un poquito
							if(tipo.equals("+") && tipo2.equals("^")){
								altob+=altotexto/2 + 3;
							}
							
							anchotexto=metricaletra.stringWidth(tipo);
							
							altoc=Math.max(Math.max(altoa,altob),altotexto);
							
							anchoc=anchoa+anchotexto+anchob+4;
							
							c=createImage(anchoc,altoc);
							gc=(Graphics2D)c.getGraphics();
							opcionesGraphics2D(gc,anchoc,altoc, tamanoLetra);
							
							gc.drawImage(a,0,(altoc-altoa)/2,this);
							if(tipo.equals("+")){ //Si es una suma hay que ponerla un poquito más arriba
								gc.drawString(tipo,anchoa+1,(altotexto+altoc)/2-2);
							}else{
								gc.drawString(tipo,anchoa+2,(altotexto+altoc)/2);
							}
							gc.drawImage(b,anchoa+4+anchotexto,(altoc-altob)/2,this);
						}
						a=c;
						altoa=a.getHeight(this);
						anchoa=a.getWidth(this);
					}
					return c;
				}
	
			}else{ //Si no es porque tiene que ser una función
				if(expresionPrefijo.getNumeroOperandos()==0){
					if(funciones.apareceLlave(tipo) || funcionesUsuario.apareceLlave(tipo)){
						tipo=tipo.concat("( )");
					}
					anchotexto=metricaletra.stringWidth(tipo);
					
					c=createImage(anchotexto,altocompletotexto);
					gc=(Graphics2D)c.getGraphics();
					opcionesGraphics2D(gc,anchotexto,altocompletotexto, tamanoLetra);
					
					gc.drawString(tipo,0,altotexto);
					
					return c;
				}else{
					a=graficaExpresion(expresionPrefijo.getOperando(1),g, tamanoLetra);
					altoa=a.getHeight(this);
					anchoa=a.getWidth(this);
					
					if(tipo.equals("abs")){
						altoc=altoa;
						anchoc=anchoa+8;
						
						c=createImage(anchoc,altoc);
						gc=(Graphics2D)c.getGraphics();
						opcionesGraphics2D(gc,anchoc,altoc, tamanoLetra);
						
						gc.drawImage(a,4,0,this);
						gc.draw(new Line2D.Double(0,0,0,altoc));
						gc.draw(new Line2D.Double(anchoc-1,0,anchoc-1,altoc));
						return c;
					}else if(tipo.equals("sqrt")){
						espacio=Math.min(Math.max(10,altoa/3),20);
						
						altoc=altoa+4;
						anchoc=anchoa+espacio+8;
						
						c=createImage(anchoc,altoc);
						gc=(Graphics2D)c.getGraphics();
						opcionesGraphics2D(gc,anchoc,altoc, tamanoLetra);
							
						gc.drawImage(a,espacio+4,4,this);
						gc.draw(new Line2D.Double(0,altoc/2,espacio/3,altoc/3));
						gc.draw(new Line2D.Double(espacio/3,altoc/3,2*espacio/3,altoc));
						gc.draw(new Line2D.Double(2*espacio/3,altoc,espacio,0));
						gc.draw(new Line2D.Double(espacio,0,anchoc-2,0));
						gc.draw(new Line2D.Double(anchoc-2,0,anchoc-1,altoc/9));
						return c;
					}else{
						tipo2=" , ";
						for(int i=2;i<=expresionPrefijo.getNumeroOperandos();i++){
							
							b=graficaExpresion(expresionPrefijo.getOperando(i),g, tamanoLetra);
							altob=b.getHeight(this);
							anchob=b.getWidth(this);
							
							anchotexto2=metricaletra.stringWidth(tipo2);
							
							altoc=Math.max(Math.max(altoa,altob),altotexto);
							anchoc=anchoa+anchotexto2+anchob+2;
							
							c=createImage(anchoc,altoc);
							gc=(Graphics2D)c.getGraphics();
							opcionesGraphics2D(gc,anchoc,altoc, tamanoLetra);
							
							gc.drawImage(a,0,(altoc-altoa)/2,this);
							gc.drawString(tipo2,anchoa+2,altoc/2+altotexto/3);//altoc-altotexto/3);
							gc.drawImage(b,anchoa+2+anchotexto2,(altoc-altob)/2,this);
							
							a=c;
							altoa=a.getHeight(this);
							anchoa=a.getWidth(this);
						}
						
						espacio=Math.min(Math.max(8,altoa/6),16);
						anchotexto=metricaletra.stringWidth(tipo);
						
						altoc=Math.max(altoa,altotexto);
						anchoc=anchotexto+anchoa+2*espacio+2;
						
						c=createImage(anchoc,altoc);
						gc=(Graphics2D)c.getGraphics();
						opcionesGraphics2D(gc,anchoc,altoc, tamanoLetra);
						
						gc.drawString(tipo,0,(altotexto+altoc)/2);
						gc.draw(new Arc2D.Double(anchotexto+2,0,2*espacio-1,altoc,100,160, Arc2D.OPEN));//);anchotexto+espacio,altoc/2,espacio,altoc,90,180);
						gc.drawImage(a,anchotexto+espacio+2,(altoc-altoa)/2,this);
						gc.draw(new Arc2D.Double(anchotexto+2+anchoa,0,2*espacio-1,altoc,280,160, Arc2D.OPEN));//anchotexto+espacio+anchoa,altoc/2,espacio,altoc,270,180);
						return c;
					}
				}
			}

		}catch(Exception e){
			e.printStackTrace();
			throw new SintaxException(e.getMessage());
		}
	} //funcion f
	
	private void opcionesGraphics2D(Graphics2D g, int ancho, int alto, int tamanoLetra){
		final BasicStroke anchoLinea = new BasicStroke(1.0f);
		Font fuente=Opciones.getTipoLetra();
		
		g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		g.setPaint(Opciones.getColorLetra());
		g.setBackground(Opciones.getColorFondo());
		g.clearRect(0,0,ancho,alto);
		g.setStroke(anchoLinea);
		fuente=new Font(fuente.getFontName(), fuente.getStyle(), tamanoLetra>=8?tamanoLetra:8);
		g.setFont(fuente);
	}
	
	/**
	 *Función que devuelve el número de imagen.
	 *Este se debe poner con setNumeroImagen
	 *@return El número relacionado con esta imagen.
	 */
	public int getNumeroImagen(){
		return numeroImagen;
	}
	
	/**
	 *Función que le asigna el número a la imagen.
	 *@param nuevoNumeroImagen El nuevo número relacionado con esta imagen.
	 */
	public void setNumeroImagen(int nuevoNumeroImagen){
		numeroImagen=nuevoNumeroImagen;
	}
	
}//fin de Evaluador