package itcr.mathexplorer.traductores;

import itcr.mathexplorer.exceptions.*;
import itcr.mathexplorer.expresion.*;
import itcr.mathexplorer.tablas.*;

/**
 *@(#)TraductorPrefijoInfijo.java
 *
 *Clase que recibe una expresión en notación prefijo
 *y la formatea en formato infijo en forma de String.
 *
 *Para formatear en String la expresión exprEnPrefijo se 
 *utiliza Obj.toString(exprEnPrefijo).
 *
 *Esta clase está pensada para utilizarse en combinación con
 *la clase TraductorPrefijo que traduce una expresión
 *matemática escrita en lenguaje normal a prefijo.
 *
 *
 *@author Alexander Borbón Alpízar
 */
public class TraductorPrefijoInfijo{

	//VARIABLES PRIVADAS
	
	private TablaOperadores operadores;
	private TablaFunciones funciones;
	private TablaConstantes constantes;
	private TablaFuncionesUsuario funcionesUsuario;


	//CONSTRUCTORES
	
	/**
	 *Traductor que no recibe la expresión por lo que la inicializa vacía.
	 *Se reciben las tablas de operadores, funciones y constantes para hacer la traducción.
	 *@param tablaoperadores La tabla con los operadores.
	 *@param tablafunciones La tabla con las funciones.
	 *@param tablaconstantes La tabla con las constantes.
	 *@param tablafuncionesusuario La tabla con las funciones definidas por el usuario.
	 */
	public TraductorPrefijoInfijo(TablaOperadores tablaoperadores, TablaFunciones tablafunciones, TablaConstantes tablaconstantes, TablaFuncionesUsuario tablafuncionesusuario){
		operadores=tablaoperadores;
		funciones=tablafunciones;
		constantes=tablaconstantes;
		funcionesUsuario=tablafuncionesusuario;
	}
	
	//FUNCIONES PÚBLICAS
	
	/**
	 *Función que recibe una expresión en prefijo y la convierte a String.
	 *@param expresionPrefijo La expresión a traducir.
	 *@return El String que representa a la expresión.
	 *@exception SintaxException Si no encuentra la expresión dada en las tablas
	 */
	public String traducir(Nodo expresionPrefijo) throws SintaxException{
		String comodin, tipo, tipo2;
		int i, j;
		
		tipo=expresionPrefijo.getTipo();
		if ((tipo.length()>5 && ("EnteroLargo DecimalLargo Constante Variable Boolean").indexOf(tipo)!=-1) || constantes.apareceLlave(tipo)){ //Si el tipo es Entero, EnteroLargo, Decimal, DecimalLargo, Constante, Variable o Boolean
			comodin=((Nodo)expresionPrefijo.getOperando(1)).getTipo();
		}else if(tipo.length()>7 && "RacionalLargo".indexOf(tipo)!=-1){
			comodin=((Nodo)expresionPrefijo.getOperando(1)).getTipo() + "/" + ((Nodo)expresionPrefijo.getOperando(2)).getTipo();
		}else if(tipo.equals("Indefinido")){
			comodin="Indefinido";
		}else if(operadores.apareceLlave(tipo)){ //Si es un operador
			comodin="";
			for(i=1;i<=expresionPrefijo.getNumeroOperandos();i++){
				tipo2=expresionPrefijo.getOperando(i).getTipo();
				if(operadores.apareceLlave(tipo2) && operadores.devuelveOperador(tipo).prioridad()>operadores.devuelveOperador(tipo2).prioridad()){
					comodin=comodin.concat(" ( ");
					comodin=comodin.concat(traducir(expresionPrefijo.getOperando(i)));
					comodin=comodin.concat(" ) ");
					if(i!=expresionPrefijo.getNumeroOperandos() || tipo.equals("!")){
						comodin=comodin.concat(tipo);
					}
				}else if(tipo2.equals("^") && operadores.devuelveOperador(tipo).prioridad()>=operadores.devuelveOperador("^").prioridad()){
					comodin=comodin.concat(" ( ");
					comodin=comodin.concat(traducir(expresionPrefijo.getOperando(i)));
					comodin=comodin.concat(" ) ");
					if(i!=expresionPrefijo.getNumeroOperandos() || tipo.equals("!")){
						comodin=comodin.concat(tipo);
					}
				}else{
					comodin=comodin.concat(traducir(expresionPrefijo.getOperando(i)));
					if(i!=expresionPrefijo.getNumeroOperandos() || tipo.equals("!")){
						comodin=comodin.concat(tipo);
					}else if(i==expresionPrefijo.getNumeroOperandos() && tipo.equals("not")){
						comodin="not(".concat(comodin.concat(")"));
					}
				}
			}
			
			//comodin=comodin.concat(traducir(expresion.getOperando(i)));
		}else if(tipo.equals("Lista")){
			comodin="{ ";
			if(expresionPrefijo.getNumeroOperandos()!=0){
				for(i=1;i<=expresionPrefijo.getNumeroOperandos()-1;i++){
					comodin=comodin.concat(traducir(expresionPrefijo.getOperando(i)));
					comodin=comodin.concat(", ");
				}
				comodin=comodin.concat(traducir(expresionPrefijo.getOperando(i)));
			}
			comodin=comodin.concat(" }");
		}else if(tipo.equals("Matriz")){
			comodin="[ ";
			if(expresionPrefijo.getNumeroOperandos()!=0){
				for(i=1;i<=expresionPrefijo.getNumeroOperandos()-1;i++){
					for(j=1;j<=expresionPrefijo.getOperando(i).getNumeroOperandos()-1;j++){
						comodin=comodin.concat(traducir(expresionPrefijo.getOperando(i).getOperando(j)));
						comodin=comodin.concat(", ");
					}
					comodin=comodin.concat(traducir(expresionPrefijo.getOperando(i).getOperando(j)));
					comodin=comodin.concat("; ");
				}
				
				for(j=1;j<=expresionPrefijo.getOperando(i).getNumeroOperandos()-1;j++){
					comodin=comodin.concat(traducir(expresionPrefijo.getOperando(i).getOperando(j)));
					comodin=comodin.concat(", ");
				}
				comodin=comodin.concat(traducir(expresionPrefijo.getOperando(i).getOperando(j)));
			}
			comodin=comodin.concat(" ]");
		}else if(funciones.apareceLlave(tipo)){ //Si es una función
			comodin=tipo;
			comodin=comodin.concat("( ");
			if(expresionPrefijo.getNumeroOperandos()!=0){
				for(i=1;i<=expresionPrefijo.getNumeroOperandos()-1;i++){
					comodin=comodin.concat(traducir(expresionPrefijo.getOperando(i)));
					comodin=comodin.concat(", ");
				}
				comodin=comodin.concat(traducir(expresionPrefijo.getOperando(i)));
			}
			comodin=comodin.concat(" )");
		}else if(tipo.equals(":=")){
			comodin=traducir(expresionPrefijo.getOperando(1));
			comodin=comodin.concat(" := ");
			comodin=comodin.concat(traducir(expresionPrefijo.getOperando(2)));
		}else if(funcionesUsuario.apareceLlave(tipo)){ //Si es una función definida por el usuario
			comodin=tipo;
			if(expresionPrefijo.getNumeroOperandos()>0){
				comodin=comodin.concat("( ");
				for(i=1;i<expresionPrefijo.getNumeroOperandos();i++){
					comodin=comodin.concat(traducir(expresionPrefijo.getOperando(i)));
					comodin=comodin.concat(", ");
				}
				comodin=comodin.concat(traducir(expresionPrefijo.getOperando(i)));
				comodin=comodin.concat(" )");
			}
		}else{
			throw new SintaxException("Expresión no encontrada en las tablas");
		}
		return comodin;
	}
	
}//fin de TraductorPrefijoInfijo